///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "BitstreamWriter.h"
#include "..\src\SceneTreeParser\SceneTreeManager.h"


CBitstreamWriter::CBitstreamWriter()
{
	m_uUsedBits		= 0;
	m_uBuffer		= 0;

	m_uPacketSeqNum	= 0;
}

CBitstreamWriter::~CBitstreamWriter()
{
	m_auBitstream.clear();
	m_auTemp.clear();
}

/************************************************************************/
// GetEachNode
// Desc		: Get the LASeR/SAF node in order
// Param	: root element pointer
// History
//	PSH (2008. 9. 22)	- first release
/************************************************************************/
lsrErr GetEachNode(const void *pRoot)
{
	lsrErr err = LASeRNoErr;

	if (pRoot)
	{
		err = SetSceneRoot(pRoot);
		CONFIRM(err);
	}

	CLASeR	*pLASeRNode	= NULL;
	CSAF	*pSAFNode	= NULL;

	CBitstreamWriter *pBitstreamWriter = new CBitstreamWriter();

	while ((pSAFNode == NULL) || (pSAFNode->GetElementType() != SAF_EndOfSAFSession))
	{
		err = GetNextObject(&pLASeRNode, &pSAFNode);
		CONFIRM(err);

		err = pBitstreamWriter->GetRealNodeType(pLASeRNode, pSAFNode);
	}

	delete pBitstreamWriter;
	pBitstreamWriter = NULL;

	return err;
}

/************************************************************************/
// GetRealNodeType
// Desc		: Get the real nod type & branch off 
// Param	: LASeR element pointer or SAF element pointer
// History
//	PSH (2008. 9. 23)	- first release
/************************************************************************/
lsrErr CBitstreamWriter::GetRealNodeType(CLASeR *pLASeRNode, CSAF *pSAFNode)
{
	lsrErr err = LASeRNoErr;

	UINT uRealNodeType = 0;

	if ((pLASeRNode != NULL) && (pSAFNode == NULL))
		uRealNodeType = pLASeRNode->GetElementType();

	else if ((pLASeRNode == NULL) && (pSAFNode != NULL))
		uRealNodeType = pSAFNode->GetElementType();

	else
		err = LASeREncoderErr;

	switch (uRealNodeType)
	{
	case SAF_SceneHeader:
		err = Encode((CSceneHeader *)pSAFNode);
	}

	return err;
}

/************************************************************************/
// Write8
// Desc		: Put it's variable into the LASeR bitstream
// Param	: 8 bits variable, flag
// History
//	PSH (2008. 9. 25)	- first release
/************************************************************************/
void CBitstreamWriter::Write8(const UINT8 uValue, UINT8 uFlag)
{
	deque<UINT8> *pBitstream = NULL;

	pBitstream = (uFlag ? &m_auBitstream : &m_auTemp);

	if (m_uUsedBits)
		WriteBits(8, (UINT)uValue);

	else
		pBitstream->push_back(uValue);
}

/************************************************************************/
// Write16
// Desc		: Put it's variable into the LASeR bitstream
// Param	: 16 bits variable, flag
// History
//	PSH (2008. 9. 25)	- first release
/************************************************************************/
void CBitstreamWriter::Write16(const UINT16 uValue, UINT8 uFlag)
{
	deque<UINT8> *pBitstream = NULL;

	pBitstream = (uFlag ? &m_auBitstream : &m_auTemp);

	if (m_uUsedBits)
		WriteBits(16, (UINT)uValue);

	else
	{
		UINT8 uTemp = 0;

		uTemp = (UINT8)(uValue >> 8);
		pBitstream->push_back(uTemp);

		uTemp = (UINT8)(uValue % 256);
		pBitstream->push_back(uTemp);
	}
}

/************************************************************************/
// Write24
// Desc		: Put it's variable into the LASeR bitstream
// Param	: 24 bits variable, flag
// History
//	PSH (2008. 9. 29)	- first release
/************************************************************************/
void CBitstreamWriter::Write24(const UINT uValue, UINT8 uFlag)
{
	deque<UINT8> *pBitstream = NULL;

	pBitstream = (uFlag ? &m_auBitstream : &m_auTemp);

	if (m_uUsedBits)
		WriteBits(24, uValue);

	else
	{
		UINT8 uTemp0, uTemp1, uTemp2;
		uTemp0 = uTemp1 = uTemp2 = 0;

		uTemp2 = (UINT8)(uValue >> 16);
		uTemp1 = (UINT8)(uValue << 8 >> 16);
		uTemp0 = (UINT8)(uValue % 256);

		pBitstream->push_back(uTemp2);
		pBitstream->push_back(uTemp1);
		pBitstream->push_back(uTemp0);
	}
}

/************************************************************************/
// Write32
// Desc		: Put it's variable into the LASeR bitstream
// Param	: 32 bits variable, flag
// History
//	PSH (2008. 9. 25)	- first release
/************************************************************************/
void CBitstreamWriter::Write32(const UINT uValue, UINT8 uFlag)
{
	deque<UINT8> *pBitstream = NULL;

	pBitstream = (uFlag ? &m_auBitstream : &m_auTemp);

	if (m_uUsedBits)
		WriteBits(24, uValue);

	else
	{
		UINT8 uTemp0, uTemp1, uTemp2, uTemp3;
		uTemp0 = uTemp1 = uTemp2 = uTemp3 = 0;

		uTemp3 = (UINT8)(uValue >> 24);
		uTemp2 = (UINT8)(uValue << 8 >> 24);
		uTemp1 = (UINT8)(uValue << 16 >> 24);
		uTemp0 = (UINT8)(uValue % 256);

		pBitstream->push_back(uTemp3);
		pBitstream->push_back(uTemp2);
		pBitstream->push_back(uTemp1);
		pBitstream->push_back(uTemp0);
	}
}

/************************************************************************/
// WriteBits
// Desc		: Put it's variable into the LASeR bitstream
// Param	: wanted bits & 8 bits variable, flag
// History
//	PSH (2008. 9. 30)	- first release
/************************************************************************/
void CBitstreamWriter::WriteBits(const UINT uBits, const UINT uValue, UINT8 uFlag)
{
	deque<UINT8> *pBitstream = NULL;

	pBitstream = (uFlag ? &m_auBitstream : &m_auTemp);

	UINT	uTempBits	= uBits;
	UINT	uTempValue	= 0;

	while ((m_uUsedBits + uTempBits) > 7)
	{
		uTempBits	-= (8 - m_uUsedBits);
		uTempValue	= uValue >> uTempBits;

		m_uBuffer += (UINT8)uTempValue;

		pBitstream->push_back(m_uBuffer);

		m_uBuffer = m_uUsedBits = 0;
	}

	uTempValue = uValue << (8 - m_uUsedBits - uTempBits);

	m_uBuffer	+= (UINT8)uTempValue;
	m_uUsedBits	+= (UINT8)uTempBits;
}

/************************************************************************/
// Flush
// Desc		: Fill the 1 byte buffer and put it's variable into the LASeR bitstream
// History
//	PSH (2008. 10. 14)	- first release
/************************************************************************/
void CBitstreamWriter::Flush(UINT8 uFlag)
{
	deque<UINT8> *pBitstream = NULL;

	pBitstream = (uFlag ? &m_auBitstream : &m_auTemp);

	pBitstream->push_back(m_uBuffer);

	m_uBuffer = m_uUsedBits = 0;
}

/************************************************************************/
// Encode
// Desc		: Encode it's variable
// Param	: the pointer of SceneHeader
// History
//	PSH (2008. 9. 22)	- first release
/************************************************************************/
lsrErr CBitstreamWriter::Encode(CSceneHeader *pSceneHeader)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// SceneHeader
	// objectTypeIndication = 9
	Write8(pSceneHeader->GetUINT8Attr(ATTR_OBJECTTYPEINDICATION, &err));
	CONFIRM(err);

	// streamType = 3
	Write8(pSceneHeader->GetUINT8Attr(ATTR_STREAMTYPE, &err));
	CONFIRM(err);

	// timeStampResolution
	Write24(pSceneHeader->GetUINTAttr(ATTR_TIMESTAMPRESOLUTION, &err));
	CONFIRM(err);

	// bufferSize
	Write16((UINT16)pSceneHeader->GetUINTAttr(ATTR_BUFFERSIZE, &err));
	CONFIRM(err);

	/************************************************************************/
	// get the LASeRHeader
	CLASeR	*pLASeRNode	= NULL;
	CSAF	*pSAFNode	= NULL;

	err = GetNextObject(&pLASeRNode, &pSAFNode);
	CONFIRM(err);

	CLASeRHeader *pLASeRHeader = (CLASeRHeader *)pLASeRNode;

	/************************************************************************/
	// LASeRHeader
	// profile
	Write8(pLASeRHeader->GetUINT8Attr(ATTR_PROFILE, &err));
	CONFIRM(err);

	// level
	Write8(pLASeRHeader->GetUINT8Attr(ATTR_LEVEL, &err));
	CONFIRM(err);

	// reserved
	WriteBits(3, 0);

	// pointsCodecType
	WriteBits(2, pLASeRHeader->GetUINT8Attr(ATTR_POINTSCODECTYPE, &err));
	CONFIRM(err);

	// pathComponents
	WriteBits(4, pLASeRHeader->GetUINT8Attr(ATTR_PATHCOMPONENTS, &err));
	CONFIRM(err);

	// useFullRequestHost
	WriteBits(1, pLASeRHeader->GetUINT8Attr(ATTR_USEFULLREQUESTHOST, &err));
	CONFIRM(err);

	UINT8	bHasTimeResolution	= (pLASeRHeader->GetUINTAttr(ATTR_TIMERESOLUTION, &err) != 0);
	CONFIRM(err);
	WriteBits(1, bHasTimeResolution);

	if (bHasTimeResolution)
	{
		// timeResolution
		Write16((UINT16)pLASeRHeader->GetUINTAttr(ATTR_TIMERESOLUTION, &err));
		CONFIRM(err);
	}

	// colorComponentBits
	WriteBits(4, (pLASeRHeader->GetUINT8Attr(ATTR_COLORCOMPONENTBITS, &err) - 1));
	CONFIRM(err);

	// resolution
	WriteBits(4, pLASeRHeader->GetUINT8Attr(ATTR_RESOLUTION, &err));
	CONFIRM(err);

	// coordBits
	WriteBits(5, pLASeRHeader->GetUINT8Attr(ATTR_COORDBITS, &err));
	CONFIRM(err);

	// scaleBits_minus_coordBits
	WriteBits(4, pLASeRHeader->GetUINT8Attr(ATTR_SCALEBITSMINUSCOORDBITS, &err));
	CONFIRM(err);

	// newSceneIndicator
	WriteBits(1, pLASeRHeader->GetUINT8Attr(ATTR_NEWSCENEINDICATOR, &err));
	CONFIRM(err);

	WriteBits(3, 0);							// for padding

	// extensionID
	WriteBits(4, 4);							// why 4? I don't know
	WriteBits(1, 0);

	WriteBits(1, 0);

	Flush();

	/************************************************************************/
	// SceneHeader Return
	WriteBits(4, GetSAFType(pSceneHeader->GetElementType()), 1);
	WriteBits(12, 0, 1);

	/************************************************************************/
	// SAF Packet Header
	EncodeSAFPacketHeader(0, pSceneHeader->GetUINTAttr(ATTR_TIME, &err), (const UINT16)m_auTemp.size());

	// merge original & temporarily bitstream
	m_auBitstream.insert(m_auBitstream.end(), m_auTemp.begin(), m_auTemp.end());
	m_auTemp.clear();

	return err;
}

/************************************************************************/
// EncodeSAFPacketHeader
// Desc		: Encode it's variable
// Param	: rap, time, length
// History
//	PSH (2008. 10. 14)	- first release
/************************************************************************/
void CBitstreamWriter::EncodeSAFPacketHeader(const UINT8 bRap, const UINT uTime, const UINT16 uLength)
{
	WriteBits(1, bRap, 1);						// 1: m_auBitstream
	WriteBits(15, m_uPacketSeqNum, 1);
	++m_uPacketSeqNum;

	WriteBits(2, 1, 1);							// 0: DTS flag, 1 : CTS flag
	WriteBits(30, uTime, 1);

	Write16(uLength + 2, 1);
}

/************************************************************************/
// GetSAFType
// Desc		: Get SAF Type
// Param	: SAF enumeration
// History
//	PSH (2008. 10. 17)	- first release
/************************************************************************/
UINT8 CBitstreamWriter::GetSAFType(const UINT8 uSAFType)
{
	switch (uSAFType)
	{
	case SAF_SceneHeader:
	case SAF_MediaHeader:
	case SAF_ImageHeader:
		return 1;
		break;

	case SAF_EndOfStream:
		return 3;
		break;

	case SAF_SceneUnit:
	case SAF_MediaUnit:
	case SAF_ImageUnit:
		return 4;
		break;

	case SAF_EndOfSAFSession:
		return 5;
		break;

	case SAF_CacheUnit:
		return 6;
		break;

	case SAF_RemoteStreamHeader:
		return 7;
		break;

	default:
		return 0;
		break;
	}
}