///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//


// DragAndDropListBox.cpp : implementation file
//

#include "stdafx.h"
#include "DragAndDropListBox.h"


#define TID_SCROLLDOWN  100
#define TID_SCROLLUP    101

// CDragAndDropListBox

IMPLEMENT_DYNAMIC(CDragAndDropListBox, CListBox)
CDragAndDropListBox::CDragAndDropListBox()
: m_nMovingIndex(-1)
, m_nMoveToIndex(-1)
, m_bCaptured(FALSE)
, m_dwInterval(0)
{
}

CDragAndDropListBox::~CDragAndDropListBox()
{
}

BEGIN_MESSAGE_MAP(CDragAndDropListBox, CListBox)
   ON_WM_LBUTTONDOWN()
   ON_WM_LBUTTONUP()
   ON_WM_MOUSEMOVE()
   ON_WM_TIMER()
END_MESSAGE_MAP()


// CDragAndDropListBox message handlers

void CDragAndDropListBox::OnLButtonDown(UINT nFlags, CPoint point)
{
	CListBox::OnLButtonDown(nFlags, point);
	
	// clear all the flags
	m_nMovingIndex	= LB_ERR;
	m_nMoveToIndex	= LB_ERR;
	m_bCaptured		= FALSE;
	m_dwInterval	= 0;
	
	UINT8 bOutside = 0;

	//find the item that they want to drag
	//and keep track of it. Later in the mouse move
	//we will capture the mouse if this value is set
	int nIndex = (int)ItemFromPoint(point, (BOOL &)bOutside);

	if (nIndex != LB_ERR && !bOutside)
	{
		m_nMovingIndex = nIndex;
		SetCurSel(nIndex);
	}
}

void CDragAndDropListBox::OnLButtonUp(UINT nFlags, CPoint point)
{
	if (m_nMovingIndex != LB_ERR && m_bCaptured)
	{
		KillTimer(TID_SCROLLDOWN);
		KillTimer(TID_SCROLLUP);

		m_dwInterval	= 0;
		m_bCaptured		= FALSE;
		ReleaseCapture();

		CRect rRect;
		GetClientRect(&rRect);

		//if they are still within the listbox window
		if (rRect.PtInRect(point))
			InsertDraggedItem();

		else
		{
			Invalidate();
			UpdateWindow();
		}

		m_nMovingIndex = LB_ERR;
		m_nMoveToIndex = LB_ERR;
	}

	CListBox::OnLButtonUp(nFlags, point);
}

void CDragAndDropListBox::OnMouseMove(UINT nFlags, CPoint point)
{
	CListBox::OnMouseMove(nFlags, point);

	if (nFlags & MK_LBUTTON)
	{
		if (m_nMovingIndex != LB_ERR && !m_bCaptured)
		{
			SetCapture();
			m_bCaptured = TRUE;
		}

		UINT8 bOutside = 0;
		int nIndex = (int)ItemFromPoint(point, (BOOL &)bOutside);

		// if they our not on a particular item
		if (bOutside)
		{
			CRect rClientRect;
			GetClientRect(&rClientRect);

			// if they are still within the listbox window, then
			// simply select the last item as the drop point
			// else if they are outside the window then scroll the items
			if (rClientRect.PtInRect(point))
			{
				KillTimer(TID_SCROLLDOWN);
				KillTimer(TID_SCROLLUP);

				m_dwInterval	= 0;
				nIndex			= LB_ERR;
				bOutside		= FALSE;
			}

			else
				DoTheScrolling(point, rClientRect);
		}

		else
		{
			KillTimer(TID_SCROLLDOWN);
			KillTimer(TID_SCROLLUP);
			m_dwInterval = 0;
		}

		if (nIndex != m_nMoveToIndex && !bOutside)
			DrawTheLines(nIndex);
	}
}

void CDragAndDropListBox::DrawTheLines(UINT uIndex)
{
	CRect rRect, rClientRect;
	GetClientRect(&rClientRect);
	
	CDC *pDC = GetDC();

	UINT uWidth = 2;

	if (m_nMoveToIndex != m_nMovingIndex)
	{
		CPen Pen(PS_SOLID, uWidth, RGB(255,255,255));

		CPen *pOldPen = pDC->SelectObject(&Pen);

		if (m_nMoveToIndex != LB_ERR)
		{
			GetItemRect(m_nMoveToIndex, &rRect);

			if (rClientRect.PtInRect(rRect.TopLeft()))
			{
				rRect.right = rClientRect.right;
				pDC->MoveTo(rRect.left, rRect.top + 1);
				pDC->LineTo(rRect.right - (uWidth / 2), rRect.top + 1);
			}
		}

		else
		{
			GetItemRect(GetCount() - 1, &rRect);

			if (rClientRect.PtInRect(CPoint(0, rRect.bottom)))
			{
				rRect.right = rClientRect.right;
				pDC->MoveTo(rRect.left, rRect.bottom);
				pDC->LineTo(rRect.right - (uWidth / 2), rRect.bottom);
			}
		}

		pDC->SelectObject(pOldPen);
	}

	m_nMoveToIndex = uIndex;

	if (m_nMoveToIndex != m_nMovingIndex)
	{
		CPen Pen(PS_SOLID, uWidth, RGB(0,0,0));

		CPen *pOldPen = pDC->SelectObject(&Pen);

		if (m_nMoveToIndex != LB_ERR)
		{
			GetItemRect(uIndex, &rRect);

			if (rClientRect.PtInRect(rRect.TopLeft()))
			{
				rRect.right = rClientRect.right; // HOTFIX
				pDC->MoveTo(rRect.left, rRect.top + 1);
				pDC->LineTo(rRect.right - (uWidth / 2), rRect.top + 1);
			}
		}

		else
		{
			GetItemRect(GetCount() - 1, &rRect);

			if (rClientRect.PtInRect(CPoint(0, rRect.bottom)))
			{
				rRect.right = rClientRect.right;
				pDC->MoveTo(rRect.left, rRect.bottom);
				pDC->LineTo(rRect.right - (uWidth / 2), rRect.bottom);
			}
		}

		pDC->SelectObject(pOldPen);
	}

	ReleaseDC(pDC);
}

void CDragAndDropListBox::InsertDraggedItem()
{
	CString	strText		= _T("");
	DWORD	dwItemData	= 0;

	GetText(m_nMovingIndex, strText);
	dwItemData = GetItemData(m_nMovingIndex);

	if (m_nMovingIndex < m_nMoveToIndex)
	{
		SetRedraw(FALSE);

		int nTopIndex	= GetTopIndex();
		int nNewIndex	= InsertString(m_nMoveToIndex, strText);

		SetItemData(nNewIndex, dwItemData);
		DeleteString(m_nMovingIndex);

		SetCurSel(nNewIndex - 1);
		SetTopIndex(nTopIndex);

		SetRedraw(TRUE);

		//we have to redraw the window since we are deleting a string
		//after we set the current selection. DeleteString causes a 
		//focus rRect to show up under the current selection
		Invalidate();
		UpdateWindow();
	}

	else if (m_nMovingIndex > m_nMoveToIndex)
	{
		SetRedraw(FALSE);

		int nTopIndex = GetTopIndex();
		DeleteString(m_nMovingIndex);

		int nNewIndex = InsertString(m_nMoveToIndex,strText);
		SetItemData(nNewIndex, dwItemData);

		SetCurSel(nNewIndex);
		SetTopIndex(nTopIndex);

		SetRedraw(TRUE);

		//we have to redraw the window since we are deleting a string
		//after we set the current selection. DeleteString causes a 
		//focus rRect to show up under the current selection
		Invalidate();
		UpdateWindow();
	}

	GetParent()->SendMessage(WM_COMMAND, MAKEWPARAM(GetDlgCtrlID(), LBN_SELCHANGE), (LPARAM)m_hWnd);
}

void CDragAndDropListBox::DoTheScrolling(CPoint Point, CRect rClientRect)
{
	if (Point.y > rClientRect.Height())
	{
		DWORD dwInterval = 250 / (1 + ((Point.y - rClientRect.Height()) / GetItemHeight(0)));

		if (dwInterval != m_dwInterval)
		{
			m_dwInterval = dwInterval;
			SetTimer(TID_SCROLLDOWN, dwInterval, NULL);
			OnTimer(TID_SCROLLDOWN);
		}
	}

	else if (Point.y < 0)
	{
		DWORD dwInterval = 250 / (1 + (abs(Point.y) / GetItemHeight(1)));

		if (dwInterval != m_dwInterval)
		{
			m_dwInterval = dwInterval;
			SetTimer(TID_SCROLLUP, dwInterval, NULL);
			OnTimer(TID_SCROLLUP);
		}
	}

	else
	{
		KillTimer(TID_SCROLLDOWN);
		KillTimer(TID_SCROLLUP);
		m_dwInterval = 0;
	}
}

void CDragAndDropListBox::OnTimer(UINT nIDEvent)
{
	if (nIDEvent == TID_SCROLLDOWN)
	{
		DrawTheLines(m_nMoveToIndex + 1);
		SetTopIndex(GetTopIndex() + 1);
	}

	else if (nIDEvent == TID_SCROLLUP)
	{
		DrawTheLines(m_nMoveToIndex - 1);
		SetTopIndex(GetTopIndex() - 1);
	}

	CListBox::OnTimer(nIDEvent);
}

// Sort the List
// Returns		- Returns true for success
// bAscending	- indicate sort order
// low			- row to start scanning from - default row is 0
// high			- row to end scan. -1 indicates last row
UINT8 CDragAndDropListBox::SortList(BOOL bAscending, int low /*= 0*/, int high /*= -1*/)
{
	// Needless?
	if (GetCount() < 2)
		return TRUE;

	if (high == -1)
		high = GetCount() - 1;

	int lo = low;
	int hi = high;

	CString	midItem		= _T("");
	CString	strTemp		= _T("");
	CString strTemp2	= _T("");

	if (hi <= lo)
		return FALSE;

	GetText((lo + hi) / 2, midItem);

	// loop through the list until indices cross
	while (lo <= hi)
	{
		// rowText will hold all column text for one row
		// CStringArray rowText;

		// find the first element that is greater than or equal to 
		// the partition element starting from the left Index.
		if (bAscending)
		{
			while (1)
			{
				if (lo >= high)
					break;

				GetText(lo, strTemp);

				if (strTemp >= midItem)
					break;

				++lo;
			}
		}

		else
		{
			while (1)
			{
				if (lo >= high)
					break;

				GetText(lo, strTemp);

				if (strTemp <= midItem)
					break;

				++lo;
			}
		}

		// find an element that is smaller than or equal to 
		// the partition element starting from the right Index.
		if (bAscending)
		{
			while (1)
			{
				if (hi <= low)
					break;

				GetText(hi, strTemp);

				if (strTemp <= midItem)
					break;

				--hi;
			}
		}

		else
		{
			while (1)
			{
				if (hi <= low)
					break;

				GetText(hi, strTemp);

				if (strTemp >= midItem)
					break;

				--hi;
			}
		}

		// if the indexes have not crossed, swap
		// and if the items are not equal
		if (lo <= hi)
		{
			GetText(lo, strTemp);
			GetText(hi, strTemp2);

			if (strTemp != strTemp2)
			{
				DeleteString(lo);
				InsertString(lo, strTemp2);
				DeleteString(hi);
				InsertString(hi, strTemp);
			}

			++lo;
			--hi;
		}
	}

	// If the right index has not reached the left side of array
	// must now sort the left partition.
	if (low < hi)
		SortList(bAscending, low, hi);

	// If the left index has not reached the right side of array
	// must now sort the right partition.
	if (lo < high)
		SortList(bAscending , lo, high);

	return TRUE;
}