///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//


#include "stdafx.h"
#include "LASeRPlayer.h"

#include "LASeRPlayerView.h"
#include "MainFrm.h"

#include "..\src\libLASeR\Datatypes.h"

#include "JS_Functions.h"

#define INTERVAL		60
#define WINDOW_WIDTH	16
#define WINDOW_HEIGHT	132


void CLASeRPlayerView::CompareEventA(CPoint point)
{
	lsrErr err = LASeRNoErr;

	deque<CLASeR *>::iterator i;
	UINT8 bSelected = 0;

	for (i = m_apA.begin(); i < m_apA.end(); ++i)
	{
		bSelected = ConfirmEventRegion((*i)->GetChildren(0), point);

		if (bSelected)	break;
	}

	if (bSelected)
	{
		// click
		if (m_uEventType == 5)
		{
			CString strXlink = ((*i)->GetStrAttr(ATTR_XLINK_HREF, &err)).c_str();
			ErrorMessage(err);

			ShellExecute(this->m_hWnd, "open", "IEXPLORE.EXE", strXlink, NULL, SW_SHOW);
		}

		// mouseover
		else if (m_uEventType == 21)
			SetCursor(LoadCursor(0, IDC_HAND));
	}
}

void CLASeRPlayerView::CompareEventListener(CPoint point)
{
	lsrErr err = LASeRNoErr;

	deque<CLASeR *>::iterator i;
	UINT8 bSelected = 0;

	for (i = m_apListener.begin(); i < m_apListener.end(); ++i)
	{
		if (m_uEventType == 35)
		{
			tstring strEventparam = (*i)->GetStrAttr(ATTR_EVENT_PARAM, &err);

			m_fDiagonal = GetDiagonalDisplaysize(point.x, point.y);

			float fEventparam = (float)atof(strEventparam.c_str());

			m_uWidthUnit = 0;

			ConvertLPtoDP(&fEventparam, &fEventparam);

//			m_uWidthUnit = 5;

			if (fEventparam <= m_fDiagonal)
			{
				bSelected = 1;
			}
			if (bSelected)
				break;
		}
		else
		{
			bSelected = ConfirmEventRegion(
				SearchLASeRElementUsingID((*i)->GetStrAttr(ATTR_OBSERVER, &err)), point);

			if (bSelected)
				break;
		}
		if (bSelected)
			break;
	}

	if (bSelected)
	{
		CLSRConditional *pHandler = SearchConditional((*i)->GetStrAttr(ATTR_HANDLER, &err));
		DrawLSRConditional(pHandler);
	}
}

void CLASeRPlayerView::ConvertLPtoDP(float *f_width, float *f_height)
{
	if (m_uWidthUnit == 5) {}   //px
	else if (m_uWidthUnit == 3) //pt  1/72 in
	{
		CPoint pt;
		pt.x = (LONG)*f_width  * 20;
		pt.y = (LONG)*f_height * 20;
		CClientDC dc(this);
		dc.SetMapMode(MM_TWIPS);
		dc.LPtoDP(&pt);
		*f_width  =  (float)pt.x;
		*f_height =  (float)-pt.y;
	}
	else if (m_uWidthUnit == 4) //pc  12pt
	{
		CPoint pt;
		pt.x = (LONG)*f_width  * 20*12;
		pt.y = (LONG)*f_height * 20*12;
		CClientDC dc(this);
		dc.SetMapMode(MM_TWIPS);
		dc.LPtoDP(&pt);
		*f_width  =  (float)pt.x;
		*f_height =  (float)-pt.y;

	}
	else if (m_uWidthUnit <= 2)
	{
		CPoint pt;
		pt.x = (LONG)*f_width * 100;
		pt.y = (LONG)*f_height * 100;
		CClientDC dc(this);

		switch (m_uWidthUnit)
		{
		case 0:   //inch
			dc.SetMapMode(MM_LOENGLISH);
			break;

		case 1:   //cm
			dc.SetMapMode(MM_LOMETRIC);
			break;

		case 2:   //mm
			dc.SetMapMode(MM_HIMETRIC);
			break;

		default:
			break;
		}
		dc.LPtoDP(&pt);
		*f_width  =  (float)pt.x;
		*f_height = (float)-pt.y;
	}
	else 
	{
		// % 
	}
}

UINT8 CLASeRPlayerView::ConfirmEventRegion(CLASeR *pLASeR, CPoint point)
{
	lsrErr	err		= LASeRNoErr;

	UINT8	bResult = 0;

	switch (pLASeR->GetElementType())
	{
	case SVG_circle:
		{
			int		nWidth	= (int)pLASeR->GetFloatAttr(ATTR_CX, &err) - point.x;
			ErrorMessage(err);
			int		nHeight	= (int)pLASeR->GetFloatAttr(ATTR_CY, &err) - point.y;
			ErrorMessage(err);
			UINT	uR		= (UINT)pLASeR->GetFloatAttr(ATTR_R, &err);
			ErrorMessage(err);

			if ((uR * uR) > (UINT)((nWidth * nWidth) + (nHeight * nHeight)))
				bResult = 1;
		}
		break;

	case SVG_rect:
		{
			int	uX1	= (int)pLASeR->GetFloatAttr(ATTR_X, &err);
			ErrorMessage(err);
			int	uY1	= (int)pLASeR->GetFloatAttr(ATTR_Y, &err);
			ErrorMessage(err);
			int	uX2	= uX1 + (int)pLASeR->GetFloatAttr(ATTR_WIDTH, &err);
			ErrorMessage(err);
			int	uY2	= uY1 + (int)pLASeR->GetFloatAttr(ATTR_HEIGHT, &err);
			ErrorMessage(err);

			if ((point.x > uX1) && (point.x < uX2) && (point.y > uY1) && (point.y < uY2))
				bResult = 1;
		}		
		break;

	case SVG_image:
		{
			int	uX1	= (int)pLASeR->GetFloatAttr(ATTR_X, &err);
			ErrorMessage(err);
			int	uY1	= (int)pLASeR->GetFloatAttr(ATTR_Y, &err);
			ErrorMessage(err);
			int	uX2	= uX1 + (int)pLASeR->GetFloatAttr(ATTR_WIDTH, &err);
			ErrorMessage(err);
			int	uY2	= uY1 + (int)pLASeR->GetFloatAttr(ATTR_HEIGHT, &err);
			ErrorMessage(err);

			if ((point.x > uX1) && (point.x < uX2) && (point.y > uY1) && (point.y < uY2))
				bResult = 1;
		}		
		break;

	case SVG_text:
		{
			int	uX1	= (int)pLASeR->GetFloatAttr(ATTR_X, &err);
			ErrorMessage(err);
			int	uY1	= (int)pLASeR->GetFloatAttr(ATTR_Y, &err);
			ErrorMessage(err);
			int	uX2	= uX1 + (int)((pLASeR->GetFloatAttr(ATTR_FONT_SIZE, &err) - 6)
				* (pLASeR->GetStrAttr(ATTR_TEXTCONTENT, &err)).size());
			ErrorMessage(err);
			int	uY2	= uY1 + (int)pLASeR->GetFloatAttr(ATTR_FONT_SIZE, &err) + 10;
			ErrorMessage(err);

			if ((point.x > uX1) && (point.x < uX2) && (point.y > uY1) && (point.y < uY2))
				bResult = 1;
		}

		break;

	default:
		/* nothing */
		break;
	}

	return bResult;
}

void CLASeRPlayerView::ReplaceCursor(CPoint point)
{
	lsrErr err = LASeRNoErr;

	CLASeR *pLASeRlink = SearchLASeRElementUsingID(m_apcursor[0]->GetStrAttr(ATTR_XLINK_HREF, &err));
	ErrorMessage(err);

	deque<CLASeR *> cursor;

	if (pLASeRlink->GetElementType() == SVG_g)
	{
		UINT8 childsize = pLASeRlink->GetChildrenSize();

		for (UINT8 i=0; i<childsize; i++)
		{
			cursor.push_back(pLASeRlink->GetChildren(i));
		}
	}
	else
	{
		cursor.push_back(pLASeRlink);
	}

	UINT num_cursor = cursor.size();

	for (UINT i=0; i<num_cursor; i++)
	{
		switch (cursor[i]->GetElementType())
		{
		case SVG_line:
			{
				GetShapeElement(cursor[i], 0);

				int uWidth  = (int)((cursor[i]->GetFloatAttr(ATTR_X2, &err) - cursor[i]->GetFloatAttr(ATTR_X1, &err)) / 2);
				int uHeight = (int)((cursor[i]->GetFloatAttr(ATTR_Y2, &err) - cursor[i]->GetFloatAttr(ATTR_Y1, &err)) / 2);

				cursor[i]->SetAttr(ATTR_X1, (float)point.x - uWidth);
				cursor[i]->SetAttr(ATTR_Y1, (float)point.y - uHeight);
				cursor[i]->SetAttr(ATTR_X2, (float)point.x + uWidth);
				cursor[i]->SetAttr(ATTR_Y2, (float)point.y + uHeight);

				GetShapeElement(cursor[i]);
			}
			break;
		}
	}
	ShowCursor(NULL);
}

void CLASeRPlayerView::DrawSVG(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	deque<float> afBox = pLASeR->GetFloatListAttr(ATTR_VIEWBOX, &err);

	float fwidth, fheight;

	if (afBox.size() != 0)
	{
		m_rCanvas.left   = (UINT)afBox[0];
		m_rCanvas.top    = (UINT)afBox[1];
		m_rCanvas.right  = (UINT)afBox[2];
		m_rCanvas.bottom = (UINT)afBox[3];
	}
	else
	{
		m_uWidthUnit     = pLASeR->GetUINT8Attr(ATTR_WIDTH_UNIT,  &err);
		m_uHeightUnit    = pLASeR->GetUINT8Attr(ATTR_HEIGHT_UNIT, &err);
		m_rCanvas.left   = 0;
		m_rCanvas.top    = 0;
		fwidth           = pLASeR->GetFloatAttr(ATTR_WIDTH,  &err);
		fheight          = pLASeR->GetFloatAttr(ATTR_HEIGHT, &err);

		ConvertLPtoDP(&fwidth, &fheight);
		m_rCanvas.right  = (LONG)fwidth;
		m_rCanvas.bottom = (LONG)fheight;
	}

	//test
	m_fDiagonal = GetDiagonalDisplaysize(m_rCanvas.right, m_rCanvas.bottom);

	CMainFrame *pMainFrame = (CMainFrame*)AfxGetMainWnd();

	m_rCanvas.right += WINDOW_WIDTH;
	m_rCanvas.bottom += WINDOW_HEIGHT;
	pMainFrame->MoveWindow(m_rCanvas);
	pMainFrame->CenterWindow();

	m_pDC = GetDC();

	CBrush brush, *pOldBrush;
	brush.CreateSolidBrush(RGB(248, 248, 248));
	pOldBrush = (CBrush *)m_pDC->SelectObject(&brush);

	CPen pen, *pOldPen;
	pen.CreatePen(PS_SOLID, 1, RGB(248, 248, 248));
	pOldPen = (CPen *)m_pDC->SelectObject(&pen);		

	m_pDC->Rectangle(m_rCanvas);

	m_pDC->SelectObject(pOldBrush);
	m_pDC->SelectObject(pOldPen);

	ReleaseDC(m_pDC);

	// test
	m_apTest = pLASeR->GetChildren();
}

void CLASeRPlayerView::SetAniCommonAttrs(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/

	AnimationAttrs *pThreadParams = new AnimationAttrs;

	pThreadParams->pView		= this;
	pThreadParams->ueletype     = pLASeR->GetElementType();
	pThreadParams->pCLASeR      = pLASeR;
	pThreadParams->pLASeR		= pLASeR->GetParent();
	pThreadParams->uIndex		= m_uAniEleCount;
	++m_uAniEleCount;

	pThreadParams->uAttrType	= pLASeR->GetUINT8Attr(ATTR_ATTRIBUTENAME, &err);
	ErrorMessage(err);

	deque<tstring> astrTemp		= pLASeR->GetStrListAttr(ATTR_BEGIN, &err);
	pThreadParams->uBegin		= (UINT)(ReadTimesType(astrTemp[0].c_str(), &err) * 1000);
	ErrorMessage(err);
	pThreadParams->afFrom		= pLASeR->GetFloatListAttr(ATTR_FROM, &err);
	ErrorMessage(err);
	pThreadParams->afMid		= pThreadParams->afFrom;
	pThreadParams->afTo			= pLASeR->GetFloatListAttr(ATTR_TO, &err);
	ErrorMessage(err);
	pThreadParams->afBy			= pLASeR->GetFloatListAttr(ATTR_BY, &err);
	ErrorMessage(err);
	pThreadParams->uDur			= (pLASeR->GetFloatAttr(ATTR_DUR, &err) * 1000);
	ErrorMessage(err);

	pThreadParams->uAccumulate	= pLASeR->GetUINT8Attr(ATTR_ACCUMULATE, &err);
	ErrorMessage(err);

	pThreadParams->uAdditive	= pLASeR->GetUINT8Attr(ATTR_ADDITIVE, &err);
	ErrorMessage(err);

	pThreadParams->uCalcMode	= pLASeR->GetUINT8Attr(ATTR_CALCMODE, &err);
	ErrorMessage(err);

	pThreadParams->afValues		= pLASeR->GetFloatListAttr(ATTR_VALUES, &err);
	ErrorMessage(err);

// 	pThreadParams->auFill		= pLASeR->GetUINT8ListAttr(ATTR_FILL, &err);
// 	ErrorMessage(err);
// 	pThreadParams->strFill		= (pLASeR->GetStrAttr(ATTR_FILL, &err));
// 	ErrorMessage(err);

	pThreadParams->uRepeatCount	= pLASeR->GetFloatAttr(ATTR_REPEATCOUNT, &err);
	ErrorMessage(err);
	pThreadParams->uRepeatDur	= pLASeR->GetFloatAttr(ATTR_REPEATDUR, &err);
	ErrorMessage(err);

	/************************************************************************/

	pThreadParams->uEleCount = (pThreadParams->afFrom).size();

	if (pThreadParams->uEleCount != (pThreadParams->afTo).size())
	{
		err = LASeRValidationErr;
		ErrorMessage(err);
	}

	if (((pThreadParams->afTo).size() == 0) && ((pThreadParams->afBy).size() !=0))
	{
		for (UINT8 i = 0; i < pThreadParams->uEleCount; ++i)
		{
			float fTempInc = (float)((pThreadParams->afBy[i]) * INTERVAL / pThreadParams->uDur);

			(pThreadParams->afIncrement).push_back(fTempInc);
		}
	}
	else
	{
		for (UINT8 i = 0; i < pThreadParams->uEleCount; ++i)
		{
			float fTempInc = (float)((pThreadParams->afTo[i] - pThreadParams->afFrom[i]) * INTERVAL / pThreadParams->uDur);

			(pThreadParams->afIncrement).push_back(fTempInc);
		}
	}

	pThreadParams->uIncCount	= (UINT)(pThreadParams->uDur / INTERVAL);
	pThreadParams->uCount		= 1;

	m_apAnimationAttrs.push_back(pThreadParams);
}

void CLASeRPlayerView::DrawLSRAnimationScroll(CLASeR *pLASeR)
{
	SetAniCommonAttrs(pLASeR);

	CWinThread *pThread = ::AfxBeginThread(ThreadAni, *(m_apAnimationAttrs.rbegin()));
}

void CLASeRPlayerView::DrawAnimate(CLASeR *pLASeR)
{
	SetAniCommonAttrs(pLASeR);

	CWinThread *pThread = ::AfxBeginThread(ThreadAni, *(m_apAnimationAttrs.rbegin()));
}

void CLASeRPlayerView::DrawAnimateColor(CLASeR *pLASeR)
{
	SetAniCommonAttrs(pLASeR);

	CWinThread *pThread = ::AfxBeginThread(ThreadAni, *(m_apAnimationAttrs.rbegin()));
}

void CLASeRPlayerView::DrawAnimateMotion(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	SetAniCommonAttrs(pLASeR);

	CLASeR *children_temp =	pLASeR->GetChildren(0);
	CLASeR *LASeRmPath = SearchLASeRElementUsingID(children_temp->GetStrAttr(ATTR_XLINK_HREF, &err));

	tstring strD = LASeRmPath->GetStrAttr(ATTR_D, &err);

	TCHAR	*pBegin	= (TCHAR *)strD.c_str();	// position of number's beginning
	TCHAR	*pBlank	= NULL;						// position of blank
	TCHAR	*pComma	= NULL;						// position of comma
	TCHAR	*pToken	= NULL;
	TCHAR	chChar	= NULL;
	UINT8	uSize	= 0;

	while (1)
	{
		chChar = *pBegin;

		pBlank = _tcschr(pBegin, ' ');
		pToken = pBlank;

		switch (chChar)
		{
		case 'L':
		case 'M':
			pBegin = pToken + 1;

			pComma = _tcschr(pBegin, ',');

			pToken = pComma;
			m_afMotionList.push_back((float)_tcstod(pBegin, &pToken));

			pBegin = pToken + 1;

			pToken = _tcschr(pBegin, ' ');
			m_afMotionList.push_back((float)_tcstod(pBegin, &pToken));
			break;

		case 'Z':
			uSize = (UINT8)m_afMotionList.size();

			pToken = pBegin;
			break;

		default:
			/* nothing */
			break;
		}
		/************************************************************************/

		pBegin = pToken + 1;

		if ((*pToken != ' ') && (*pToken != ','))
			break;
	}

	CWinThread *pThread = ::AfxBeginThread(ThreadAni, *(m_apAnimationAttrs.rbegin()));
}

void CLASeRPlayerView::DrawAnimateTransform(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	SetAniCommonAttrs(pLASeR);

	UINT8 uAttr_type = pLASeR->GetUINT8Attr(ATTR_TYPE, &err);
	ErrorMessage(err);

	switch (uAttr_type)
	{
	case 0:
		(*(m_apAnimationAttrs.rbegin()))->uAttrType = ATTR_LSR_TRANSLATION;
		break;
	case 1:
		(*(m_apAnimationAttrs.rbegin()))->uAttrType = ATTR_LSR_SCALE;
		break;
	case 2:
		(*(m_apAnimationAttrs.rbegin()))->uAttrType = ATTR_LSR_ROTATION;
		break;
	default:
		break;
	}

	CWinThread *pThread = ::AfxBeginThread(ThreadAni, *(m_apAnimationAttrs.rbegin()));
}

void CLASeRPlayerView::DrawDefs(CLASeR *pLASeR)
{
	deque<CLASeR *> m_apchild = pLASeR->GetChildren();
	int m_childsize = pLASeR->GetChildrenSize();
	for (int i=0; i<m_childsize; i++)
	{
		(m_apchild[i])->SetDeactivate();
	}
}

void CLASeRPlayerView::DrawScript(CString strScript)
{
	lsrErr err = LASeRNoErr;

	// create a JS runtime
	m_pRuntime = JS_NewRuntime(1024 * 1024);

	if (!m_pRuntime)
	{
		err = LASeRJSEngineErr;
		ErrorMessage(err);
	}

	// create a JS context
	m_pContext = JS_NewContext(m_pRuntime, 8192);

	if (!m_pContext)
	{
		err = LASeRJSEngineErr;
		ErrorMessage(err);
	}

	// create a JS object
	m_pGlobal = JS_NewObject(m_pContext, NULL, NULL, NULL);

	if (!m_pGlobal)
	{
		err = LASeRJSEngineErr;
		ErrorMessage(err);
	}

	// init global class & function
	if (!JS_InitStandardClasses(m_pContext, m_pGlobal))
	{
		err = LASeRJSEngineErr;
		ErrorMessage(err);
	}

	/************************************************************************/
	// defining event class
	JSClass clsEvent = {"evt", 0,
		JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
		JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
	};

	m_clsEvent = clsEvent;

	static JSPropertySpec apropEvent[] = {
		{"target", 1, 0},
		{0}
	};

	static JSFunctionSpec afuncEvent[] = {
		{"getTarget", JGetTarget, 0, 0, 0},
		{0}
	};

	m_pEvent = JS_InitClass(m_pContext, m_pGlobal, NULL, &m_clsEvent, NULL, 0, apropEvent, afuncEvent, NULL, NULL);

	jsval jsEvent = OBJECT_TO_JSVAL(m_pEvent);
	/************************************************************************/

	// run javascript
	uintN size = strScript.GetLength();

	JSBool ok = JS_EvaluateScript(m_pContext, m_pGlobal, (LPCTSTR)strScript, size, NULL, 0, &m_jsRVal);
}

void CLASeRPlayerView::DrawSet(CLASeR *pLASeR)
{
	SetAniCommonAttrs(pLASeR);

	CWinThread *pThread = ::AfxBeginThread(ThreadAni, *(m_apAnimationAttrs.rbegin()));
}

void CLASeRPlayerView::DrawVideo(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	CRect rVideo;

	rVideo.left		= (int)pLASeR->GetFloatAttr(ATTR_X, &err);
	ErrorMessage(err);
	rVideo.top		= (int)pLASeR->GetFloatAttr(ATTR_Y, &err);
	ErrorMessage(err);
	rVideo.right	= rVideo.left + (int)pLASeR->GetFloatAttr(ATTR_WIDTH, &err);
	ErrorMessage(err);
	rVideo.bottom	= rVideo.top + (int)pLASeR->GetFloatAttr(ATTR_HEIGHT, &err);
	ErrorMessage(err);

	m_VideoPanel.MoveWindow(rVideo);

	CMainFrame *pMainFrame = (CMainFrame *)AfxGetMainWnd();

	CString strVideoSource	= (GetSourceUsingHref(pLASeR)).c_str();
	CString	strPath			= pMainFrame->GetSource();

	if (strVideoSource.Find(':') == -1)
		strVideoSource = strPath.Left(strPath.ReverseFind('\\') + 1) + strVideoSource;

	pMainFrame->DoPlayFile(strVideoSource);
}

void CLASeRPlayerView::DrawLSRConditional(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	CLSRConditional *temp = (CLSRConditional *)pLASeR;
	
	UINT8 command_count;
	command_count = temp->GetChildrenSize();
	for (int i=0; i<command_count; i++)
	{
		lsrErr err = LASeRNoErr;
		CSAF	*pSAFForDrawing		= NULL;
		err		= GetNextObject(&pLASeR, &pSAFForDrawing);

		ProcessCommand(pLASeR);
	}

	GetShapeElement(SearchLASeRElementUsingID(pLASeR->GetStrAttr(ATTR_REF, &err)));
}

void CLASeRPlayerView::DrawLSRRectClip(CLASeR *pLASeR, UINT8 bDraw)
{
	lsrErr err = LASeRNoErr;

	m_pDC = GetDC();

	/************************************************************************/
	// get rectClip attributes
	deque<float>    afsize          = pLASeR->GetFloatListAttr(ATTR_SIZE, &err);
	ErrorMessage(err);
	float           fx              = pLASeR->GetFloatAttr(ATTR_X, &err);
	ErrorMessage(err);
	float           fy              = pLASeR->GetFloatAttr(ATTR_Y, &err);
	ErrorMessage(err);
	float           fwidth          = afsize[0];
	float           fheight         = afsize[1];

	/************************************************************************/
	
	RectF rectF;

	if (fx == 0)
	{
		CRect rect;
		GetClientRect(rect);
		rectF.X = (REAL)rect.left + (REAL)(rect.right/2);
	}	
	else
	{
		rectF.X = fx - (fwidth/2);
	}

	if (fy == 0)
	{
		CRect rect;
		GetClientRect(rect);
		rectF.Y = (REAL)rect.top + (REAL)(rect.bottom/2);
	}	
	else
	{
		rectF.Y = fy - (fheight/2);
	}

	rectF.Width = fwidth;
	rectF.Height = fheight;

	if (bDraw)
	{
		Region region(rectF);

		Graphics graphics(m_pDC->m_hDC);

		graphics.SetClip(&region);

		deque<CLASeR *> m_apchild = pLASeR->GetChildren();
		int m_childsize = pLASeR->GetChildrenSize();
		for (int i=0; i<m_childsize; i++)
		{
			DrawRect(m_apchild[i], &graphics);
			(m_apchild[i])->SetDeactivate();
		}
	}
	else
	{
		deque<CLASeR *> m_apchild = pLASeR->GetChildren();
		int m_childsize = pLASeR->GetChildrenSize();
		for (int i=0; i<m_childsize; i++)
		{
			GetShapeElement(m_apchild[i], FALSE);
		}
	}
	ReleaseDC(m_pDC);
}

void CLASeRPlayerView::DrawUse(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	CLASeR *pLASeRlink = SearchLASeRElementUsingID(pLASeR->GetStrAttr(ATTR_XLINK_HREF, &err));
}

UINT ThreadAni(LPVOID pParam)
{
	AnimationAttrs		*pAniParams	= (AnimationAttrs *)pParam;
	CLASeRPlayerView	*pAppView	= (CLASeRPlayerView *)pAniParams->pView;

	Sleep(pAniParams->uBegin);

	pAppView->SetTimer(pAniParams->uIndex, INTERVAL, NULL);

	return 0;
}
