///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "stdafx.h"
#include "LASeRPlayer.h"

#include "LASeRPlayerView.h"
#include "MainFrm.h"

#define WINDOW_WIDTH	16
#define WINDOW_HEIGHT	132

void CLASeRPlayerView::DrawCircle(CLASeR *pLASeR, UINT8 bDraw /* = TRUE */)
{
	lsrErr err = LASeRNoErr;

	m_pDC = GetDC();

	/************************************************************************/
	// get rect attributes
	UINT8           uVisibility     = pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);
	deque<UINT8>	auFill			= pLASeR->GetUINT8ListAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	tstring			strFill			= pLASeR->GetStrAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	UINT8			uFillOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_FILL_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	deque<UINT8>	auStroke		= pLASeR->GetUINT8ListAttr(ATTR_STROKE, &err);
	ErrorMessage(err);
	UINT8			uStrokeOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_STROKE_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	float			fStrokeWidth	= pLASeR->GetFloatAttr(ATTR_STROKE_WIDTH, &err);
	ErrorMessage(err);

	deque<float>	afRotate		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
	ErrorMessage(err);
	deque<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
	ErrorMessage(err);
	deque<float>	afTranslate		= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
	ErrorMessage(err);

	float    		nCX				= pLASeR->GetFloatAttr(ATTR_CX, &err);
	ErrorMessage(err);
	float			nCY				= pLASeR->GetFloatAttr(ATTR_CY, &err);
	ErrorMessage(err);
	float			nR				= pLASeR->GetFloatAttr(ATTR_R, &err);
	ErrorMessage(err);

	ConvertLPtoDP(&nR, &nR);

	float             nX              = nCX - nR;
	float             nY              = nCY - nR;
	float             nWidth          = 2 * nR;
	float             nHeight         = 2 * nR;

	/************************************************************************/

	if (uVisibility == 1) { /*hidden*/ }
	else
	{
		Graphics graphics(m_pDC->m_hDC);

		if (bDraw)
		{
			if (strFill != _T(""))
			{
				// call DrawLin~
				CLASeR *LASeRGradient = SearchLASeRElementUsingID(strFill);
				UINT8 Gradient_type = LASeRGradient->GetElementType();

				if (Gradient_type == SVG_linearGradient)
				{
					UINT8 strgradientunit = LASeRGradient->GetUINT8Attr(ATTR_GRADIENTUNITS, &err);
					ErrorMessage(err);

					float x1     = LASeRGradient->GetFloatAttr(ATTR_X1, &err);
					float y1     = LASeRGradient->GetFloatAttr(ATTR_Y1, &err);
					float x2     = LASeRGradient->GetFloatAttr(ATTR_X2, &err);
					float y2     = LASeRGradient->GetFloatAttr(ATTR_Y2, &err);

					if (strgradientunit == 1)
					{
						x1     = nX;
						y1     = nY;
						x2     = nX + nWidth;
						y2     = nY + nHeight;
					}

					deque<CLASeR *> child_Stop = LASeRGradient->GetChildren();

					deque<UINT8> Stop_color0      = child_Stop[0]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
					ErrorMessage(err);
					UINT8        ustop_opacity0	  = (UINT8)(child_Stop[0]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
					ErrorMessage(err);
					float        fstop_offset0    = child_Stop[0]->GetFloatAttr(ATTR_OFFSET, &err);
					ErrorMessage(err);

					x1 += fstop_offset0 * nWidth;
					y1 += fstop_offset0 * nHeight;

					deque<UINT8> Stop_color1      = child_Stop[1]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
					ErrorMessage(err);
					UINT8        ustop_opacity1	  = (UINT8)(child_Stop[1]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
					ErrorMessage(err);
					float        fstop_offset1    = child_Stop[1]->GetFloatAttr(ATTR_OFFSET, &err);
					ErrorMessage(err);

					x2 =  fstop_offset1 * x2;
					y2 =  fstop_offset1 * y2;				

					LinearGradientBrush linerBrush(Point((INT)x1,(INT)y1), Point((INT)x2,(INT)y2), 
						Color((BYTE)ustop_opacity0, Stop_color0[0], Stop_color0[1], Stop_color0[2]),
						Color((BYTE)ustop_opacity1, Stop_color1[0], Stop_color1[1], Stop_color1[2]));

					graphics.FillEllipse(&linerBrush, nX, nY, nWidth, nHeight);
				}
				else if (Gradient_type == SVG_radialGradient)
				{
					UINT8 strgradientunit = LASeRGradient->GetUINT8Attr(ATTR_GRADIENTUNITS, &err);
					ErrorMessage(err);

					float cx = LASeRGradient->GetFloatAttr(ATTR_CX, &err);
					float cy = LASeRGradient->GetFloatAttr(ATTR_CY, &err);
					float r  = LASeRGradient->GetFloatAttr(ATTR_R, &err);

					if (strgradientunit == 1)
					{
						cx     = nX + nWidth/2;
						cy     = nY + nHeight/2;
						r      = nWidth/2;
					}

					deque<CLASeR *> child_Stop = LASeRGradient->GetChildren();

					deque<UINT8> Stop_color0      = child_Stop[0]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
					ErrorMessage(err);
					UINT8        ustop_opacity0	  = (UINT8)(child_Stop[0]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
					ErrorMessage(err);
					float        fstop_offset0    = child_Stop[0]->GetFloatAttr(ATTR_OFFSET, &err);
					ErrorMessage(err);

					cx += fstop_offset0 * nWidth;
					cy += fstop_offset0 * nHeight;

					deque<UINT8> Stop_color1      = child_Stop[1]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
					ErrorMessage(err);
					UINT8        ustop_opacity1	  = (UINT8)(child_Stop[1]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
					ErrorMessage(err);
					float        fstop_offset1    = child_Stop[1]->GetFloatAttr(ATTR_OFFSET, &err);
					ErrorMessage(err);

					r =  fstop_offset1 * r;		

					GraphicsPath path;
					path.AddEllipse(cx-r, cy-r, 2*r, 2*r);

					int count =1;
					PathGradientBrush pathBrush(&path);
					pathBrush.SetCenterColor(Color((BYTE)ustop_opacity0, Stop_color0[0], Stop_color0[1], Stop_color0[2]));
					pathBrush.SetSurroundColors(&Color((BYTE)ustop_opacity1, Stop_color1[0], Stop_color1[1], Stop_color1[2]),&count);
					graphics.FillEllipse(&pathBrush, nX, nY, nWidth, nHeight);
				}
				graphics.TranslateTransform(afTranslate[0], afTranslate[1]);
				graphics.ScaleTransform(afScale[0], afScale[1]);
				graphics.RotateTransform(afRotate[0]);

			}
			else if (auFill.size() == 3)			// rgb
			{
				SolidBrush brush(Color(uFillOpacity, auFill[0], auFill[1], auFill[2]));
				graphics.FillEllipse(&brush, nX, nY, nWidth, nHeight);
			}

			if (auStroke.size() == 3)
			{
				Pen pen(Color(uStrokeOpacity, auStroke[0], auStroke[1], auStroke[2]), fStrokeWidth);
				graphics.DrawEllipse(&pen, nX, nY, nWidth, nHeight);
			}
		}
		else
		{
			SolidBrush brush(Color(255, 248, 248, 248));
			graphics.FillEllipse(&brush, nX, nY, nWidth, nHeight);

			Pen pen(Color(255, 248, 248, 248), fStrokeWidth);
			graphics.DrawEllipse(&pen, nX, nY, nWidth, nHeight);
		}
		ReleaseDC(m_pDC);
	}
}

void CLASeRPlayerView::DrawCircle(CLASeR *pLASeR, Graphics *graphics, UINT8 bDraw /* = TRUE */)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// get rect attributes
	UINT8           uVisibility     = pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);
	deque<UINT8>	auFill			= pLASeR->GetUINT8ListAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	tstring			strFill			= (pLASeR->GetStrAttr(ATTR_FILL, &err));
	ErrorMessage(err);
	UINT8			uFillOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_FILL_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	deque<UINT8>	auStroke		= pLASeR->GetUINT8ListAttr(ATTR_STROKE, &err);
	ErrorMessage(err);
	UINT8			uStrokeOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_STROKE_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	float			fStrokeWidth	= pLASeR->GetFloatAttr(ATTR_STROKE_WIDTH, &err);
	ErrorMessage(err);

	deque<float>	afRotate		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
	ErrorMessage(err);
	deque<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
	ErrorMessage(err);
	deque<float>	afTranslate		= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
	ErrorMessage(err);

	float				nCX				= pLASeR->GetFloatAttr(ATTR_CX, &err);
	ErrorMessage(err);
	float				nCY				= pLASeR->GetFloatAttr(ATTR_CY, &err);
	ErrorMessage(err);
	float				nR				= pLASeR->GetFloatAttr(ATTR_R, &err);
	ErrorMessage(err);
	ConvertLPtoDP(&nR, &nR);

	float             nX              = nCX - nR;
	float             nY              = nCY - nR;
	float             nWidth          = 2 * nR;
	float             nHeight         = 2 * nR;

	/************************************************************************/

	if (uVisibility == 1) { /*hidden*/	}
	else
	{
		if (bDraw)
		{
			if (strFill != _T(""))
			{
				// call DrawLin~
				CLASeR *LASeRGradient = SearchLASeRElementUsingID(strFill);
				UINT8 Gradient_type = LASeRGradient->GetElementType();

				if (Gradient_type == SVG_linearGradient)
				{
					UINT8 strgradientunit = LASeRGradient->GetUINT8Attr(ATTR_GRADIENTUNITS, &err);
					ErrorMessage(err);

					float x1     = LASeRGradient->GetFloatAttr(ATTR_X1, &err);
					float y1     = LASeRGradient->GetFloatAttr(ATTR_Y1, &err);
					float x2     = LASeRGradient->GetFloatAttr(ATTR_X2, &err);
					float y2     = LASeRGradient->GetFloatAttr(ATTR_Y2, &err);

					if (strgradientunit == 1)
					{
						x1     = nX;
						y1     = nY;
						x2     = nX + nWidth;
						y2     = nY + nHeight;
					}

					deque<CLASeR *> child_Stop = LASeRGradient->GetChildren();
					
					deque<UINT8> Stop_color0      = child_Stop[0]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
					ErrorMessage(err);
					UINT8        ustop_opacity0	  = (UINT8)(child_Stop[0]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
					ErrorMessage(err);
					float        fstop_offset0    = child_Stop[0]->GetFloatAttr(ATTR_OFFSET, &err);
					ErrorMessage(err);

					x1 += fstop_offset0 * nWidth;
					y1 += fstop_offset0 * nHeight;

					deque<UINT8> Stop_color1      = child_Stop[1]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
					ErrorMessage(err);
					UINT8        ustop_opacity1	  = (UINT8)(child_Stop[1]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
					ErrorMessage(err);
					float        fstop_offset1    = child_Stop[1]->GetFloatAttr(ATTR_OFFSET, &err);
					ErrorMessage(err);

					x2 =  fstop_offset1 * x2;
					y2 =  fstop_offset1 * y2;	

					LinearGradientBrush linerBrush(Point((INT)x1,(INT)y1), Point((INT)x2,(INT)y2), 
						Color((BYTE)ustop_opacity0, Stop_color0[0], Stop_color0[1], Stop_color0[2]),
						Color((BYTE)ustop_opacity1, Stop_color1[0], Stop_color1[1], Stop_color1[2]));

					graphics->FillRectangle(&linerBrush, nX, nY, nWidth, nHeight);
				}
				else if (Gradient_type == SVG_radialGradient)
				{
					UINT8 strgradientunit = LASeRGradient->GetUINT8Attr(ATTR_GRADIENTUNITS, &err);
					ErrorMessage(err);

					float cx = LASeRGradient->GetFloatAttr(ATTR_CX, &err);
					float cy = LASeRGradient->GetFloatAttr(ATTR_CY, &err);
					float r  = LASeRGradient->GetFloatAttr(ATTR_R, &err);

					if (strgradientunit == 1)
					{
						cx     = nX + nWidth/2;
						cy     = nY + nHeight/2;
						r      = nWidth/2;
					}

					deque<CLASeR *> child_Stop = LASeRGradient->GetChildren();
					
					deque<UINT8> Stop_color0      = child_Stop[0]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
					ErrorMessage(err);
					UINT8        ustop_opacity0	  = (UINT8)(child_Stop[0]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
					ErrorMessage(err);
					float        fstop_offset0    = child_Stop[0]->GetFloatAttr(ATTR_OFFSET, &err);
					ErrorMessage(err);

					cx += fstop_offset0 * nWidth;
					cy += fstop_offset0 * nHeight;

					deque<UINT8> Stop_color1      = child_Stop[1]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
					ErrorMessage(err);
					UINT8        ustop_opacity1	  = (UINT8)(child_Stop[1]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
					ErrorMessage(err);
					float        fstop_offset1    = child_Stop[1]->GetFloatAttr(ATTR_OFFSET, &err);
					ErrorMessage(err);

					r =  fstop_offset1 * r;			

					GraphicsPath path;
					path.AddEllipse(cx-r, cy-r, 2*r, 2*r);

					int count =1;
					PathGradientBrush pathBrush(&path);
					pathBrush.SetCenterColor(Color((BYTE)ustop_opacity0, Stop_color0[0], Stop_color0[1], Stop_color0[2]));
					pathBrush.SetSurroundColors(&Color((BYTE)ustop_opacity1, Stop_color1[0], Stop_color1[1], Stop_color1[2]),&count);
					graphics->FillEllipse(&pathBrush, nX, nY, nWidth, nHeight);
				}
				graphics->TranslateTransform(afTranslate[0], afTranslate[1]);
				graphics->ScaleTransform(afScale[0], afScale[1]);
				graphics->RotateTransform(afRotate[0]);

			}
			else if (auFill.size() == 3)			// rgb
			{
				SolidBrush brush(Color(uFillOpacity, auFill[0], auFill[1], auFill[2]));
				graphics->FillEllipse(&brush, nX, nY, nWidth, nHeight);
			}

			if (auStroke.size() == 3)
			{
				Pen pen(Color(uStrokeOpacity, auStroke[0], auStroke[1], auStroke[2]), fStrokeWidth);
				graphics->DrawEllipse(&pen, nX, nY, nWidth, nHeight);
			}
		}
		else
		{
			SolidBrush brush(Color(255, 248, 248, 248));
			graphics->FillEllipse(&brush, nX, nY, nWidth, nHeight);

			Pen pen(Color(255, 248, 248, 248), fStrokeWidth);
			graphics->DrawEllipse(&pen, nX, nY, nWidth, nHeight);
		}
	}
}


void CLASeRPlayerView::DrawRect(CLASeR *pLASeR, UINT8 bDraw /* = TRUE */)
{
	lsrErr err = LASeRNoErr;

	m_pDC = GetDC();

	/************************************************************************/
	// get rect attributes
	UINT8           uVisibility     = pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);

	UINT8			u_display		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);

	deque<UINT8>	auFill			= pLASeR->GetUINT8ListAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	tstring			strFill			= pLASeR->GetStrAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	UINT8			uFillOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_FILL_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	deque<UINT8>	auStroke		= pLASeR->GetUINT8ListAttr(ATTR_STROKE, &err);
	ErrorMessage(err);
	UINT8			uStrokeOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_STROKE_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	float			fStrokeWidth	= pLASeR->GetFloatAttr(ATTR_STROKE_WIDTH, &err);
	ErrorMessage(err);

	deque<float>	afRotate		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
	ErrorMessage(err);
	deque<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
	ErrorMessage(err);
	deque<float>	afTranslate		= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
	ErrorMessage(err);

	UINT8			uVectoreffect	= pLASeR->GetUINT8Attr(ATTR_VECTOR_EFFECT, &err);
	ErrorMessage(err);

	float			nWidth			= pLASeR->GetFloatAttr(ATTR_WIDTH, &err);
	ErrorMessage(err);
	float			nHeight			= pLASeR->GetFloatAttr(ATTR_HEIGHT, &err);
	ErrorMessage(err);
	float			nX				= pLASeR->GetFloatAttr(ATTR_X, &err);
	ErrorMessage(err);
	float			nY				= pLASeR->GetFloatAttr(ATTR_Y, &err);
	ErrorMessage(err);

	ConvertLPtoDP(&nWidth, &nHeight);
	/************************************************************************/

	// 090601 YongChul
	if(u_display == 1)
	{
		pLASeR->SetDeactivate(); /*none*/
	}
	else
	{
		if (uVisibility == 1) { /*hidden*/ }
		else //
		{
			Graphics graphics(m_pDC->m_hDC);

			if (bDraw)
			{
				if (strFill != _T(""))
				{
					// call DrawLin~
					CLASeR *LASeRGradient = SearchLASeRElementUsingID(strFill);
					UINT8 Gradient_type = LASeRGradient->GetElementType();

					if (Gradient_type == SVG_linearGradient)
					{
						UINT8 strgradientunit = LASeRGradient->GetUINT8Attr(ATTR_GRADIENTUNITS, &err);
						ErrorMessage(err);

						float x1     = LASeRGradient->GetFloatAttr(ATTR_X1, &err);
						float y1     = LASeRGradient->GetFloatAttr(ATTR_Y1, &err);
						float x2     = LASeRGradient->GetFloatAttr(ATTR_X2, &err);
						float y2     = LASeRGradient->GetFloatAttr(ATTR_Y2, &err);

						if (strgradientunit == 1)
						{
							x1     = nX;
							y1     = nY;
							x2     = nX + nWidth;
							y2     = nY + nHeight;
						}

						deque<CLASeR *> child_Stop = LASeRGradient->GetChildren();
						
						deque<UINT8> Stop_color0      = child_Stop[0]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
						ErrorMessage(err);
						UINT8        ustop_opacity0	  = (UINT8)(child_Stop[0]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
						ErrorMessage(err);
						float        fstop_offset0    = child_Stop[0]->GetFloatAttr(ATTR_OFFSET, &err);
						ErrorMessage(err);

						x1 += fstop_offset0 * nWidth;
						y1 += fstop_offset0 * nHeight;

						deque<UINT8> Stop_color1      = child_Stop[1]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
						ErrorMessage(err);
						UINT8        ustop_opacity1	  = (UINT8)(child_Stop[1]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
						ErrorMessage(err);
						float        fstop_offset1    = child_Stop[1]->GetFloatAttr(ATTR_OFFSET, &err);
						ErrorMessage(err);

						x2 =  fstop_offset1 * x2;
						y2 =  fstop_offset1 * y2;	

						LinearGradientBrush linerBrush(Point((INT)x1,(INT)y1), Point((INT)x2,(INT)y2), 
							Color((BYTE)ustop_opacity0, Stop_color0[0], Stop_color0[1], Stop_color0[2]),
							Color((BYTE)ustop_opacity1, Stop_color1[0], Stop_color1[1], Stop_color1[2]));
						graphics.FillRectangle(&linerBrush, nX, nY, nWidth, nHeight);
					}
					else if (Gradient_type == SVG_radialGradient)
					{
						UINT8 strgradientunit = LASeRGradient->GetUINT8Attr(ATTR_GRADIENTUNITS, &err);
						ErrorMessage(err);

						float cx = LASeRGradient->GetFloatAttr(ATTR_CX, &err);
						float cy = LASeRGradient->GetFloatAttr(ATTR_CY, &err);
						float r  = LASeRGradient->GetFloatAttr(ATTR_R, &err);

						if (strgradientunit == 1)
						{
							cx     = nX + nWidth/2;
							cy     = nY + nHeight/2;
							r      = nWidth/2;
						}
						deque<CLASeR *> child_Stop = LASeRGradient->GetChildren();
						
						deque<UINT8> Stop_color0      = child_Stop[0]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
						ErrorMessage(err);
						UINT8        ustop_opacity0	  = (UINT8)(child_Stop[0]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
						ErrorMessage(err);
						float        fstop_offset0    = child_Stop[0]->GetFloatAttr(ATTR_OFFSET, &err);
						ErrorMessage(err);

						cx += fstop_offset0 * nWidth;
						cy += fstop_offset0 * nHeight;

						deque<UINT8> Stop_color1      = child_Stop[1]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
						ErrorMessage(err);
						UINT8        ustop_opacity1	  = (UINT8)(child_Stop[1]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
						ErrorMessage(err);
						float        fstop_offset1    = child_Stop[1]->GetFloatAttr(ATTR_OFFSET, &err);
						ErrorMessage(err);

						r =  fstop_offset1 * r;	

						GraphicsPath path;
						path.AddEllipse(cx-r, cy-r, 2*r, 2*r);

						int count =1;
						PathGradientBrush pathBrush(&path);
						pathBrush.SetCenterColor(Color((BYTE)ustop_opacity0, Stop_color0[0], Stop_color0[1], Stop_color0[2]));
						pathBrush.SetSurroundColors(&Color((BYTE)ustop_opacity1, Stop_color1[0], Stop_color1[1], Stop_color1[2]),&count);
						graphics.FillRectangle(&pathBrush, nX, nY, nWidth, nHeight);
					}
					graphics.TranslateTransform(afTranslate[0], afTranslate[1]);
					graphics.ScaleTransform(afScale[0], afScale[1]);
					graphics.RotateTransform(afRotate[0]);

				}
				else if (auFill.size() == 3)			// rgb
				{
					SolidBrush brush(Color(uFillOpacity, auFill[0], auFill[1], auFill[2]));
					graphics.TranslateTransform(afTranslate[0], afTranslate[1]);
					if (uVectoreffect != 1)
					{
						graphics.ScaleTransform(afScale[0], afScale[1]);
					}
					graphics.RotateTransform(afRotate[0]);
					graphics.FillRectangle(&brush, nX, nY, nWidth, nHeight);
				}

				if (auStroke.size() == 3)
				{
					Pen pen(Color(uStrokeOpacity, auStroke[0], auStroke[1], auStroke[2]), fStrokeWidth);
					graphics.DrawRectangle(&pen, (INT)nX, (INT)nY, (INT)nWidth, (INT)nHeight);
				}
			}
			else
			{
				SolidBrush brush(Color(255, 248, 248, 248));
				graphics.TranslateTransform(afTranslate[0], afTranslate[1]);
				if (uVectoreffect != 1)
				{
					graphics.ScaleTransform(afScale[0], afScale[1]);
				}
				graphics.RotateTransform(afRotate[0]);
				graphics.FillRectangle(&brush, nX, nY, nWidth, nHeight);

				Pen pen(Color(255, 248, 248, 248), fStrokeWidth);
				graphics.DrawRectangle(&pen, nX, nY, nWidth, nHeight);
			}
			ReleaseDC(m_pDC);
		}
	}
}

void CLASeRPlayerView::DrawRect(CLASeR *pLASeR, Graphics *graphics, UINT8 bDraw /* = TRUE */)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// get rect attributes
	UINT8           uVisibility     = pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);

	UINT8			u_display		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);

	deque<UINT8>	auFill			= pLASeR->GetUINT8ListAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	tstring			strFill			= (pLASeR->GetStrAttr(ATTR_FILL, &err));
	ErrorMessage(err);
	UINT8			uFillOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_FILL_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	deque<UINT8>	auStroke		= pLASeR->GetUINT8ListAttr(ATTR_STROKE, &err);
	ErrorMessage(err);
	UINT8			uStrokeOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_STROKE_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	float			fStrokeWidth	= pLASeR->GetFloatAttr(ATTR_STROKE_WIDTH, &err);
	ErrorMessage(err);

	deque<float>	afRotate		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
	ErrorMessage(err);
	deque<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
	ErrorMessage(err);
	deque<float>	afTranslate		= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
	ErrorMessage(err);

	float			nWidth			= pLASeR->GetFloatAttr(ATTR_WIDTH, &err);
	ErrorMessage(err);
	float			nHeight			= pLASeR->GetFloatAttr(ATTR_HEIGHT, &err);
	ErrorMessage(err);
	float			nX				= pLASeR->GetFloatAttr(ATTR_X, &err);
	ErrorMessage(err);
	float			nY				= pLASeR->GetFloatAttr(ATTR_Y, &err);
	ErrorMessage(err);

	ConvertLPtoDP(&nWidth, &nHeight);
	/************************************************************************/

	// 090601 YongChul
	if(u_display == 1)
	{
		pLASeR->SetDeactivate(); /*none*/
	}
	else
	{
		if (uVisibility == 1) { /*hidden*/	}
		else
		{
			if (bDraw)
			{
				if (strFill != _T(""))
				{
					// call DrawLin~
					CLASeR *LASeRGradient = SearchLASeRElementUsingID(strFill);
					UINT8 Gradient_type = LASeRGradient->GetElementType();

					if (Gradient_type == SVG_linearGradient)
					{
						UINT8 strgradientunit = LASeRGradient->GetUINT8Attr(ATTR_GRADIENTUNITS, &err);
						ErrorMessage(err);

						float x1     = LASeRGradient->GetFloatAttr(ATTR_X1, &err);
						float y1     = LASeRGradient->GetFloatAttr(ATTR_Y1, &err);
						float x2     = LASeRGradient->GetFloatAttr(ATTR_X2, &err);
						float y2     = LASeRGradient->GetFloatAttr(ATTR_Y2, &err);

						if (strgradientunit == 1)
						{
							x1     = nX;
							y1     = nY;
							x2     = nX + nWidth;
							y2     = nY + nHeight;
						}

						deque<CLASeR *> child_Stop = LASeRGradient->GetChildren();
						
						deque<UINT8> Stop_color0      = child_Stop[0]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
						ErrorMessage(err);
						UINT8        ustop_opacity0	  = (UINT8)(child_Stop[0]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
						ErrorMessage(err);
						float        fstop_offset0    = child_Stop[0]->GetFloatAttr(ATTR_OFFSET, &err);
						ErrorMessage(err);

						x1 += fstop_offset0 * nWidth;
						y1 += fstop_offset0 * nHeight;

						deque<UINT8> Stop_color1      = child_Stop[1]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
						ErrorMessage(err);
						UINT8        ustop_opacity1	  = (UINT8)(child_Stop[1]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
						ErrorMessage(err);
						float        fstop_offset1    = child_Stop[1]->GetFloatAttr(ATTR_OFFSET, &err);
						ErrorMessage(err);

						x2 =  fstop_offset1 * x2;
						y2 =  fstop_offset1 * y2;	

						LinearGradientBrush linerBrush(Point((int)x1,(int)y1), Point((int)x2,(int)y2), 
							Color((BYTE)ustop_opacity0, Stop_color0[0], Stop_color0[1], Stop_color0[2]),
							Color((BYTE)ustop_opacity1, Stop_color1[0], Stop_color1[1], Stop_color1[2]));
						graphics->FillRectangle(&linerBrush, nX, nY, nWidth, nHeight);
					}
					else if (Gradient_type == SVG_radialGradient)
					{
						UINT8 strgradientunit = LASeRGradient->GetUINT8Attr(ATTR_GRADIENTUNITS, &err);
						ErrorMessage(err);

						float cx = LASeRGradient->GetFloatAttr(ATTR_CX, &err);
						float cy = LASeRGradient->GetFloatAttr(ATTR_CY, &err);
						float r  = LASeRGradient->GetFloatAttr(ATTR_R, &err);

						if (strgradientunit == 1)
						{
							cx     = nX + nWidth/2;
							cy     = nY + nHeight/2;
							r      = nWidth/2;
						}

						deque<CLASeR *> child_Stop = LASeRGradient->GetChildren();

						deque<UINT8> Stop_color0      = child_Stop[0]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
						ErrorMessage(err);
						UINT8        ustop_opacity0	  = (UINT8)(child_Stop[0]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
						ErrorMessage(err);
						float        fstop_offset0    = child_Stop[0]->GetFloatAttr(ATTR_OFFSET, &err);
						ErrorMessage(err);

						cx += fstop_offset0 * nWidth;
						cy += fstop_offset0 * nHeight;

						deque<UINT8> Stop_color1      = child_Stop[1]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
						ErrorMessage(err);
						UINT8        ustop_opacity1	  = (UINT8)(child_Stop[1]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
						ErrorMessage(err);
						float        fstop_offset1    = child_Stop[1]->GetFloatAttr(ATTR_OFFSET, &err);
						ErrorMessage(err);

						r =  fstop_offset1 * r;	
						GraphicsPath path;
						path.AddEllipse(cx-r, cy-r, 2*r, 2*r);

						int count =1;
						PathGradientBrush pathBrush(&path);
						pathBrush.SetCenterColor(Color((BYTE)ustop_opacity0, Stop_color0[0], Stop_color0[1], Stop_color0[2]));
						pathBrush.SetSurroundColors(&Color((BYTE)ustop_opacity1, Stop_color1[0], Stop_color1[1], Stop_color1[2]),&count);
						graphics->FillRectangle(&pathBrush, nX, nY, nWidth, nHeight);
					}
					graphics->TranslateTransform(afTranslate[0], afTranslate[1]);
					graphics->ScaleTransform(afScale[0], afScale[1]);
					graphics->RotateTransform(afRotate[0]);

				}
				else if (auFill.size() == 3)			// rgb
				{
					SolidBrush brush(Color(uFillOpacity, auFill[0], auFill[1], auFill[2]));
					graphics->FillRectangle(&brush, nX, nY, nWidth, nHeight);
				}

				if (auStroke.size() == 3)
				{
					Pen pen(Color(uStrokeOpacity, auStroke[0], auStroke[1], auStroke[2]), fStrokeWidth);
					graphics->DrawRectangle(&pen, nX, nY, nWidth, nHeight);
				}
			}
			else
			{
				SolidBrush brush(Color(255, 248, 248, 248));
				graphics->FillRectangle(&brush, nX, nY, nWidth, nHeight);

				Pen pen(Color(255, 248, 248, 248), fStrokeWidth);
				graphics->DrawRectangle(&pen, nX, nY, nWidth, nHeight);
			}
		}
	}
}

void CLASeRPlayerView::DrawEllipse(CLASeR *pLASeR, UINT8 bDraw /* = TRUE */)
{
	lsrErr err = LASeRNoErr;

	m_pDC = GetDC();

	/************************************************************************/
	// get rect attributes
	UINT8           uVisibility     = pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);

	UINT8			u_display		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);

	deque<UINT8>	auFill			= pLASeR->GetUINT8ListAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	tstring			strFill			= pLASeR->GetStrAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	UINT8			uFillOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_FILL_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	deque<UINT8>	auStroke		= pLASeR->GetUINT8ListAttr(ATTR_STROKE, &err);
	ErrorMessage(err);
	UINT8			uStrokeOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_STROKE_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	float			fStrokeWidth	= pLASeR->GetFloatAttr(ATTR_STROKE_WIDTH, &err);
	ErrorMessage(err);

	deque<float>	afRotate		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
	ErrorMessage(err);
	deque<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
	ErrorMessage(err);
	deque<float>	afTranslate		= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
	ErrorMessage(err);

	UINT8			uVectoreffect	= pLASeR->GetUINT8Attr(ATTR_VECTOR_EFFECT, &err);
	ErrorMessage(err);

	float				nCX				= pLASeR->GetFloatAttr(ATTR_CX, &err);
	ErrorMessage(err);
	float				nCY				= pLASeR->GetFloatAttr(ATTR_CY, &err);
	ErrorMessage(err);
	float				nRX				= pLASeR->GetFloatAttr(ATTR_RX, &err);
	ErrorMessage(err);
	float				nRY				= pLASeR->GetFloatAttr(ATTR_RY, &err);
	ErrorMessage(err);

	ConvertLPtoDP(&nRX, &nRY);

	float             nX              = nCX - nRX;
	float             nY              = nCY - nRY;
	float             nWidth          = 2 * nRX;
	float             nHeight         = 2 * nRY;

	/************************************************************************/

	// 090529 YongChul
	if(u_display == 1)
	{
		pLASeR->SetDeactivate(); /*none*/
	}
	else
	{
		if (uVisibility == 1) { /*hidden*/ }
		else
		{
			Graphics graphics(m_pDC->m_hDC);

			if (bDraw)
			{
				if (strFill != _T(""))
				{
					// call DrawLin~
					CLASeR *LASeRGradient = SearchLASeRElementUsingID(strFill);
					UINT8 Gradient_type = LASeRGradient->GetElementType();

					if (Gradient_type == SVG_linearGradient)
					{
						UINT8 strgradientunit = LASeRGradient->GetUINT8Attr(ATTR_GRADIENTUNITS, &err);
						ErrorMessage(err);

						float x1     = LASeRGradient->GetFloatAttr(ATTR_X1, &err);
						float y1     = LASeRGradient->GetFloatAttr(ATTR_Y1, &err);
						float x2     = LASeRGradient->GetFloatAttr(ATTR_X2, &err);
						float y2     = LASeRGradient->GetFloatAttr(ATTR_Y2, &err);

						if (strgradientunit == 1)
						{
							x1     = nX;
							y1     = nY;
							x2     = nX + nWidth;
							y2     = nY + nHeight;
						}

						deque<CLASeR *> child_Stop = LASeRGradient->GetChildren();
						
						deque<UINT8> Stop_color0      = child_Stop[0]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
						ErrorMessage(err);
						UINT8        ustop_opacity0	  = (UINT8)(child_Stop[0]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
						ErrorMessage(err);
						float        fstop_offset0    = child_Stop[0]->GetFloatAttr(ATTR_OFFSET, &err);
						ErrorMessage(err);

						x1 += fstop_offset0 * nWidth;
						y1 += fstop_offset0 * nHeight;

						deque<UINT8> Stop_color1      = child_Stop[1]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
						ErrorMessage(err);
						UINT8        ustop_opacity1	  = (UINT8)(child_Stop[1]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
						ErrorMessage(err);
						float        fstop_offset1    = child_Stop[1]->GetFloatAttr(ATTR_OFFSET, &err);
						ErrorMessage(err);

						x2 =  fstop_offset1 * x2;
						y2 =  fstop_offset1 * y2;

						LinearGradientBrush linerBrush(Point((int)x1,(int)y1), Point((int)x2,(int)y2), 
							Color((BYTE)ustop_opacity0, Stop_color0[0], Stop_color0[1], Stop_color0[2]),
							Color((BYTE)ustop_opacity1, Stop_color1[0], Stop_color1[1], Stop_color1[2]));
						graphics.FillEllipse(&linerBrush, nX, nY, nWidth, nHeight);
					}
					else if (Gradient_type == SVG_radialGradient)
					{
						UINT8 strgradientunit = LASeRGradient->GetUINT8Attr(ATTR_GRADIENTUNITS, &err);
						ErrorMessage(err);

						float cx = LASeRGradient->GetFloatAttr(ATTR_CX, &err);
						float cy = LASeRGradient->GetFloatAttr(ATTR_CY, &err);
						float r  = LASeRGradient->GetFloatAttr(ATTR_R, &err);

						if (strgradientunit == 1)
						{
							cx     = nX + nWidth/2;
							cy     = nY + nHeight/2;
							r      = nWidth/2;
						}

						deque<CLASeR *> child_Stop = LASeRGradient->GetChildren();
						
						deque<UINT8> Stop_color0      = child_Stop[0]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
						ErrorMessage(err);
						UINT8        ustop_opacity0	  = (UINT8)(child_Stop[0]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
						ErrorMessage(err);
						float        fstop_offset0    = child_Stop[0]->GetFloatAttr(ATTR_OFFSET, &err);
						ErrorMessage(err);

						cx += fstop_offset0 * nWidth;
						cy += fstop_offset0 * nHeight;

						deque<UINT8> Stop_color1      = child_Stop[1]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
						ErrorMessage(err);
						UINT8        ustop_opacity1	  = (UINT8)(child_Stop[1]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
						ErrorMessage(err);
						float        fstop_offset1    = child_Stop[1]->GetFloatAttr(ATTR_OFFSET, &err);
						ErrorMessage(err);

						r =  fstop_offset1 * r;		

						GraphicsPath path;
						path.AddEllipse(cx-r, cy-r, 2*r, 2*r);

						int count =1;
						PathGradientBrush pathBrush(&path);
						pathBrush.SetCenterColor(Color((BYTE)ustop_opacity0, Stop_color0[0], Stop_color0[1], Stop_color0[2]));
						pathBrush.SetSurroundColors(&Color((BYTE)ustop_opacity1, Stop_color1[0], Stop_color1[1], Stop_color1[2]),&count);
						graphics.FillEllipse(&pathBrush, nX, nY, nWidth, nHeight);
					}
					graphics.TranslateTransform(afTranslate[0], afTranslate[1]);
					graphics.ScaleTransform(afScale[0], afScale[1]);
					graphics.RotateTransform(afRotate[0]);

				}
				else if (auFill.size() == 3)			// rgb
				{
					SolidBrush brush(Color(uFillOpacity, auFill[0], auFill[1], auFill[2]));
					graphics.TranslateTransform(afTranslate[0], afTranslate[1]);
					if (uVectoreffect != 1)
					{
						graphics.ScaleTransform(afScale[0], afScale[1]);
					}
					graphics.RotateTransform(afRotate[0]);
					graphics.FillEllipse(&brush, nX, nY, nWidth, nHeight);
				}

				if (auStroke.size() == 3)
				{
					Pen pen(Color(uStrokeOpacity, auStroke[0], auStroke[1], auStroke[2]), fStrokeWidth);
					graphics.DrawEllipse(&pen, nX, nY, nWidth, nHeight);
				}
			}
			else
			{
				SolidBrush brush(Color(255, 248, 248, 248));
				graphics.TranslateTransform(afTranslate[0], afTranslate[1]);
				if (uVectoreffect != 1)
				{
					graphics.ScaleTransform(afScale[0], afScale[1]);
				}
				graphics.RotateTransform(afRotate[0]);
				graphics.FillEllipse(&brush, nX, nY, nWidth, nHeight);

				Pen pen(Color(255, 248, 248, 248), fStrokeWidth);
				graphics.DrawEllipse(&pen, nX, nY, nWidth, nHeight);
			}
			ReleaseDC(m_pDC);
		}
	}
}

void CLASeRPlayerView::DrawEllipse(CLASeR *pLASeR, Graphics *graphics, UINT8 bDraw /* = TRUE */)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// get rect attributes
	UINT8           uVisibility     = pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);
	
	UINT8			u_display		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);

	deque<UINT8>	auFill			= pLASeR->GetUINT8ListAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	tstring			strFill			= (pLASeR->GetStrAttr(ATTR_FILL, &err));
	ErrorMessage(err);
	UINT8			uFillOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_FILL_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	deque<UINT8>	auStroke		= pLASeR->GetUINT8ListAttr(ATTR_STROKE, &err);
	ErrorMessage(err);
	UINT8			uStrokeOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_STROKE_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	float			fStrokeWidth	= pLASeR->GetFloatAttr(ATTR_STROKE_WIDTH, &err);
	ErrorMessage(err);

	deque<float>	afRotate		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
	ErrorMessage(err);
	deque<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
	ErrorMessage(err);
	deque<float>	afTranslate		= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
	ErrorMessage(err);

	UINT8			uVectoreffect	= pLASeR->GetUINT8Attr(ATTR_VECTOR_EFFECT, &err);
	ErrorMessage(err);

	float				nCX				= pLASeR->GetFloatAttr(ATTR_CX, &err);
	ErrorMessage(err);
	float				nCY				= pLASeR->GetFloatAttr(ATTR_CY, &err);
	ErrorMessage(err);
	float			    nRX				= pLASeR->GetFloatAttr(ATTR_RX, &err);
	ErrorMessage(err);
	float				nRY				= pLASeR->GetFloatAttr(ATTR_RY, &err);
	ErrorMessage(err);

	ConvertLPtoDP(&nRX, &nRY);

	float             nX              = nCX - nRX;
	float             nY              = nCY - nRY;
	float             nWidth          = 2 * nRX;
	float             nHeight         = 2 * nRY;

	/************************************************************************/

	// 090529 YongChul
	if(u_display == 1)
	{
		pLASeR->SetDeactivate(); /*none*/
	}
	else
	{
		if (uVisibility == 1) { /*hidden*/	}
		else
		{
			if (bDraw)
			{
				if (strFill != _T(""))
				{
					// call DrawLin~
					CLASeR *LASeRGradient = SearchLASeRElementUsingID(strFill);
					UINT8 Gradient_type = LASeRGradient->GetElementType();

					if (Gradient_type == SVG_linearGradient)
					{
						UINT8 strgradientunit = LASeRGradient->GetUINT8Attr(ATTR_GRADIENTUNITS, &err);
						ErrorMessage(err);

						float x1     = LASeRGradient->GetFloatAttr(ATTR_X1, &err);
						float y1     = LASeRGradient->GetFloatAttr(ATTR_Y1, &err);
						float x2     = LASeRGradient->GetFloatAttr(ATTR_X2, &err);
						float y2     = LASeRGradient->GetFloatAttr(ATTR_Y2, &err);

						if (strgradientunit == 1)
						{
							x1     = nX;
							y1     = nY;
							x2     = nX + nWidth;
							y2     = nY + nHeight;
						}

						deque<CLASeR *> child_Stop = LASeRGradient->GetChildren();
						
						deque<UINT8> Stop_color0      = child_Stop[0]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
						ErrorMessage(err);
						UINT8        ustop_opacity0	  = (UINT8)(child_Stop[0]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
						ErrorMessage(err);
						float        fstop_offset0    = child_Stop[0]->GetFloatAttr(ATTR_OFFSET, &err);
						ErrorMessage(err);

						x1 += fstop_offset0 * nWidth;
						y1 += fstop_offset0 * nHeight;

						deque<UINT8> Stop_color1      = child_Stop[1]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
						ErrorMessage(err);
						UINT8        ustop_opacity1	  = (UINT8)(child_Stop[1]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
						ErrorMessage(err);
						float        fstop_offset1    = child_Stop[1]->GetFloatAttr(ATTR_OFFSET, &err);
						ErrorMessage(err);

						x2 =  fstop_offset1 * x2;
						y2 =  fstop_offset1 * y2;

						LinearGradientBrush linerBrush(Point((int)x1,(int)y1), Point((int)x2,(int)y2), 
							Color((BYTE)ustop_opacity0, Stop_color0[0], Stop_color0[1], Stop_color0[2]),
							Color((BYTE)ustop_opacity1, Stop_color1[0], Stop_color1[1], Stop_color1[2]));
						graphics->FillRectangle(&linerBrush, nX, nY, nWidth, nHeight);
					}
					else if (Gradient_type == SVG_radialGradient)
					{
						UINT8 strgradientunit = LASeRGradient->GetUINT8Attr(ATTR_GRADIENTUNITS, &err);
						ErrorMessage(err);

						float cx = LASeRGradient->GetFloatAttr(ATTR_CX, &err);
						float cy = LASeRGradient->GetFloatAttr(ATTR_CY, &err);
						float r  = LASeRGradient->GetFloatAttr(ATTR_R, &err);

						if (strgradientunit == 1)
						{
							cx     = nX + nWidth/2;
							cy     = nY + nHeight/2;
							r      = nWidth/2;
						}

						deque<CLASeR *> child_Stop = LASeRGradient->GetChildren();
						
						deque<UINT8> Stop_color0      = child_Stop[0]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
						ErrorMessage(err);
						UINT8        ustop_opacity0	  = (UINT8)(child_Stop[0]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
						ErrorMessage(err);
						float        fstop_offset0    = child_Stop[0]->GetFloatAttr(ATTR_OFFSET, &err);
						ErrorMessage(err);

						cx += fstop_offset0 * nWidth;
						cy += fstop_offset0 * nHeight;

						deque<UINT8> Stop_color1      = child_Stop[1]->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
						ErrorMessage(err);
						UINT8        ustop_opacity1	  = (UINT8)(child_Stop[1]->GetFloatAttr(ATTR_STOP_OPACITY, &err) * 255.0);
						ErrorMessage(err);
						float        fstop_offset1    = child_Stop[1]->GetFloatAttr(ATTR_OFFSET, &err);
						ErrorMessage(err);

						r =  fstop_offset1 * r;		

						GraphicsPath path;
						path.AddEllipse(cx-r, cy-r, 2*r, 2*r);

						int count =1;
						PathGradientBrush pathBrush(&path);
						pathBrush.SetCenterColor(Color((BYTE)ustop_opacity0, Stop_color0[0], Stop_color0[1], Stop_color0[2]));
						pathBrush.SetSurroundColors(&Color((BYTE)ustop_opacity1, Stop_color1[0], Stop_color1[1], Stop_color1[2]),&count);
						graphics->FillEllipse(&pathBrush, nX, nY, nWidth, nHeight);
					}
					graphics->TranslateTransform(afTranslate[0], afTranslate[1]);
					graphics->ScaleTransform(afScale[0], afScale[1]);
					graphics->RotateTransform(afRotate[0]);

				}
				else if (auFill.size() == 3)			// rgb
				{
					SolidBrush brush(Color(uFillOpacity, auFill[0], auFill[1], auFill[2]));
					graphics->TranslateTransform(afTranslate[0], afTranslate[1]);
					if (uVectoreffect != 1)
					{
						graphics->ScaleTransform(afScale[0], afScale[1]);
					}
					graphics->RotateTransform(afRotate[0]);
					graphics->FillEllipse(&brush, nX, nY, nWidth, nHeight);
				}

				if (auStroke.size() == 3)
				{
					Pen pen(Color(uStrokeOpacity, auStroke[0], auStroke[1], auStroke[2]), fStrokeWidth);
					graphics->DrawEllipse(&pen, nX, nY, nWidth, nHeight);
				}
			}
			else
			{
				SolidBrush brush(Color(255, 248, 248, 248));
				graphics->TranslateTransform(afTranslate[0], afTranslate[1]);
				if (uVectoreffect != 1)
				{
					graphics->ScaleTransform(afScale[0], afScale[1]);
				}
				graphics->RotateTransform(afRotate[0]);
				graphics->FillEllipse(&brush, nX, nY, nWidth, nHeight);

				Pen pen(Color(255, 248, 248, 248), fStrokeWidth);
				graphics->DrawEllipse(&pen, nX, nY, nWidth, nHeight);
			}
		}
	}
}

void CLASeRPlayerView::DrawImage(CLASeR *pLASeR, UINT8 bDraw /* = TRUE */)
{
	lsrErr err = LASeRNoErr;

	m_pDC = GetDC();

	/************************************************************************/
	// get rect attributes
	UINT8           uVisibility     = pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);
	
	UINT8			u_display		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);

	deque<float>	afRotate		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
	ErrorMessage(err);
	deque<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
	ErrorMessage(err);
	deque<float>	afTranslate		= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
	ErrorMessage(err);

	float			nWidth			= pLASeR->GetFloatAttr(ATTR_WIDTH, &err);
	ErrorMessage(err);
	float			nHeight			= pLASeR->GetFloatAttr(ATTR_HEIGHT, &err);
	ErrorMessage(err);
	int				nX				= (int)pLASeR->GetFloatAttr(ATTR_X, &err);
	ErrorMessage(err);
	int				nY				= (int)pLASeR->GetFloatAttr(ATTR_Y, &err);
	ErrorMessage(err);

	ConvertLPtoDP(&nWidth, &nHeight);
	/************************************************************************/

	// 090624 YongChul
	if(u_display == 1)
	{
		pLASeR->SetDeactivate(); /*none*/
	}
	else
	{
		if (uVisibility == 1) { /*hidden*/ }
		else
		{
			Graphics graphics(m_pDC->m_hDC);

			if (bDraw)
			{
				tstring image_source;

				CSAF *SAFimage = SearchSAFElementUsingID(pLASeR->GetStrAttr(ATTR_XLINK_HREF, &err));
				CLASeR *pLASeRtemp = NULL;
				err = GetNextObject(&pLASeRtemp, &SAFimage);

				CSAFUnit *ESSAFUnit = (CSAFUnit *)SAFimage;

				image_source = ESSAFUnit->GetStrAttr(ATTR_SOURCE, &err);
				CStringW str_soource(image_source.c_str());
				Image image(str_soource.GetBuffer());

				graphics.TranslateTransform(afTranslate[0], afTranslate[1]);
				graphics.ScaleTransform(afScale[0], afScale[1]);
				graphics.RotateTransform(afRotate[0]);

				// Display image
				graphics.DrawImage(&image, (int)nX, (int)nY, (int)nWidth, (int)nHeight);
			}
			else
			{
				SolidBrush brush(Color(255, 248, 248, 248));
				graphics.FillRectangle(&brush, (int)nX, (int)nY, (int)nWidth, (int)nHeight);
			}
			ReleaseDC(m_pDC);
		}
	}
}

void CLASeRPlayerView::DrawImage(CLASeR *pLASeR, Graphics *graphics, UINT8 bDraw /* = TRUE */)
{
	lsrErr err = LASeRNoErr;

	m_pDC = GetDC();

	/************************************************************************/
	// get rect attributes
	UINT8           uVisibility     = pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);

	UINT8			u_display		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);

	deque<float>	afRotate		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
	ErrorMessage(err);
	deque<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
	ErrorMessage(err);
	deque<float>	afTranslate		= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
	ErrorMessage(err);

	float			nWidth			= pLASeR->GetFloatAttr(ATTR_WIDTH, &err);
	ErrorMessage(err);
	float			nHeight			= pLASeR->GetFloatAttr(ATTR_HEIGHT, &err);
	ErrorMessage(err);
	int				nX				= (int)pLASeR->GetFloatAttr(ATTR_X, &err);
	ErrorMessage(err);
	int				nY				= (int)pLASeR->GetFloatAttr(ATTR_Y, &err);
	ErrorMessage(err);

	ConvertLPtoDP(&nWidth, &nHeight);
	/************************************************************************/

	// 090624 YongChul
	if(u_display == 1)
	{
		pLASeR->SetDeactivate(); /*none*/
	}
	else
	{
		if (uVisibility == 1) { /*hidden*/ }
		else
		{
			if (bDraw)
			{
				tstring image_source;

				CSAF *SAFimage = SearchSAFElementUsingID(pLASeR->GetStrAttr(ATTR_XLINK_HREF, &err));
				CLASeR *pLASeRtemp = NULL;
				err = GetNextObject(&pLASeRtemp, &SAFimage);

				CSAFUnit *ESSAFUnit = (CSAFUnit *)SAFimage;

				image_source = ESSAFUnit->GetStrAttr(ATTR_SOURCE, &err);
				CStringW str_soource(image_source.c_str());
				Image image(str_soource.GetBuffer());

				graphics->TranslateTransform(afTranslate[0], afTranslate[1]);
				graphics->ScaleTransform(afScale[0], afScale[1]);
				graphics->RotateTransform(afRotate[0]);

				// Display image
				graphics->DrawImage(&image, (int)nX, (int)nY, (int)nWidth, (int)nHeight);
			}
		}
		ReleaseDC(m_pDC);
	}
}

void CLASeRPlayerView::DrawLine(CLASeR *pLASeR, UINT8 bDraw /* = TRUE */)
{
	lsrErr err = LASeRNoErr;

	m_pDC = GetDC();

	/************************************************************************/
	// get rect attributes
	UINT8           uVisibility     = pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);

	UINT8			u_display		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);

	deque<UINT8>	auStroke		= pLASeR->GetUINT8ListAttr(ATTR_STROKE, &err);
	ErrorMessage(err);
	UINT8			uStrokeOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_STROKE_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	float			fStrokeWidth	= pLASeR->GetFloatAttr(ATTR_STROKE_WIDTH, &err);
	ErrorMessage(err);

	deque<float>	afRotate		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
	ErrorMessage(err);
	deque<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
	ErrorMessage(err);
	deque<float>	afTranslate		= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
	ErrorMessage(err);

	UINT8			uVectoreffect	= pLASeR->GetUINT8Attr(ATTR_VECTOR_EFFECT, &err);
	ErrorMessage(err);

	int				nX1				= (int)pLASeR->GetFloatAttr(ATTR_X1, &err);
	ErrorMessage(err);
	int				nY1				= (int)pLASeR->GetFloatAttr(ATTR_Y1, &err);
	ErrorMessage(err);
	int				nX2				= (int)pLASeR->GetFloatAttr(ATTR_X2, &err);
	ErrorMessage(err);
	int				nY2				= (int)pLASeR->GetFloatAttr(ATTR_Y2, &err);
	ErrorMessage(err);

	/************************************************************************/

	// 090528 YongChul
	if(u_display == 1)
	{
		pLASeR->SetDeactivate(); /*none*/
	}
	else
	{
		if (uVisibility == 1) { /*hidden*/	}
		else
		{
			Graphics graphics(m_pDC->m_hDC);

			if (bDraw)
			{
				if (auStroke.size() == 3)
				{
					Pen pen(Color(uStrokeOpacity, auStroke[0], auStroke[1], auStroke[2]), fStrokeWidth);
					graphics.TranslateTransform(afTranslate[0], afTranslate[1]);
					if (uVectoreffect != 1)
					{
						graphics.ScaleTransform(afScale[0], afScale[1]);
					}
					graphics.RotateTransform(afRotate[0]);
					graphics.DrawLine(&pen, nX1, nY1, nX2, nY2);
				}
			}
			else
			{
				Pen pen(Color(255, 248, 248, 248), fStrokeWidth);
				graphics.TranslateTransform(afTranslate[0], afTranslate[1]);
				if (uVectoreffect != 1)
				{
					graphics.ScaleTransform(afScale[0], afScale[1]);
				}
				graphics.RotateTransform(afRotate[0]);
				graphics.DrawLine(&pen, nX1, nY1, nX2, nY2);
			}
			ReleaseDC(m_pDC);
		}
	}
}

void CLASeRPlayerView::DrawLine(CLASeR *pLASeR, Graphics *graphics, UINT8 bDraw /* = TRUE */)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// get rect attributes
	UINT8           uVisibility     = pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);

	UINT8			u_display		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);

	deque<UINT8>	auStroke		= pLASeR->GetUINT8ListAttr(ATTR_STROKE, &err);
	ErrorMessage(err);
	UINT8			uStrokeOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_STROKE_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	float			fStrokeWidth	= pLASeR->GetFloatAttr(ATTR_STROKE_WIDTH, &err);
	ErrorMessage(err);

	deque<float>	afRotate		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
	ErrorMessage(err);
	deque<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
	ErrorMessage(err);
	deque<float>	afTranslate		= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
	ErrorMessage(err);

	UINT8			uVectoreffect	= pLASeR->GetUINT8Attr(ATTR_VECTOR_EFFECT, &err);
	ErrorMessage(err);

	int				nX1				= (int)pLASeR->GetFloatAttr(ATTR_X1, &err);
	ErrorMessage(err);
	int				nY1				= (int)pLASeR->GetFloatAttr(ATTR_Y1, &err);
	ErrorMessage(err);
	int				nX2				= (int)pLASeR->GetFloatAttr(ATTR_X2, &err);
	ErrorMessage(err);
	int				nY2				= (int)pLASeR->GetFloatAttr(ATTR_Y2, &err);
	ErrorMessage(err);

	/************************************************************************/

 	//090528 YongChul
 	if(u_display == 1)
 	{
 		pLASeR->SetDeactivate(); /*none*/
 	}
 	else
	{
		if (uVisibility == 1) { /*hidden*/	}
		else
		{
			if (bDraw)
			{
				if (auStroke.size() == 3)
				{
					Pen pen(Color(uStrokeOpacity, auStroke[0], auStroke[1], auStroke[2]), fStrokeWidth);
					graphics->TranslateTransform(afTranslate[0], afTranslate[1]);
					if (uVectoreffect != 1)
					{
						graphics->ScaleTransform(afScale[0], afScale[1]);
					}
					graphics->RotateTransform(afRotate[0]);
					graphics->DrawLine(&pen, nX1, nY1, nX2, nY2);
				}
			}
			else
			{
				Pen pen(Color(255, 248, 248, 248), fStrokeWidth);
				graphics->TranslateTransform(afTranslate[0], afTranslate[1]);
				if (uVectoreffect != 1)
				{
					graphics->ScaleTransform(afScale[0], afScale[1]);
				}
				graphics->RotateTransform(afRotate[0]);
				graphics->DrawLine(&pen, nX1, nY1, nX2, nY2);
			}
			ReleaseDC(m_pDC);
		}
	}
}

void CLASeRPlayerView::DrawPath(CLASeR *pLASeR, UINT8 bDraw /* = TRUE */)
{
	lsrErr err = LASeRNoErr;

	m_pDC = GetDC();

	/************************************************************************/
	// get path attributes
	UINT8           uVisibility     = pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);
	deque<UINT8>	auFill			= pLASeR->GetUINT8ListAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	CString			strFill			= (pLASeR->GetStrAttr(ATTR_FILL, &err)).c_str();
	ErrorMessage(err);
	UINT8			uFillOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_FILL_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	deque<UINT8>	auStroke		= pLASeR->GetUINT8ListAttr(ATTR_STROKE, &err);
	ErrorMessage(err);
	UINT8			uStrokeOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_STROKE_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	float			fStrokeWidth	= pLASeR->GetFloatAttr(ATTR_STROKE_WIDTH, &err);
	ErrorMessage(err);

	deque<float>	afRotate		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
	ErrorMessage(err);
	deque<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
	ErrorMessage(err);
	deque<float>	afTranslate		= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
	ErrorMessage(err);

	tstring         strD            = pLASeR->GetStrAttr(ATTR_D, &err);
	ErrorMessage(err);
	/************************************************************************/

	if (uVisibility == 1) { /*hidden*/	}
	else
	{
		GraphicsPath gp; //path 

		deque<float> afList;

		TCHAR	*pBegin	= (TCHAR *)strD.c_str();	// position of number's beginning
		TCHAR	*pBlank	= NULL;						// position of blank
		TCHAR	*pComma	= NULL;						// position of comma
		TCHAR	*pToken	= NULL;
		TCHAR	chChar	= NULL;
		UINT8	uSize	= 0;

		while (1)
		{
			chChar = *pBegin;

			pBlank = _tcschr(pBegin, ' ');
			pToken = pBlank;

			switch (chChar)
			{
			case 'C':
				for (UINT8 i = 0; i < 3; ++i)
				{
					pBegin = pToken + 1;

					pComma = _tcschr(pBegin, ',');

					pToken = pComma;
					afList.push_back((float)_tcstod(pBegin, &pToken));

					pBegin = pToken + 1;

					pToken = _tcschr(pBegin, ' ');
					afList.push_back((float)_tcstod(pBegin, &pToken));
				}

				uSize = (UINT8)afList.size();

				{
					PointF Bezier_point[] = {PointF(afList[uSize - 8], afList[uSize - 7]),
						PointF(afList[uSize - 6], afList[uSize - 5]),
						PointF(afList[uSize - 4], afList[uSize - 3]),
						PointF(afList[uSize - 2], afList[uSize - 1])};
					// Ŀ ߰
					gp.AddBezier(Bezier_point[0], Bezier_point[1], Bezier_point[2], Bezier_point[3]);
				}

				break;

			case 'L':
			case 'M':
				pBegin = pToken + 1;

				pComma = _tcschr(pBegin, ',');

				pToken = pComma;
				afList.push_back((float)_tcstod(pBegin, &pToken));

				pBegin = pToken + 1;

				pToken = _tcschr(pBegin, ' ');
				afList.push_back((float)_tcstod(pBegin, &pToken));

				if (chChar == 'L')
				{
					uSize = (UINT8)afList.size();

					PointF line_point[] = {PointF(afList[uSize - 4], afList[uSize - 3]),
						PointF(afList[uSize - 2], afList[uSize - 1])}; 
					gp.AddLines(line_point, 2);
				}

				break;

			case 'Z':
				uSize = (UINT8)afList.size();

				pToken = pBegin;

				{
					PointF line_point[] = {PointF(afList[uSize - 2], afList[uSize - 1]),
						PointF(afList[0], afList[1])}; 
					gp.AddLines(line_point, 2);
				}

				break;

			default:
				/* nothing */
				break;
			}
			/************************************************************************/

			pBegin = pToken + 1;

			if ((*pToken != ' ') && (*pToken != ','))
				break;
		}

		Graphics graphics(m_pDC->m_hDC);

		// transform
		graphics.TranslateTransform(afTranslate[0], afTranslate[1]);
		graphics.ScaleTransform(afScale[0], afScale[1]);
		graphics.RotateTransform(afRotate[0]);

		if (bDraw)
		{
			if (strFill != _T(""))
			{
				// call DrawLin~
			}

			else if (auFill.size() == 3)			// rgb
			{
				SolidBrush brush(Color(uFillOpacity, auFill[0], auFill[1], auFill[2]));
				graphics.FillPath(&brush,&gp);
			}

			if (auStroke.size() == 3)
			{
				Pen pen(Color(uStrokeOpacity, auStroke[0], auStroke[1], auStroke[2]), fStrokeWidth);
				graphics.DrawPath(&pen,&gp);
			}
		}

		else
		{
			SolidBrush brush(Color(255, 248, 248, 248));
			graphics.FillPath(&brush,&gp);

			Pen pen(Color(255, 248, 248, 248), fStrokeWidth);
			graphics.DrawPath(&pen,&gp);
		}
		ReleaseDC(m_pDC);
	}
}

void CLASeRPlayerView::DrawPolygon(CLASeR *pLASeR, UINT8 bDraw /* = TRUE */)
{
	lsrErr err = LASeRNoErr;

	m_pDC = GetDC();

	/************************************************************************/
	// get rect attributes
	UINT8           uVisibility     = pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);

	UINT8			u_display		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);

	deque<UINT8>	auFill			= pLASeR->GetUINT8ListAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	tstring			strFill			= pLASeR->GetStrAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	UINT8			uFillOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_FILL_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	deque<UINT8>	auStroke		= pLASeR->GetUINT8ListAttr(ATTR_STROKE, &err);
	ErrorMessage(err);
	UINT8			uStrokeOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_STROKE_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	float			fStrokeWidth	= pLASeR->GetFloatAttr(ATTR_STROKE_WIDTH, &err);
	ErrorMessage(err);

	deque<float>	afRotate		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
	ErrorMessage(err);
	deque<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
	ErrorMessage(err);
	deque<float>	afTranslate		= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
	ErrorMessage(err);

	UINT8			uVectoreffect	= pLASeR->GetUINT8Attr(ATTR_VECTOR_EFFECT, &err);
	ErrorMessage(err);

	deque<float>    p_points        = pLASeR->GetFloatListAttr(ATTR_POINTS, &err);
	ErrorMessage(err);

	/************************************************************************/

	UINT8 len_points = (UINT8)(int)(p_points.size() / 2);
	len_points++;
	PointF *point;
	point = new PointF[len_points];
	len_points--;
	UINT8 k =0; //
	for (int i=0; i<len_points; i++)
	{
		point[i].X = p_points[k++];
		point[i].Y = p_points[k++];
	}
	/************************************************************************/

	// 090529 YongChul
	if(u_display == 1)
	{
		pLASeR->SetDeactivate(); /*none*/
	}
	else
	{
		if (uVisibility == 1) { /*hidden*/ }
		else
		{
			Graphics graphics(m_pDC->m_hDC);

			if (bDraw)
			{
				
				if (auFill.size() == 3)			// rgb
				{
					SolidBrush brush(Color(uFillOpacity, auFill[0], auFill[1], auFill[2]));
					graphics.TranslateTransform(afTranslate[0], afTranslate[1]);
					if (uVectoreffect != 1)
					{
						graphics.ScaleTransform(afScale[0], afScale[1]);
					}
					graphics.RotateTransform(afRotate[0]);
					graphics.FillPolygon(&brush, point, len_points);
				}

				if (auStroke.size() == 3)
				{
					Pen pen(Color(uStrokeOpacity, auStroke[0], auStroke[1], auStroke[2]), fStrokeWidth);
					graphics.DrawPolygon(&pen, point, len_points);
				}
			}
			else
			{
				SolidBrush brush(Color(255, 248, 248, 248));
				graphics.TranslateTransform(afTranslate[0], afTranslate[1]);
				if (uVectoreffect != 1)
				{
					graphics.ScaleTransform(afScale[0], afScale[1]);
				}
				graphics.RotateTransform(afRotate[0]);
				graphics.FillPolygon(&brush, point, len_points);

				Pen pen(Color(255, 248, 248, 248), fStrokeWidth);
				graphics.DrawPolygon(&pen, point, len_points);
			}
			ReleaseDC(m_pDC);
		}
	}
}

void CLASeRPlayerView::DrawPolygon(CLASeR *pLASeR, Graphics *graphics, UINT8 bDraw /* = TRUE */)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// get rect attributes
	UINT8           uVisibility     = pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);
	
	UINT8			u_display		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);

	deque<UINT8>	auFill			= pLASeR->GetUINT8ListAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	tstring			strFill			= pLASeR->GetStrAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	UINT8			uFillOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_FILL_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	deque<UINT8>	auStroke		= pLASeR->GetUINT8ListAttr(ATTR_STROKE, &err);
	ErrorMessage(err);
	UINT8			uStrokeOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_STROKE_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	float			fStrokeWidth	= pLASeR->GetFloatAttr(ATTR_STROKE_WIDTH, &err);
	ErrorMessage(err);

	deque<float>	afRotate		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
	ErrorMessage(err);
	deque<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
	ErrorMessage(err);
	deque<float>	afTranslate		= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
	ErrorMessage(err);

	UINT8			uVectoreffect	= pLASeR->GetUINT8Attr(ATTR_VECTOR_EFFECT, &err);
	ErrorMessage(err);

	deque<float>    p_points        = pLASeR->GetFloatListAttr(ATTR_POINTS, &err);
	ErrorMessage(err);

	/************************************************************************/

	UINT8 len_points = (UINT8)(int)(p_points.size() / 2);
	len_points++;
	PointF *point;
	point = new PointF[len_points];
	len_points--;
	UINT8 k =0; //
	for (int i=0; i<len_points; i++)
	{
		point[i].X = p_points[k++];
		point[i].Y = p_points[k++];
	}
	/************************************************************************/

	// 090529 YongChul
	if(u_display == 1)
	{
		pLASeR->SetDeactivate(); /*none*/
	}
	else
	{
		if (uVisibility == 1) { /*hidden*/ }
		else
		{
			if (bDraw)
			{

				if (auFill.size() == 3)			// rgb
				{
					SolidBrush brush(Color(uFillOpacity, auFill[0], auFill[1], auFill[2]));
					graphics->TranslateTransform(afTranslate[0], afTranslate[1]);
					if (uVectoreffect != 1)
					{
						graphics->ScaleTransform(afScale[0], afScale[1]);
					}
					graphics->RotateTransform(afRotate[0]);
					graphics->FillPolygon(&brush, point, len_points);
				}

				if (auStroke.size() == 3)
				{
					Pen pen(Color(uStrokeOpacity, auStroke[0], auStroke[1], auStroke[2]), fStrokeWidth);
					graphics->DrawPolygon(&pen, point, len_points);
				}
			}
			else
			{
				SolidBrush brush(Color(255, 248, 248, 248));
				graphics->TranslateTransform(afTranslate[0], afTranslate[1]);
				if (uVectoreffect != 1)
				{
					graphics->ScaleTransform(afScale[0], afScale[1]);
				}
				graphics->RotateTransform(afRotate[0]);
				graphics->FillPolygon(&brush, point, len_points);

				Pen pen(Color(255, 248, 248, 248), fStrokeWidth);
				graphics->DrawPolygon(&pen, point, len_points);
			}
		}
	}
}

void CLASeRPlayerView::DrawPolyline(CLASeR *pLASeR, UINT8 bDraw /* = TRUE */)
{
	lsrErr err = LASeRNoErr;

	m_pDC = GetDC();

	/************************************************************************/
	// get rect attributes
	UINT8           uVisibility     = pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);

	UINT8			u_display		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);

	deque<UINT8>	auFill			= pLASeR->GetUINT8ListAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	tstring			strFill			= pLASeR->GetStrAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	UINT8			uFillOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_FILL_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	deque<UINT8>	auStroke		= pLASeR->GetUINT8ListAttr(ATTR_STROKE, &err);
	ErrorMessage(err);
	UINT8			uStrokeOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_STROKE_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	float			fStrokeWidth	= pLASeR->GetFloatAttr(ATTR_STROKE_WIDTH, &err);
	ErrorMessage(err);

	deque<float>	afRotate		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
	ErrorMessage(err);
	deque<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
	ErrorMessage(err);
	deque<float>	afTranslate		= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
	ErrorMessage(err);

	UINT8			uVectoreffect	= pLASeR->GetUINT8Attr(ATTR_VECTOR_EFFECT, &err);
	ErrorMessage(err);

	deque<float>    p_points        = pLASeR->GetFloatListAttr(ATTR_POINTS, &err);
	ErrorMessage(err);

	/************************************************************************/

	UINT8 len_points = (UINT8)(int)(p_points.size() / 2);
	len_points++;
	PointF *point;
	point = new PointF[len_points];
	len_points--;
	UINT8 k =0; //
	for (int i=0; i<len_points; i++)
	{
		point[i].X = p_points[k++];
		point[i].Y = p_points[k++];
	}
	/************************************************************************/

	// 090529 YongChul
	if(u_display == 1)
	{
		pLASeR->SetDeactivate(); /*none*/
	}
	else
	{
		if (uVisibility == 1) { /*hidden*/ }
		else
		{
			Graphics graphics(m_pDC->m_hDC);

			if (bDraw)
			{

				if (auFill.size() == 3)			// rgb
				{
					SolidBrush brush(Color(uFillOpacity, auFill[0], auFill[1], auFill[2]));
					graphics.TranslateTransform(afTranslate[0], afTranslate[1]);
					if (uVectoreffect != 1)
					{
						graphics.ScaleTransform(afScale[0], afScale[1]);
					}
					graphics.RotateTransform(afRotate[0]);
					graphics.FillPolygon(&brush, point, len_points);
				}

				if (auStroke.size() == 3)
				{
					Pen pen(Color(uStrokeOpacity, auStroke[0], auStroke[1], auStroke[2]), fStrokeWidth);
					graphics.DrawPolygon(&pen, point, len_points);
				}
			}
			else
			{
				SolidBrush brush(Color(255, 248, 248, 248));
				graphics.TranslateTransform(afTranslate[0], afTranslate[1]);
				if (uVectoreffect != 1)
				{
					graphics.ScaleTransform(afScale[0], afScale[1]);
				}
				graphics.RotateTransform(afRotate[0]);
				graphics.FillPolygon(&brush, point, len_points);

				Pen pen(Color(255, 248, 248, 248), fStrokeWidth);
				graphics.DrawPolygon(&pen, point, len_points);
			}
			ReleaseDC(m_pDC);
		}
	}
}

void CLASeRPlayerView::DrawPolyline(CLASeR *pLASeR, Graphics *graphics, UINT8 bDraw /* = TRUE */)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// get rect attributes
	UINT8           uVisibility     = pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);

	UINT8			u_display		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);

	deque<UINT8>	auFill			= pLASeR->GetUINT8ListAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	tstring			strFill			= pLASeR->GetStrAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	UINT8			uFillOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_FILL_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	deque<UINT8>	auStroke		= pLASeR->GetUINT8ListAttr(ATTR_STROKE, &err);
	ErrorMessage(err);
	UINT8			uStrokeOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_STROKE_OPACITY, &err) * 255.0);
	ErrorMessage(err);
	float			fStrokeWidth	= pLASeR->GetFloatAttr(ATTR_STROKE_WIDTH, &err);
	ErrorMessage(err);

	deque<float>	afRotate		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
	ErrorMessage(err);
	deque<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
	ErrorMessage(err);
	deque<float>	afTranslate		= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
	ErrorMessage(err);

	UINT8			uVectoreffect	= pLASeR->GetUINT8Attr(ATTR_VECTOR_EFFECT, &err);
	ErrorMessage(err);

	deque<float>    p_points        = pLASeR->GetFloatListAttr(ATTR_POINTS, &err);
	ErrorMessage(err);

	/************************************************************************/

	UINT8 len_points = (UINT8)(int)(p_points.size() / 2);
	len_points++;
	PointF *point;
	point = new PointF[len_points];
	len_points--;
	UINT8 k =0; //
	for (int i=0; i<len_points; i++)
	{
		point[i].X = p_points[k++];
		point[i].Y = p_points[k++];
	}
	/************************************************************************/

	// 090529 YongChul
	if(u_display == 1)
	{
		pLASeR->SetDeactivate(); /*none*/
	}
	else
	{
		if (uVisibility == 1) { /*hidden*/ }
		else
		{
			if (bDraw)
			{

				if (auFill.size() == 3)			// rgb
				{
					SolidBrush brush(Color(uFillOpacity, auFill[0], auFill[1], auFill[2]));
					graphics->TranslateTransform(afTranslate[0], afTranslate[1]);
					if (uVectoreffect != 1)
					{
						graphics->ScaleTransform(afScale[0], afScale[1]);
					}
					graphics->RotateTransform(afRotate[0]);
					graphics->FillPolygon(&brush, point, len_points);
				}

				if (auStroke.size() == 3)
				{
					Pen pen(Color(uStrokeOpacity, auStroke[0], auStroke[1], auStroke[2]), fStrokeWidth);
					graphics->DrawPolygon(&pen, point, len_points);
				}
			}
			else
			{
				SolidBrush brush(Color(255, 248, 248, 248));
				graphics->TranslateTransform(afTranslate[0], afTranslate[1]);
				if (uVectoreffect != 1)
				{
					graphics->ScaleTransform(afScale[0], afScale[1]);
				}
				graphics->RotateTransform(afRotate[0]);
				graphics->FillPolygon(&brush, point, len_points);

				Pen pen(Color(255, 248, 248, 248), fStrokeWidth);
				graphics->DrawPolygon(&pen, point, len_points);
			}
		}
	}
}

void CLASeRPlayerView::DrawLSRSimpleLayout(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// get attributes
	deque<float>	delta			= pLASeR->GetFloatListAttr(ATTR_LSR_DELTA, &err);
	ErrorMessage(err);

	UINT8			u_display		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);

// 	deque<float>	afRotate		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
// 	ErrorMessage(err);
// 	deque<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
// 	ErrorMessage(err);
// 	deque<float>	afTranslate		= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
// 	ErrorMessage(err);

	deque<CLASeR *> child_simpleLayout = pLASeR->GetChildren();
	int sL_child_size = pLASeR->GetChildrenSize();

	// 090605 YongChul
	if(u_display == 1)
	{
		pLASeR->SetDeactivate(); /*none*/
	}
	else
	{
		for (int i=0; i<sL_child_size; i++)
		{
			if (child_simpleLayout[i]->GetElementType() == SVG_circle)
			{
				float circle_r = child_simpleLayout[i]->GetFloatAttr(ATTR_R, &err);
				child_simpleLayout[i]->SetAttr(ATTR_CX, 4+i*delta[0]+circle_r);
				child_simpleLayout[i]->SetAttr(ATTR_CY, 4+i*delta[1]+circle_r);
			}
			else
			{
				child_simpleLayout[i]->SetAttr(ATTR_X, 4+i*delta[0]);
				child_simpleLayout[i]->SetAttr(ATTR_Y, 4+i*delta[1]);
			}
			GetShapeElement(child_simpleLayout[i]);
			(child_simpleLayout[i])->SetDeactivate();
		}
	}
}

void CLASeRPlayerView::DrawSwitchcase(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	m_rCanvas.left   = 0;
	m_rCanvas.top    = 0;
	
	//float m_right = pDlg->m_nDisPlay;
	//float m_bottom = pDlg->m_nDisPlay;

	//if (m_uWidthUnit != 0)
	//{
	//	m_uWidthUnit = 0;
	//	ConvertLPtoDP(&m_right, &m_bottom);
	//	m_uWidthUnit = 5;

	//	m_rCanvas.right  = m_right;
	//	m_rCanvas.bottom = m_bottom;


	//	CMainFrame *pMainFrame = (CMainFrame*)AfxGetMainWnd();

	//	m_rCanvas.right += WINDOW_WIDTH;
	//	m_rCanvas.bottom += WINDOW_HEIGHT;
	//	pMainFrame->MoveWindow(m_rCanvas);
	//}
	//
	//test
	m_fDiagonal = GetDiagonalDisplaysize(m_rCanvas.right, m_rCanvas.bottom);


	UINT uminDisplaySize = pLASeR->GetUINTAttr(ATTR_LSR_MINDISPLAYSIZE, &err);
	ErrorMessage(err);

	deque<float> afminMemory = pLASeR->GetFloatListAttr(ATTR_LSR_MINMEMORY, &err);
	ErrorMessage(err);

	UINT uminResolution = pLASeR->GetUINTAttr(ATTR_LSR_MINRESOLUTION, &err);
	ErrorMessage(err);

	UINT           uminSize           = pLASeR->GetUINTAttr(ATTR_LSR_MINSIZE, &err);
	ErrorMessage(err);
	
	UINT8           uminSizeBehaviour  = pLASeR->GetUINT8Attr(ATTR_LSR_MINSIZEBEHAVIOR, &err);
	ErrorMessage(err);

	if (pDlg->m_nDisPlay < uminDisplaySize)
	{
		UINT8 uChildrenSize = pLASeR->GetChildrenSize();
		for (UINT8 i=0; i<uChildrenSize; i++)
		{
			CLASeR *pLASeRchild = pLASeR->GetChildren(i);		
			pLASeRchild->SetDeactivate();
		}
	}

	if (afminMemory.size() != 0)
	{
		float ftotalmemory = 0;

		ftotalmemory += afminMemory[0] * 4;
		ftotalmemory += afminMemory[1];
		ftotalmemory += afminMemory[2] * 1024;
		ftotalmemory += afminMemory[3] * 1024;
		ftotalmemory += afminMemory[4];

		ftotalmemory /= 1024;
		
		if (pDlg->m_nAvailablePhys < ftotalmemory)
		{
			UINT8 uChildrenSize = pLASeR->GetChildrenSize();
			for (UINT8 i=0; i<uChildrenSize; i++)
			{
				CLASeR *pLASeRchild = pLASeR->GetChildren(i);		
				pLASeRchild->SetDeactivate();
			}
		}
		pDlg->m_nAdaptationSelected = 1;
	}

	if (pDlg->m_nDpiX < uminResolution)
	{
		UINT8 uChildrenSize = pLASeR->GetChildrenSize();
		for (UINT8 i=0; i<uChildrenSize; i++)
		{
			CLASeR *pLASeRchild = pLASeR->GetChildren(i);		
			pLASeRchild->SetDeactivate();
		}
	}

	if (uminSizeBehaviour == 0)
	{
		UINT8 uchildnum = pLASeR->GetChildrenSize();
		for (UINT8 i=0; i<uchildnum; i++)
		{
			CLASeR *pChild = pLASeR->GetChildren(i);
			UINT   uFont_size = (UINT)pChild->GetFloatAttr(ATTR_FONT_SIZE, &err);
			ErrorMessage(err);

			if (uFont_size < uminSize)
			{
				pChild->SetAttr(ATTR_FONT_SIZE, (float)uminSize);
			}
		}
	}
	else if (uminSizeBehaviour == 1)
	{
		UINT8 uchildnum = pLASeR->GetChildrenSize();
		for (UINT8 i=0; i<uchildnum; i++)
		{
			CLASeR *pChild = pLASeR->GetChildren(i);
			UINT   uFont_size = (UINT)pChild->GetFloatAttr(ATTR_FONT_SIZE, &err);
			ErrorMessage(err);
			if (uFont_size < uminSize)
			{
				pChild->SetDeactivate();
			}
		}
	}
}

void CLASeRPlayerView::DrawparsingSwitchcase(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	UINT8 umode = pLASeR->GetUINT8Attr(ATTR_LSR_MODE, &err);
	ErrorMessage(err);

	if (umode == 0)
	{
		UINT8 uChildrenSize = pLASeR->GetChildrenSize();
		for (UINT8 i=0; i<uChildrenSize; i++)
		{
			CLASeR *pLASeRchild = pLASeR->GetChildren(i);

			deque<float> afminMemory = pLASeRchild->GetFloatListAttr(ATTR_LSR_MINMEMORY, &err);
			ErrorMessage(err);

			if (afminMemory.size() != 0)
			{
				float ftotalmemory = 0;

				ftotalmemory += afminMemory[0] * 4;
				ftotalmemory += afminMemory[1];
				ftotalmemory += afminMemory[2] * 1024;
				ftotalmemory += afminMemory[3] * 1024;
				ftotalmemory += afminMemory[4];

				ftotalmemory /= 1024;

				if (pDlg->m_nAvailablePhys > ftotalmemory)
				{
					for (int k=i+1; k<uChildrenSize; k++)
					{
						CLASeR *pLASeRchilds = pLASeR->GetChildren(k);
						pLASeRchilds->SetDeactivate();
					}
				}
			}
		}
	}
	else if (umode == 1)
	{
		UINT8 uChildrenSize = pLASeR->GetChildrenSize();
		uChildrenSize -= 1;
		for (int i=uChildrenSize; i>=0; i--)
		{
			CLASeR *pLASeRchild = pLASeR->GetChildren(i);

			deque<float> afminMemory = pLASeRchild->GetFloatListAttr(ATTR_LSR_MINMEMORY, &err);
			ErrorMessage(err);

			if (afminMemory.size() != 0)
			{
				float ftotalmemory = 0;

				ftotalmemory += afminMemory[0] * 4;
				ftotalmemory += afminMemory[1];
				ftotalmemory += afminMemory[2] * 1024;
				ftotalmemory += afminMemory[3] * 1024;
				ftotalmemory += afminMemory[4];

				ftotalmemory /= 1024;

				if (pDlg->m_nAvailablePhys > ftotalmemory)
				{
					for (int k=i-1; k>=0; k--)
					{
						CLASeR *pLASeRchilds = pLASeR->GetChildren(k);
						pLASeRchilds->SetDeactivate();
					}
				}
			}
		}
	}
	else if (umode == 2)
	{
		UINT8 uChildrenSize = pLASeR->GetChildrenSize();
		for (UINT8 i=0; i<uChildrenSize; i++)
		{
			CLASeR *pLASeRchild = pLASeR->GetChildren(i);

			deque<float> afminMemory = pLASeRchild->GetFloatListAttr(ATTR_LSR_MINMEMORY, &err);
			ErrorMessage(err);

			if (afminMemory.size() != 0)
			{
				float ftotalmemory = 0;

				ftotalmemory += afminMemory[0] * 4;
				ftotalmemory += afminMemory[1];
				ftotalmemory += afminMemory[2] * 1024;
				ftotalmemory += afminMemory[3] * 1024;
				ftotalmemory += afminMemory[4];

				ftotalmemory /= 1024;

				if (pDlg->m_nAvailablePhys > ftotalmemory)
				{
					for (int k=i+1; k<uChildrenSize; k++)
					{
						CLASeR *pLASeRchilds = pLASeR->GetChildren(k);
						pLASeRchilds->SetDeactivate();
					}
				}
				else
				{
					for (int k=i; k<uChildrenSize; k++)
					{
						CLASeR *pLASeRchilds = pLASeR->GetChildren(k);
						pLASeRchilds->SetDeactivate();
					}
				}
			}
		}
	}
}

// 090625 YongChul
void CLASeRPlayerView::DrawText(CLASeR *pLASeR, UINT8 bDraw /* = TRUE */)
{
	lsrErr err = LASeRNoErr;

	m_pDC = GetDC();

	/************************************************************************/

	UINT8           uVisibility		= pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);

	UINT8			u_display		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);

	tstring         strFont_family	= pLASeR->GetStrAttr(ATTR_FONT_FAMILY, &err);
	ErrorMessage(err);
	CStringW str_font_family(strFont_family.c_str());
	UINT            uFont_size		= (UINT)pLASeR->GetFloatAttr(ATTR_FONT_SIZE, &err);
	ErrorMessage(err);
	UINT8			uFont_style		= pLASeR->GetUINT8Attr(ATTR_FONT_STYLE, &err);
	ErrorMessage(err);
	UINT8			uFont_variant	= pLASeR->GetUINT8Attr(ATTR_FONT_VARIANT, &err);
	ErrorMessage(err);

	tstring			strText_value	= pLASeR->GetStrAttr(ATTR_TEXTCONTENT, &err);
	ErrorMessage(err);
	CStringW str_text(strText_value.c_str());

	deque<UINT8>	auFill			= pLASeR->GetUINT8ListAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	tstring			strFill			= pLASeR->GetStrAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	UINT8			uFillOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_FILL_OPACITY, &err) * 255.0);
	ErrorMessage(err);

	int				nX				= (int)pLASeR->GetFloatAttr(ATTR_X, &err);
	ErrorMessage(err);
	int				nY				= (int)pLASeR->GetFloatAttr(ATTR_Y, &err);
	ErrorMessage(err);

	/************************************************************************/

	if(u_display == 1)
	{
		pLASeR->SetDeactivate(); /*none*/
	}
	else
	{
		if (uVisibility == 1) { /*hidden*/ }
		else
		{
			Graphics graphics(m_pDC->m_hDC);

			if (bDraw)
			{
				FontFamily fontFamily(str_font_family);
				Font font(&fontFamily, uFont_size, uFont_style, UnitPixel);
				PointF pointF(nX, nY);
				SolidBrush brush(Color(uFillOpacity, auFill[0], auFill[1], auFill[2]));

				graphics.DrawString(str_text, (INT)wcslen(str_text), &font, pointF, &brush);
			}
			ReleaseDC(m_pDC);
		}
	}
}

// 090626 YongChul
void CLASeRPlayerView::DrawText(CLASeR *pLASeR, Graphics *graphics, UINT8 bDraw /* = TRUE */)
{
	lsrErr err = LASeRNoErr;

	m_pDC = GetDC();

	/************************************************************************/

	UINT8           uVisibility		= pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);

	UINT8			u_display		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);

	tstring         strFont_family	= pLASeR->GetStrAttr(ATTR_FONT_FAMILY, &err);
	ErrorMessage(err);
	CStringW str_font_family(strFont_family.c_str());
	UINT            uFont_size		= (UINT)pLASeR->GetFloatAttr(ATTR_FONT_SIZE, &err);
	ErrorMessage(err);
	UINT8			uFont_style		= pLASeR->GetUINT8Attr(ATTR_FONT_STYLE, &err);
	ErrorMessage(err);
	UINT8			uFont_variant	= pLASeR->GetUINT8Attr(ATTR_FONT_VARIANT, &err);
	ErrorMessage(err);

	tstring			strText_value	= pLASeR->GetStrAttr(ATTR_TEXTCONTENT, &err);
	ErrorMessage(err);
	CStringW str_text(strText_value.c_str());

	deque<UINT8>	auFill			= pLASeR->GetUINT8ListAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	tstring			strFill			= pLASeR->GetStrAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	UINT8			uFillOpacity	= (UINT8)(pLASeR->GetFloatAttr(ATTR_FILL_OPACITY, &err) * 255.0);
	ErrorMessage(err);

	int				nX				= (int)pLASeR->GetFloatAttr(ATTR_X, &err);
	ErrorMessage(err);
	int				nY				= (int)pLASeR->GetFloatAttr(ATTR_Y, &err);
	ErrorMessage(err);

	/************************************************************************/

	if(u_display == 1)
	{
		pLASeR->SetDeactivate(); /*none*/
	}
	else
	{
		if (uVisibility == 1) { /*hidden*/ }
		else
		{
			if (bDraw)
			{
				FontFamily fontFamily(str_font_family);
				Font font(&fontFamily, uFont_size, uFont_style, UnitPixel);
				PointF pointF(nX, nY);
				SolidBrush brush(Color(uFillOpacity, auFill[0], auFill[1], auFill[2]));

				graphics->DrawString(str_text, (INT)wcslen(str_text), &font, pointF, &brush);
			}
			ReleaseDC(m_pDC);
		}
	}
}
