///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "stdafx.h"
#include "JS_Functions.h"

#include "jsstr.h"

#include "..\src\SceneTreeParser\SceneTreeManager.h"

/************************************************************************/
// element
JSObject *element;

static JSPropertySpec apropElement[] = {
	// regular
	{"parentNode", 1, 0},
	// for convenient
	{"element", 1, 0},
	{0},
};

static JSFunctionSpec afuncElement[] = {
	{"appendChild", JAppendChild, 0, 0, 0},
	{"getAttribute", JGetAttribute, 0, 0, 0},
	{"getOwnerDocument", JGetOwnerDocument, 0, 0, 0},
	{"getStyle", JGetStyle, 0, 0, 0},
	{"removeChild", JRemoveChild, 0, 0, 0},
	{"setAttribute", JSetAttribute, 0, 0, 0},
	{0},
};

JSBool SetElementProperties(JSContext *ctx, JSObject *obj, jsval jsElement)
{
	if (jsElement == 0)
		return JS_TRUE;

	JSObject	*parent		= NULL;
	jsval		jsParent	= 0;

	CLASeR		*pLASeRElement	= (CLASeR *)JSVAL_TO_OBJECT(jsElement);
	jsval		jsLASeRParent	= OBJECT_TO_JSVAL(pLASeRElement->GetParent());

	if (!InitClass(ctx, &parent, afuncElement))
		return JS_FALSE;

	if (!JS_DefineProperties(ctx, parent, apropElement))
		return JS_FALSE;

	jsParent = OBJECT_TO_JSVAL(parent);

	if (!JS_SetProperty(ctx, obj, "element", &jsElement))
		return JS_FALSE;

	if (!JS_SetProperty(ctx, obj, "parentNode", &jsParent))
		return JS_FALSE;

	if (!JS_SetProperty(ctx, parent, "element", &jsLASeRParent))
		return JS_FALSE;

	return JS_TRUE;
}

/************************************************************************/
// doc
JSObject *doc;

static JSFunctionSpec afuncDoc[] = {
	{"createElement", JCreateElement, 0, 0, 0},
	{"getDocumentElement", JGetDocumentElement, 0, 0, 0},
	{"getElementById", JGetElementById, 0, 0, 0},
	{0},
};

/************************************************************************/
// style
JSObject *style;

static JSPropertySpec apropStyle[] = {
	{"properties", 1, 0},
	{0},
};

static JSFunctionSpec afuncStyle[] = {
	{"getPropertyValue", JGetPropertyValue, 0, 0, 0},
	{"setProperty", JSetProperty, 0, 0, 0},
	{0},
};


/************************************************************************/
JSBool InitClass(JSContext *ctx, JSObject **obj, JSFunctionSpec *afuncA)
{
	*obj = JS_NewObject(ctx, NULL, NULL, NULL);

	if (!(*obj))
		return JS_FALSE;

	if (!JS_InitStandardClasses(ctx, *obj))
		return JS_FALSE;

	if (!JS_DefineFunctions(ctx, *obj, afuncA))
		return JS_FALSE;

	return JS_TRUE;
}

JSBool GetAttrValue(JSContext *ctx, void *pElement, TCHAR *szParam, jsval *rval)
{
	lsrErr err = LASeRNoErr;

	CLASeR	*pLASeR			= (CLASeR *)pElement;

	// get the attribute type & value type
	UINT	uAttrType		= pLASeR->GetAttrType(szParam);
	UINT8	uAttrValType	= pLASeR->GetAttrValType(uAttrType);

	float			fValue		= 0.0;
	UINT8			uValue8		= 0;
	UINT			uValue		= 0;
	tstring			strValue	= _T("");
	deque<UINT8>	auValue8;

	switch (uAttrValType)
	{
	case FLOAT_TYPE:
		fValue	= pLASeR->GetFloatAttr(uAttrType, &err);
		*rval	= DOUBLE_TO_JSVAL((double)fValue);
		break;

	case UINT8_TYPE:
		uValue8	= pLASeR->GetUINT8Attr(uAttrType, &err);
		*rval	= INT_TO_JSVAL((int)uValue8);
		break;

	case UINT32_TYPE:
		uValue	= pLASeR->GetUINTAttr(uAttrType, &err);
		*rval	= INT_TO_JSVAL((int)uValue);
		break;

	case STRING_TYPE:
		// need to add
		break;

	case UINT8_LIST_TYPE:
		auValue8 = pLASeR->GetUINT8ListAttr(uAttrType, &err);

		/************************************************************************/
		// exception case (color)
		{
			char *szBuf = (char *)JS_malloc(ctx, 10);

			if (!szBuf)
				return JS_FALSE;

			if (auValue8[0] == 255)
			{
				if ((auValue8[1] == 0) && (auValue8[2] == 0))
					_tcscpy(szBuf, "red");

				if ((auValue8[1] == 192) && (auValue8[2] == 203))
					_tcscpy(szBuf, "pink");

				else if ((auValue8[1] == 255) && (auValue8[2] == 0))
					_tcscpy(szBuf, "yellow");

				else if ((auValue8[1] == 0) && (auValue8[2] == 255))
					_tcscpy(szBuf, "magenta");
			}

			else if (auValue8[0] == 0)
			{
				if ((auValue8[1] == 255) && (auValue8[2] == 0))
					_tcscpy(szBuf, "green");

				else if ((auValue8[1] == 255) && (auValue8[2] == 255))
					_tcscpy(szBuf, "cyan");

				else if ((auValue8[1] == 0) && (auValue8[2] == 255))
					_tcscpy(szBuf, "blue");
			}

			*rval = STRING_TO_JSVAL(JS_NewString(ctx, szBuf, strlen(szBuf)));
		}

		break;

	default:
		/* nothing */
		break;
	}

	return JS_TRUE;
}

JSBool SetAttrValue(JSContext *ctx, void *pElement, jsval *argv)
{
	CLASeR	*pLASeR		= (CLASeR *)pElement;

	TCHAR	*szParam1	= NULL;
	TCHAR	*szParam2	= NULL;
	double	dParam2		= 0.0;

	// convert 1st parameter
	if (JSVAL_IS_STRING(argv[0]))
		szParam1 = JS_GetStringBytes(JS_ValueToString(ctx, argv[0]));

	else
		return JS_FALSE;

	// convert 2nd parameter
	if (JSVAL_IS_NUMBER(argv[1]))
		JS_ValueToNumber(ctx, argv[1], &dParam2);

	else if (JSVAL_IS_STRING(argv[1]))
		szParam2 = JS_GetStringBytes(JS_ValueToString(ctx, argv[1]));

	else
		return JS_FALSE;

	// get attribute type
	UINT uAttrType = pLASeR->GetAttrType(szParam1);

	// set new attribute value
	if (dParam2 != 0.0)
		pLASeR->SetAttr(uAttrType, (float)dParam2);

	else if (uAttrType == ATTR_ID)
		pLASeR->SetID(szParam2);

	else
		pLASeR->SetAttr(uAttrType, szParam2);

	return JS_TRUE;
}

/************************************************************************/
JSBool JGetTarget(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (!InitClass(ctx, &element, afuncElement))
		return JS_FALSE;

	if (!JS_DefineProperties(ctx, element, apropElement))
		return JS_FALSE;

	jsval jsElement = 0;

	if (!JS_GetProperty(ctx, obj, "target", &jsElement))
		return JS_FALSE;

	// define properties
	if (!SetElementProperties(ctx, element, jsElement))
		return JS_FALSE;

	*rval = OBJECT_TO_JSVAL(element);

	return JS_TRUE;
}

JSBool JAppendChild(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 1)
		return JS_FALSE;

	jsval jsParent = 0;

	if (!JS_GetProperty(ctx, obj, "element", &jsParent))
		return JS_FALSE;

	// get the svg element
	CLASeR *pParent = (CLASeR *)JSVAL_TO_OBJECT(jsParent);

	/************************************************************************/
	JSObject	*jsTargetObject = NULL;
	jsval		jsNewElement	= 0;

	if (JSVAL_IS_OBJECT(argv[0]))
		JS_ValueToObject(ctx, argv[0], &jsTargetObject);

	// get new LASeR element
	if (!JS_GetProperty(ctx, jsTargetObject, "element", &jsNewElement))
		return JS_FALSE;

	CLASeR *pNew = (CLASeR *)JSVAL_TO_OBJECT(jsNewElement);

	// why 256? Maximum children : 255
	pParent->InsertChild(256, pNew);

	// if having ID, save element pointer
	if (pNew->CheckHavingID())
		AddLASeRElementID(pNew);

	// set new parent object
	if (!InitClass(ctx, &element, afuncElement))
		return JS_FALSE;

	if (!JS_DefineProperties(ctx, element, apropElement))
		return JS_FALSE;

	jsval jsParentObject = OBJECT_TO_JSVAL(element);

	// set new LASeR element's parent
	if (!JS_SetProperty(ctx, jsTargetObject, "parentNode", &jsParentObject))
		return JS_FALSE;

	// set new LASeR element
	if (!JS_SetProperty(ctx, element, "element", &jsParent))
		return JS_FALSE;

	return JS_TRUE;
}

JSBool JGetAttribute(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 1)
		return JS_FALSE;

	jsval jsElement = 0;

	if (!JS_GetProperty(ctx, obj, "element", &jsElement))
		return JS_FALSE;

	// get the LASeR element
	void *pElement = (void *)JSVAL_TO_OBJECT(jsElement);

	// get the function parameter
	TCHAR *szParam = NULL;

	if (JSVAL_IS_STRING(argv[0]))
		szParam = JS_GetStringBytes(JS_ValueToString(ctx, argv[0]));

	else
		return JS_FALSE;

	// get the attribute value
	if (!GetAttrValue(ctx, pElement, szParam, rval))
		return JS_FALSE;

	return JS_TRUE;
}

JSBool JGetOwnerDocument(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (!InitClass(ctx, &doc, afuncDoc))
		return JS_FALSE;

	*rval = OBJECT_TO_JSVAL(doc);

	return JS_TRUE;
}

JSBool JGetStyle(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (!InitClass(ctx, &style, afuncStyle))
		return JS_FALSE;

	if (!JS_DefineProperties(ctx, style, apropStyle))
		return JS_FALSE;

	jsval jsProperties = 0;

	if (!JS_GetProperty(ctx, obj, "element", &jsProperties))
		return JS_FALSE;

	if (!JS_SetProperty(ctx, style, "properties", &jsProperties))
		return JS_FALSE;

	*rval = OBJECT_TO_JSVAL(style);

	return JS_TRUE;
}

JSBool JRemoveChild(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 1)
		return JS_FALSE;

	jsval jsParent = 0;

	if (!JS_GetProperty(ctx, obj, "element", &jsParent))
		return JS_FALSE;

	// get the svg element
	CLASeR *pParent = (CLASeR *)JSVAL_TO_OBJECT(jsParent);

	/************************************************************************/
	JSObject	*jsTargetObject	= NULL;
	jsval		jsElement		= 0;

	if (JSVAL_IS_OBJECT(argv[0]))
		JS_ValueToObject(ctx, argv[0], &jsTargetObject);

	// get the new LASeR element
	if (!JS_GetProperty(ctx, jsTargetObject, "element", &jsElement))
		return JS_FALSE;

	CLASeR *pRemove = (CLASeR *)JSVAL_TO_OBJECT(jsElement);

	// if having ID, delete element pointer
	if (pRemove->CheckHavingID())
		DeleteLASeRElementID(pRemove);

	// why 256? Maximum children : 255
	pParent->DeleteChild(pRemove->GetIndex());

	return JS_TRUE;
}

JSBool JSetAttribute(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 2)
		return JS_FALSE;

	jsval jsElement = 0;

	if (!JS_GetProperty(ctx, obj, "element", &jsElement))
		return JS_FALSE;

	// get the LASeR element
	void *pElement = (void *)JSVAL_TO_OBJECT(jsElement);

	// set the attribute value
	if (!SetAttrValue(ctx, pElement, argv))
		return JS_FALSE;

	return JS_TRUE;
}

JSBool JCreateElement(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 1)
		return JS_FALSE;

	if (!InitClass(ctx, &element, afuncElement))
		return JS_FALSE;

	/************************************************************************/
	// create element
	TCHAR	*szParam		= NULL;
	CLASeR	*pNewElement	= NULL;

	if (JSVAL_IS_STRING(argv[0]))
		szParam = JS_GetStringBytes(JS_ValueToString(ctx, argv[0]));

	else
		return JS_FALSE;

	if (szParam != NULL)
	{
		if (!_tcscmp(szParam, "circle"))
			pNewElement = new CSVGCircle();

		else if (!_tcscmp(szParam, "line"))
			pNewElement = new CSVGLine();

		else if (!_tcscmp(szParam, "path"))
			pNewElement = new CSVGPath();

		else if (!_tcscmp(szParam, "polygon"))
			pNewElement = new CSVGPolygon(SVG_polygon);

		else if (!_tcscmp(szParam, "rect"))
			pNewElement = new CSVGRect();
	}

	jsval jsNewElement = OBJECT_TO_JSVAL(pNewElement);

	if (!JS_SetProperty(ctx, element, "element", &jsNewElement))
		return JS_FALSE;
	/************************************************************************/

	*rval = OBJECT_TO_JSVAL(element);

	return JS_TRUE;
}

JSBool JGetDocumentElement(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (!InitClass(ctx, &element, afuncElement))
		return JS_FALSE;

	if (!JS_DefineProperties(ctx, element, apropElement))
		return JS_FALSE;

	jsval jsSvgRoot = OBJECT_TO_JSVAL(GetSVG());

	if (!JS_SetProperty(ctx, element, "element", &jsSvgRoot))
		return JS_FALSE;

	*rval = OBJECT_TO_JSVAL(element);

	return JS_TRUE;
}

JSBool JGetElementById(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 1)
		return JS_FALSE;

	if (!InitClass(ctx, &element, afuncElement))
		return JS_FALSE;

	TCHAR *szParam = NULL;

	/************************************************************************/
	if (JSVAL_IS_STRING(argv[0]))
		szParam = JS_GetStringBytes(JS_ValueToString(ctx, argv[0]));

	else
		return JS_FALSE;

	CLASeR	*pElement	= SearchLASeRElementUsingID(szParam);
	jsval	jsElement	= OBJECT_TO_JSVAL(pElement);

	// define properties
	if (!SetElementProperties(ctx, element, jsElement))
		return JS_FALSE;

	*rval = OBJECT_TO_JSVAL(element);

	return JS_TRUE;
}

JSBool JGetPropertyValue(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 1)
		return JS_FALSE;

	jsval jsProperties = 0;

	if (!JS_GetProperty(ctx, obj, "properties", &jsProperties))
		return JS_FALSE;

	// get the LASeR element
	void *pElement = (void *)JSVAL_TO_OBJECT(jsProperties);

	// get the function parameter
	TCHAR *szParam = NULL;

	if (JSVAL_IS_STRING(argv[0]))
		szParam = JS_GetStringBytes(JS_ValueToString(ctx, argv[0]));

	else
		return JS_FALSE;

	// get the attribute value
	if (!GetAttrValue(ctx, pElement, szParam, rval))
		return JS_FALSE;

	return JS_TRUE;
}

JSBool JSetProperty(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 2)
		return JS_FALSE;

	jsval jsProperties = 0;

	if (!JS_GetProperty(ctx, obj, "properties", &jsProperties))
		return JS_FALSE;

	// get the LASeR element
	void *pElement = (void *)JSVAL_TO_OBJECT(jsProperties);

	// set the attribute value
	if (!SetAttrValue(ctx, pElement, argv))
		return JS_FALSE;

	return JS_TRUE;
}