///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

// LASeRPlayerView.cpp : implementation of the CLASeRPlayerView class
//

#include "stdafx.h"
#include "LASeRPlayer.h"

#include "LASeRPlayerView.h"
#include "MainFrm.h"
#include <math.h>


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLASeRPlayerView

IMPLEMENT_DYNCREATE(CLASeRPlayerView, CView)

BEGIN_MESSAGE_MAP(CLASeRPlayerView, CView)
	//{{AFX_MSG_MAP(CLASeRPlayerView)
	ON_WM_SIZE()
	ON_WM_ERASEBKGND()
	ON_WM_LBUTTONDBLCLK()
	//}}AFX_MSG_MAP
	ON_WM_TIMER()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(ID_TERMINAL_STATE, &CLASeRPlayerView::OnTerminalState)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLASeRPlayerView construction/destruction

CLASeRPlayerView::CLASeRPlayerView()
{
	m_bHaveVideo	= FALSE;
	m_uVideoWidth	= 320;
	m_uVideoHeight	= 240;

	m_bLASeR		= FALSE;

	m_pDC			= NULL;

	m_uAniEleCount	= 0;

	m_uEventType	= 0;

	m_strScript		= _T("");

	m_pRuntime		= NULL;
	m_pContext		= NULL;
	m_pGlobal		= NULL;
	m_pEvent		= NULL;
	m_jsRVal		= 0;

	pDlg = new CTerminal_State;
}

CLASeRPlayerView::~CLASeRPlayerView()
{
	ClearInfo();
}

void CLASeRPlayerView::ClearInfo()
{
	// kill timers
	for (UINT8 i = 0; i < m_uAniEleCount; ++i)
	{
		if ((m_apAnimationAttrs[i])->uCount < (m_apAnimationAttrs[i])->uIncCount)
			KillTimer(i);
	}

	// delete memory allocation for animation
	for (deque<AnimationAttrs *>::iterator j = m_apAnimationAttrs.begin();
		j < m_apAnimationAttrs.end(); ++j)
	{
		if (*j)
		{
			delete *j;
			*j = NULL;
		}
	}

	if (m_apThread.size() > 0)
	{
		for (deque<CWinThread *>::iterator k = m_apThread.begin();
			k < m_apThread.end(); ++k)
			::WaitForSingleObject(*k, INFINITE);
	}

	// destroy context & runtime
	if ((m_pContext != NULL) && (m_pRuntime != NULL))
	{
		JS_DestroyContext(m_pContext);
		JS_DestroyRuntime(m_pRuntime);
		JS_ShutDown();

		m_pContext	= NULL;
		m_pRuntime	= NULL;
	}

	// clear LASeR scene tree
	CSAFSession *pRoot = GetSceneRoot();

	if (pRoot)
	{
		delete pRoot;
		pRoot = NULL;

		SetSceneRoot(NULL);
	}

	m_bHaveVideo	= FALSE;
	m_uVideoWidth	= 320;
	m_uVideoHeight	= 240;

	m_bLASeR		= FALSE;

	m_pDC			= NULL;

	m_uAniEleCount	= 0;

	m_apAnimationAttrs.clear();
	m_apThread.clear();

	m_afMotionList.clear();

	m_uEventType	= 0;

	m_apA.clear();
	m_apListener.clear();

	m_apTest.clear();
}

BOOL CLASeRPlayerView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CLASeRPlayerView drawing

void CLASeRPlayerView::OnDraw(CDC* pDC)
{
	// TODO: add draw code for native data here
	if (m_bLASeR)
		DrawShapeElement();
}

/////////////////////////////////////////////////////////////////////////////
// CLASeRPlayerView diagnostics

#ifdef _DEBUG
void CLASeRPlayerView::AssertValid() const
{
	CView::AssertValid();
}

void CLASeRPlayerView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CLASeRPlayerDoc* CLASeRPlayerView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CLASeRPlayerDoc)));
	return (CLASeRPlayerDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CLASeRPlayerView message handlers

void CLASeRPlayerView::OnInitialUpdate() 
{
	CMainFrame *pMainFrame = (CMainFrame*)AfxGetMainWnd();

	CBitmap bmpLoad;

	m_VideoPanel.Create(_T(""),
		WS_CHILD | WS_VISIBLE | SS_BITMAP | SS_CENTERIMAGE, CRect(0, 0, 10, 10),
		this);
	bmpLoad.LoadBitmap(IDB_BITMAP_VIDEOPANEL);
	m_VideoPanel.SetBitmap(bmpLoad);
	bmpLoad.Detach();

	// call MainFrame.SetMPlayerVideoWindow() here to set the video window handle
	// i have no way to call this in MainFrame, no initial function like CView
	pMainFrame->SetMPlayerVideoWindow();

	CView::OnInitialUpdate();
}

HWND CLASeRPlayerView::GetMPlayerVideoWindow(void)
{
	if (m_VideoPanel.m_hWnd != NULL)
		return m_VideoPanel.m_hWnd;

	return NULL;
}

void CLASeRPlayerView::PreparePanels(UINT8 bHaveVideo)
{
	CRect rClient;
	GetClientRect(&rClient);

	// save, for OnSize(), call KeepAspectRatio()?
	m_bHaveVideo = bHaveVideo;

	if (bHaveVideo)
		m_VideoPanel.ShowWindow(SW_SHOW);

	else
		m_VideoPanel.ShowWindow(SW_HIDE);

	if (!m_bLASeR)
	{
		m_VideoPanel.MoveWindow(rClient);
		m_bLASeR = FALSE;
	}
}

void CLASeRPlayerView::SetVideoWidth(int nVideoWidth)
{
	m_uVideoWidth = nVideoWidth;
}

void CLASeRPlayerView::SetVideoHeight(int nVideoHeight)
{
	m_uVideoHeight = nVideoHeight;
}

void CLASeRPlayerView::KeepAspectRatio(CRect rClientRect, CRect &rNewRect)
{
	float	fVideoAspect	= 0.0;
	float	fClientAspect	= 0.0;
	CPoint	pointCenter;
	UINT	uWidth			= 0;
	UINT	uHeight			= 0;

	rNewRect = rClientRect;

	if (m_uVideoHeight > 0)
		fVideoAspect = (float)(m_uVideoWidth / m_uVideoHeight);

	else
		fVideoAspect = 1;

	if (rClientRect.Height() > 0)
		fClientAspect = (float)(rClientRect.Width() / rClientRect.Height());

	else
		fClientAspect = 1;

	pointCenter = rClientRect.CenterPoint();

	if (fClientAspect > fVideoAspect)
	{
		uWidth = (UINT)(fVideoAspect * rClientRect.Height());
		rNewRect.left = pointCenter.x - uWidth / 2;
		rNewRect.right = rNewRect.left + uWidth;
	}

	else
	{
		uHeight = (UINT)((1.0 / fVideoAspect) * rClientRect.Width());
		rNewRect.top = pointCenter.y - uHeight / 2;
		rNewRect.bottom = rNewRect.top + uHeight;
	}
}

void CLASeRPlayerView::EnableClientEdge(UINT8 bEnable)
{
	if (bEnable)
		ModifyStyleEx(0, WS_EX_CLIENTEDGE);

	else
		ModifyStyleEx(WS_EX_CLIENTEDGE, 0);
}

BOOL CLASeRPlayerView::OnEraseBkgnd(CDC* pDC) 
{
	CRect rRect;
	GetClientRect(&rRect);
	pDC->FillSolidRect(rRect, RGB(248, 248, 248));

	return TRUE;
}

void CLASeRPlayerView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	CMainFrame *pMainFrame = (CMainFrame*)AfxGetMainWnd();
	pMainFrame->PostMessage(UWM_VIEW_MPLAYER_FULLSCREEN);

	CView::OnLButtonDblClk(nFlags, point);
}

void CLASeRPlayerView::DrawScene()
{
	m_bLASeR = TRUE;

	DrawNonShapeElement();
	DrawShapeElement();
}

void CLASeRPlayerView::DrawShapeElement()
{
	lsrErr err = LASeRNoErr;

	CSAF *pRoot = GetSceneRoot();

	if (!pRoot)
		return;

	CLASeR	*pLASeR	= NULL;
	CSAF	*pSAF	= pRoot;

	while (1)
	{
		err = GetNextObject(&pLASeR, &pSAF);
		ErrorMessage(err);

		if ((pLASeR != NULL) && (pSAF == NULL))
			GetShapeElement(pLASeR);

		else if ((pLASeR == NULL) && (pSAF != NULL))
		{
			if (SAF_EndOfSAFSession == pSAF->GetElementType())
				break;
		}
	}
}

void CLASeRPlayerView::DrawNonShapeElement()
{
	lsrErr err = LASeRNoErr;

	CSAF *pRoot = GetSceneRoot();

	if (!pRoot)
		return;

	CLASeR	*pLASeR	= NULL;
	CSAF	*pSAF	= pRoot;

	while (1)
	{
		err = GetNextObject(&pLASeR, &pSAF);
		ErrorMessage(err);

		if ((pLASeR != NULL) && (pSAF == NULL))
			GetNonShapeElement(pLASeR);

		else if ((pLASeR == NULL) && (pSAF != NULL))
		{
			if (SAF_EndOfSAFSession == pSAF->GetElementType())
				break;
		}
	}
}

void CLASeRPlayerView::GetShapeElement(CLASeR *pLASeR, UINT8 bDraw /* = TRUE */)
{
	UINT8 pElementType = pLASeR->GetElementType();

	switch (pElementType)
	{
	case SVG_circle:
		DrawCircle(pLASeR, bDraw);
		break;

	case SVG_ellipse:
		DrawEllipse(pLASeR, bDraw);
		break;

	case SVG_image:
		DrawImage(pLASeR, bDraw);
		break;

	case SVG_g:
		DrawSwitchcase(pLASeR);
		break;

	case SVG_line:
 		DrawLine(pLASeR, bDraw);
 		break;

	case SVG_path:
		DrawPath(pLASeR, bDraw);
		break;

	case SVG_polygon:
		DrawPolygon(pLASeR, bDraw);
		break;

	case SVG_polyline:
		DrawPolyline(pLASeR, bDraw);
		break;

	case SVG_rect:
		DrawRect(pLASeR, bDraw);
		break;

	case LSR_rectClip:
		DrawLSRRectClip(pLASeR);
		break;

	case LSR_simpleLayout:
		DrawLSRSimpleLayout(pLASeR);
		break;

	case SVG_text:
		DrawText(pLASeR, bDraw);
		break;

	default:
		/* nothing */
		break;
	}
}

void CLASeRPlayerView::GetNonShapeElement(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	UINT8 pElementType = pLASeR->GetElementType();

	switch (pElementType)
	{
	case SVG_a:
		{
			UINT8 u_display = pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);

			if(u_display != 1)
				m_apA.push_back(pLASeR);
			else
			{
				UINT8 u_numofchild = pLASeR->GetChildrenSize();
				for(UINT8 i=0; i<u_numofchild; i++)
				{
					CLASeR *childLASeR = pLASeR->GetChildren(i);
					childLASeR->SetDeactivate();
				}
				break;
			}
		}

		break;
		
	case SVG_animate:
		DrawAnimate(pLASeR);
		break;

	case SVG_animateColor:
		DrawAnimateColor(pLASeR);
		break;

	case SVG_animateMotion:
		DrawAnimateMotion(pLASeR);
		break;

	case SVG_animateTransform:
		DrawAnimateTransform(pLASeR);
		break;

	case SVG_defs:
		DrawDefs(pLASeR);
		break;

	case SVG_set:
		DrawSet(pLASeR);
		break;

	case SVG_script:
		m_strScript = pLASeR->GetStrAttr(ATTR_TEXTCONTENT, &err).c_str();
		ErrorMessage(err);

		DrawScript(m_strScript);
		break;

	case SVG_svg:
		DrawSVG(pLASeR);
		break;

	case SVG_video:
		DrawVideo(pLASeR);
		break;

	case LSR_animateScroll:
		DrawLSRAnimationScroll(pLASeR);
		break;

	case LSR_cursorManager:
		m_apcursor.push_back(pLASeR);
		break;

	case LSR_parsingSwitch:
		DrawparsingSwitchcase(pLASeR);
		break;

	case EV_listener:
		m_apListener.push_back(pLASeR);
		break;

	default:
		/* nothing */
		break;
	}
}

tstring	CLASeRPlayerView::GetSourceUsingHref(CLASeR *LASeRForDrawing)
{
	lsrErr err = LASeRNoErr;

	CSAF	*pSAF	= SearchSAFElementUsingID(LASeRForDrawing->GetStrAttr(ATTR_XLINK_HREF, &err));
	ErrorMessage(err);
	CLASeR	*pLASeR	= NULL;

	err = GetNextObject(&pLASeR, &pSAF);
	ErrorMessage(err);

	CSAFUnit *pES = (CSAFUnit *)pSAF;

	return pES->GetStrAttr(ATTR_SOURCE, &err);
}

void CLASeRPlayerView::OnTimer(UINT_PTR nIDEvent)
{
	// animation element type
	if ((m_apAnimationAttrs[nIDEvent])->pCLASeR->GetElementType() == LSR_animateScroll)
	{
		lsrErr err = LASeRNoErr;

		CLASeR *LASeRrectClip = SearchLASeRElementUsingID
			(m_apAnimationAttrs[nIDEvent]->pCLASeR->GetStrAttr(ATTR_XLINK_HREF, &err));

		DrawLSRRectClip(LASeRrectClip, FALSE);

		if (m_apAnimationAttrs[nIDEvent]->pCLASeR->GetUINT8Attr(ATTR_LSR_DIRECTION, &err) == 2)
		{
			UINT uX = (UINT)LASeRrectClip->GetFloatAttr(ATTR_X, &err);

			LASeRrectClip->SetAttr(ATTR_X, uX + 1);
		}

		DrawLSRRectClip(LASeRrectClip);
	}

	else
	{
		// animation element's parent
		CLASeR *pLASeR = (m_apAnimationAttrs[nIDEvent])->pLASeR;

		// cumulative
		if((m_apAnimationAttrs[nIDEvent])->uAccumulate == 0)
			GetShapeElement(pLASeR, TRUE);
		else
			// clear element
			GetShapeElement(pLASeR, FALSE);

		// in/decrease
		for (UINT i = 0; i < (m_apAnimationAttrs[nIDEvent])->uEleCount; ++i)
			(m_apAnimationAttrs[nIDEvent])->afMid[i] += (m_apAnimationAttrs[nIDEvent])->afIncrement[i];

		// update scene tree
		switch ((m_apAnimationAttrs[nIDEvent])->uAttrType)
		{
		case ATTR_FILL:
			{
				// change RGB from float to String 
				char szRColor[4], szGColor[4], szBColor[4];

				_itoa((int)(m_apAnimationAttrs[nIDEvent])->afMid[0], szRColor, 10);
				_itoa((int)(m_apAnimationAttrs[nIDEvent])->afMid[1], szGColor, 10);
				_itoa((int)(m_apAnimationAttrs[nIDEvent])->afMid[2], szBColor, 10);

				string	strRColor	= szRColor;
				string	strGColor	= szGColor;
				string	strBColor	= szBColor;

				tstring strRGBColor = _T("rgb(") + strRColor + _T(",") + strGColor + _T(",") + strBColor + _T(")");
				pLASeR->SetAttr((m_apAnimationAttrs[nIDEvent])->uAttrType, strRGBColor);
			}
			break;

		default:
			pLASeR->SetAttr((m_apAnimationAttrs[nIDEvent])->uAttrType, (m_apAnimationAttrs[nIDEvent])->afMid[0]);
			break;
		}

		// redraw element
		GetShapeElement(pLASeR, TRUE);
	}

	// count
	if ((m_apAnimationAttrs[nIDEvent])->uCount < (m_apAnimationAttrs[nIDEvent])->uIncCount)
		++(m_apAnimationAttrs[nIDEvent])->uCount;

	else
		KillTimer(nIDEvent);
}

void CLASeRPlayerView::OnLButtonDown(UINT nFlags, CPoint point)
{
	// TODO: ⿡ ޽ ó ڵ带 ߰ /Ǵ ⺻ ȣմϴ.
	m_uEventType = 5;

	if (m_apA.size() != 0)
		CompareEventA(point);

	if (m_apListener.size() != 0)
		CompareEventListener(point);

	// for test
	deque<CLASeR *>::iterator i;
	UINT8 bSelected = 0;

	for (i = m_apTest.begin(); i < m_apTest.end(); ++i)
	{
		bSelected = ConfirmEventRegion(*i, point);

		if (bSelected)	break;
	}
/*
	if (i != m_apTest.end())
	{
		lsrErr err = LASeRNoErr;

		CLASeR *pSelected = *i;

		if (pSelected->GetID() != _T(""))
		{
			jsval	jsTarget	= OBJECT_TO_JSVAL(pSelected);
			jsval	jsEvent		= OBJECT_TO_JSVAL(m_pEvent);

			if (!JS_SetProperty(m_pContext, m_pEvent, "target", &jsTarget))
			{
				err = LASeRJSEngineErr;
				ErrorMessage(err);
			}

			JS_CallFunctionName(m_pContext, m_pGlobal, (pSelected->GetID()).c_str(), 1, &jsEvent, &m_jsRVal);

			GetNonShapeElement(GetSVG());
			DrawShapeElement();
		}
	}
*/
	CView::OnLButtonDown(nFlags, point);
}

void CLASeRPlayerView::OnMouseMove(UINT nFlags, CPoint point)
{
	// TODO: ⿡ ޽ ó ڵ带 ߰ /Ǵ ⺻ ȣմϴ.
	m_uEventType = 21;
	
	if (m_apA.size() != 0)
		CompareEventA(point);

	else if (m_apcursor.size() !=0)
	{
		ReplaceCursor(point);
	}

	CView::OnMouseMove(nFlags, point);
}

void CLASeRPlayerView::OnSize(UINT nType, int cx, int cy)
{
	CView::OnSize(nType, cx, cy);

	m_uEventType = 35;

	if (m_apListener.size() !=0)
	{
		CPoint point;
		point.x = cx; 
		point.y = cy;
		CompareEventListener(point);
	}
}

float CLASeRPlayerView::GetDiagonalDisplaysize(int i_width, int i_height)
{
	float m_fDiagonal = sqrt((float)(i_width*i_width) + (float)(i_height*i_height));
	return m_fDiagonal;
}
void CLASeRPlayerView::OnTerminalState()
{
	// TODO: ⿡  ó ڵ带 ߰մϴ.
	pDlg->Create(IDD_TERMINAL_STATE, this);
	pDlg->ShowWindow(SW_SHOW);
}
