///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

// LASeRPlayerView.h : interface of the CLASeRPlayerView class
//
/////////////////////////////////////////////////////////////////////////////

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "LASeRPlayerDoc.h"
#include "Terminal_State.h"

#include "..\src\SceneTreeParser\SceneTreeManager.h"

#define XP_WIN
#include "jsapi.h"


struct AnimationAttrs
{
	CView			*pView;						// active view
	CLASeR			*pLASeR;					// target LASeR element
	CLASeR          *pCLASeR;                   // Current LASeR element
	UINT8           ueletype;                   // element type
	UINT8			uIndex;						// animation & thread index

	UINT8			uAttrType;					// target attribute
	UINT			uBegin;						// begin time
	deque<float>	afFrom;
	deque<float>	afMid;						// in/decrease
	deque<float>	afTo;
 	deque<float>	afBy;
	//UINT			uDur;
	float           uDur;
	UINT			uEleCount;					// attribate's count ex) color type : 3

	deque<float>	afIncrement;				// increment

	UINT			uIncCount;					// count max
	UINT			uCount;						// real count

	UINT8			uAccumulate;				// cumulative
	UINT8			uAdditive;					// additive

	UINT8			uCalcMode;					// calcMode
	deque<float>	afValues;					// values

// 	deque<UINT8>	auFill;
// 	tstring			strFill;

	UINT			uRepeatCount;				// repeatCount
	UINT			uRepeatDur;					// repeatDur
};

class CLASeRPlayerView : public CView
{
protected:										// create from serialization only
	CLASeRPlayerView();
	DECLARE_DYNCREATE(CLASeRPlayerView)

// Attributes
public:

private:
	CStatic m_VideoPanel;

	UINT8	m_bHaveVideo;
	UINT	m_uVideoWidth;
	UINT	m_uVideoHeight;

	UINT8   m_uWidthUnit;                      // SVG width type
	UINT8   m_uHeightUnit;                     // SVG height type

	UINT8	m_bLASeR;

	CRect	m_rCanvas;

	CDC		*m_pDC;

	/* --- adaptation --- */
	CTerminal_State *pDlg;

	/* --- animation --- */
	UINT8	m_uAniEleCount;

	deque<AnimationAttrs *> m_apAnimationAttrs;
	deque<CWinThread *>		m_apThread;

	deque<float>			m_afMotionList;

	/* --- event --- */
	UINT8   m_uEventType;

	float   m_fDiagonal;

	deque<CLASeR *>	m_apA;
	deque<CLASeR *>	m_apListener;
	deque<CLASeR *> m_apcursor;

	/* --- script --- */
	CString m_strScript;

	JSRuntime	*m_pRuntime;
	JSContext	*m_pContext;
	JSObject	*m_pGlobal;
	JSObject	*m_pEvent;

	JSClass		m_clsEvent;
	jsval		m_jsRVal;

	/* --- temporary test --- */
	deque<CLASeR *>	m_apTest;

// Operations
private:
	void	DrawAnimatedElement();
	void	DrawShapeElement();
	void    DrawNonShapeElement();
	void    Delete_rectClip(CLASeR *LASeRForDrawing);

	void	GetShapeElement(CLASeR *pLASeR, UINT8 bDraw = TRUE);
	void	GetAnimatedElement(CLASeR *pLASeR);
	void    GetNonShapeElement(CLASeR *pLASeR);

	tstring	GetSourceUsingHref(CLASeR *pLASeR);
	float   GetDiagonalDisplaysize(int i_width, int i_height);

	// Shape
	void    DrawCircle(CLASeR *pLASeR, UINT8 bDraw = TRUE);
	void    DrawCircle(CLASeR *pLASeR, Graphics *graphics, UINT8 bDraw = TRUE);
	void    DrawDefs(CLASeR *pLASeR);
	void	DrawEllipse(CLASeR *pLASeR, UINT8 bDraw = TRUE);
	void	DrawEllipse(CLASeR *pLASeR, Graphics *graphics, UINT8 bDraw = TRUE);
	void    DrawImage(CLASeR *pLASeR, UINT8 bDraw = TRUE);
	void    DrawImage(CLASeR *pLASeR, Graphics *graphics, UINT8 bDraw = TRUE);
	void    DrawLine(CLASeR *pLASeR, UINT8 bDraw = TRUE);
	void    DrawLine(CLASeR *pLASeR, Graphics *graphics, UINT8 bDraw = TRUE);
	void    DrawPath(CLASeR *pLASeR, UINT8 bDraw = TRUE);
	void    DrawPolygon(CLASeR *pLASeR, UINT8 bDraw = TRUE);
	void    DrawPolygon(CLASeR *pLASeR, Graphics *graphics, UINT8 bDraw = TRUE);
	void    DrawPolyline(CLASeR *pLASeR, UINT8 bDraw = TRUE);
	void    DrawPolyline(CLASeR *pLASeR, Graphics *graphics, UINT8 bDraw = TRUE);
	void	DrawRect(CLASeR *pLASeR, UINT8 bDraw = TRUE);
	void	DrawRect(CLASeR *pLASeR, Graphics *graphics, UINT8 bDraw = TRUE);
	void    DrawLSRSimpleLayout(CLASeR *pLASeR);
	void	DrawText(CLASeR *pLASeR, UINT8 bDraw = TRUE);
	void	DrawText(CLASeR *pLASeR, Graphics *graphics, UINT8 bDraw = TRUE);
	void    DrawUse(CLASeR *pLASeR);


	// non shape
	void	SetAniCommonAttrs(CLASeR *pLASeR);
	void    ConvertLPtoDP(float *f_width, float *f_height);

	void	DrawAnimate(CLASeR *pLASeR);
	void    DrawAnimateColor(CLASeR *pLASeR);
	void    DrawAnimateMotion(CLASeR *pLASeR);
	void    DrawAnimateTransform(CLASeR *pLASeR);
	void    DrawScript(CString strScript);
	void	DrawSVG(CLASeR *pLASeR);
	void    DrawSwitchcase(CLASeR *pLASeR);
	void	DrawVideo(CLASeR *pLASeR);
	void    DrawLSRAnimationScroll(CLASeR *pLASeR);
	void    DrawLSRConditional(CLASeR *pLASeR);
	void    DrawLSRRectClip(CLASeR *pLASeR, UINT8 bDraw = TRUE);
	void    DrawparsingSwitchcase(CLASeR *pLASeR);
	void    DrawSet(CLASeR *pLASeR);

	// event
	void    CompareEventA(CPoint point);
	void	CompareEventListener(CPoint point);
	UINT8	ConfirmEventRegion(CLASeR *pLASeR, CPoint point);
	void    ReplaceCursor(CPoint point);

public:
	CLASeRPlayerDoc* GetDocument();

	HWND	GetMPlayerVideoWindow();
	void	PreparePanels(UINT8 bHaveVideo);
	void	SetVideoWidth(int nVideoWidth);
	void	SetVideoHeight(int nVideoHeight);
	void	KeepAspectRatio(CRect ClientRect, CRect &NewRect);
	void	EnableClientEdge(UINT8 bEnable);

	void	ClearInfo();
	void	DrawScene();

	CRect	GetCanvas() const	{return m_rCanvas;}

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CLASeRPlayerView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual void OnInitialUpdate();
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CLASeRPlayerView();

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CLASeRPlayerView)
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

public:
	afx_msg void OnTimer(UINT_PTR nIDEvent);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnTerminalState();
};

UINT ThreadAni(LPVOID pParam);

#ifndef _DEBUG  // debug version in LASeRPlayerView.cpp
inline CLASeRPlayerDoc* CLASeRPlayerView::GetDocument()
   { return (CLASeRPlayerDoc*)m_pDocument; }
#endif