///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

// MPlayer.cpp : implementation file
//

#include "stdafx.h"
#include "MPlayer.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// TIMER
#define TIMER_HALF_SECOND          1
#define TIMER_ONE_SECOND           2
#define TIMER_MPLAYER_STARTING     3

#define MAX_VIDEO_WIDTH  9999
#define MAX_VIDEO_HEIGHT 9999


/////////////////////////////////////////////////////////////////////////////
// CMPlayer

BEGIN_MESSAGE_MAP(CMPlayer, CStatic)
	//{{AFX_MSG_MAP(CMPlayer)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMPlayer message handlers

// Construction of CMPlayer class
CMPlayer::CMPlayer()
{
	m_uOSMajorVersion		= 0;				// os major version
	m_strMPlayerPathName	= _T("");			// path name
	m_strSource				= _T("");			// source file name, for play
	m_strCmdLn				= _T("");			// command line
	m_hVideoPanel			= NULL;

	m_uStatus				= MPLAYER_STOP;		// current status, play, stop, pause, ...
	m_uVolume				= 100;
	m_bPendingVolume		= FALSE;			// No Pending Volume to be set
	m_bHaveVideo			= FALSE;

	m_fCacheFill			= 0.0;

	m_hReadFromChild		= NULL;
	m_hWriteToParent		= NULL;
	m_hWriteToChild			= NULL;
	m_hReadFromParent		= NULL;

	m_pThreadSendCmd		= NULL;
	m_pThreadMPlayerMonitor = NULL;

	m_bDisableMPlayerMonitor = FALSE;

	Reset();
}

// Deconstruction of CMPlayer class
CMPlayer::~CMPlayer()
{
	Stop();

	TerminateThread(m_piProcessInfo.hThread, 0);
	TerminateProcess(m_piProcessInfo.hProcess, 0);

	DestroyCmdPipe();

	// Do not call CleanUp() here, it will call KillTimer() which will cause
	// an exception in debug version
	TerminateSendCmdThread();
	TerminateMPlayerMonitorThread();
}

// Reset some member variables, called before start playing every time
void CMPlayer::Reset(void)
{
	m_uTimePos			= 0;
	m_bHaveVideo		= TRUE;

	m_fCacheFill		= 0.0;

	// Clip information (MP3 ID3 or something)
	m_uClipLength		= 0;

	// Audio information
	m_uAudioNCH			= 0;

	// Video information
	m_uVideoWidth		= 320;
	m_uVideoHeight		= 240;

	strcpy(m_szCmdString, "");
	m_bAllowToSendCmd	= TRUE;

	// We must GetCurrentTimer here, or the MPlayerMonitor will call Stop() immediately
	m_timeLastReadCmd = CTime::GetCurrentTime();
}

// Set the embedded video window handle (see MPlayer doc: slave.txt)
// Parameters:
//  [in]  HWND : handle of the video panel window
void CMPlayer::SetVideoPanelHandle(HWND hVideoPanel)
{
	m_hVideoPanel = hVideoPanel;
}

// Set the operating system major version number
// so i can determine to send pipe command or not
// Parameters:
//  [in]  int : operating system major version number
void CMPlayer::SetOSMajorVersion(UINT8 nOSMajorVersion)
{
	m_uOSMajorVersion = nOSMajorVersion;
}

// Set the source (media clip filename) for CMPlayer
// Parameters:
//  [in]  CString : source path name
void CMPlayer::SetSource(CString strSource)
{
	m_strSource = strSource;
}

// Set MPlayer Binary file path name
void CMPlayer::SetMPlayerPathName(CString strMPlayerPathName)
{
	m_strMPlayerPathName = strMPlayerPathName;
}

// Check if MPlayer binary file exist
UINT8 CMPlayer::CheckEngine()
{
	CFileFind fFind;
	CString strMPlayerBinaryFile = m_strMPlayerPathName;

	if (!fFind.FindFile(strMPlayerBinaryFile))
		return FALSE;

	return TRUE;
}

// Starting play back, create MPlayer.exe process, set MPLAYER_STARTING state
void CMPlayer::Play()
{
	if (m_uStatus != MPLAYER_STOP)
		return;

	if (CreateCmdPipe() != 1)
		return;

	Reset();

	if (StartSendCmdThread() != 1)
		return;

	if (!m_bDisableMPlayerMonitor)
	{
		// Start MPlayerMonitor Thread
		if (StartMPlayerMonitorThread() != 1)
			return;
	}

	memset(&m_piProcessInfo, 0, sizeof(m_piProcessInfo));
	memset(&m_siStartupInfo, 0, sizeof(m_siStartupInfo));

	m_siStartupInfo.cb = sizeof(m_siStartupInfo);
	m_siStartupInfo.hStdError = m_hWriteToParent;
	m_siStartupInfo.hStdOutput = m_hWriteToParent;
	m_siStartupInfo.hStdInput = m_hReadFromParent;
	m_siStartupInfo.wShowWindow = SW_HIDE;
	m_siStartupInfo.dwFlags |= STARTF_USESTDHANDLES | STARTF_USESHOWWINDOW;

	// Makeup Command Line
	MakeupCmdLn();

	// We need a writable buffer for the command (silly Windows restriction)
	LPTSTR cmd = m_strCmdLn.GetBuffer(m_strCmdLn.GetLength() + 1);
	BOOL bStarted;
	bStarted = ::CreateProcess(NULL,			// command is part of input string
		cmd,									// (writable) command string
		NULL,									// process security
		NULL,									// thread security
		TRUE,									// inherit handles flag
		CREATE_NO_WINDOW | DETACHED_PROCESS | CREATE_SUSPENDED,
		NULL,									// inherit environment
		NULL,									// inherit directory
		&m_siStartupInfo,						// STARTUPINFO
		&m_piProcessInfo);						// PROCESS_INFORMATION

	m_strCmdLn.ReleaseBuffer();

	if (bStarted)
	{
		// Now close the output pipes so we get true EOF/broken pipe
		::CloseHandle(m_hWriteToParent);
		::CloseHandle(m_hReadFromParent);

		// Set MPLAYER_STARTING state
		m_uStatus = MPLAYER_STARTING;
		// Start timer, read MPlayer.exe output via pipe in OnTimer()
		SetTimer(TIMER_MPLAYER_STARTING, 200, NULL);
		// Post a message to mainframe
		AfxGetApp()->m_pMainWnd->PostMessage(UWM_MPLAYER_STARTING, 0, 0);
	}

	else
	{
		// Failed to start
		DestroyCmdPipe();
		// Stop timer, terminate thread
		CleanUp();
		AfxGetApp()->m_pMainWnd->PostMessage(UWM_MPLAYER_FAILEDTOSTART, 0, 0);
	}
}

// Pause/Resume the playback
// Parameters:
//  [in]  BOOL : TRUE pause, FALSE resume
void CMPlayer::Pause(UINT8 bPause)
{
	if ((m_uStatus == MPLAYER_PLAYING) || (m_uStatus == MPLAYER_PAUSE))
	{
		if (bPause == TRUE)
		{
			Command("pause");
			m_uStatus = MPLAYER_PAUSE;
		}

		else
		{
			m_uStatus = MPLAYER_PLAYING;

			// If user set volume when pausing, set the value now
			if (m_bPendingVolume)
			{
				CString strVolume;
				strVolume.Format("volume %d 1 pause", m_uVolume);
				Command(strVolume);
				m_bPendingVolume = FALSE;
			}

			else
				Command("pause");
		}
	}
}

// Mute/Unmute the MPlayer
void CMPlayer::Mute()
{
	if ((m_uStatus == MPLAYER_PLAYING) || (m_uStatus == MPLAYER_PAUSE))
		Command("mute");
}

// Stop the playback, abort the starting up
void CMPlayer::Stop()
{
	if ((m_uStatus == MPLAYER_PLAYING) || (m_uStatus == MPLAYER_PAUSE) || (m_uStatus == MPLAYER_STARTING))
	{
		CWaitCursor	wait;
		CString		strOutput			= _T("");
		INT			nReturn				= 0;
		UINT		uCounter			= 0;
		UINT8		bCheckMPlayerQuit	= 0;

		if (m_uOSMajorVersion < 5)				// Below Windows 2000
		{
			bCheckMPlayerQuit = FALSE;			// Needless to check MPlayer output, because there is no ouput
			Sleep(200);							// Wait a little bit
		}

		else
			bCheckMPlayerQuit = TRUE;			// Check MPlayer output

		// Check MPlayer quit message can avoid noise when TerminateProcess
		while (bCheckMPlayerQuit)
		{
			Command("quit");

			Sleep(100);
			uCounter++;

			nReturn = ReadMPlayerOutput(strOutput);

			if (strOutput.Find("Exit") != -1)
				// MPlayer quited normally, good
				break;

			if (nReturn == -2)
				// MPlayer has terminated already, exit now
				break;

			// 50 for 5 seconds
			if ((nReturn != 1) && (uCounter > 50))
				// Waited too long, and nothing can be read, stop it
				break;
		}

		TerminateThread(m_piProcessInfo.hThread, 0);
		TerminateProcess(m_piProcessInfo.hProcess, 0);

		DestroyCmdPipe();
		CleanUp();

		m_uStatus = MPLAYER_STOP;
	}
}

// Set absolute volume
// Parameters:
//  [in]  int : volume to be set
void CMPlayer::SetVolume(UINT8 uVolume)
{
	if ((m_uStatus == MPLAYER_PLAYING) || (m_uStatus == MPLAYER_PAUSE) || (m_uStatus == MPLAYER_STOP) || (m_uStatus == MPLAYER_STARTING))
	{
		CString strVolume;

		if ((uVolume < 0) || (uVolume > 100))
			uVolume = 100;

		m_uVolume = uVolume;

		if (m_uStatus == MPLAYER_PLAYING)
		{
			strVolume.Format("volume %d 1", uVolume);
			Command(strVolume);
		}

		else
			m_bPendingVolume = TRUE;
	}
}

// Get current absolute volume
// Return values:
//  int : current absolute volume
UINT8 CMPlayer::GetVolume()
{
	return m_uVolume;
}

// Terminate MPlayer process, destroy command pipe
//  Called when MPlayer is terminated itself (usually the end of play)
void CMPlayer::AutoStop()
{
	if (m_uStatus == MPLAYER_PLAYING)
	{
		TerminateThread(m_piProcessInfo.hThread, 0);
		TerminateProcess(m_piProcessInfo.hProcess, 0);

		DestroyCmdPipe();
		CleanUp();

		m_uStatus = MPLAYER_STOP;
	}
}

// Get current MPlayer status
// Return values:
//  int : one of the MPlayer's status (ie: MPLAYER_PLAYING, MPLAYER_STOP)
UINT8 CMPlayer::GetStatus()
{
	return m_uStatus;
}

// Send command to MPlayer to get current time position
void CMPlayer::RequireTimePos()
{
	// Return if MPlayer is not running
	if (CheckActivate() != 0)
		return;

	// Don't send command when MPlayer is paused, or you will resume it?
	if (m_uStatus == MPLAYER_PAUSE)
		return;

	Command("get_time_pos");
}

// Get MPlayer current time position
// Return values:
//  long : MPlayer current time position
UINT64 CMPlayer::GetTimePos(void)
{
	return m_uTimePos;
}

// Seek to an absolute time position
// Parameters:
//  [in]  DWORD : time position to be seek
void CMPlayer::SeekTime(UINT64 uSeekTime)
{
	if (m_uStatus == MPLAYER_PLAYING)
	{
		CString strSeek;
		strSeek.Format("seek %ul 2", uSeekTime);

		Command(strSeek);
	}
}

// Seek relative time
// Parameters:
//  [in]  int : relative time position
//  [out] 
void CMPlayer::SeekRelativeTime(int nRelativeTime)
{
	if (m_uStatus == MPLAYER_PLAYING)
	{
		CString strSeek;
		strSeek.Format("seek %d 0", nRelativeTime);

		Command(strSeek);
	}
}

// Send command to MPlayer process by using pipe
// Parameters:
//  [in]  CString : command
// Return values:
//  int : 0 means success
//        -1 means fail
INT8 CMPlayer::Command(CString strCommand)
{
	// If we are not running Windows 2000 (or above), return immediately
	if ((m_uOSMajorVersion < 5) || (m_hWriteToChild == NULL) || (CheckActivate() != 0))
		return -1;

	strCommand = strCommand + "\n";

	if (WaitForSingleObject(m_hEventSendCmdDone, 0) == WAIT_OBJECT_0)
		m_bAllowToSendCmd = TRUE;

	if (m_bAllowToSendCmd)
	{
		m_bAllowToSendCmd = FALSE;

		// limit to 39 characters maximum
		strncpy(m_szCmdString, (LPSTR)(LPCTSTR)strCommand, 39);
		m_szCmdString[39] = '\x0';

		SetEvent(m_hEventSendCmdStart);
	}

	return 0;
}

// Create the command pipe, to communite with MPlayer process
// Return values:
//  int : non-zero means sucess, zero means failure or already created
INT8 CMPlayer::CreateCmdPipe(void)
{
	if (m_hReadFromChild != NULL)
		// already created
		return 0;

	// create pipe

	// The SECURITY_ATTRIBUTES is used to create inheritable handles
	// the last member of the structure is set to TRUE so that the handle
	// will be inheritable.
	m_saAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
	m_saAttr.lpSecurityDescriptor = NULL;
	m_saAttr.bInheritHandle = TRUE;

	HANDLE hReadFromChild;

	// The CreatePipe operation creates a single unidirectional anonymous pipe,
	// and returns handles to its read and write ends. They are, by the
	// SECURITY_ATTRIBUTES, inheritable handles.
	if (!::CreatePipe(&hReadFromChild, &m_hWriteToParent, &m_saAttr, 8192))
		// pipe failed
		return 0;

	if (!::DuplicateHandle(::GetCurrentProcess(), hReadFromChild, ::GetCurrentProcess(),
		&m_hReadFromChild, 0, TRUE, DUPLICATE_SAME_ACCESS))
	{
		// duplicate failed
		::CloseHandle(hReadFromChild);
		::CloseHandle(m_hWriteToParent);
		return 0;
	}

	::CloseHandle(hReadFromChild);

	HANDLE hWriteToChild;

	if (!::CreatePipe(&m_hReadFromParent, &hWriteToChild, &m_saAttr, 0))
		// pipe failed
		return 0;

	if (!::DuplicateHandle(::GetCurrentProcess(), hWriteToChild, ::GetCurrentProcess(),
		&m_hWriteToChild, 0, TRUE, DUPLICATE_SAME_ACCESS))
	{
		::CloseHandle(m_hReadFromParent);
		::CloseHandle(hWriteToChild);
		return 0;
	}

	::CloseHandle(hWriteToChild);

	return 1;
}

// Destroy the command pipe
// Return values:
//  int : -1 means no command pipe to be destroyed
//         0 means success
INT8 CMPlayer::DestroyCmdPipe(void)
{
	if (m_hReadFromChild == NULL)
		return -1;

	m_hReadFromChild = NULL;
	m_hWriteToParent = NULL;
	m_hWriteToChild = NULL;
	m_hReadFromParent = NULL;
	
	return 0;
}

// Check MPlayer process activate state
// Return values:
//  int :  0 MPlayer process is activated
//        -1 MPlayer process is terminated
INT8 CMPlayer::CheckActivate()
{
	DWORD nExitCode = 0;

	if (GetExitCodeProcess(m_piProcessInfo.hProcess, &nExitCode))
	{
		if (nExitCode == STILL_ACTIVE)
			return 0;

		else
			return -1;
	}

	return -2;
}

// Return if current media file has video stream
// Return values:
//  BOOL : media file has video or not
UINT8 CMPlayer::HaveVideo()
{
	return m_bHaveVideo;
}

void CMPlayer::MakeupCmdLn(void)
{
	CString strTemp;

	// Initialization
	m_strCmdLn = _T("");

	// MPlayer Exe Pathname
	strTemp.Format("\"%s\"", m_strMPlayerPathName);
	AddArgument(strTemp);

	// Video Panel Handler
	strTemp.Format("-wid %d", m_hVideoPanel);
	AddArgument(strTemp);

	AddArgument(_T("-priority normal"));

	AddArgument(_T("-quiet"));
	AddArgument(_T("-slave"));
	AddArgument(_T("-identify"));
	AddArgument(_T("-colorkey 0xF8F8F8"));

	AddArgument(_T("-double"));

	AddArgument(_T("-noautosub"));

	AddArgument(_T("-vo directx"));
	AddArgument(_T("-ao dsound"));

	// Source (Clip to be play)
	strTemp.Format("\"%s\"", _T(m_strSource));
	AddArgument(strTemp);
}

void CMPlayer::AddArgument(CString strArgument)
{
	m_strCmdLn += strArgument;
	m_strCmdLn += _T(" ");
}

// Get clip information: Length
// Return values:
//  long : ClipInfo Length
UINT64 CMPlayer::GetClipLength() const
{
	return m_uClipLength;
}

// return Audio Channel info
INT8 CMPlayer::GetAudioNCH() const
{
	return m_uAudioNCH;
}

// Get video information: Width
// Return values:
//  int : VideoInfo Width
int CMPlayer::GetVideoWidth() const
{
	if (m_uVideoWidth > MAX_VIDEO_WIDTH)
		return MAX_VIDEO_WIDTH;

	else
		return m_uVideoWidth;
}

// Get video information: Height
// Return values:
//  int : VideoInfo Height
int CMPlayer::GetVideoHeight() const
{
	if (m_uVideoHeight > MAX_VIDEO_HEIGHT)
		return MAX_VIDEO_HEIGHT;

	else
		return m_uVideoHeight;
}

// Get MPlayer startup information
// Return values:
//  int :  1 MPlayer started :)
//        -1 MPlayer exited, failed to start :(
//        -2 MPlayer interrupted ?
//        -3 MPlayer is too slow, no responce :( (obsoleted)
//        -4 MPlayer process is not even started :( (obsoleted)
//        -5 MPlayer started and terminated :(
//         0 N/A
INT8 CMPlayer::GetMPlayerStartupInfo()
{
	CString	strOutput	= _T("");
	INT8	nReturn		= 0;
	INT8	nReturnCode	= 0;					// MPlayer started or failed?

	// Return if MPlayer is not running
	if (CheckActivate() != 0)
		return -5;

	if (m_uStatus != MPLAYER_STARTING)
		return 0;

	while (1)
	{
		nReturn = ReadMPlayerOutput(strOutput);

		// MPlayer process is terminated?
		if (nReturn == -2)
		{
			nReturnCode = -5;
			break;
		}

		// No output? empty?
		if (nReturn != 1)
			break;

		// Cache fill: 12.50% (131072 bytes)
		if (strOutput.Find("Cache fill:") != -1)
		{
			strOutput = strOutput.Mid(12);
			sscanf(strOutput, "%f", &m_fCacheFill);

			if (m_fCacheFill > 100)
				m_fCacheFill = 100;

			continue;
		}

		// Check if no video
		if (strOutput.Find("Video: no video") != -1)
		{
			m_bHaveVideo = FALSE;
			continue;
		}

		if (strOutput.Find("ID_LENGTH") != -1)
		{
			strOutput = strOutput.Mid(10);
			sscanf(strOutput, "%ld", &m_uClipLength);

			// hard limit (99:59:59)
			if (m_uClipLength >= 359999L)
				m_uClipLength = 359999L;

			continue;
		}

		if (strOutput.Find("ID_AUDIO_NCH") != -1)
		{
			strOutput = strOutput.Mid(13);
			sscanf(strOutput, "%d", &m_uAudioNCH);
			continue;
		}

		if (strOutput.Find("ID_VIDEO_WIDTH") != -1)
		{
			strOutput = strOutput.Mid(15);
			sscanf(strOutput, "%d", &m_uVideoWidth);
			continue;
		}

		if (strOutput.Find("ID_VIDEO_HEIGHT") != -1)
		{
			strOutput = strOutput.Mid(16);
			sscanf(strOutput, "%d", &m_uVideoHeight);
			continue;
		}

		// for Audio-Only stream, wait for "Starting" signal
		if (m_bHaveVideo == FALSE)
		{
			if (strOutput.Find("Starting") != -1)
			{
				// MPlayer finally starting playback the clip, return
				nReturnCode = 1;
				break;
			}
		}

		// for A/V stream (or Video-Only), wait for "VO: [" signal
		// or you will crash MPlayerGUI in CMainFrame::RestoreFromMaxMin() if pipe buffer
		// is small and have some delay when empty the reading pipe
		// very weird!
		if (m_bHaveVideo == TRUE)
		{
			if (strOutput.Find("VO: [") == 0)
			{
				// MPlayer finally starting playback, return
				nReturnCode = 1;
				break;
			}
		}

		// MPlayer failed to play the clip, return
		if (strOutput.Find("Exiting") != -1)
		{
			nReturnCode = -1;
			break;
		}

		// Interrupted?
		if (strOutput.Find("interrupted") != -1)
		{
			nReturnCode = -2;
			break;
		}
	}

	return nReturnCode;
}

// Read MPlayer stdout via command pipe
// Parameters:
//  [out] CString & : MPlayer stdout string
// Return values:
//  int : -1 means pipe not available
//        -2 means MPlayer process is not running
//        -3 means MPlayer output nothing
//         1 means successed
INT8 CMPlayer::ReadMPlayerOutput(CString &strOutput)
{
	// Return if Pipe not available
	if (m_hReadFromChild == NULL)
		return -1;

	// Return if MPlayer is not running
	if (CheckActivate() != 0)
		return -2;

	DWORD	lBytesRemain;
	DWORD	lRead;
	char	chTestBuffer[100], chBuffer[5];

	strOutput = _T("*no output*");

	PeekNamedPipe(m_hReadFromChild, chTestBuffer, 80, &lBytesRemain, 0, 0);

	m_timeLastReadCmd = CTime::GetCurrentTime();

	// Only call ReadFile() if lBytesRemain is in range
	if ((lBytesRemain > 0) && (lBytesRemain < 100))
	{
		strOutput = _T("");

		for (UINT8 i = 0; i < lBytesRemain; ++i)
		{
			chBuffer[0] = '\x0';

			ReadFile(m_hReadFromChild, chBuffer, 1, &lRead, NULL);

			// newline, return
			if ((chBuffer[0] == '\n') || (chBuffer[0] == '\r'))
				break;
			
			strOutput = strOutput + chBuffer[0];
		}
	}

	else
		return -3;

	return 1;
}

// Get MPlayer output and store useful information to member variables
void CMPlayer::GetMPlayerInfo()
{
	CString	strOutput	= _T("");
	INT8	nReturn		= 0;

	// Return if MPlayer is not running
	if (CheckActivate() != 0)
		return;

	while (1)
	{
		strOutput = _T("");
		nReturn = ReadMPlayerOutput(strOutput);

		// No output? empty?
		if (nReturn != 1)
			break;

		if (strOutput.Find("ANS_TIME_POSITION") != -1)
		{
			UINT64 uTimePos = 0;

			strOutput = strOutput.Mid(18);
			sscanf(strOutput, "%ld", &uTimePos);

			// Only update m_uTimePos when legal (in range)
			if (uTimePos >= 0)
				m_uTimePos = uTimePos;

			// hard limit (99:59:59)
			if (m_uTimePos >= 359999L)
				m_uTimePos = 359999L;
			
			continue;
		}
	}
}

// After get enough info in OnTimer() -> GetMPlayerStartupInfo(), confirm MPlayer
// is actually started or just failed 
void CMPlayer::ConfirmMPlayerPlaying(INT8 nStatus)
{
	// MPlayer is actually started
	if (nStatus == 1)
	{
		if (CheckActivate() == 0)
		{
			// Stop the needless timer
			KillTimer(TIMER_MPLAYER_STARTING);
			// Start the routine timer
			SetTimer(TIMER_HALF_SECOND, 500, NULL);
			SetTimer(TIMER_ONE_SECOND, 1000, NULL);

			m_uStatus = MPLAYER_PLAYING;
			AfxGetApp()->m_pMainWnd->PostMessage(UWM_MPLAYER_STARTPLAYING, 0, 0);
		}
	}

	// MPlayer failed to start (various reasons)
	if ((nStatus == -1) || (nStatus == -2) || (nStatus == -5))
	{
		KillTimer(TIMER_MPLAYER_STARTING); // stop the needless timer
		// Cheating...
		m_uStatus = MPLAYER_PLAYING;
		// Called to cleanup
		AutoStop();

		m_uStatus = MPLAYER_STOP;
		AfxGetApp()->m_pMainWnd->PostMessage(UWM_MPLAYER_FAILEDTOSTART, 0, 0);
	}
}


// Check if MPlayer is stopped (by itself)
// Post a message to CMainFrame if MPlayer is stopped
void CMPlayer::CheckAutoStop()
{
	if (m_uStatus == MPLAYER_PLAYING)
	{
		if (CheckActivate() != 0)
		{
			AutoStop();

			AfxGetApp()->m_pMainWnd->PostMessage(UWM_MPLAYER_AUTOSTOPPED, 0, 0);
		}
	}
}

// After ::Stop ::AutoStop do some cleanup job
void CMPlayer::CleanUp()
{
	// Stop timers
	KillTimer(TIMER_HALF_SECOND);
	KillTimer(TIMER_ONE_SECOND);
	KillTimer(TIMER_MPLAYER_STARTING);

	TerminateSendCmdThread();
	TerminateMPlayerMonitorThread();
}

float CMPlayer::GetCacheFill()
{
	return m_fCacheFill;
}

UINT CMPlayer::SendCmd(LPVOID p)
{
	CMPlayer *me = (CMPlayer*)p;
	me->SendCmd();
	return 0;
}

void CMPlayer::SendCmd()
{
	DWORD	nWritten		= 0;
	HANDLE	hWriteToChild	= NULL;
	char	chCmdString[40];

	while (1)
	{
		if (WaitForSingleObject(m_hEventSendCmdStart, INFINITE) == WAIT_OBJECT_0)
		{
			TRACE("*** Thread: SendCmdStart\n");
			hWriteToChild = GetWriteToChildHandle();

			// terminate thread if the hWriteToChild is NULL
			//  which means MPlayer.exe process has already terminated, and pipe is closed
			if (hWriteToChild == NULL)
				break;

			// strcpy(chBuffer, m_chBuffer);
			GetCmdString(chCmdString);
			WriteFile(hWriteToChild, chCmdString, strlen(chCmdString), &nWritten, NULL);
			SetEvent(m_hEventSendCmdDone);
		}
	}
}

// for SendCmd Thread to get member variable: m_szCmdString
void CMPlayer::GetCmdString(char szCmdString[])
{
	strncpy(szCmdString, m_szCmdString, 39);
	szCmdString[39] = '\x0';
}

HANDLE CMPlayer::GetWriteToChildHandle()
{
	return m_hWriteToChild;
}

// Start (Create) the SendCmdThread
//  return code:
//    int : 0, already created
//          1, success
//         -1, failed
INT8 CMPlayer::StartSendCmdThread()
{
	if (m_pThreadSendCmd == NULL)
	{
		m_hEventSendCmdStart = CreateEvent(NULL, FALSE, FALSE, NULL);
		m_hEventSendCmdDone = CreateEvent(NULL, FALSE, FALSE, NULL);

		m_pThreadSendCmd = AfxBeginThread(SendCmd, this, 0, CREATE_SUSPENDED);

		if (!m_pThreadSendCmd)
		{
			// thread failed to create, close events and return
			CloseHandle(m_hEventSendCmdStart);
			CloseHandle(m_hEventSendCmdDone);

			m_hEventSendCmdStart = NULL;
			m_hEventSendCmdDone = NULL;

			return -1;
		}

		m_pThreadSendCmd->m_bAutoDelete = FALSE;
		m_pThreadSendCmd->ResumeThread();
		TRACE("Thread SendCmd created.\n");

		return 1;
	}

	return 0;
}

// Try to terminate the SendCmdThread
//  by repeatably SetEvent(m_hEventSendCmdStart), the SendCmdThread will realize the
//  hWriteToChild is NULL and terminate itself automatically
void CMPlayer::TerminateSendCmdThread()
{
	if (m_pThreadSendCmd != NULL)
	{
		TRACE("try to terminate thread: SendCmd\n");
		DWORD dwExitCode = 0;

		while (1)
		{
			SetEvent(m_hEventSendCmdStart);
			::GetExitCodeThread(m_pThreadSendCmd->m_hThread, &dwExitCode);

			if (dwExitCode != STILL_ACTIVE)
				break;

			if (WaitForSingleObject(m_pThreadSendCmd->m_hThread, 100) == WAIT_OBJECT_0)
				break;
		}

		delete m_pThreadSendCmd;
		m_pThreadSendCmd = NULL;

		CloseHandle(m_hEventSendCmdStart);
		CloseHandle(m_hEventSendCmdDone);

		m_hEventSendCmdStart = NULL;
		m_hEventSendCmdDone = NULL;

		TRACE("Thread: SendCmd terminated normally.\n");
	}
}

UINT CMPlayer::MPlayerMonitor(LPVOID p)
{
	CMPlayer *me = (CMPlayer*)p;
	me->MPlayerMonitor();
	return 0;
}

void CMPlayer::MPlayerMonitor()
{
	// HANDLE hWriteToChild;
	CTime timeCurrent;

	while (TRUE)
	{
		if (WaitForSingleObject(m_hEventMPlayerMonitorQuit, 1000) == WAIT_OBJECT_0)
		{
			TRACE("*** Thread: m_hEventMPlayerMonitorQuit\n");
			break;
		}

		// Compare time, if more than 20 seconds the ReadMPlayerOutput() is not called
		// terminate the MPlayer process
		timeCurrent = CTime::GetCurrentTime();

		// m_timeLastReadCmd is set in ReadMPlayerOutput()
		if ((timeCurrent - m_timeLastReadCmd) > 20)
		{
			// Stop(); // don't call Stop() here, or we will crash the application
			// TODO: analyze the reason why Stop() crash the app here?
			// Terminate MPlayer process by use the TerminateProcess(), die hard!
			TerminateThread(m_piProcessInfo.hThread, 0);
			TerminateProcess(m_piProcessInfo.hProcess, 0);
		}

		TRACE("MPlayerMonitor running... Check: %d\n", timeCurrent - m_timeLastReadCmd);
	}

	TRACE("Yes, MPlayerMonitor thread terminated!\n");
}

int CMPlayer::StartMPlayerMonitorThread()
{
	if (m_pThreadMPlayerMonitor == NULL)
	{
		m_hEventMPlayerMonitorQuit = CreateEvent(NULL, FALSE, FALSE, NULL);

		m_pThreadMPlayerMonitor = AfxBeginThread(MPlayerMonitor, this, 0, CREATE_SUSPENDED);

		if (!m_pThreadMPlayerMonitor)
		{
			// thread failed to create, close events and return
			CloseHandle(m_hEventMPlayerMonitorQuit);
			m_hEventMPlayerMonitorQuit = NULL;

			return -1;
		}

		m_pThreadMPlayerMonitor->m_bAutoDelete = FALSE;
		m_pThreadMPlayerMonitor->ResumeThread();
		TRACE("MPlayerMonitor Thread created.\n");

		return 1;
	}

	return 0;
}

void CMPlayer::TerminateMPlayerMonitorThread()
{
	if (m_pThreadMPlayerMonitor != NULL)
	{
		TRACE("try to terminate thread: MPlayerMonitor\n");
		DWORD dwExitCode;

		// To terminate MPlayerMonitor thread...
		while (TRUE)
		{
			SetEvent(m_hEventMPlayerMonitorQuit);
			::GetExitCodeThread(m_pThreadMPlayerMonitor->m_hThread, &dwExitCode);

			if (dwExitCode != STILL_ACTIVE)
			{
				TRACE("MPlayerMonitor Terminated: STILL_ACTIVE\n");
				break;
			}

			if (WaitForSingleObject(m_pThreadMPlayerMonitor->m_hThread, 1000) == WAIT_OBJECT_0)
			{
				TRACE("MPlayerMonitor Terminated: WAIT_OBJECT_0\n");
				break;
			}

			TRACE("Waiting for termination of MPlayerMonitor...\n");
		}

		// Release memory
		delete m_pThreadMPlayerMonitor;
		m_pThreadMPlayerMonitor = NULL;

		// Close handle
		CloseHandle(m_hEventMPlayerMonitorQuit);
		m_hEventMPlayerMonitorQuit = NULL;

		TRACE("MPlayerMonitor Thread terminated normally.\n");
	}
}

void CMPlayer::SetDisableMPlayerMonitor(UINT8 bDisableMPlayerMonitor)
{
	m_bDisableMPlayerMonitor = bDisableMPlayerMonitor;
}

// WM_TIMER message handler
void CMPlayer::OnTimer(UINT nIDEvent) 
{
	switch (nIDEvent)
	{
	case TIMER_MPLAYER_STARTING:
		ResumeThread(m_piProcessInfo.hThread);

		{
			// When MPlayer is starting, read its output
			INT8 nReturn = 0;
			nReturn = GetMPlayerStartupInfo();
			ConfirmMPlayerPlaying(nReturn);
			break;
		}

		break;

	case TIMER_HALF_SECOND:
		RequireTimePos();						// Requiest time position
		GetMPlayerInfo();						// Get info (read pipe) every half second
		break;

	case TIMER_ONE_SECOND:
		CheckAutoStop();						// Check if MPlayer is stopped (end of play)
		break;

	default:
		/* nothing */
		break;
	}

	CStatic::OnTimer(nIDEvent);
}