///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000
// MPlayer.h : header file
//

// MPlayer status
enum MPLAYER_STATUS
{
  MPLAYER_STARTING,
  MPLAYER_PLAYING,
  MPLAYER_STOP,
  MPLAYER_PAUSE,
  MPLAYER_ERROR,
};

/////////////////////////////////////////////////////////////////////////////
// CMPlayer window

// CMPlayer is inherited from CStatic, so i can use the TIMER now
class CMPlayer : public CStatic
{
// Construction
public:
	CMPlayer();

// Attributes
public:
	// thread related
	char	m_szCmdString[40];
	CWinThread *m_pThreadSendCmd;
	HANDLE	m_hEventSendCmdStart, m_hEventSendCmdDone;
	UINT8	m_bAllowToSendCmd;

	// MPlayer Monitor
	CWinThread *m_pThreadMPlayerMonitor;
	HANDLE	m_hEventMPlayerMonitorQuit;
	
	CTime	m_timeLastReadCmd;
	UINT8	m_bDisableMPlayerMonitor;

private:
	// essential member variables
	CString	m_strMPlayerPathName;				// MPlayer Binary file path name
	CString	m_strSource;						// source filename, media clip filename
	CString	m_strCmdLn;							// command line
	HWND	m_hVideoPanel;						// handle of the embedded video panel

	// MPlayer information (general)
	UINT8	m_uStatus;							// current status, play, stop, pause, ...
	UINT64	m_uTimePos;							// current time position
	UINT8	m_uVolume;							// MPlayer volume
	UINT8	m_bPendingVolume;					// Pending Volume to be set (typacially when paused)
	UINT8	m_bHaveVideo;						// clip have video or not

	float	m_fCacheFill;						// cache fill

	// MPlayer information, these values will be set when MPlayer.exe is starting
	// via the pipe.
	// MPlayer information (clip information)
	UINT64	m_uClipLength;

	// MPlayer information (audio information)
	UINT8	m_uAudioNCH;

	// MPlayer information (video information)
	UINT	m_uVideoWidth;
	UINT	m_uVideoHeight;

	// Command pipe related member variables
	SECURITY_ATTRIBUTES m_saAttr;
	HANDLE	m_hReadFromChild;
	HANDLE	m_hWriteToParent;
	HANDLE	m_hWriteToChild;
	HANDLE	m_hReadFromParent;

	// MPlayer process related member variables
	STARTUPINFO m_siStartupInfo;
	PROCESS_INFORMATION m_piProcessInfo;

	// miscellaneous member variables
	UINT8	m_uOSMajorVersion;					// os major version

// Operations
public:
	void	Reset();							// when user start play a clip, reset media clip related info

	// set embedded video panel
	void	SetVideoPanelHandle(HWND hInnerPanel);

	// interface
	void	SetOSMajorVersion(UINT8 nOSMajorVersion);
	void	SetSource(CString strSource);
	void	SetMPlayerPathName(CString strMPlayerPathName);

	UINT8	CheckEngine();

	// essential interface
	void	Play();
	void	Pause(UINT8 bPause);
	void	Mute();
	void	Stop();

	void	SetVolume(UINT8 uVolume);			// set absolute volume
	UINT8	GetVolume();						// get absolute volume

	void	AutoStop();							// called after MPlayer process ends
	UINT8	GetStatus();

	void	RequireTimePos();					// send a get_time_pos command to MPlayer
	UINT64	GetTimePos();						// get current playing time

	void	SeekTime(UINT64 uSeekTime);			// seek to an absolute position of <value> seconds.
	// seek relative time
	void	SeekRelativeTime(int nRelativeTime);

	INT8	Command(CString strCommand);		// send command to MPlayer via pipe
	INT8	CreateCmdPipe();					// create the command pipe
	INT8	DestroyCmdPipe();					// destroy the command pipe
	INT8	CheckActivate();					// check if MPlayer process is still running

	UINT8	HaveVideo();						// return if media clip has video stream

	void	MakeupCmdLn(void);
	void	AddArgument(CString strArgument);

	// clip information interface (should be const)
	UINT64	GetClipLength() const;

	// audio information interface (should be const)
	INT8	GetAudioNCH() const;

	// video information interface (should be const)
	int		GetVideoWidth() const;
	int		GetVideoHeight() const;

	INT8	GetMPlayerStartupInfo();			// get MPlayer startup information
	// read MPlayer output via pipe
	INT8	ReadMPlayerOutput(CString &strOutput);
	void	GetMPlayerInfo(void);				// get MPlayer output via pipe
	// after MPlayer startup is done, confirm
	void	ConfirmMPlayerPlaying(INT8 nStatus);

	void	CheckAutoStop();					// check if MPlayer is stopped and post message to CMainFrame
	void	CleanUp();							// cleanup something after playback
	float	GetCacheFill();

	// thread related
	static UINT SendCmd(LPVOID p);
	void	SendCmd();

	void	GetCmdString(char[]);
	HANDLE	GetWriteToChildHandle();
	INT8	StartSendCmdThread();
	void	TerminateSendCmdThread();

	// MPlayer Monitor
	static UINT MPlayerMonitor(LPVOID p);
	void	MPlayerMonitor();

	int		StartMPlayerMonitorThread();
	void	TerminateMPlayerMonitorThread();

	void	SetDisableMPlayerMonitor(UINT8 bDisableMPlayerMonitor);

// Implementation
public:
	virtual ~CMPlayer();

	// Generated message map functions
protected:
	//{{AFX_MSG(CMPlayer)
	afx_msg void OnTimer(UINT nIDEvent);
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};