///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "LASeRPlayer.h"

#include "MainFrm.h"
#include "LASeRPlayerView.h"

#include "OpenURLDlg.h"

// general functions
#include "functions.h"
// CSplitPath class
#include "PathInfo.h"

#include "..\src\XML2SceneTree\XMLFileManager.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#define MINIMUM_WINDOW_WIDTH 300

// TIMER
#define TIMER_HALF_SECOND          1
#define TIMER_ONE_SECOND           2
#define TIMER_CACHEFILL            3
#define TIMER_FULLSCREENMOUSEHIDER 4

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_FILE_OPENURL, OnFileOpenurl)
	ON_COMMAND(ID_FILE_CLOSE_FILE, OnFileCloseFile)
	ON_WM_CLOSE()
	ON_COMMAND(ID_VIEW_FULLSCREEN, OnViewFullscreen)
	ON_COMMAND(ID_VIEW_PLAYLIST, OnViewPlaylist)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PLAYLIST, OnUpdateViewPlaylist)
	ON_WM_GETMINMAXINFO()
	ON_COMMAND(ID_PLAY_PLAY, OnPlayPlay)
	ON_UPDATE_COMMAND_UI(ID_PLAY_PLAY, OnUpdatePlayPlay)
	ON_COMMAND(ID_PLAY_PAUSE, OnPlayPause)
	ON_UPDATE_COMMAND_UI(ID_PLAY_PAUSE, OnUpdatePlayPause)
	ON_COMMAND(ID_PLAY_STOP, OnPlayStop)
	ON_UPDATE_COMMAND_UI(ID_PLAY_REPEAT, OnUpdatePlayRepeat)
	ON_COMMAND(ID_PLAY_PREVIOUS, OnPlayPrevious)
	ON_COMMAND(ID_PLAY_NEXT, OnPlayNext)
	ON_COMMAND_RANGE(ID_PLAY_FF10S, ID_PLAY_REW10M, OnPlayForwardRewind)
	ON_COMMAND(ID_VOLUME_MUTE, OnVolumeMute)
	ON_COMMAND(ID_PLAY_REPEAT, OnPlayRepeat)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
	ON_MESSAGE(UWM_MPLAYER_STARTING, OnMPlayerStarting)
	ON_MESSAGE(UWM_MPLAYER_STARTPLAYING, OnMPlayerStartPlaying)
	ON_MESSAGE(UWM_MPLAYER_FAILEDTOSTART, OnMPlayerFailedToStart)
	ON_MESSAGE(UWM_MPLAYER_AUTOSTOPPED, OnMPlayerAutoStopped)
	ON_MESSAGE(UWM_VIEW_MPLAYER_FULLSCREEN, OnViewMPlayerFullScreen)
	ON_MESSAGE(UWM_SLIDER_PROGRESS, OnSliderProgress)
	ON_MESSAGE(UWM_SLIDER_VOLUME, OnSliderVolume)
	ON_MESSAGE(UWM_PLAYLIST_PLAY, OnPlayListPlay)
	ON_MESSAGE(UWM_PLAYLIST_DLG_HIDE, OnPlaylistDlgHide)
	ON_MESSAGE(UWM_STARTUP_OPENFILE, OnStartupOpenfile)
	ON_WM_DROPFILES()
END_MESSAGE_MAP()

// status bar indicators
static UINT indicators[] =
{
	ID_INDICATOR_STATUS,						// status
	ID_INDICATOR_ABPS,							// audio bps
	ID_INDICATOR_TIME,							// current time
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	char szAppPath[MAX_PATH] = "";

	// Get the Module pathname
	::GetModuleFileName(0, szAppPath, sizeof(szAppPath) - 1);

	// Extract directory
	m_strAppDirectory = szAppPath;
	m_strAppDirectory = m_strAppDirectory.Left(m_strAppDirectory.ReverseFind('\\'));

	// Initialization
	m_clsLASeRPlayerStatus.strSource					= _T("");

#ifdef _DEBUG
	m_clsLASeRPlayerStatus.strMPlayerPathName			= _T("C:\\mplayer.exe");
#else
	m_clsLASeRPlayerStatus.strMPlayerPathName			= _T("C:\\mplayer.exe");
#endif // _DEBUG

	m_clsLASeRPlayerStatus.nPercent						= -1;
	m_clsLASeRPlayerStatus.uTimePosOld					= 0;
	m_clsLASeRPlayerStatus.nSeekToPercent				= 0;
	m_clsLASeRPlayerStatus.fCacheFill					= 0;
	m_clsLASeRPlayerStatus.bRepeatMode					= FALSE;
	m_clsLASeRPlayerStatus.bFullScreen					= FALSE;
	m_clsLASeRPlayerStatus.bToolbarVisible				= FALSE;
	m_clsLASeRPlayerStatus.bStatusBarVisible			= FALSE;
	m_clsLASeRPlayerStatus.bProgressBarVisible			= FALSE;
	m_clsLASeRPlayerStatus.bPlaylistVisible				= FALSE;
	m_clsLASeRPlayerStatus.nSkipOneUpdateOfProgressBar	= 0;
	m_clsLASeRPlayerStatus.nVolumeLoaded				= 100;

	// set MPlayer binary pathname according to m_clsLASeRP layerCfg.bMPlayerPathName
	m_clsMPlayer.SetMPlayerPathName(m_clsLASeRPlayerStatus.strMPlayerPathName);

	// Get OS major version number
	// Check if we are running Windows 2000 (or above)
	m_clsMPlayer.SetOSMajorVersion(OS_GetMajorVersion());

	// Set pointer to playlist dialog (modaless) to null
	m_pPlaylistDlg = NULL;

	m_bLASeR = FALSE;
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndStatusBar.Create(this, IDD_PLAYER_STATUSBAR, CBRS_ALIGN_BOTTOM, IDD_PLAYER_STATUSBAR))
	{
		TRACE0("Failed to create xxx dialogbar\n");
		return -1;
	}
	
	if (!m_wndMyToolBar.Create(this))
		 return -1;
	 
	 if (!m_wndProgressBar.Create(this, IDD_PROGRESS_DLGBAR, CBRS_ALIGN_BOTTOM, IDD_PROGRESS_DLGBAR))
	 {
		 TRACE0("Failed to create progress dialogbar\n");
		 return -1;
	 }

	 // Create the playlist dialog
	 m_pPlaylistDlg = new CPlaylistDlg;
	 m_pPlaylistDlg->Create(IDD_PLAYLIST, this); // GetDesktopWindow());
	 m_pPlaylistDlg->ShowWindow(SW_HIDE);

	 // Set status pane stretch and 75 pixel wide minimal
	 // m_wndStatusBar.SetPaneInfo(0, ID_INDICATOR_STATUS, SBPS_STRETCH, 75);
	 // Reset Statusbar
	 m_wndStatusBar.Reset();

	 // Show/Hide Toolbar
	 ShowControlBar(&m_wndMyToolBar, TRUE, TRUE);
	 // Show/Hide status bar
	 ShowControlBar(&m_wndStatusBar, TRUE, TRUE);
	 // Show/Hide progress dlgbar
	 ShowControlBar(&m_wndProgressBar, TRUE, TRUE);

	 // Show/Hide playlist dlg
	 if (m_clsLASeRPlayerStatus.bPlaylistVisible)
		 m_pPlaylistDlg->ShowWindow(SW_SHOW);
	 
	 // Create the m_clsMPlayer
	 if (!m_clsMPlayer.Create(_T(""), WS_CHILD, CRect(0, 0, 0, 0), this))
		 return -1;
	 
	 // Load Volume
	 m_clsMPlayer.SetVolume(m_clsLASeRPlayerStatus.nVolumeLoaded);
	 m_wndMyToolBar.SetSliderVolume(m_clsLASeRPlayerStatus.nVolumeLoaded);
	 
	 DragAcceptFiles(TRUE);						// Accept user dropped file
	 
	 // Move the MainFrame Window to the center of desktop
	 CenterWindow();
	 
	 return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs
	
	// set the window's initial size to 200x20 (actually it will be limited by
	//   OnGetMinMaxInfo())
	cs.cx = 200;
	cs.cy = 20;
	
	// TODO: Find a way to disable/enable Tilte
	// No Title
	cs.style &= ~FWS_ADDTOTITLE;
	
	// return CFrameWnd::PreCreateWindow(cs);
	if (!CFrameWnd::PreCreateWindow(cs))
		return FALSE;
	
	// This is the Class Name to prevent multi-instance
	cs.lpszClass = MPLAYERGUI_WND_CLASS_NAME;
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

// Call from CView, i must wait until the VideoPanel object in CView is created
void CMainFrame::SetMPlayerVideoWindow(void)
{
	CLASeRPlayerView *pView;
	pView = (CLASeRPlayerView*)GetActiveView();
	
	m_hVideoPanel = pView->GetMPlayerVideoWindow();
	m_clsMPlayer.SetVideoPanelHandle(m_hVideoPanel);
	
	// Pre-arrange panels
	pView->PreparePanels(FALSE);
}

// File->Open handler
void CMainFrame::OnFileOpen() 
{
	const TCHAR szFileDialogFilter[] = _T("all files (*.*)|*.*||");
	const TCHAR szFileDialogExt[] = _T("rm");
	
	CString strFilename = _T("");
	
	// Popup the File Open dialog box
	CFileDialog fileDialog(TRUE, szFileDialogExt, NULL, OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_READONLY, szFileDialogFilter);
	
	// Call DoFileOpen() if user picked a file
	if (fileDialog.DoModal() == IDOK)
	{
		strFilename = fileDialog.GetPathName();
		DoFileOpen(_T(strFilename));
	}
}

void CMainFrame::DoFileOpen(CString strFileName)
{
	CString strExtension = PathInfo::getExtension(_T(strFileName));

	// If user opened a playlist file, load it in the playlist and play from the first
	if ((strExtension.CompareNoCase(_T("txt")) == 0)
		|| (strExtension.CompareNoCase(_T("pls")) == 0)
		|| (strExtension.CompareNoCase(_T("m3u")) == 0)
		|| (strExtension.CompareNoCase(_T("ram")) == 0))
	{
		DoPlayStop();

		m_pPlaylistDlg->LoadPlayList(strFileName);

		m_pPlaylistDlg->PlayFromFirst();
	}

	// User opened a media file (i hope), add it to the playlist and play it
	else
	{
		// Stop the playback
		// Clean the Playlist
		m_pPlaylistDlg->DoPlayListRemoveAll();
		// Add the media file as one entry
		m_pPlaylistDlg->AddPlayListEntry(strFileName);
		// Play from first
		m_pPlaylistDlg->PlayFromFirst();
	}
}

// File->Open URL handle
void CMainFrame::OnFileOpenurl() 
{
	COpenURLDlg OpenURLDlg;

	if (OpenURLDlg.DoModal() == IDOK)
		DoFileOpen(OpenURLDlg.m_strOpenURL);
}

// Play->Play handler
void CMainFrame::OnPlayPlay() 
{
	UINT8 uStatus = m_clsMPlayer.GetStatus();
	
	// Return if .strSource is empty (not set yet)
	if (m_clsLASeRPlayerStatus.strSource == "")
		return;
	
	// Action to do if the current status is STOP
	if (uStatus == MPLAYER_STOP)
		// Since this is the situation that STOP -> PLAY, by PLAY button
		// Call DoPlayPlay()
		DoPlayPlay();
	
	else if (uStatus == MPLAYER_PAUSE) // Action to do if the current status is PAUSE
		OnPlayPause(); // Toggle PLAY/PAUSE mode
}

// ?
void CMainFrame::OnUpdatePlayPlay(CCmdUI* pCmdUI) 
{
	UINT8 nStatus = m_clsMPlayer.GetStatus();
	
	if ((nStatus == MPLAYER_PLAYING) || (nStatus == MPLAYER_PAUSE))
		pCmdUI->SetCheck(TRUE);
	
	else
		pCmdUI->SetCheck(FALSE);
}

// Actually do the play action
void CMainFrame::DoPlayPlay()
{
	UINT uStatus = m_clsMPlayer.GetStatus();
	
	// Return if .strSource is empty
	if (m_clsLASeRPlayerStatus.strSource == "")
		return;
	
	// Check if the MPlayer engine is available
	if (!m_clsMPlayer.CheckEngine())
	{
		m_wndStatusBar.SetMessage(_T("Engine not found!"));
		return;
	}
	
	if (uStatus == MPLAYER_STOP)
		// Ask MPlayer to start, and let it to get startup information
		m_clsMPlayer.Play();
}

// Play->Stop handler
void CMainFrame::OnPlayStop() 
{
	UINT8 uStatus = m_clsMPlayer.GetStatus();
	
	// Do action when status is PLAYING or PAUSE or STARTING
	if ((uStatus == MPLAYER_PLAYING) || (uStatus == MPLAYER_PAUSE) || (uStatus == MPLAYER_STARTING))
	{
		// Turn off full screen
		if (m_clsLASeRPlayerStatus.bFullScreen)
			OnViewFullscreen();
		
		DoPlayStop();
	}	
}

void CMainFrame::DoPlayStop(void)
{
	UINT8 uStatus = m_clsMPlayer.GetStatus();
	
	// Do action when status is PLAYING or PAUSE or STARTING
	if ((uStatus == MPLAYER_PLAYING) || (uStatus == MPLAYER_PAUSE) || (uStatus == MPLAYER_STARTING))
	{
		// Ask MPlayer to stop
		m_clsMPlayer.Stop();
		
		// Clean up status variables
		PlayerCleanUp();
	}
}

// Clean up status variables after stop
void CMainFrame::PlayerCleanUp(void)
{
	// Stop Timers
	KillTimer(TIMER_HALF_SECOND);
	KillTimer(TIMER_ONE_SECOND);
	KillTimer(TIMER_CACHEFILL);
	
	// Turn back to logo panel
	FitWindow(FALSE, 320, 240);
	
	// Reset status
	m_clsLASeRPlayerStatus.fCacheFill = 0;
	m_clsLASeRPlayerStatus.nSeekToPercent = 0;
	m_clsLASeRPlayerStatus.nPercent = -1;
	m_clsLASeRPlayerStatus.uTimePosOld = 0;
	m_clsLASeRPlayerStatus.nSkipOneUpdateOfProgressBar = 0;
	
	m_wndStatusBar.Reset();
	
	// Disable progress bar
	m_wndProgressBar.EnableProgressBar(FALSE);
	
	// Status bar: Stop
	m_wndStatusBar.SetMessage(_T("Stopped"));
}

// After starting play back, get MPlayer reported clip duration
void CMainFrame::GetTimeLength(void)
{
	UINT8	uStatus = m_clsMPlayer.GetStatus();
	INT64	nDuration;
	
	if ((uStatus == MPLAYER_PLAYING) || (uStatus == MPLAYER_PAUSE))
	{
		nDuration = m_clsMPlayer.GetClipLength();
		
		// -1 means MPlayer don't respond me :(
		if (nDuration != -1)
			m_wndStatusBar.SetDuration(nDuration);
	}
}

// Play->Pause handler
void CMainFrame::OnPlayPause() 
{
	UINT8 uStatus = m_clsMPlayer.GetStatus();
	
	if (uStatus == MPLAYER_PLAYING)
	{
		m_clsMPlayer.Pause(TRUE);
		
		// Status bar: Paused
		m_wndStatusBar.SetMessage(_T("Paused"));
	}
	
	else if (uStatus == MPLAYER_PAUSE)
	{
		m_clsMPlayer.Pause(FALSE);
		
		// Status bar: Playing... or Playing... (list)
		m_wndStatusBar.SetMessage(_T("Playing"));
	}	
}

void CMainFrame::OnUpdatePlayPause(CCmdUI* pCmdUI) 
{
	UINT8 uStatus = m_clsMPlayer.GetStatus();
	
	if (uStatus == MPLAYER_PAUSE)
		pCmdUI->SetCheck(TRUE);
	
	else
		pCmdUI->SetCheck(FALSE);
}

// File->Close handler
void CMainFrame::OnFileCloseFile() 
{
	DoPlayStop();
	m_clsLASeRPlayerStatus.strSource = "";
	ChangeTitle("");
}

// ?
void CMainFrame::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI) 
{
	if (m_clsLASeRPlayerStatus.bFullScreen)
	{
		lpMMI->ptMaxSize.x		= m_clsLASeRPlayerStatus.rFullScreenWindowRect.Width();
		lpMMI->ptMaxSize.y		= m_clsLASeRPlayerStatus.rFullScreenWindowRect.Height();
		lpMMI->ptMaxTrackSize.x	= lpMMI->ptMaxSize.x;
		lpMMI->ptMaxTrackSize.y	= lpMMI->ptMaxSize.y;
	}
	
	else
	{
		lpMMI->ptMinTrackSize.x	= MINIMUM_WINDOW_WIDTH;
		lpMMI->ptMinTrackSize.y	= GetFrameMinusViewHeight();
	}

	CFrameWnd::OnGetMinMaxInfo(lpMMI);
}

// Change Frame-Window size according to Clip video size
void CMainFrame::FitWindow(UINT8 bHaveVideo, UINT uVideoWidth, UINT uVideoHeight)
{
	CRect rFrame, rView;
	
	CLASeRPlayerView *pView = (CLASeRPlayerView*)GetActiveView();

	DWORD dwViewExStyle = pView->GetExStyle();
	
	if (m_clsLASeRPlayerStatus.bFullScreen)
	{
		// Do not resize window anymore, but let view resize its video panel
		// CView, show/hide video panels
		pView->PreparePanels(bHaveVideo);
		
		// Tell view the video width/height for keep aspect ratio
		pView->SetVideoWidth(uVideoWidth);
		pView->SetVideoHeight(uVideoHeight);
		
		// Tell view to resize once to keep aspect ratio (if)
		pView->PostMessage(WM_SIZE);
		return;
	}
	
	if (bHaveVideo)
	{
		// Turn Window to normal status
		RestoreFromMaxMin();
		
		// Get Frame-Window rectangle
		GetWindowRect(&rFrame);
		
		CPoint	pointCenter		= rFrame.CenterPoint();
		UINT	uVideoWidthTemp	= uVideoWidth;
		
		if (uVideoWidthTemp < MINIMUM_WINDOW_WIDTH)
			uVideoWidthTemp = MINIMUM_WINDOW_WIDTH;

		UINT	uWidth	= uVideoWidthTemp + GetFrameMinusViewWidth();
		UINT	uHeight	= uVideoHeight + GetFrameMinusViewHeight();
		
		if (dwViewExStyle & WS_EX_CLIENTEDGE)
			uWidth	+= 2 * GetSystemMetrics(SM_CXEDGE);
		
		if (dwViewExStyle & WS_EX_CLIENTEDGE)
			uHeight	+= 2 * GetSystemMetrics(SM_CYEDGE);
		
		rFrame.left	= pointCenter.x - uWidth / 2;
		rFrame.top	= pointCenter.y - uHeight / 2;
		
		// Pull back the Window a little bit
		if (rFrame.top < 0)
			rFrame.top = 0;
		
		rFrame.right = rFrame.left + uWidth;
		rFrame.bottom = rFrame.top + uHeight;

		MoveWindow(rFrame);
	}
	
	// CView, show/hide video panels
	pView->PreparePanels(bHaveVideo);
	
	// Tell view the video width/height for keep aspect ratio
	pView->SetVideoWidth(uVideoWidth);
	pView->SetVideoHeight(uVideoHeight);
	
	// Tell view to resize once to keep aspect ratio (if)
	pView->PostMessage(WM_SIZE);
}

// handle the UWM_SLIDER_PROGRESS message
LRESULT CMainFrame::OnSliderProgress(WPARAM wParam, LPARAM lParam)
{
	UINT8	uStatus		= m_clsMPlayer.GetStatus();
	UINT64	uDuration	= 0;
	
	if (uStatus == MPLAYER_PLAYING)
	{
		// Seek if new position is required
		if (m_clsLASeRPlayerStatus.nSeekToPercent != (UINT8)wParam)
		{
			m_clsLASeRPlayerStatus.nSeekToPercent = (UINT8)wParam;
			uDuration = m_clsMPlayer.GetClipLength();
			
			// Calculate where to seek
			DWORD dwSeekTime = (DWORD)(uDuration * m_clsLASeRPlayerStatus.nSeekToPercent / 100);
			m_clsMPlayer.SeekTime(dwSeekTime);
			
			// Skip once in UpdateProgress() avoid jummpy
			m_clsLASeRPlayerStatus.nSkipOneUpdateOfProgressBar = 2;
		}
	}
	
	return 1;
}

// handle the UWM_SLIDER_VOLUME message
LRESULT CMainFrame::OnSliderVolume(WPARAM wParam, LPARAM lParam)
{
	UINT8 uVolume = m_clsMPlayer.GetVolume();
	
	if (uVolume != (UINT8)wParam)
	{
		uVolume = (UINT8)wParam;
		
		m_clsMPlayer.SetVolume(uVolume);
	}
	
	return 1;
}

// Play->FF10S, REW10S, FF1M, REW1M, FF10M, REW10M handler
void CMainFrame::OnPlayForwardRewind(UINT nID)
{
	int nTime;
	
	switch (nID)
	{
	case ID_PLAY_FF10S:
		nTime = 10;
		break;
	
	case ID_PLAY_REW10S:
		nTime = -10;
		break;
	
	case ID_PLAY_FF1M:
		nTime = 60;
		break;
	
	case ID_PLAY_REW1M:
		nTime = -60;
		break;
	
	case ID_PLAY_FF10M:
		nTime = 600;
		break;
	
	case ID_PLAY_REW10M:
		nTime = -600;
		break;
	
	default:
		nTime = 0;
		break;
	}
	
	UINT8 uStatus = m_clsMPlayer.GetStatus();
	
	if (uStatus == MPLAYER_PLAYING)
	{
		m_clsMPlayer.SeekRelativeTime(nTime);
		
		// Avoid the jummpy progress bar
		m_clsLASeRPlayerStatus.nSkipOneUpdateOfProgressBar = 2;
	}
}

// Play->Value->Mute handler
void CMainFrame::OnVolumeMute() 
{
	UINT8 uStatus = m_clsMPlayer.GetStatus();
	
	if (uStatus == MPLAYER_PLAYING)
		m_clsMPlayer.Mute();
}

// Change frame window title
void CMainFrame::ChangeTitle(CString strTitle)
{
	// CString title(MAKEINTRESOURCE(IDR_MAINFRAME));
	CString title = "LASeRPlayer";
	
	if (strTitle.GetLength() > 0)
		title = strTitle + _T(" - ") + title;

	SetWindowText(title);
}

// Dropped file handler
void CMainFrame::OnDropFiles(HDROP hDropInfo) 
{
	TCHAR szFileName[_MAX_PATH];
	UINT uFiles = ::DragQueryFile(hDropInfo, (UINT) -1, NULL, 0);
	
	// If there are more than 1 files has been dropped, call Playlist handle
	if (uFiles > 1)
	{
		// Clean the Playlist
		m_pPlaylistDlg->DoPlayListRemoveAll();
		// Add dropped files to Playlist
		m_pPlaylistDlg->OnDropFiles(hDropInfo);
		// Play from first
		m_pPlaylistDlg->PlayFromFirst();
	}
	
	// If there is only 1 file has been dropped, pass it to the DoFileOpen()
	else
	{
		// Only process one file (first one)
		::DragQueryFile(hDropInfo, 0, szFileName, _MAX_PATH);
		DoFileOpen(_T(szFileName));
	}
	
	::DragFinish(hDropInfo);
}

// Restore to normal window if the current window is minimal/maximal
UINT8 CMainFrame::RestoreFromMaxMin()
{
	WINDOWPLACEMENT wp;
	wp.length = sizeof (WINDOWPLACEMENT);
	
	GetWindowPlacement(&wp);
	
	if ((wp.showCmd == SW_SHOWMAXIMIZED) || (wp.showCmd == SW_SHOWMINIMIZED))
	{
		wp.showCmd = SW_RESTORE;
		SetWindowPlacement(&wp);
		return 1;
	}
	
	return 0;
}

// Toggle full screen mode
void CMainFrame::OnViewFullscreen() 
{
	CRect r;
	CLASeRPlayerView *pView = (CLASeRPlayerView*)GetActiveView();
	
	if (m_clsLASeRPlayerStatus.bFullScreen == FALSE)
	{
		m_clsLASeRPlayerStatus.bFullScreen = TRUE;
		
		// Save the current window rect
		GetWindowRect(&m_clsLASeRPlayerStatus.rLastWindowRect);
		
		// Get current visible state of bars, and hide them
		m_clsLASeRPlayerStatus.bToolbarVisible = (UINT8)m_wndMyToolBar.IsWindowVisible();
		
		if (m_clsLASeRPlayerStatus.bToolbarVisible)
			ShowControlBar(&m_wndMyToolBar, FALSE, FALSE);
		
		m_clsLASeRPlayerStatus.bStatusBarVisible = (UINT8)m_wndStatusBar.IsWindowVisible();
		
		if (m_clsLASeRPlayerStatus.bStatusBarVisible)
			ShowControlBar(&m_wndStatusBar, FALSE, FALSE);
		
		m_clsLASeRPlayerStatus.bProgressBarVisible = (UINT8)m_wndProgressBar.IsWindowVisible();
		
		if (m_clsLASeRPlayerStatus.bProgressBarVisible)
			ShowControlBar(&m_wndProgressBar, FALSE, FALSE);
		
		m_clsLASeRPlayerStatus.bPlaylistVisible = (UINT8)m_pPlaylistDlg->IsWindowVisible();
		
		if (m_clsLASeRPlayerStatus.bPlaylistVisible)
			m_pPlaylistDlg->ShowWindow(SW_HIDE);
		
		// Start full screen...
		// Get full screen rect
		GetDesktopWindow()->GetWindowRect(&m_clsLASeRPlayerStatus.rFullScreenWindowRect);
		// Hide caption, thickframe, ...
		ModifyStyle(WS_CAPTION | WS_THICKFRAME, NULL, SWP_NOZORDER);
		// Hide menu
		::SetMenu(m_hWnd, NULL);
		// Turn off view client edge
		pView->EnableClientEdge(FALSE);
		// For convinence
		r = m_clsLASeRPlayerStatus.rFullScreenWindowRect;
		// Set window position
		SetWindowPos(NULL, r.left, r.top, r.Width(), r.Height(), SWP_NOZORDER | SWP_NOSENDCHANGING);
		
		// Grab the input focus
		SetFocus();
		
		// start FULLSCREEN mouse cursor hider timer
		SetTimer(TIMER_FULLSCREENMOUSEHIDER, 1000, NULL);
	}
	
	else
	{
		m_clsLASeRPlayerStatus.bFullScreen = FALSE;
		
		// Restore bars to ex-full screen state
		if (m_clsLASeRPlayerStatus.bToolbarVisible)
			ShowControlBar(&m_wndMyToolBar, TRUE, FALSE);
		
		if (m_clsLASeRPlayerStatus.bStatusBarVisible)
			ShowControlBar(&m_wndStatusBar, TRUE, FALSE);
		
		if (m_clsLASeRPlayerStatus.bProgressBarVisible)
			ShowControlBar(&m_wndProgressBar, TRUE, FALSE);
		
		if (m_clsLASeRPlayerStatus.bPlaylistVisible)
			m_pPlaylistDlg->ShowWindow(SW_SHOW);
		
		// Show caption, thickframe, ...
		ModifyStyle(NULL, WS_CAPTION | WS_THICKFRAME, SWP_NOZORDER);
		// Show menu
		::SetMenu(m_hWnd, m_hMenuDefault);
		// Turn on view client edge
		pView->EnableClientEdge(TRUE);
		// For convinence
		r = m_clsLASeRPlayerStatus.rLastWindowRect;
		// Set window position
		SetWindowPos(NULL, r.left, r.top, r.Width(), r.Height(), SWP_NOZORDER | SWP_NOSENDCHANGING);
		
		// kill FULLSCREEN mouse cursor hider timer
		KillTimer(TIMER_FULLSCREENMOUSEHIDER);
	}
}

// UWM_PLAYLIST_PLAY message handler
LRESULT CMainFrame::OnPlayListPlay(WPARAM wParam, LPARAM lParam)
{
	CString strPlayListName = _T("");

	CLASeRPlayerView *pView = (CLASeRPlayerView *)GetActiveView();

	m_bLASeR = FALSE;							// LASeR disable flag
	pView->ClearInfo();							// clear LASeR info
	
	// Get playlist filename from PlayListDlgBar
	m_pPlaylistDlg->GetFileToPlay(strPlayListName);
	CString	strExtension = PathInfo::getExtension(_T(strPlayListName));

	m_clsLASeRPlayerStatus.strSource = strPlayListName;
	m_clsMPlayer.SetSource(m_clsLASeRPlayerStatus.strSource);

	if (strExtension.CompareNoCase(_T("xml")) == 0)
	{
		lsrErr err = LASeRNoErr;

		void *pRoot = NULL;
		m_bLASeR = TRUE;

		err = OpenXMLFile(m_clsLASeRPlayerStatus.strSource, &pRoot);
		ErrorMessage(err);

		if (pRoot)
		{
			err = SetSceneRoot(pRoot);
			ErrorMessage(err);
		}

		pView->DrawScene();

		return 1;
	}
	
	DoPlayStop();
	DoPlayPlay();
	
	return 1;
}

void CMainFrame::DoPlayFile(CString strFileName)
{
	DoPlayStop();

	UINT uStatus = m_clsMPlayer.GetStatus();

	m_clsMPlayer.SetSource(strFileName);

	// Check if the MPlayer engine is available
	if (!m_clsMPlayer.CheckEngine())
	{
		m_wndStatusBar.SetMessage(_T("Engine not found!"));
		return;
	}

	if (uStatus == MPLAYER_STOP)
		// Ask MPlayer to start, and let it to get startup information
		m_clsMPlayer.Play();
}

// Try to play next in playlist
void CMainFrame::PlayListTryNextFile()
{
	int		nPlayListReturn	= m_pPlaylistDlg->RequestNextFile();
	CString	strPlayListName	= _T("");
	
	if ((m_clsLASeRPlayerStatus.bRepeatMode == TRUE) && (nPlayListReturn == -1))
	{
		// Try to rewind and check if there is at least 1 file in playlist
		if (m_pPlaylistDlg->RewindPlayList() == 1)
			nPlayListReturn = 1;				// cheating
	}
	
	if (nPlayListReturn != -1)
	{
		// Get filename from PlayList
		m_pPlaylistDlg->GetFileToPlay(strPlayListName);
		m_clsLASeRPlayerStatus.strSource = strPlayListName;
		m_clsMPlayer.SetSource(m_clsLASeRPlayerStatus.strSource);
		
		DoPlayStop();
		DoPlayPlay();
	}
	
	else
	{
		// Otherwise there is no more file in playlist, and no repeat, keep in stop state
		// Turn off full screen
		if (m_clsLASeRPlayerStatus.bFullScreen)
			OnViewFullscreen();
	}
}

// Play->Repeat handler
void CMainFrame::OnPlayRepeat() 
{
	m_clsLASeRPlayerStatus.bRepeatMode = !m_clsLASeRPlayerStatus.bRepeatMode;
}

void CMainFrame::OnUpdatePlayRepeat(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_clsLASeRPlayerStatus.bRepeatMode);
}

// Say goodbye to LASeRPlayer, about to close
void CMainFrame::OnClose() 
{
	// About to terminate, do some last minute stuff
	DoLastMinuteCleanUp();
	
	CFrameWnd::OnClose();
}

// Last minute clean up job
void CMainFrame::DoLastMinuteCleanUp()
{
	// Quitting...
	m_wndStatusBar.SetMessage(_T("Quitting..."));
	
	// Make sure that MPlayer is terminated
	m_clsMPlayer.Stop();
	
	// Cleanup
	PlayerCleanUp();
	
	// Delete PlaylistDlg (modaless)
	if (m_pPlaylistDlg)
		delete m_pPlaylistDlg;
}

// View->PlayList handler
void CMainFrame::OnViewPlaylist() 
{
	if (m_pPlaylistDlg->IsWindowVisible())
		m_pPlaylistDlg->ShowWindow(SW_HIDE);
	
	else
	{
		CRect rc_this, rc_pl;
		
		m_pPlaylistDlg->GetWindowRect(&rc_pl);
		GetWindowRect(&rc_this);
		
		// Move playlist window to the bottom of this window (mainframe)
		m_pPlaylistDlg->MoveWindow(rc_this.left, rc_this.bottom, rc_pl.Width(), rc_pl.Height(), FALSE);
		
		m_pPlaylistDlg->ShowWindow(SW_SHOW);
	}
}

// UWM_PLAYLIST_DLG_HIDE handler
LRESULT CMainFrame::OnPlaylistDlgHide(WPARAM wParam, LPARAM lParam)
{
	m_pPlaylistDlg->ShowWindow(SW_HIDE);
	return 1;
}

void CMainFrame::OnUpdateViewPlaylist(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_pPlaylistDlg->IsWindowVisible());
}

// Called when user double clicked an assoicated file
LRESULT CMainFrame::OnStartupOpenfile(WPARAM wParam, LPARAM lParam)
{
	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	
	AfxGetApp()->ParseCommandLine(cmdInfo);
	
	if (cmdInfo.m_nShellCommand == CCommandLineInfo::FileOpen)
		DoFileOpen(_T(cmdInfo.m_strFileName));
	
	return 1;
}

// Play->Previous handler
void CMainFrame::OnPlayPrevious() 
{
	CString	strPlayListName;
	UINT8	uStatus				= m_clsMPlayer.GetStatus();
	int		nPlayListReturn		= m_pPlaylistDlg->RequestPreviousFile();;

	if (uStatus != MPLAYER_PLAYING)
		return;

	if (nPlayListReturn != -1)
	{
		// Get filename from PlayList
		m_pPlaylistDlg->GetFileToPlay(strPlayListName);
		m_clsLASeRPlayerStatus.strSource = strPlayListName;
		m_clsMPlayer.SetSource(m_clsLASeRPlayerStatus.strSource);

		DoPlayStop();
		DoPlayPlay();
	}
}

// Play->Next handler
void CMainFrame::OnPlayNext() 
{
	CString	strPlayListName;
	UINT8	uStatus				= m_clsMPlayer.GetStatus();
	UINT8	uPlayListReturn		= m_pPlaylistDlg->RequestNextFile();;

	if (uStatus != MPLAYER_PLAYING)
		return;

	if (uPlayListReturn != -1)
	{
		// Get filename from PlayList
		m_pPlaylistDlg->GetFileToPlay(strPlayListName);
		m_clsLASeRPlayerStatus.strSource = strPlayListName;
		m_clsMPlayer.SetSource(m_clsLASeRPlayerStatus.strSource);

		DoPlayStop();
		DoPlayPlay();
	}
}

// UWM_VIEW_MPLAYER_FULLSCREEN handler
LRESULT CMainFrame::OnViewMPlayerFullScreen(WPARAM wParam, LPARAM lParam)
{
	OnViewFullscreen();
	return 1;
}

BOOL CMainFrame::PreTranslateMessage(MSG* pMsg) 
{
	// ESC to quit FullScreen mode
	if(pMsg->message == WM_KEYDOWN)
	{
		if((pMsg->wParam == VK_ESCAPE) && (m_clsLASeRPlayerStatus.bFullScreen == TRUE))
		{
			OnViewFullscreen();
			return TRUE;
		}
	}

	return CFrameWnd::PreTranslateMessage(pMsg);
}

// Handlers for CMPlayer message

// Called when MPlayer is starting
LRESULT CMainFrame::OnMPlayerStarting(WPARAM wParam, LPARAM lParam)
{
	// Starting...
	m_wndStatusBar.SetMessage(_T("Starting..."));
	
	SetTimer(TIMER_HALF_SECOND, 500, NULL);
	SetTimer(TIMER_ONE_SECOND, 1000, NULL);
	SetTimer(TIMER_CACHEFILL, 300, NULL);
	
	// StatusBar: Buffering
	m_wndStatusBar.SetStatusMode(STATUSMODE_IDLE);
	
	return 1;
}

// Called right after MPlayer started to play
LRESULT CMainFrame::OnMPlayerStartPlaying(WPARAM wParam, LPARAM lParam)
{
	KillTimer(TIMER_CACHEFILL);

	// Set Labels in View
	CLASeRPlayerView *pView = (CLASeRPlayerView*)GetActiveView();
	
	// Update panel: Duration
	GetTimeLength();
	
	// Get video information
	UINT8	bHaveVideo	= m_clsMPlayer.HaveVideo();
	UINT	uWidth		= m_clsMPlayer.GetVideoWidth();
	UINT	uHeight		= m_clsMPlayer.GetVideoHeight();

	if (!m_bLASeR)
	{
		// Resize the main frame to video size
		if (bHaveVideo == TRUE)
		{
			if (m_clsLASeRPlayerStatus.bFullScreen == TRUE)
				FitWindow(TRUE, uWidth, uHeight);

			else
			{
				if (m_clsLASeRPlayerStatus.bFullScreen)
					OnViewFullscreen();
			}

			FitWindow(TRUE, uWidth, uHeight);

			if (m_clsLASeRPlayerStatus.bFullScreen)
				OnViewFullscreen();
		}

		else
			FitWindow(FALSE, uWidth, uHeight);
	}
	
	UINT64 uDuration = m_clsMPlayer.GetClipLength();
	
	// Enable progress bar (enable seek)
	if (uDuration > 0)
		m_wndProgressBar.EnableProgressBar(TRUE);
	
	// Get title
	CString	strTitle	= PathInfo::getTitle(_T(m_clsLASeRPlayerStatus.strSource));
	CString	strName		= PathInfo::getName(_T(m_clsLASeRPlayerStatus.strSource));
	
	ChangeTitle(strName);
	
	m_wndStatusBar.SetAudioMode(m_clsMPlayer.GetAudioNCH());
	// FIXME: set to corresponding media type
	m_wndStatusBar.SetStatusMode(STATUSMODE_LOCALMEDIA);
	m_wndStatusBar.SetMessage(_T("Playing"));
	
	return 1;
}

// Called after MPlayer failed to start
LRESULT CMainFrame::OnMPlayerFailedToStart(WPARAM wParam, LPARAM lParam)
{
	PlayerCleanUp();

	m_wndStatusBar.SetMessage(_T("MPlayer failed to start"));
	// Failed to Start
	m_wndStatusBar.SetStatusMode(STATUSMODE_FAILED);

	return 1;
}

// Called after MPlayer is stopped (by itself, ex: Finished Playing)
LRESULT CMainFrame::OnMPlayerAutoStopped(WPARAM wParam, LPARAM lParam)
{
	// Clean up status variables
	PlayerCleanUp();

	return 1;
}

UINT8 CMainFrame::GetFrameMinusViewHeight()
{
	UINT8 uHeight = 0;
	
	// the Menu height
	// (SM_CYMENU for menu bar height, SM_CYMENUSIZE for menu bar button height)
	uHeight += (UINT8)::GetSystemMetrics(SM_CYMENU);	// SM_CYMENUSIZE
	
	DWORD style = GetStyle();
	
	if (style & WS_CAPTION)
		uHeight += (UINT8)GetSystemMetrics(SM_CYCAPTION);
	
	if (style & WS_THICKFRAME)
		uHeight += (UINT8)GetSystemMetrics((style & WS_CAPTION) ? SM_CYSIZEFRAME : SM_CYFIXEDFRAME) * 2;
	
	if (IsWindow(m_wndMyToolBar.m_hWnd))
	{
		if (m_wndMyToolBar.IsWindowVisible())
			uHeight += (UINT8)m_wndMyToolBar.CalcFixedLayout(FALSE, TRUE).cy;
	}
	
	if (IsWindow(m_wndStatusBar.m_hWnd))
	{
		if (m_wndStatusBar.IsWindowVisible())
			uHeight += (UINT8)m_wndStatusBar.CalcFixedLayout(FALSE, TRUE).cy;
	}
	
	if (IsWindow(m_wndProgressBar.m_hWnd))
	{
		if (m_wndProgressBar.IsWindowVisible())
			uHeight += (UINT8)m_wndProgressBar.CalcFixedLayout(FALSE, TRUE).cy;
	}
	
	return uHeight;
}

UINT8 CMainFrame::GetFrameMinusViewWidth()
{
	UINT8 uWidth = 0;
	
	DWORD style = GetStyle();
	
	if (style & WS_THICKFRAME)
		uWidth += (UINT8)GetSystemMetrics(SM_CXSIZEFRAME) * 2;
	
	return uWidth;
}

void CMainFrame::RecalcLayout(BOOL bNotify) 
{
	CFrameWnd::RecalcLayout(bNotify);
	
	// Resize the framewindow after toggle the control bars
	CRect r;
	GetWindowRect(&r);

	MINMAXINFO mmi;
	memset(&mmi, 0, sizeof(mmi));
	
	SendMessage(WM_GETMINMAXINFO, 0, (LPARAM)&mmi);
	r |= CRect(r.TopLeft(), CSize(r.Width(), mmi.ptMinTrackSize.y));

	MoveWindow(&r);
}

void CMainFrame::OnTimer(UINT nIDEvent) 
{
	switch (nIDEvent)
	{
	case TIMER_HALF_SECOND:
		UpdateTimePos();
		break;

    case TIMER_ONE_SECOND:
		UpdateProgress();
		break;

    case TIMER_CACHEFILL:
		UpdateCacheFill();
		break;

    case TIMER_FULLSCREENMOUSEHIDER:
		{
			CPoint p;
			GetCursorPos(&p);
			CRect r;
			GetWindowRect(r);
			
			// only SetCursor to NULL if in rect and cursor is not null
			if ((r.PtInRect(p)) && (GetCursor() != NULL))
				SetCursor(NULL);
			
			break;
		}
	
	default:
		/* nothing */
		break;
	}
	
	CFrameWnd::OnTimer(nIDEvent);
}

void CMainFrame::UpdateTimePos()
{
	// Get MPlayer Time Position
	UINT8	uStatus		= m_clsMPlayer.GetStatus();
	UINT64	uTimePos	= 0;
	
	if (uStatus == MPLAYER_PLAYING)
	{
		uTimePos = m_clsMPlayer.GetTimePos();
		
		// ?
		// -1 means mplayer don't respond to me :(
		if (uTimePos != -1)
		{
			// Compare to determine if a refeash is needed
			if (m_clsLASeRPlayerStatus.uTimePosOld != uTimePos)
			{
				m_clsLASeRPlayerStatus.uTimePosOld = uTimePos;
				m_wndStatusBar.SetTime(uTimePos);
			}
		}
	}
}

void CMainFrame::UpdateProgress()
{
	INT8 nPercent = 0;

	// Since the MPlayer gives me wrong percent value when playing RealMedia clip
	// calculate the percent by hand
	float	fTimePos	= (float)m_clsMPlayer.GetTimePos();
	float	fTimeLength	= (float)m_clsMPlayer.GetClipLength();
	
	if (m_clsLASeRPlayerStatus.nSkipOneUpdateOfProgressBar > 0)
	{
		m_clsLASeRPlayerStatus.nSkipOneUpdateOfProgressBar--;
		return;
	}
	
	if (fTimePos > fTimeLength)
		return;
	
	if (fTimeLength == 0.0)
		nPercent = 0;
	
	else
		nPercent = (UINT8)(fTimePos * 100 / fTimeLength);
	
	// ?
	// -1 means mplayer don't respond me :(
	if (nPercent != -1)
	{
		// Check if a refeash is needed
		if (m_clsLASeRPlayerStatus.nPercent != nPercent)
		{
			m_clsLASeRPlayerStatus.nPercent = nPercent;
			m_wndProgressBar.SetSliderProgress(nPercent);
		}
	}
}

void CMainFrame::UpdateCacheFill()
{
	float	fCacheFill	= m_clsMPlayer.GetCacheFill();;
	CString	strStatus	= _T("");
	
	// ?
	if (m_clsLASeRPlayerStatus.fCacheFill != fCacheFill)
	{
		m_clsLASeRPlayerStatus.fCacheFill = fCacheFill;
		strStatus.Format("Cache fill: %2.02f%%", fCacheFill);
		m_wndStatusBar.SetMessage(strStatus);
	}
}