///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

// MainFrm.h : interface of the CMainFrame class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MAINFRM_H__A154A451_51C5_4DFF_82E1_137B439B6ED1__INCLUDED_)
#define AFX_MAINFRM_H__A154A451_51C5_4DFF_82E1_137B439B6ED1__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "MPlayer.h"
#include "ProgressDlgBar.h"
#include "PlaylistDlg.h"
#include "MyToolBar.h"
#include "PlayerStatusbar.h"

class CMainFrame : public CFrameWnd
{
public:
	CMainFrame();

protected:										// create from serialization only
	DECLARE_DYNCREATE(CMainFrame)

private:
	// LASeRPlayer running status
	typedef struct
	{
		CString	strSource;						// MPlayer source media filename (to play)
		CString	strMPlayerPathName;				// user specified MPlayer binary path name
		INT8	nPercent;						// for comparison
		UINT64	uTimePosOld;					// for comparison
		INT8	nSeekToPercent;					// for comparison
		float	fCacheFill;						// for comparison
		UINT8	bRepeatMode;					// current repeat mode
		UINT8	bFullScreen;					// is full screen?
		UINT8	bToolbarVisible;				// tool bar visible?
		UINT8	bStatusBarVisible;				// status bar visible?
		UINT8	bProgressBarVisible;			// progress bar visible?
		UINT8	bPlaylistVisible;				// playlist visible?
		CRect	rFullScreenWindowRect;
		CRect	rLastWindowRect;
		UINT8	nSkipOneUpdateOfProgressBar;	// skip one update message
		UINT8	nVolumeLoaded;
	} LASeRPlayerStatus;

	LASeRPlayerStatus m_clsLASeRPlayerStatus;

	CMPlayer		m_clsMPlayer;				// The MPlayer Core Interface
	HWND			m_hVideoPanel;				// Handle of MPlayer Video Window (Embedded)

	CString			m_strAppDirectory;			// module path
	CPlaylistDlg	*m_pPlaylistDlg;			// play list modaless dialog

	// LASeRPlayer
	UINT8			m_bLASeR;

// Attributes
public:

// Operations
public:
	void	SetMPlayerVideoWindow();

	void	DoPlayFile(CString strFileName);

	CString	GetSource() const	{return m_clsLASeRPlayerStatus.strSource;}

protected:
	void	DoPlayPlay();
	void	DoPlayStop();
	void	PlayerCleanUp();

	void	GetTimeLength();

	void	FitWindow(UINT8 bHaveVideo, UINT uVideoWidth, UINT uVideoHeight);

	void	ChangeTitle(CString strTitle);
	UINT8	RestoreFromMaxMin();
	void	PlayListTryNextFile();
	void	DoLastMinuteCleanUp();

	void	DoFileOpen(CString strFileName);

	UINT8	GetFrameMinusViewHeight();
	UINT8	GetFrameMinusViewWidth();
	void	UpdateTimePos();
	void	UpdateProgress();
	void	UpdateCacheFill();

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMainFrame)
	public:
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	virtual void RecalcLayout(BOOL bNotify = TRUE);
	//}}AFX_VIRTUAL

// Implementation
public:

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:  // control bar embedded members
	CPlayerStatusbar m_wndStatusBar;
	CProgressDlgBar m_wndProgressBar;
	CMyToolBar m_wndMyToolBar;

// Generated message map functions
protected:
	//{{AFX_MSG(CMainFrame)
	afx_msg int	OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnFileOpen();
	afx_msg void OnFileOpenurl();
	afx_msg void OnPlayPlay();
	afx_msg void OnUpdatePlayPlay(CCmdUI* pCmdUI);
	afx_msg void OnPlayStop();
	afx_msg void OnPlayPause();
	afx_msg void OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI);
	afx_msg void OnUpdatePlayPause(CCmdUI* pCmdUI);
	afx_msg void OnFileCloseFile();
	afx_msg LRESULT OnSliderProgress(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnSliderVolume(WPARAM wParam, LPARAM lParam);
	afx_msg void OnPlayForwardRewind(UINT nID);
	afx_msg void OnVolumeMute();
	afx_msg void OnDropFiles(HDROP hDropInfo);
	afx_msg void OnViewFullscreen();
	afx_msg LRESULT OnPlayListPlay(WPARAM wParam, LPARAM lParam);
	afx_msg void OnPlayRepeat();
	afx_msg void OnUpdatePlayRepeat(CCmdUI* pCmdUI);
	afx_msg void OnClose();
	afx_msg void OnViewPlaylist();
	afx_msg LRESULT OnPlaylistDlgHide(WPARAM wParam, LPARAM lParam);
	afx_msg void OnUpdateViewPlaylist(CCmdUI* pCmdUI);
	afx_msg LRESULT OnStartupOpenfile(WPARAM wParam, LPARAM lParam);
	afx_msg void OnPlayPrevious();
	afx_msg void OnPlayNext();
	afx_msg LRESULT OnViewMPlayerFullScreen(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnMPlayerStarting(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnMPlayerStartPlaying(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnMPlayerFailedToStart(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnMPlayerAutoStopped(WPARAM wParam, LPARAM lParam);
	afx_msg void OnTimer(UINT nIDEvent);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MAINFRM_H__A154A451_51C5_4DFF_82E1_137B439B6ED1__INCLUDED_)
