///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
//
// This notice must be included in all copies or derivative works.

#include "stdafx.h"
#include "LASeRPlayer.h"
#include "MyToolBar.h"


CMyToolBar::CMyToolBar()
{
}

CMyToolBar::~CMyToolBar()
{
}

UINT8 CMyToolBar::Create(CWnd *pParentWnd)
{
	if (!CToolBar::Create(pParentWnd, WS_CHILD | WS_VISIBLE | CBRS_ALIGN_BOTTOM))
		return FALSE;

	if (!LoadToolBar(IDR_MYTOOLBAR))
		return FALSE;

	ModifyStyle(0, TBSTYLE_FLAT);

	SetButtonStyle(0, TBBS_BUTTON);				// PLAY
	SetButtonStyle(1, TBBS_BUTTON);				// PAUSE
	SetButtonStyle(2, TBBS_BUTTON);				// STOP
	SetButtonStyle(3, TBBS_SEPARATOR);			// Separator
	SetButtonStyle(4, TBBS_BUTTON);				// PREVIOUS
	SetButtonStyle(5, TBBS_BUTTON);				// NEXT
	SetButtonStyle(6, TBBS_SEPARATOR);			// Separator
	SetButtonStyle(7, TBBS_BUTTON);				// REPEAT
	SetButtonStyle(8, TBBS_SEPARATOR);			// Separator
	SetButtonStyle(9, TBBS_BUTTON);				// PLAYLIST
	SetButtonStyle(10, TBBS_SEPARATOR);			// Separator
	SetButtonStyle(11, TBBS_BUTTON);			// MUTE
	SetButtonStyle(12, TBBS_CHECKBOX);			// SLIDER

	if (!m_clsSliderVolume.Create(WS_CHILD | WS_VISIBLE | TBS_HORZ,
		CRect(0, 0, 0, 0),
		this,
		IDC_SLIDER_VOLUME))
		return FALSE;

	// init the slider volume
	m_clsSliderVolume.SetLineSize(1);
	m_clsSliderVolume.SetPageSize(20);
	m_clsSliderVolume.SetRange(0, 100);
	m_clsSliderVolume.SetPos(100);

	return TRUE;
}

void CMyToolBar::ArrangeControls()
{
	if(!::IsWindow(m_clsSliderVolume.m_hWnd))
		return;

	CRect r;
	GetClientRect(&r);

	m_clsSliderVolume.MoveWindow(r.right - 50, r.top + 2, 50, r.bottom - 2);

	// Trying to set the Mute Button's Position
	// Get Button:Playlist information, especially i need the right edge of the rectangle
	CRect r9;
	GetItemRect(9, &r9);

	// Set Button:10 (Separator) 's Width, so the Mute will move also :)
	// r.right - 50    is the Volume Control's left edge
	// r9.right        is the Playlist Button's right edge
	// r9.Width()      is the Toolbar Button's Width
	// 8               is the space between two buttons on Toolbar
	SetButtonInfo(10, GetItemID(10), TBBS_SEPARATOR, r.right - 50 - r9.right - r9.Width() - 8);
}


BEGIN_MESSAGE_MAP(CMyToolBar, CToolBar)
	ON_WM_SIZE()
	ON_WM_HSCROLL()
END_MESSAGE_MAP()


void CMyToolBar::OnSize(UINT nType, int cx, int cy)
{
	CToolBar::OnSize(nType, cx, cy);

	ArrangeControls();
}


void CMyToolBar::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	if ((CSliderCtrl*)pScrollBar == (CSliderCtrl*)GetDlgItem(IDC_SLIDER_VOLUME))
	{
		UINT uSliderPos = m_clsSliderVolume.GetPos();

		AfxGetApp()->m_pMainWnd->PostMessage(UWM_SLIDER_VOLUME, uSliderPos, 0);
	}

	CToolBar::OnHScroll(nSBCode, nPos, pScrollBar);
}


void CMyToolBar::SetSliderVolume(UINT8 uPos)
{
	if (GetDlgItem(IDC_SLIDER_VOLUME) != NULL)
	{
		if ((uPos < 0) || (uPos > 100))
			uPos = 0;

		m_clsSliderVolume.SetPos(uPos);
	}
}

UINT8 CMyToolBar::GetSliderVolume(void)
{
	if (GetDlgItem(IDC_SLIDER_VOLUME) != NULL)
		return m_clsSliderVolume.GetPos();

	else
		return 0;
}