///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

// ----------
// PathInfo.h
// ----------
/**
* @file
* @brief Re: PathInfo
* @author Achim Klein
* @version 1.3
*/


// -------------------
// One-Definition-Rule
// -------------------
#ifndef PATHINFO_H_INCLUDE_NR1
#define PATHINFO_H_INCLUDE_NR1


// -------
// Pragmas
// -------
#ifdef WIN32
#pragma warning(disable: 4786)
#endif


// --------
// Includes
// --------
#include <vector>
#include <string>


// ----------
// Namespaces
// ----------
using namespace std;


// --------------------------------
// Definition of the PathInfo class
// --------------------------------
/**
 * This class separates the different sections of a path.
 */
class PathInfo
{

public:

	// ------------
	// Construction
	// ------------

	/// standard-constructor
	PathInfo();

	/// virtual destructor
	virtual ~PathInfo();


	// -----------------
	// Public attributes
	// -----------------

	/// this character separates two folders (may be '/' or '\')
	static const char separator;


	// --------------
	// Static methods
	// --------------

	/// returns the drive's letter from the passed path
	static const char* getDrive(const char* Path);

	/// returns the directory from the passed path
	static const char* getDirectory(const char* Path);

	/// returns the file's folder from the passed path
	static const char* getFolder(const char* Path);

	/// returns the file's name from the passed path
	static const char* getName(const char* Path);

	/// returns the file's title from the passed path
	static const char* getTitle(const char* Path);

	/// returns the file's extension from the passed path
	static const char* getExtension(const char* Path);

	/// returns the passed path with this new extension
	static const char* getPathAs(const char* Path, const char* Extension);

	/// returns the absolute path starting at Origin
	static const char* getAbsPath(const char* RelPath, const char* Origin);

	/// returns the relative path starting at Origin
	static const char* getRelPath(const char* AbsPath, const char* Origin);


protected:

	// ----------------
	// Internal methods
	// ----------------

	/// divides a path up into pieces
	static int doSplitPath(const string& Path, vector<string>& Pieces);


private:

	// ----------
	// Attributes
	// ----------

	/// this buffer is used by any static method
	static string m_buffer;
};


#endif	// #ifndef PATHINFO_H_INCLUDE_NR1
