///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
//
// PlaylistDlg.cpp : implementation file
//

#include "stdafx.h"
#include "LASeRPlayer.h"
#include "PlaylistDlg.h"

#include "PathInfo.h"
// #include "splitpath.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define PLAYLIST_HARDLIMIT 500

/////////////////////////////////////////////////////////////////////////////
// CPlaylistDlg dialog


CPlaylistDlg::CPlaylistDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPlaylistDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CPlaylistDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CPlaylistDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPlaylistDlg)
	DDX_Control(pDX, IDC_LIST_PLAYLIST, m_clsListboxPlaylist);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPlaylistDlg, CDialog)
	//{{AFX_MSG_MAP(CPlaylistDlg)
	ON_LBN_DBLCLK(IDC_LIST_PLAYLIST, OnDblclkListPlaylist)
	ON_BN_CLICKED(IDC_BUTTON_PL_ADD, OnButtonPlAdd)
	ON_BN_CLICKED(IDC_BUTTON_PL_REMOVE, OnButtonPlRemove)
	ON_BN_CLICKED(IDC_BUTTON_PL_REMOVEALL, OnButtonPlRemoveall)
	ON_BN_CLICKED(IDC_BUTTON_PL_STARTPLAY, OnButtonPlStartplay)
	ON_BN_CLICKED(IDC_BUTTON_PL_LOAD, OnButtonPlLoad)
	ON_BN_CLICKED(IDC_BUTTON_PL_SAVE, OnButtonPlSave)
	ON_BN_CLICKED(IDC_BUTTON_PL_SORT, OnButtonPlSort)
	//}}AFX_MSG_MAP
  ON_WM_DROPFILES()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPlaylistDlg message handlers

void CPlaylistDlg::OnCancel() 
{
	CDialog::OnCancel();
	AfxGetApp()->m_pMainWnd->PostMessage(UWM_PLAYLIST_DLG_HIDE, 0, 0);
}

void CPlaylistDlg::OnOK() 
{
	CDialog::OnOK();
	AfxGetApp()->m_pMainWnd->PostMessage(UWM_PLAYLIST_DLG_HIDE, 0, 0);
}

BOOL CPlaylistDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	m_strFileToPlay = "";
	m_uCurrentPlayIndex = 0;

	// m_clsListboxPlaylist.SetHorizontalExtent(1000);
	DragAcceptFiles(TRUE);						// Accept user dropped file
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CPlaylistDlg::GetFileToPlay(CString &strFileToPlay)
{
	strFileToPlay = m_strFileToPlay;
}


INT8 CPlaylistDlg::RequestNextFile(void)
{
	int	nCurSel	= m_clsListboxPlaylist.GetCurSel();
	int	nCount	= m_clsListboxPlaylist.GetCount();

	// if playlist has no more files to be play
	if ((int)(m_uCurrentPlayIndex + 1) >= nCount)
		return -1;

	// select next file in list
	++m_uCurrentPlayIndex;

	// check if the file can be selected
	if (m_clsListboxPlaylist.SetCurSel(m_uCurrentPlayIndex) == LB_ERR)
		return -1;

	if (nCurSel != LB_ERR)
		m_clsListboxPlaylist.GetText(nCurSel, m_strFileToPlay);

	return 1;
}

INT8 CPlaylistDlg::RequestPreviousFile(void)
{
	int	nCurSel	= m_clsListboxPlaylist.GetCurSel();
	int	nCount	= m_clsListboxPlaylist.GetCount();

	// if current file is the first already
	if (m_uCurrentPlayIndex == 0)
		return -1;

	// select previous file in list
	--m_uCurrentPlayIndex;

	// check if the file can be selected
	if (m_clsListboxPlaylist.SetCurSel(m_uCurrentPlayIndex) == LB_ERR)
		return -1;

	if (nCurSel != LB_ERR)
		m_clsListboxPlaylist.GetText(nCurSel, m_strFileToPlay);

	return 1;
}

// -1 means no file in playlist
INT8 CPlaylistDlg::RewindPlayList(void)
{
	if (m_clsListboxPlaylist.SetCurSel(0) != LB_ERR)
	{
		// set the index to 0
		m_uCurrentPlayIndex = 0;
		// prepare the filename for first rewind play
		m_clsListboxPlaylist.GetText(0, m_strFileToPlay);

		return 1;
	}

	else
		return -1;
}

int CPlaylistDlg::AddPlayListEntry(CString strEntry)
{
	int		nCount	= m_clsListboxPlaylist.GetCount();;
	INT8	nReturn	= 0;

	if (nCount < PLAYLIST_HARDLIMIT)
	{
		m_clsListboxPlaylist.AddString(strEntry);

		// set the horizontal extent
		SetExtendListbox();
		nReturn = 1;
	}

	else
		nReturn = -1;

	return nReturn;
}

void CPlaylistDlg::DoPlayListPlay(void)
{
	int nCurSel = m_clsListboxPlaylist.GetCurSel();

	if (nCurSel != LB_ERR)
	{
		// save start point to a member variable
		m_uCurrentPlayIndex = nCurSel;

		m_clsListboxPlaylist.GetText(nCurSel, m_strFileToPlay);
		// inform main frame that a file is ready to be played
		AfxGetApp()->m_pMainWnd->PostMessage(UWM_PLAYLIST_PLAY, 0, 0);
	}
}

void CPlaylistDlg::DoPlayListAdd(void)
{
	// define filters for use with the file dialog
	const TCHAR szFileDialogFilter[]	= _T("all files (*.*)|*.*||");
	const TCHAR szFileDialogExt[]		= _T("");
	CString		strFilename				= _T("");
	CString		strPathName				= _T("");
	CString		str(_T(" "), 10000);
	POSITION	pos;

	CFileDialog fileDialog(TRUE,
		szFileDialogExt,
		NULL,
		OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_READONLY | OFN_ALLOWMULTISELECT,
		szFileDialogFilter);

	fileDialog.m_ofn.lpstrFile	= str.GetBuffer(10000);
	fileDialog.m_ofn.nMaxFile	= 10000;
	str.ReleaseBuffer();

	if (fileDialog.DoModal() == IDOK)
	{
		pos = fileDialog.GetStartPosition();

		// to avoid refresh
		m_clsListboxPlaylist.SetRedraw(FALSE);
		// show the wait cursor
		CWaitCursor wait;

		while (pos != NULL)
		{
			strPathName = fileDialog.GetNextPathName(pos);

			// call routine to add entry, maximum PLAYLIST_HARDLIMIT entries
			if (AddPlayListEntry(strPathName) != 1)
				break;
		}

		// enable drawing and make sure list box is redrawn
		m_clsListboxPlaylist.SetRedraw(TRUE);
		m_clsListboxPlaylist.Invalidate();
	}
}

void CPlaylistDlg::DoPlayListRemove(void)
{
	int nCurSel = m_clsListboxPlaylist.GetCurSel();

	if (nCurSel != LB_ERR)
	{
		m_clsListboxPlaylist.DeleteString(nCurSel);

		// set the horizontal extent
		SetExtendListbox();

		// try to select next string
		if (m_clsListboxPlaylist.GetCount() > 0)
		{
			if (m_clsListboxPlaylist.SetCurSel(nCurSel) == LB_ERR)
				m_clsListboxPlaylist.SetCurSel(nCurSel - 1);
		}
	}
}

void CPlaylistDlg::DoPlayListRemoveAll(void)
{
	m_clsListboxPlaylist.ResetContent();

	// set the horizontal extent
	SetExtendListbox();
}

void CPlaylistDlg::OnDblclkListPlaylist() 
{
	DoPlayListPlay();
}

void CPlaylistDlg::OnButtonPlAdd() 
{
	DoPlayListAdd();
}

void CPlaylistDlg::OnButtonPlRemove() 
{
	DoPlayListRemove();
}

void CPlaylistDlg::OnButtonPlRemoveall() 
{
	DoPlayListRemoveAll();
}

void CPlaylistDlg::OnButtonPlStartplay() 
{
	DoPlayListPlay();
}

void CPlaylistDlg::LoadPlayList(CString strFileName)
{
	CStdioFile	file;
	CString		strTemp			= _T("");
	CString		strPathPrefix	= _T("");

	// reset content
	m_clsListboxPlaylist.ResetContent();

	if (!file.Open(strFileName, CFile::modeRead | CFile::typeText))
	{
		MessageBox("unable to open file");
		return;
	}

	// get strFileName path (for absolute path prefix)
	strPathPrefix = PathInfo::getDirectory(strFileName);

	// to avoid refresh
	m_clsListboxPlaylist.SetRedraw(FALSE);
	// show the wait cursor
	CWaitCursor wait;

	while (file.ReadString(strTemp))
	{
		strTemp.TrimLeft();
		strTemp.TrimRight();

		if (strTemp.GetLength() > 0)
		{
			// entries preceded by '#' and ';' and '[' are comments, just ignore them
			if (strTemp.Find("#") == 0)
				continue;
			if (strTemp.Find(";") == 0)
				continue;
			if (strTemp.Find("[") == 0)
				continue;

			// no "://" found, means not a Net Radio Stream URL
			if (strTemp.Find("://") == -1)
			{
				// found ":\" in filename, if not found, means relative path
				if (strTemp.Find(":\\") == -1)
					// strTemp = strPathPrefix + strTemp;
					// MessageBox(strPathPrefix);
					// get absolute path
					strTemp = PathInfo::getAbsPath(strTemp, strPathPrefix);
			}

			// call AddPlayListEntry(), PLAYLIST_HARDLIMIT limit
			if (AddPlayListEntry(strTemp) != 1)
				break;
		}
	}

	// enable drawing and make sure list box is redrawn
	m_clsListboxPlaylist.SetRedraw(TRUE);
	m_clsListboxPlaylist.Invalidate();
}

void CPlaylistDlg::OnButtonPlLoad() 
{
	// define filters for use with the file dialog
	const TCHAR	szFileDialogFilter[]	= _T("playlist files (*.txt, *.pls)|*.txt;*.pls||");
	const TCHAR	szFileDialogExt[]		= _T("txt");
	CString		strPathName				= _T("");

	CFileDialog fileDialog(TRUE,
		szFileDialogExt,
		NULL,
		OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_READONLY,
		szFileDialogFilter);

	if (fileDialog.DoModal() == IDOK)
	{
		strPathName = fileDialog.GetPathName();
		strPathName.TrimLeft();
		strPathName.TrimRight();

		if (strPathName.GetLength() > 0)
			LoadPlayList(strPathName);
	}
}

void CPlaylistDlg::PlayFromFirst(void)
{
	int nCount = m_clsListboxPlaylist.GetCount();

	// if there is no item in list, return immediately
	if (nCount < 1)
		return;

	// select the first one
	m_clsListboxPlaylist.SetCurSel(0);

	// start play
	DoPlayListPlay();
}

void CPlaylistDlg::SetExtendListbox(void) 
{
	// Find the longest string in the list box. 
	CString str; 
	CSize sz; 
	int dx = 0; 
	TEXTMETRIC tm; 
	CDC* pDC = m_clsListboxPlaylist.GetDC(); 
	CFont* pFont = m_clsListboxPlaylist.GetFont(); 

	// Select the listbox font, save the old font 
	CFont* pOldFont = pDC->SelectObject(pFont); 
	// Get the text metrics for avg char width 
	pDC->GetTextMetrics(&tm); 

	for (UINT i = 0; i < (UINT)m_clsListboxPlaylist.GetCount(); ++i) 
	{ 
		m_clsListboxPlaylist.GetText(i, str); 
		sz = pDC->GetTextExtent(str); 

		// Add the avg width to prevent clipping 
		sz.cx += tm.tmAveCharWidth; 

		if (sz.cx > dx) 
			dx = sz.cx; 
	}
	// Select the old font back into the DC 
	pDC->SelectObject(pOldFont); 
	m_clsListboxPlaylist.ReleaseDC(pDC); 

	// Set the horizontal extent so every character of all strings 
	// can be scrolled to. 
	m_clsListboxPlaylist.SetHorizontalExtent(dx);
}

void CPlaylistDlg::OnButtonPlSave() 
{
	// define filters for use with the file dialog
	const TCHAR	szFileDialogFilter[]	= _T("playlist files (*.txt, *.pls)|*.txt;*.pls||");
	const TCHAR	szFileDialogExt[]		= _T("txt");
	CString		strPathName				= _T("");

	CFileDialog fileDialog(FALSE,
		szFileDialogExt,
		NULL,
		OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		szFileDialogFilter);

	if (fileDialog.DoModal() == IDOK)
	{
		strPathName = fileDialog.GetPathName();
		strPathName.TrimLeft();
		strPathName.TrimRight();

		if (strPathName.GetLength() > 0)
			SavePlayList(strPathName);
	}
}

void CPlaylistDlg::SavePlayList(CString strFileName)
{
	CStdioFile file;
	CString strAbsolutePathName	= _T("");
	CString strRelativePath		= _T("");
	CString	strRelativePathName	= _T("");

	// get path to be related to (eg: "D:\user\album.pls", "D:\user")
	strRelativePath = PathInfo::getDirectory(strFileName);

	if (!file.Open(strFileName, CFile::modeCreate | CFile::modeWrite | CFile::typeText))
	{
		MessageBox("unable to open file");
		return;
	}

	for (UINT i = 0; i < (UINT)m_clsListboxPlaylist.GetCount(); ++i) 
	{ 
		// get absolute pathname from list (eg: "d:\music\1.mp3")
		m_clsListboxPlaylist.GetText(i, strAbsolutePathName);
		// absolute -> relative path (eg: "..\music\1.mp3")
		strRelativePathName = PathInfo::getRelPath(strAbsolutePathName, strRelativePath);
		strRelativePathName += "\n";
		// save it
		file.WriteString(strRelativePathName);
	}
}

void CPlaylistDlg::DoPlaylistSort(void)
{
	int n = m_clsListboxPlaylist.GetCount();

	// needless to do sort?
	if (n < 2)
		return;

	// to avoid refresh
	m_clsListboxPlaylist.SetRedraw(FALSE);
	// show the wait cursor
	CWaitCursor wait;

	// Call it's own sort algorithm
	m_clsListboxPlaylist.SortList(TRUE);

	// enable drawing and make sure list box is redrawn
	m_clsListboxPlaylist.SetRedraw(TRUE);
	m_clsListboxPlaylist.Invalidate();
}

void CPlaylistDlg::OnButtonPlSort() 
{
	DoPlaylistSort();	
}

// Dropped file handler
void CPlaylistDlg::OnDropFiles(HDROP hDropInfo) 
{
	TCHAR szFileName[_MAX_PATH];

	UINT uFiles = ::DragQueryFile(hDropInfo, (UINT) -1, NULL, 0);

	// Add the whole list to playlist
	for (UINT uIndex = 0; uIndex < uFiles; ++uIndex)
	{
		::DragQueryFile(hDropInfo, uIndex, szFileName, _MAX_PATH);
		AddPlayListEntry(szFileName);
	}

	::DragFinish(hDropInfo);
}