///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

// SeekSliderCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "LASeRPlayer.h"
#include "SeekSliderCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSeekSliderCtrl

CSeekSliderCtrl::CSeekSliderCtrl() : m_cnMin(0), m_cnMax(100), m_cnThumbWidth(25)
{
	m_uPos		= 0;
	m_bDragging	= FALSE;
	m_bEnable	= FALSE;
	m_nY		= 0;
}

CSeekSliderCtrl::~CSeekSliderCtrl()
{
}


BEGIN_MESSAGE_MAP(CSeekSliderCtrl, CSliderCtrl)
	//{{AFX_MSG_MAP(CSeekSliderCtrl)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_ERASEBKGND()
	ON_WM_SETFOCUS()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSeekSliderCtrl message handlers

void CSeekSliderCtrl::OnPaint() 
{
	CPaintDC dc0(this);							// device context for painting
	CMemDC dc(&dc0);							// flickless drawing, see memdc.h for details

	COLORREF
		colorWhite		= GetSysColor(COLOR_WINDOW),
		colorShadow		= GetSysColor(COLOR_3DSHADOW), 
		colorLight		= GetSysColor(COLOR_3DHILIGHT), 
		colorBKG		= GetSysColor(COLOR_BTNFACE),
		colorHilight	= GetSysColor(COLOR_HIGHLIGHT),
		color3dface		= GetSysColor(COLOR_3DFACE),
		color3DDKShadow	= GetSysColor(COLOR_3DDKSHADOW);

	// get old pen/brush for restore
	CBrush	*pOldBrush	= dc.GetCurrentBrush();;
	CPen	*pOldPen	= dc.GetCurrentPen();

	CBrush	Brush3DFACE(color3dface);
	CPen	PenBlack(PS_SOLID, 0, color3DDKShadow);

	// client
	{
		CRect rClient;
		GetClientRect(&rClient);
		// dc.SelectStockObject(LTGRAY_BRUSH);
		dc.FillSolidRect(rClient, color3dface);
	}

	// channel
	{
		CRect rChannel;
		rChannel = GetChannelRect();
		dc.Draw3dRect(rChannel, colorShadow, colorLight);
		rChannel.DeflateRect(1, 1);
		dc.Draw3dRect(rChannel, 0, color3dface);
		rChannel.DeflateRect(1, 1);

		if (m_bEnable)
			// fill shadow color if enabled
			dc.FillSolidRect(rChannel, colorShadow);
	}

	// thumb
	if (m_bEnable)
	{
		CRect rThumb;
		CPoint ptThumb1, ptThumb2;

		rThumb = GetThumbRect();
		dc.FillSolidRect(rThumb, color3dface);
		dc.Draw3dRect(rThumb, colorLight, color3DDKShadow);
		rThumb.DeflateRect(1, 1, 1, 1);
		dc.Draw3dRect(rThumb, color3dface, colorShadow);

		ptThumb1.x = rThumb.CenterPoint().x;
		ptThumb2.x = rThumb.CenterPoint().x;
		ptThumb1.y = rThumb.top + 2;
		ptThumb2.y = rThumb.bottom - 2;

		pOldPen = dc.SelectObject(&PenBlack);
		dc.MoveTo(ptThumb1);
		dc.LineTo(ptThumb2);
		dc.SelectObject(pOldPen);
	}

	// realplayer like channel (left, played)
	if (1)
	{
		CRect rPlayed;
		CRect rThumb;
		CRect rChannel;

		rChannel = GetChannelRect();
		rThumb = GetThumbRect();

		rPlayed.left = 0;
		rPlayed.top = rChannel.top;
		rPlayed.bottom = rChannel.bottom;
		rPlayed.right = rThumb.left;
		rPlayed.DeflateRect(3, 2, 0, 2);
		dc.FillSolidRect(rPlayed, color3DDKShadow);
	}

	// restore old pen/brush to avoid gdi resource leak
	dc.SelectObject(pOldPen);
	dc.SelectObject(pOldBrush);

	// Do not call CSliderCtrl::OnPaint() for painting messages
}

void CSeekSliderCtrl::SetPos(UINT8 uPos)
{
	if (!m_bDragging)
	{
		if ((uPos >= m_cnMin) && (uPos <= m_cnMax))
		{
			if (m_uPos != uPos)
			{
				m_uPos = uPos;
				Invalidate();
			}
		}
	}
}

UINT8 CSeekSliderCtrl::GetPos(void)
{
	return m_uPos;
}

void CSeekSliderCtrl::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if (!m_bEnable)
		// return if disabled
		return;

	if (GetThumbRect().PtInRect(point))
	{
		m_bDragging = TRUE;
		m_nY = point.y;

		SetCapture();
		SetFocus();
	}

	else
	{
		m_bDragging = TRUE;
		m_nY = point.y;

		MoveThumb(point);
		SetCapture();
		SetFocus();
		Invalidate();
	}
}

void CSeekSliderCtrl::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if (m_bDragging == TRUE)
	{
		ReleaseCapture();
		m_bDragging = FALSE;

		// Post message with SB_THUMBPOSITION prefix, to filter out Home/End message
		GetParent()->PostMessage(WM_HSCROLL, MAKEWPARAM(SB_THUMBPOSITION, (WORD)m_uPos), (LPARAM)m_hWnd);
	}
}

void CSeekSliderCtrl::OnMouseMove(UINT nFlags, CPoint point) 
{
	CWnd* w = GetCapture();
	if(w && w->m_hWnd == m_hWnd && (nFlags & MK_LBUTTON))
	{
		if (abs(m_nY - point.y) > 100)
		{
			m_bDragging = FALSE;
			ReleaseCapture();
		}

		else
		{
			MoveThumb(point);
			Invalidate();
		}
	}
}

CRect CSeekSliderCtrl::GetChannelRect(void)
{
	CRect r;

	GetClientRect(&r);

	r.DeflateRect(1, 3);
	// r.bottom = r.top + 20;

	return (r);
}

CRect CSeekSliderCtrl::GetThumbRect(void)
{
	CRect r, rChannel;

	rChannel = GetChannelRect();
	r = rChannel;

	int nHalfHeight	= r.Height() / 2;
	int x			= r.left + (int)((r.Width() - m_cnThumbWidth - 4) * m_uPos / 100);
	int y			= r.CenterPoint().y;

	r.SetRect(x, y, x, y);
	r.InflateRect(0, nHalfHeight, m_cnThumbWidth, nHalfHeight);

	r.left += 2;
	r.right += 2;
	r.top = rChannel.top + 2;
	r.bottom = rChannel.bottom - 2;

	if (r.right > rChannel.right - 2)
	{
		r.right = rChannel.right - 2;
		r.left = r.right - m_cnThumbWidth;
	}

	return r;
}


void CSeekSliderCtrl::MoveThumb(CPoint point)
{
	CRect r = GetChannelRect();

	point.x -= m_cnThumbWidth / 2;

	if (point.x < r.left)
		m_uPos = (UINT8)m_cnMin;

	else if (point.x >= (r.right - m_cnThumbWidth))
		m_uPos = (UINT8)m_cnMax;

	else
		m_uPos = (UINT8)(point.x * 100 / (r.Width() - m_cnThumbWidth));
}

void CSeekSliderCtrl::SetEnable(UINT8 bEnable)
{
	if (m_bEnable != bEnable)
	{
		m_bEnable = bEnable;

		if (!m_bEnable)
		{
			// set m_uPos to 0 if disabled
			m_uPos = 0;

			// disable drag just in case
			if (m_bDragging)
			{
				m_bDragging = FALSE;
				ReleaseCapture();
			}
		}

		// redraw the control
		Invalidate();
	}
}

BOOL CSeekSliderCtrl::PreTranslateMessage(MSG* pMsg) 
{
	if(pMsg->message == WM_KEYDOWN)
	{
		if((pMsg->wParam == VK_ESCAPE) && (m_bDragging == TRUE))
		{
			m_bDragging = FALSE;
			ReleaseCapture();
			return TRUE;
		}
	}

	return CSliderCtrl::PreTranslateMessage(pMsg);
}

void CSeekSliderCtrl::OnSetFocus(CWnd* pOldWnd) 
{
	CSliderCtrl::OnSetFocus(pOldWnd);
	
	AfxGetMainWnd()->SetFocus();				// don't focus on us, parents will take care of our positioning	
}