///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

// Terminal_State.cpp :  Դϴ.
//

#include "stdafx.h"
#include "LASeRPlayer.h"
#include "Terminal_State.h"
#include <math.h>


static CString g_strDisPlay[] = {"0", "1 inch", "2 inch", "3 inch", "4 inch", "5 inch",
								 "6 inch", "7 inch", "8 inch", "9 inch", "10 inch",
								 "11 inch", "12 inch", "13 inch", "14 inch", "15 inch",
								 "16 inch", "17 inch", "18 inch", "19 inch", "20 inch",
								 "21 inch", "22 inch", "23 inch", "24 inch", "25 inch"};

// CTerminal_State ȭ Դϴ.

IMPLEMENT_DYNAMIC(CTerminal_State, CDialog)

CTerminal_State::CTerminal_State(CWnd* pParent /*=NULL*/)
	: CDialog(CTerminal_State::IDD, pParent)
	, m_nTotalMemory(0)
	, m_nAvailablePhys(0)
	, m_nResolutionX(0)
	, m_nResolutionY(0)
	, m_nDpiX(0)
	, m_nDpiY(0)
{
	m_nDisPlay = -1;
	m_nAdaptationSelected = 0;
}

CTerminal_State::~CTerminal_State()
{
}

void CTerminal_State::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CComboDlg)
	DDX_Control(pDX, IDC_DISPLAY, m_ctrlDisPlay);
	DDX_CBIndex(pDX, IDC_DISPLAY, m_nDisPlay);
	//}}AFX_DATA_MAP
	DDX_Text(pDX, IDC_TOTALMEMORY, m_nTotalMemory);
	DDX_Text(pDX, IDC_AVAILABLEPHYS, m_nAvailablePhys);
	DDX_Text(pDX, IDC_RESOLUTIONX, m_nResolutionX);
	DDX_Text(pDX, IDC_RESOLUTIONY, m_nResolutionY);
	DDX_Text(pDX, IDC_DPIX, m_nDpiX);
	DDX_Text(pDX, IDC_DPIY, m_nDpiY);
}


BEGIN_MESSAGE_MAP(CTerminal_State, CDialog)
	ON_BN_CLICKED(IDC_BUTTON_OK, &CTerminal_State::OnBnClickedButtonOk)
	ON_BN_CLICKED(IDC_BUTTON1, &CTerminal_State::OnBnClickedButton1)
END_MESSAGE_MAP()


// CTerminal_State ޽ óԴϴ.

BOOL CTerminal_State::OnInitDialog()
{
	CDialog::OnInitDialog();

	// TODO:  ⿡ ߰ ʱȭ ۾ ߰մϴ.

	SetCurMemory();     //Set Memory Size
	SetCurResolution(); //Set Resolution XY
	SetCurDpi();

	for (int i=0; i<25; i++)
	{
		m_ctrlDisPlay.AddString(g_strDisPlay[i]);
	}

	SetCurDisplay();    //Set Display Size

	return TRUE;  // return TRUE unless you set the focus to a control
	// : OCX Ӽ  FALSE ȯؾ մϴ.
}

void CTerminal_State::SetCurMemory()
{
	//Get Memory size
	MEMORYSTATUS stat;
	GlobalMemoryStatus(&stat);

	UINT uTotalPhys = stat.dwTotalPhys;
	UINT uAvailPhys = stat.dwAvailPhys;

	m_nTotalMemory   = uTotalPhys/1024;
	m_nAvailablePhys = uAvailPhys/1024;

	UpdateData(FALSE);
}

void CTerminal_State::SetCurResolution()
{
	//Get Monitor info
	HMONITOR hMonitor;
	MONITORINFO mi;
	RECT        rc;

	hMonitor = MonitorFromRect(&rc, MONITOR_DEFAULTTONEAREST);
	mi.cbSize = sizeof(mi);
	GetMonitorInfo(hMonitor, &mi);

	m_nResolutionX = mi.rcMonitor.right;
	m_nResolutionY = mi.rcMonitor.bottom;

	UpdateData(FALSE);
}

void CTerminal_State::SetCurDisplay()
{
	HMONITOR    hMonitor;
	MONITORINFO mi;
	RECT        rc;
	CPoint      pt;
	CClientDC   dc(this);

	hMonitor = MonitorFromRect(&rc, MONITOR_DEFAULTTONEAREST);
	mi.cbSize = sizeof(mi);
	GetMonitorInfo(hMonitor, &mi);

	pt.x = mi.rcMonitor.right;
	pt.y = mi.rcMonitor.bottom;
	dc.SetMapMode(MM_LOENGLISH);
	dc.DPtoLP(&pt);

	float fDisplayX   =  (float)pt.x/100;
	float fDisplayY   =  (float)-pt.y/100;
	float fDisplay    =  sqrt((fDisplayX*fDisplayX) + (fDisplayY*fDisplayY));
	int u_displaysize =  ceil(fDisplay);
	
	m_ctrlDisPlay.SetCurSel(u_displaysize);

	UpdateData(TRUE);
}

void CTerminal_State::SetCurDpi()
{
	CDC *m_pDC;
	m_pDC = GetDC();
	Graphics graphics(m_pDC->m_hDC);

	m_nDpiX = graphics.GetDpiX();
	m_nDpiY = graphics.GetDpiY();
	UpdateData(FALSE);
}

void CTerminal_State::OnBnClickedButton1()
{
	// TODO: ⿡ Ʈ ˸ ó ڵ带 ߰մϴ.
	SetCurMemory();     //Set Memory Size
	SetCurResolution(); //Set Resolution XY
	SetCurDisplay();    //Set Display Size
}

void CTerminal_State::OnBnClickedButtonOk()
{
	// TODO: ⿡ Ʈ ˸ ó ڵ带 ߰մϴ.
	UpdateData(TRUE);
	UpdateData(FALSE);
}