///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

// VolumeSliderCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "LASeRPlayer.h"
#include "VolumeSliderCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVolumeSliderCtrl

CVolumeSliderCtrl::CVolumeSliderCtrl() : m_cnMin(0), m_cnMax(100), m_cnThumbWidth(10)
{
	m_uPos		= 0;
	m_bDragging = FALSE;
}

CVolumeSliderCtrl::~CVolumeSliderCtrl()
{
}


BEGIN_MESSAGE_MAP(CVolumeSliderCtrl, CSliderCtrl)
	//{{AFX_MSG_MAP(CVolumeSliderCtrl)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_ERASEBKGND()
	ON_WM_SETFOCUS()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVolumeSliderCtrl message handlers

void CVolumeSliderCtrl::OnPaint() 
{
	CPaintDC dc(this);							// device context for painting

	COLORREF 
		colorWhite		= GetSysColor(COLOR_WINDOW),
		colorShadow		= GetSysColor(COLOR_3DSHADOW), 
		colorLight		= GetSysColor(COLOR_3DHILIGHT), 
		colorBKG		= GetSysColor(COLOR_BTNFACE),
		colorHilight	= GetSysColor(COLOR_HIGHLIGHT),
		color3dface		= GetSysColor(COLOR_3DFACE),
		color3DDKShadow	= GetSysColor(COLOR_3DDKSHADOW);

	// get old pen/brush for restore
	CBrush	*pOldBrush	= dc.GetCurrentBrush();
	CPen	*pOldPen	= dc.GetCurrentPen();

	CBrush brush3DFACE(color3dface);
	CBrush brush3DDKShadow(color3DDKShadow);
	CPen penWhite(PS_SOLID, 0, colorLight);
	CPen penShadow(PS_SOLID, 0, colorShadow);

	// client
	{
		CRect rClient;
		GetClientRect(&rClient);
		dc.FillSolidRect(rClient, color3dface);
	}

	// channel
	{
		CRect rChannel;
		rChannel = GetChannelRect();
		rChannel.DeflateRect(0, 1, 1, 4);

		CPoint ptPoint[3];
		ptPoint[0].x = rChannel.right;
		ptPoint[0].y = rChannel.top;

		ptPoint[1].x = rChannel.right;
		ptPoint[1].y = rChannel.bottom;

		ptPoint[2].x = rChannel.left;
		ptPoint[2].y = rChannel.bottom;

		pOldPen = dc.SelectObject(&penWhite);

		dc.MoveTo(ptPoint[0]);
		dc.LineTo(ptPoint[1]);

		dc.MoveTo(ptPoint[1]);
		dc.LineTo(ptPoint[2]);

		dc.SelectObject(&penShadow);
		dc.MoveTo(ptPoint[2]);
		dc.LineTo(ptPoint[0]);

		dc.SelectObject(pOldPen);
	}

	// thumb
	{
		CRect rThumb;

		rThumb = GetThumbRect();
		dc.FillSolidRect(rThumb, color3dface);
		dc.Draw3dRect(rThumb, colorLight, color3DDKShadow);
		rThumb.DeflateRect(1, 1, 1, 1);
		dc.Draw3dRect(rThumb, color3dface, colorShadow);
	}

	// restore old pen/brush to avoid gdi resource leak
	dc.SelectObject(pOldPen);
	dc.SelectObject(pOldBrush);

	// Do not call CSliderCtrl::OnPaint() for painting messages
}


void CVolumeSliderCtrl::SetPos(UINT8 uPos)
{
	if (!m_bDragging)
	{
		if ((uPos >= m_cnMin) && (uPos <= m_cnMax))
			m_uPos = uPos;

		Invalidate();
	}
}

UINT8 CVolumeSliderCtrl::GetPos()
{
	return m_uPos;
}

void CVolumeSliderCtrl::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if (GetThumbRect().PtInRect(point))
	{
		m_bDragging = TRUE;

		SetCapture();
		SetFocus();
	}

	else
	{
		m_bDragging = TRUE;

		MoveThumb(point);
		SetCapture();
		SetFocus();
		Invalidate();
	}
}

void CVolumeSliderCtrl::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if (m_bDragging == TRUE)
	{
		ReleaseCapture();
		m_bDragging = FALSE;

		GetParent()->PostMessage(WM_HSCROLL, MAKEWPARAM(SB_THUMBPOSITION, (WORD)m_uPos), (LPARAM)m_hWnd);
	}
}

void CVolumeSliderCtrl::OnMouseMove(UINT nFlags, CPoint point) 
{
	CWnd* w = GetCapture();

	if (w && w->m_hWnd == m_hWnd && (nFlags & MK_LBUTTON))
	{
		MoveThumb(point);
		Invalidate();
	}
}

CRect CVolumeSliderCtrl::GetChannelRect(void)
{
	CRect r;

	GetClientRect(&r);

	return (r);
}

CRect CVolumeSliderCtrl::GetThumbRect(void)
{
	CRect r, rClient;

	GetClientRect(&rClient);
	r = GetChannelRect();

	int nHalfHeight;

	int x = r.left + (int)((r.Width() - m_cnThumbWidth) * m_uPos / 100);
	int y = r.CenterPoint().y;

	nHalfHeight = r.Height() / 2;

	r.SetRect(x, y, x, y);
	r.InflateRect(0, nHalfHeight, m_cnThumbWidth, nHalfHeight);

	return (r);
}

void CVolumeSliderCtrl::MoveThumb(CPoint point)
{
	CRect r = GetChannelRect();

	point.x -= m_cnThumbWidth / 2;

	if (point.x < r.left)
		m_uPos = (UINT8)m_cnMin;

	else if (point.x >= (r.right - m_cnThumbWidth))
		m_uPos = (UINT8)m_cnMax;

	else
		m_uPos = (UINT8)(point.x * 100 / (r.Width() - m_cnThumbWidth));
}

void CVolumeSliderCtrl::OnSetFocus(CWnd* pOldWnd) 
{
	CSliderCtrl::OnSetFocus(pOldWnd);
	
	AfxGetMainWnd()->SetFocus();				// don't focus on us, parents will take care of our positioning	
}