///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "SceneTreeManager.h"
#include "..\libLASeR\LASeR.h"
#include "..\libLASeR\SAF.h"
#include "..\libLASeR\Datatypes.h"

#include <algorithm>


static CSceneTreeManager SceneTreeMgr;

CSceneTreeManager::CSceneTreeManager()
{
	m_pRoot				= NULL;

	m_pCurLASeRElement	= NULL;
	m_pCurSAFElement	= NULL;

	m_uNextIndex		= 0;
}

CSceneTreeManager::~CSceneTreeManager()
{
	if (m_pRoot)
	{
		delete m_pRoot;
		m_pRoot = NULL;
	}

	m_apNamedLASeRElement.clear();
	m_apNamedSAFElement.clear();

	m_apConditionalElement.clear();
}

lsrErr CSceneTreeManager::SetRoot(void *pRoot)
{
	lsrErr err = LASeRNoErr;

	m_pRoot = (CSAFSession *)pRoot;

	// for init
	if (!m_pRoot)
		return err;

	// first traveling, current is root
	SetCurSAFElement(m_pRoot);
	SetCurLASeRElement(NULL);

	// set additional info
	CLASeR	*pLASeR	= NULL;
	CSAF	*pSAF	= NULL;

	while (1)
	{
		err = GetNextObject(&pLASeR, &pSAF);
		CONFIRM(err);

		if ((pLASeR != NULL) && (pSAF == NULL))
		{
			if (pLASeR != NULL)
			{
				// save named element
				if (pLASeR->CheckHavingID())
					SceneTreeMgr.m_apNamedLASeRElement.push_back(pLASeR);

				// save conditional element
				if (pLASeR->GetElementType() == LSR_conditional)
					SceneTreeMgr.m_apConditionalElement.push_back((CLSRConditional *)pLASeR);
			}
		}

		else if ((pLASeR == NULL) && (pSAF != NULL))
		{
			// save named element
			if (pSAF != NULL)
			{
				if (pSAF->CheckHavingID())
					SceneTreeMgr.m_apNamedSAFElement.push_back((CESHeader *)pSAF);
			}

			if (SAF_EndOfSAFSession == pSAF->GetElementType())
				break;
		}
	}

	// for next traveling, current is root
	SetCurSAFElement(m_pRoot);
	SetCurLASeRElement(NULL);

	return LASeRNoErr;
}

/************************************************************************/
// SetSceneRoot
// Desc		: Set the scene root element
// Param	: root element pointer
// History
//	PSH (2008. 7. 23)	- first release
/************************************************************************/
extern "C" _declspec(dllexport)
lsrErr __stdcall SetSceneRoot(const void *pRoot)
{
	lsrErr err = LASeRNoErr;

	err = SceneTreeMgr.SetRoot((void *)pRoot);

	return err;
}

/************************************************************************/
// GetSceneRoot
// Desc		: Get the scene root element
// Param	: none
// History
//	PSH (2008. 7. 24)	- first release
/************************************************************************/
extern "C" _declspec(dllexport)
CSAFSession * __stdcall GetSceneRoot()
{
	return SceneTreeMgr.GetRoot();
}

/************************************************************************/
// GetSVG
// Desc		: Get the svg element
// Param	: none
// History
//	PSH (2009. 2. 4)	- first release
/************************************************************************/
extern "C" _declspec(dllexport)
CLASeR * __stdcall GetSVG()
{
	CSceneUnit	*pSceneUnit	= SceneTreeMgr.GetRoot()->GetSceneUnit(0);
	UINT8		uSize		= pSceneUnit->GetChildrenSize();

	UINT8 i = 0;

	for (i = 0; i < uSize; ++i)
	{
		if (pSceneUnit->GetChildren(i)->GetElementType() == LSR_NewScene)
			break;
	}

	if (i < uSize)
		// svg
		return pSceneUnit->GetChildren(i)->GetChildren(0);

	else
		return NULL;
}

/************************************************************************/
// GetNextObject
// Desc		: Travel in the scene tree (preorder type)
// Param	: LASeR element pointer or SAF element pointer (for outgoing)
// History
//	PSH (2008. 7. 23)	- first release
/************************************************************************/
extern "C" _declspec(dllexport)
lsrErr __stdcall GetNextObject(CLASeR **pLASeRElement, CSAF **pSAFElement)
{
	lsrErr err = LASeRNoErr;

	CLASeR	*pCurLASeRElement	= NULL;
	CSAF	*pCurSAFElement		= NULL;

	// normal
	if ((*pLASeRElement == NULL) && (*pSAFElement == NULL))
	{
		pCurLASeRElement	= SceneTreeMgr.GetCurLASeRElement();
		pCurSAFElement		= SceneTreeMgr.GetCurSAFElement();
	}

	// for searching
	else if ((*pLASeRElement == NULL) || (*pSAFElement == NULL))
	{
		pCurLASeRElement	= *pLASeRElement;
		pCurSAFElement		= *pSAFElement;

		*pLASeRElement		= NULL;
		*pSAFElement		= NULL;
	}

	// LASeR Element
	if (pCurLASeRElement && !pCurSAFElement)
	{
		UINT8 uChildrenSize = pCurLASeRElement->GetChildrenSize();

		/************************************************************************/
		// if the current element has one more children
		if (uChildrenSize > SceneTreeMgr.GetIndex())
		{
			deque<CLASeR *> apChildren = pCurLASeRElement->GetChildren();

			*pLASeRElement		= apChildren[SceneTreeMgr.GetIndex()];

			pCurLASeRElement	= *pLASeRElement;
			pCurSAFElement		= NULL;
		}

		else
		{
			/************************************************************************/
			// if the current element has no child and a cousin
			err = SceneTreeMgr.GetCousin(pCurLASeRElement, pLASeRElement);
			CONFIRM(err);

			if (*pLASeRElement)
			{
				SceneTreeMgr.SetIndex(0);			// because parent node was changed

				pCurLASeRElement	= *pLASeRElement;
				pCurSAFElement		= NULL;
			}

			/************************************************************************/
			// if the current element has no child and cousin
			else
			{
				/************************************************************************/
				// if the current element has a SAF parent

				// save current element's index
				SceneTreeMgr.SetIndex(pCurLASeRElement->GetIndex() + 1);

				UINT8 uCurLASeRElementType = pCurLASeRElement->GetElementType();

				if (uCurLASeRElementType == LSR_LASeRHeader)
				{
					CLASeRHeader *pTemp = (CLASeRHeader *)pCurLASeRElement;

					pCurLASeRElement	= NULL;
					pCurSAFElement		= (CSAF *)pTemp->GetParent();
				}

				else if ((uCurLASeRElementType >= LSR_StringIDTable) && (uCurLASeRElementType <= LSR_ReleaseResource))
				{
					CLASeRUnit *pTemp	= (CLASeRUnit *)pCurLASeRElement;

					if (pTemp->GetParentElementType() >= SAF_SAFSession)
					{
						pCurLASeRElement	= NULL;
						pCurSAFElement		= (CSAF *)pTemp->GetParent();
					}

					else
					{
						pCurLASeRElement	= (CLASeR *)pTemp->GetParent();
						pCurSAFElement		= NULL;
					}
				}
				/************************************************************************/

				// normal case
				else
				{
					pCurLASeRElement	= pCurLASeRElement->GetParent();
					pCurSAFElement		= NULL;
				}
			}
		}

		SceneTreeMgr.SetCurLASeRElement(pCurLASeRElement);
		SceneTreeMgr.SetCurSAFElement(pCurSAFElement);
	}

	// SAF Element
	else if (!pCurLASeRElement && pCurSAFElement)
	{
		UINT8 uChildrenSize	= pCurSAFElement->GetChildrenSize();

		/************************************************************************/
		// if the current element has one more children
		if (uChildrenSize > SceneTreeMgr.GetIndex())
		{
			/************************************************************************/
			// if the current element has a LASeR child
			UINT8 uCurSAFElemetType = pCurSAFElement->GetElementType();

			if (uCurSAFElemetType == SAF_SceneHeader)
			{
				CSceneHeader *pTemp = (CSceneHeader *)pCurSAFElement;

				*pLASeRElement		= (CLASeR *)pTemp->GetChildren();

				pCurLASeRElement	= *pLASeRElement;
				pCurSAFElement		= NULL;
			}

			else if (uCurSAFElemetType == SAF_SceneUnit)
			{
				CSceneUnit *pTemp	= (CSceneUnit *)pCurSAFElement;

				// to be modified
				*pLASeRElement		= (CLASeR *)pTemp->GetChildren(SceneTreeMgr.GetIndex());

				pCurLASeRElement	= *pLASeRElement;
				pCurSAFElement		= NULL;
			}
			/************************************************************************/

			// normal case
			else
			{
				deque<CSAF *> apChildren = pCurSAFElement->GetChildren();

				*pSAFElement		= apChildren[SceneTreeMgr.GetIndex()];

				pCurLASeRElement	= NULL;
				pCurSAFElement		= *pSAFElement;
			}
		}

		/************************************************************************/
		// if the current element has no child and a cousin
		else
		{
			err = SceneTreeMgr.GetCousin(pCurSAFElement, pSAFElement);
			CONFIRM(err);

			if (*pSAFElement)
			{
				SceneTreeMgr.SetIndex(0);			// because parent node was changed

				pCurLASeRElement	= NULL;
				pCurSAFElement		= *pSAFElement;
			}

			/************************************************************************/
			// if the current element has no child and cousin
			else
			{
				// save current element's index
				SceneTreeMgr.SetIndex(pCurSAFElement->GetIndex() + 1);

				pCurLASeRElement	= NULL;
				pCurSAFElement		= pCurSAFElement->GetParent();
			}
		}

		SceneTreeMgr.SetCurLASeRElement(pCurLASeRElement);
		SceneTreeMgr.SetCurSAFElement(pCurSAFElement);
	}

	else
		err = LASeRSceneTreeErr;

	if (((*pLASeRElement == NULL) && (*pSAFElement == NULL))
		|| ((*pLASeRElement != NULL) && ((*pLASeRElement)->GetActivation())))
		GetNextObject(pLASeRElement, pSAFElement);

	return err;
}

/************************************************************************/
// GetCousion
// Desc		: Get cousin
// Param	: Current LASeR element, cousin element
// History
//	PSH (2008. 7. 24)	- first release
/************************************************************************/
lsrErr CSceneTreeManager::GetCousin(CLASeR *pCurLASeRElement, CLASeR **pLASeRElement)
{
	lsrErr err = LASeRNoErr;

	CLASeR	*pLASeRParent = NULL;

	*pLASeRElement = NULL;

	UINT8 uCurLASeRElementType = pCurLASeRElement->GetParentElementType();

	// if parent is SAF element
	if (uCurLASeRElementType >= SAF_SAFSession)
	{
		if ((uCurLASeRElementType >= LSR_StringIDTable) && (uCurLASeRElementType <= LSR_ReleaseResource))
		{
			CLASeRUnit	*pTemp		= (CLASeRUnit *)pCurLASeRElement;
			CSceneUnit	*pSAFParent	= (CSceneUnit *)pTemp->GetParent();

			if (pSAFParent->GetChildSize() > (UINT8)(pCurLASeRElement->GetIndex() + 1))
				// current + 1 = cousin
				*pLASeRElement = pSAFParent->GetChildren(pCurLASeRElement->GetIndex() + 1);
			// If the 'pLASeRElement' is null pointer, current LASeR element has no cousin.
		}
	}

	// if parent is LASeR element
	else
	{
		pLASeRParent = pCurLASeRElement->GetParent();

		// +1 : difference size to index
		if (pLASeRParent->GetChildrenSize() > pCurLASeRElement->GetIndex() + 1)
			// current + 1 = cousin
			*pLASeRElement = pLASeRParent->GetChildren(pCurLASeRElement->GetIndex() + 1);
		// If the 'pLASeRElement' is null pointer, current LASeR element has no cousin.
	}

	return err;
}

/************************************************************************/
// GetCousion
// Desc		: Get cousin
// Param	: Current SAF element, cousin element
// History
//	PSH (2008. 7. 24)	- first release
/************************************************************************/
lsrErr CSceneTreeManager::GetCousin(CSAF *pCurSAFElement, CSAF **pSAFElement)
{
	lsrErr err = LASeRNoErr;

	*pSAFElement = NULL;

	CSAF *pSAFParent = pCurSAFElement->GetParent();

	// +1 : difference size to index
	if (pSAFParent->GetChildrenSize() > (UINT)(pCurSAFElement->GetIndex() + 1))
		// current + 1 = cousin
		*pSAFElement = pSAFParent->GetChildren(pCurSAFElement->GetIndex() + 1);
	// If the 'pSAFElement' is null pointer, current LASeR element has no cousin.

	return err;
}

/************************************************************************/
// SearchLASeRElementUsingID
// Desc			: Search needed element & return the LASeR element
// Param		: element ID string
// Return value	: if found, return found pointer. if not found, return NULL pointer.
// History
//	PSH (2008. 8. 21)	- first release
/************************************************************************/
extern "C" _declspec(dllexport)
CLASeR * __stdcall SearchLASeRElementUsingID(tstring strID)
{
	deque<CLASeR *>::iterator i;

	for (i = SceneTreeMgr.m_apNamedLASeRElement.begin();
		i < SceneTreeMgr.m_apNamedLASeRElement.end(); ++i)
	{
		if ((*i)->GetID() == strID)
			break;
	}

	if (i == SceneTreeMgr.m_apNamedLASeRElement.end())
		return NULL;

	else
		return (CLASeR *)*i;
}

/************************************************************************/
// AddLASeRElementID
// Desc			: add LASeR element pointer having ID
// Param		: element pointer
// History
//	PSH (2009. 2. 15)	- first release
/************************************************************************/
extern "C" _declspec(dllexport)
lsrErr __stdcall AddLASeRElementID(CLASeR *pNew)
{
	if (!pNew)
		return LASeRBadParamErr;

	SceneTreeMgr.m_apNamedLASeRElement.push_back(pNew);

	return LASeRNoErr;
}

/************************************************************************/
// SearchSAFElementUsingID
// Desc			: Search needed element & return the SAF element
// Param		: element ID string
// Return value	: if found, return found pointer. if not found, return NULL pointer.
// History
//	PSH (2008. 8. 21)	- first release
/************************************************************************/
extern "C" _declspec(dllexport)
lsrErr __stdcall DeleteLASeRElementID(CLASeR *pElement)
{
	if (!pElement)
		return LASeRBadParamErr;

	deque<CLASeR *>::iterator iterElement = find(SceneTreeMgr.m_apNamedLASeRElement.begin(),
		SceneTreeMgr.m_apNamedLASeRElement.end(),
		pElement);

	if (iterElement == SceneTreeMgr.m_apNamedLASeRElement.end())
		return LASeRSceneTreeErr;

	SceneTreeMgr.m_apNamedLASeRElement.erase(iterElement);

	return LASeRNoErr;
}

/************************************************************************/
// SearchSAFElementUsingID
// Desc			: Search needed element & return the SAF element
// Param		: element ID string
// Return value	: if found, return found pointer. if not found, return NULL pointer.
// History
//	PSH (2008. 8. 21)	- first release
/************************************************************************/
extern "C" _declspec(dllexport)
CSAF * __stdcall SearchSAFElementUsingID(tstring strID)
{
	deque<CESHeader *>::iterator i;

	for (i = SceneTreeMgr.m_apNamedSAFElement.begin();
		i < SceneTreeMgr.m_apNamedSAFElement.end(); ++i)
	{
		if ((*i)->GetStreamID() == strID)
			break;
	}

	if (i == SceneTreeMgr.m_apNamedSAFElement.end())
		return NULL;

	else
		return (CSAF *)*i;
}

/************************************************************************/
// SearchConditional
// Desc			: Search wanted conditional element
// Param		: conditional begin string
// Return value	: if found, return found pointer. if not found, return NULL pointer.
// History
//	PSH (2008. 8. 21)	- first release
/************************************************************************/
extern "C" _declspec(dllexport)
CLSRConditional * __stdcall SearchConditional(tstring strValue)
{
	lsrErr err = LASeRNoErr;

	deque<tstring> astrTime;
	deque<CLSRConditional *>::iterator i;

	for (i = SceneTreeMgr.m_apConditionalElement.begin();
		i < SceneTreeMgr.m_apConditionalElement.end(); ++i)
	{
		astrTime = (*i)->GetStrListAttr(ATTR_BEGIN, &err);

		if (err != LASeRNoErr)
			return NULL;

		if (find(astrTime.begin(), astrTime.end(), strValue) != astrTime.end())
			break;
	}

	if (i == SceneTreeMgr.m_apConditionalElement.end())
		return NULL;

	else
		return *i;
}

/************************************************************************/
// ProcessCommand
// Desc			: Process command
// Param		: target LASeR element, target element's attribute name, value, command type
// History
//	PSH (2008. 8. 25)	- first release
/************************************************************************/
extern "C" _declspec(dllexport)
lsrErr __stdcall ProcessCommand(CLASeR *pCommand)
{
	lsrErr err = LASeRNoErr;

	UINT uCommandType = pCommand->GetElementType();

	/************************************************************************/
	// ref
	// get the pointer of the element on which addition will be applied
	CLASeR *pElement = SearchLASeRElementUsingID(pCommand->GetStrAttr(ATTR_REF, &err));
	CONFIRM(err);
	
	switch (uCommandType)
	{
	case LSR_Add:
		{
			// get attribute type using it's name
			UINT		uTempAttrType		= pElement->GetAttrType(pCommand->GetStrAttr(ATTR_ATTRIBUTENAME, &err));
			CONFIRM(err);

			// get attribute value type using it's type
			UINT8		uTempAttrValueType	= pElement->GetAttrValType(uTempAttrType);

			/************************************************************************/
			// value
			// get id of the element from which the added value taken
			tstring	strOperandID	= pCommand->GetStrAttr(ATTR_OPERANDELEMENTID, &err);
			CONFIRM(err);

			// the added value
			tstring strValue = _T("");

			if (strOperandID == _T(""))
			{
				strValue = pCommand->GetStrAttr(ATTR_VALUE, &err);
				CONFIRM(err);
			}

			/************************************************************************/
			// operandElementID
			else
			{
				// get the pointer of the operand element
				CLASeR	*pOperand			= SearchLASeRElementUsingID(strOperandID);
				CONFIRM(err);

				// get the attribute name
				tstring	strOperandAttrName	= pCommand->GetStrAttr(ATTR_OPERANDATTRIBUTE, &err);
				CONFIRM(err);

				UINT	uOperandAttrType	= pOperand->GetAttrType(strOperandAttrName);

				strValue = pOperand->GetStrAttr(uOperandAttrType, &err);
				CONFIRM(err);
			}
			/************************************************************************/

			char szBuf[11];
			memset(szBuf, 0, 11);

			switch (uTempAttrValueType)
			{
			case FLOAT_TYPE:
				{
					float fTemp = pElement->GetFloatAttr(uTempAttrType, &err);
					CONFIRM(err);

					fTemp += (float)_tstof(strValue.c_str());

					pElement->SetAttr(uTempAttrType, fTemp);
				}
				break;

			case UINT32_TYPE:
				{
					UINT uTemp32 = pElement->GetUINTAttr(uTempAttrType, &err);
					CONFIRM(err);

					uTemp32 += (UINT)_tstoi(strValue.c_str());

					pElement->SetAttr(uTempAttrType, (float)uTemp32);
				}
				break;

			case STRING_TYPE:
				{
					tstring strTemp = pElement->GetStrAttr(uTempAttrType, &err);
					CONFIRM(err);

					strTemp += strValue;

					pElement->SetAttr(uTempAttrType, strTemp);
				}
				break;

			case FLOAT_LIST_TYPE:
				{
					switch (uTempAttrType)
					{
					case ATTR_SIZE:
						{
							deque<float>	afTarget	= pElement->GetFloatListAttr(uTempAttrType, &err);
							CONFIRM(err);

							// the added value
							const TCHAR		*szValue	= (pCommand->GetStrAttr(ATTR_VALUE, &err)).c_str();
							CONFIRM(err);

							deque<float>	afTemp		= ReadFloatListType(szValue, ' ', &err);
							CONFIRM(err);

							afTarget[0]	+= afTemp[0];
							afTarget[1] += afTemp[1];

// 							tstring strTemp = _gcvt(afTarget[0], 10, szBuf);
// 							memset(szBuf, 0, 11);
// 
// 							// blank
// 							strTemp += _T(" ");
// 
// 							strTemp += _gcvt(afTarget[1], 10, szBuf);
// 
// 							pElement->SetAttr(uTempAttrType, strTemp);
						}
						break;

					default:
						err = LASeRAddErr;
						break;
					}
				}

			default:
				break;
			}
		}

		break;

	case LSR_Clean:
		break;

	case LSR_Delete:
		{
			/************************************************************************/
			// index
			// if index existed
			if (((int)(pCommand->GetUINTAttr(ATTR_INDEX, &err)) != -1) && (err == LASeRNoErr))
			{
				UINT uIndex = pElement->GetUINTAttr(ATTR_INDEX, &err);
				CONFIRM(err);

				/************************************************************************/
				// attributeName
				// default "children"
				// delete pElement's child using index
				if ((pCommand->GetStrAttr(ATTR_ATTRIBUTENAME, &err) == _T("children")) && (err == LASeRNoErr))
				{
					if (pElement->GetChildren((UINT8)uIndex)->CheckHavingID())
						SceneTreeMgr.m_apNamedLASeRElement.erase(
							find(SceneTreeMgr.m_apNamedLASeRElement.begin(),
							SceneTreeMgr.m_apNamedLASeRElement.end(),
							pElement->GetChildren((UINT8)uIndex)));

					pElement->DeleteChild((UINT8)uIndex);
				}

				// delete attribute's n-th value
				else
				{
					UINT	uTempAttrType		= pElement->GetAttrType(pCommand->GetStrAttr(ATTR_ATTRIBUTENAME, &err));
					CONFIRM(err);

					UINT	uTempAttrValType	= pElement->GetAttrValType(uTempAttrType);
					CONFIRM(err);

					tstring	strTemp				= pCommand->GetStrAttr(ATTR_VALUE, &err);
					CONFIRM(err);

					char szBuf[11];
					memset(szBuf, 0, 11);

					if (uTempAttrValType == UINT8_LIST_TYPE)
					{
						deque<UINT8> auTemp = pElement->GetUINT8ListAttr(uTempAttrType, &err);
						CONFIRM(err);

						deque<UINT8>::iterator i = auTemp.begin();
						advance(i, uIndex - 1);
						auTemp.erase(i);

						strTemp = _T("");

						for (i = auTemp.begin(); i < auTemp.end(); ++i)
						{
// 							_itot(*i, szBuf, 10);
// 							strTemp += szBuf + ' ';
						}
					}

					else if (uTempAttrValType == FLOAT_LIST_TYPE)
					{
						deque<float> afTemp = pElement->GetFloatListAttr(uTempAttrType, &err);
						CONFIRM(err);

						deque<float>::iterator i = afTemp.begin();
						advance(i, uIndex - 1);
						afTemp.erase(i);

						strTemp = _T("");

						for (i = afTemp.begin(); i < afTemp.end(); ++i)
						{
// 							_gcvt(*i, 10, szBuf);
// 							strTemp += szBuf + ' ';
						}
					}

					pElement->SetAttr(uTempAttrType, strTemp);
				}
			}

			// if index is not existed
			else
			{
				// delete ref (I)
				if ((pCommand->GetStrAttr(ATTR_ATTRIBUTENAME, &err) == _T("children")) && (err == LASeRNoErr))
				{
					CLASeR *pParent = pElement->GetParent();

					if (pElement->CheckHavingID())
						SceneTreeMgr.m_apNamedLASeRElement.erase(
						find(SceneTreeMgr.m_apNamedLASeRElement.begin(),
						SceneTreeMgr.m_apNamedLASeRElement.end(),
						pElement));

					pParent->DeleteChild(pElement->GetIndex());
				}

				// delete all attribute's value
				else
				{
					UINT uTempAttrType = pElement->GetAttrType(pCommand->GetStrAttr(ATTR_ATTRIBUTENAME, &err));
					CONFIRM(err);

					pElement->SetAttr(uTempAttrType, _T(""));
				}
			}
		}

		break;

	case LSR_Insert:
		{
			/************************************************************************/
			// index
			// if index existed
			if (((int)(pCommand->GetUINTAttr(ATTR_INDEX, &err)) != -1) && (err == LASeRNoErr))
			{
				UINT uIndex = pElement->GetUINTAttr(ATTR_INDEX, &err);
				CONFIRM(err);

				/************************************************************************/
				// attributeName
				// default "children"
				// insert child
				if ((pCommand->GetStrAttr(ATTR_ATTRIBUTENAME, &err) == _T("children")) && (err == LASeRNoErr))
				{
					// pCommand->GetChildren(0) : insert command's child
					if (pCommand->GetChildren(0)->CheckHavingID())
						SceneTreeMgr.m_apNamedLASeRElement.push_back(pCommand->GetChildren(0));

					pElement->InsertChild(uIndex, pCommand->GetChildren(0));
				}

				// insert attribute's n-th value
				else
				{
					UINT	uTempAttrType		= pElement->GetAttrType(pCommand->GetStrAttr(ATTR_ATTRIBUTENAME, &err));
					CONFIRM(err);

					UINT	uTempAttrValType	= pElement->GetAttrValType(uTempAttrType);
					CONFIRM(err);

					tstring	strTemp				= pCommand->GetStrAttr(ATTR_VALUE, &err);
					CONFIRM(err);

					char szBuf[11];
					memset(szBuf, 0, 11);

					if (uTempAttrValType == UINT8_LIST_TYPE)
					{
						deque<UINT8> auTemp = pElement->GetUINT8ListAttr(uTempAttrType, &err);
						CONFIRM(err);

						deque<UINT8>::iterator i = auTemp.begin();
						advance(i, uIndex - 1);
						auTemp.insert(i, (UINT8)_tstoi(strTemp.c_str()));

						strTemp = _T("");

						for (i = auTemp.begin(); i < auTemp.end(); ++i)
						{
// 							_itot(*i, szBuf, 10);
// 							strTemp += szBuf + ' ';
						}
					}

					else if (uTempAttrValType == FLOAT_LIST_TYPE)
					{
						deque<float> afTemp = pElement->GetFloatListAttr(uTempAttrType, &err);
						CONFIRM(err);

						deque<float>::iterator i = afTemp.begin();
						advance(i, uIndex - 1);
						afTemp.insert(i, (float)_tstof(strTemp.c_str()));

						strTemp = _T("");

						for (i = afTemp.begin(); i < afTemp.end(); ++i)
						{
// 							_gcvt(*i, 10, szBuf);
// 							strTemp += szBuf + ' ';
						}
					}

					pElement->SetAttr(uTempAttrType, strTemp);
				}
			}

			// if index is not existed
			else
			{
				// insert child at the end of children list
				if ((pCommand->GetStrAttr(ATTR_ATTRIBUTENAME, &err) == _T("children")) && (err == LASeRNoErr))
				{
					if (pCommand->GetChildren(0)->CheckHavingID())
						SceneTreeMgr.m_apNamedLASeRElement.push_back(pCommand->GetChildren(0));

					// why 256? Maximum children : 255
					pElement->InsertChild(256, pCommand->GetChildren(0));
				}

				// insert value at the end of attribute's value
				else
				{
					UINT	uTempAttrType		= pElement->GetAttrType(pCommand->GetStrAttr(ATTR_ATTRIBUTENAME, &err));
					CONFIRM(err);

					UINT	uTempAttrValType	= pElement->GetAttrValType(uTempAttrType);
					CONFIRM(err);

					tstring	strTemp				= pCommand->GetStrAttr(ATTR_VALUE, &err);
					CONFIRM(err);

					char szBuf[11];
					memset(szBuf, 0, 11);

					if (uTempAttrValType == UINT8_LIST_TYPE)
					{
						deque<UINT8> auTemp = pElement->GetUINT8ListAttr(uTempAttrType, &err);
						CONFIRM(err);

						deque<UINT8>::iterator i = auTemp.begin();
						auTemp.push_back((UINT8)_tstoi(strTemp.c_str()));

						strTemp = _T("");

						for (deque<UINT8>::iterator i = auTemp.begin(); i < auTemp.end(); ++i)
						{
// 							_itot(*i, szBuf, 10);
// 							strTemp += szBuf + ' ';
						}
					}

					else if (uTempAttrValType == FLOAT_LIST_TYPE)
					{
						deque<float> afTemp = pElement->GetFloatListAttr(uTempAttrType, &err);
						CONFIRM(err);

						deque<float>::iterator i = afTemp.begin();
						afTemp.push_back((float)_tstof(strTemp.c_str()));

						strTemp = _T("");

						for (deque<float>::iterator i = afTemp.begin(); i < afTemp.end(); ++i)
						{
// 							_gcvt(*i, 10, szBuf);
// 							strTemp += szBuf + ' ';
						}
					}

					pElement->SetAttr(uTempAttrType, strTemp);
				}
			}
		}

		break;

	case LSR_Replace:
		{
			/************************************************************************/
			// index
			// if index existed
			if (((int)(pCommand->GetUINTAttr(ATTR_INDEX, &err)) != -1) && (err == LASeRNoErr))
			{
				UINT uIndex = pElement->GetUINTAttr(ATTR_INDEX, &err);
				CONFIRM(err);

				/************************************************************************/
				// attributeName
				// default "children"
				// replace child
				if ((pCommand->GetStrAttr(ATTR_ATTRIBUTENAME, &err) == _T("children")) && (err == LASeRNoErr))
					// pCommand->GetChildren(0) : replace command's child
					pElement->ReplaceChild(uIndex, pCommand->GetChildren(0));

				else
				{
					/************************************************************************/
					// operandElementID
					tstring	strOperandID	= pCommand->GetStrAttr(ATTR_OPERANDELEMENTID, &err);
					CONFIRM(err);

					UINT	uTempAttrType	= pElement->GetAttrType(pCommand->GetStrAttr(ATTR_ATTRIBUTENAME, &err));
					CONFIRM(err);

					if (strOperandID != _T(""))
					{
						// get the pointer of the operand element
						CLASeR	*pOperand			= SearchLASeRElementUsingID(strOperandID);
						CONFIRM(err);

						// get the attribute name
						tstring	strOperandAttrName	= pCommand->GetStrAttr(ATTR_OPERANDATTRIBUTE, &err);
						CONFIRM(err);

						UINT	uOperandAttrType	= pOperand->GetAttrType(strOperandAttrName);

						tstring strTemp				= pOperand->GetStrAttr(uOperandAttrType, &err);
						CONFIRM(err);

						pElement->GetChildren((UINT8)uIndex)->SetAttr(uTempAttrType, strTemp);
						CONFIRM(err);
					}

					else
					{
						pElement->GetChildren((UINT8)uIndex)->SetAttr(uTempAttrType, pCommand->GetStrAttr(ATTR_VALUE, &err));
						CONFIRM(err);
					}
				}
			}

			else
			{
				// replace 'ref'
				CLASeR *pParent = pElement->GetParent();

				/************************************************************************/
				// attributeName
				// default "children"
				// replace child
				if ((pCommand->GetStrAttr(ATTR_ATTRIBUTENAME, &err) == _T("children")) && (err == LASeRNoErr))
					// pCommand->GetChildren(0) : replace command's child
					pParent->ReplaceChild(pElement->GetIndex(), pCommand->GetChildren(0));

				else
				{
					/************************************************************************/
					// operandElementID
					tstring	strOperandID	= pCommand->GetStrAttr(ATTR_OPERANDELEMENTID, &err);
					CONFIRM(err);

					UINT	uTempAttrType	= pElement->GetAttrType(pCommand->GetStrAttr(ATTR_ATTRIBUTENAME, &err));
					CONFIRM(err);

					if (strOperandID != _T(""))
					{
						// get the pointer of the operand element
						CLASeR	*pOperand			= SearchLASeRElementUsingID(strOperandID);
						CONFIRM(err);

						// get the attribute name
						tstring	strOperandAttrName	= pCommand->GetStrAttr(ATTR_OPERANDATTRIBUTE, &err);
						CONFIRM(err);

						UINT	uOperandAttrType	= pOperand->GetAttrType(strOperandAttrName);

						tstring strTemp				= pOperand->GetStrAttr(uOperandAttrType, &err);
						CONFIRM(err);

						pElement->SetAttr(uTempAttrType, strTemp);
						CONFIRM(err);
					}

					else
					{
						pElement->SetAttr(uTempAttrType, pCommand->GetStrAttr(ATTR_VALUE, &err));
						CONFIRM(err);
					}
				}
			}
		}

		break;

	case LSR_Restore:
		break;

	case LSR_Save:
		break;

	case LSR_SendEvent:
		break;

	case LSR_Activate:
		pElement->SetActivate();
		break;

	case LSR_Deactivate:
		pElement->SetDeactivate();
		break;

	case LSR_ReleaseResource:
		break;

	default:
		err = LASeRBadParamErr;
		break;
	}

	return err;
}