///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include <io.h>
#include <fstream>

#include "XMLFileManager.h"
#include "SceneCreator.h"


CXMLFileManager::CXMLFileManager()
{
	m_pReader = new XNode;
}

CXMLFileManager::~CXMLFileManager()
{
	m_pReader->Close();

	DELETEMEMORY(m_pReader);
}

/************************************************************************/
// OpenXMLFile
// Desc		: Get XML file name and call LASeREncoder class' 'CreateScene' function
// Param	: XML file name
// History
//	PSH (2008. 7. 14)	- first release
/************************************************************************/
extern "C" _declspec(dllexport)
lsrErr __stdcall OpenXMLFile(const TCHAR *szFileName, void **pRoot)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// parsing XML
	CXMLFileManager *pFileMgr = new CXMLFileManager();

	if (!pFileMgr)
	{
		err = LASeRNoMoreMemoryErr;
		return err;
	}

	err = pFileMgr->ParseXML(szFileName);
	CONFIRM(err);
	/************************************************************************/

	CSceneCreator *pLASeREncoder = new CSceneCreator();

	if (!pLASeREncoder)
	{
		err = LASeRNoMoreMemoryErr;
		return err;
	}

	// convert DOM tree to scene tree
	err		= pLASeREncoder->CreateScene(pFileMgr->GetDoc());
	*pRoot	= pLASeREncoder->GetScene();

	delete pLASeREncoder;
	pLASeREncoder = NULL;

	delete pFileMgr;
	pFileMgr = NULL;

	return err;
}

/************************************************************************/
// ParseXML
// Desc		: Open and parse XML file
// Param	: XML file name
// History
//	PSH (2008. 7. 14)	- first release
/************************************************************************/
lsrErr CXMLFileManager::ParseXML(const TCHAR *szFileName)
{
	lsrErr err = LASeRNoErr;

	// for checking file existence
	err = (lsrErr)_taccess(szFileName, 4);
	CONFIRM(err);

	ifstream	file(szFileName, ios::in|ios::binary);
	UINT		uFileSize	= 0;
	TCHAR		*szFileData	= NULL;

	// move to file's end
	file.seekg(-1, ios_base::end);

	// get file size
	uFileSize = file.tellg() / sizeof(TCHAR) + 1;

	szFileData = new TCHAR[uFileSize];

	if (!szFileData)
	{
		err = LASeRNoMoreMemoryErr;
		return err;
	}

	memset(szFileData, 0, uFileSize);

	// move to file's start point to read correctly
	file.seekg(0, ios_base::beg);
	file.read(szFileData, uFileSize);
	file.close();

	if (!(m_pReader->Load(szFileData)))
		err = LASeRFileIOErr;

	// remove comment in XML
	else
	{
		LPXNode	pFound			= NULL;
		LPXNode	pFoundsParent	= NULL;

		while (1)
		{
			pFound = m_pReader->Find(_T("#COMMENT"));

			if (pFound)
			{
				// abnormal
				pFoundsParent	= pFound->parent->parent;

				pFoundsParent->RemoveChild(pFound);
				pFound = pFoundsParent = NULL;
			}

			else
				break;
		}
	}
	
	delete [] szFileData;
	szFileData = NULL;

	return err;
}