///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#ifndef _COMPLEXTYPE_H_
#define _COMPLEXTYPE_H_

#include "base.h"

/************************************************************************/
// SVG ComplexType
class CXMLBase
{
public:
	tstring	m_strBase;
	UINT8	m_uSpace;
	tstring	m_strLang;
};

class CSVGObject : public CLASeR
{
public:
	tstring		m_strID;
	CXMLBase	*m_pXMLBase;
	tstring		m_strClass;
	tstring		m_strStyle;

	CSVGObject();
	virtual ~CSVGObject();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);

	tstring	GetID() const			{return m_strID;}
	void	SetID(tstring strNewID);

	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);

	UINT	GetAttrType(tstring strAttrName);
	UINT8	GetAttrValType(UINT uAttrType);
};

class CSVGProperties
{
public:
	float		m_fAudioLevel;				// zero to one
	deque<UINT8>	m_auColor;
	UINT8		m_uColorRendering;
	UINT8		m_uDisplay;
	UINT8		m_uDisplayAlign;
	deque<UINT8>	m_auFill;
	tstring		m_strFillUrl;
	float		m_fFillOpacity;				// zero to one
	UINT8		m_uFillRule;
	tstring		m_strFontFamily;
	float		m_fFontSize;
	UINT8		m_uFontStyle;
	UINT8		m_uTextDecoration;
	UINT8		m_uFontWeight;
	UINT8		m_uFontVariant;
	UINT8		m_uImageRendering;
	float		m_fLineIncrement;
	UINT8		m_uPointerEvents;
	UINT8		m_uShapeRendering;
	deque<UINT8>	m_auSolidColor;
	float		m_fSolidOpacity;			// zero to one
	deque<UINT8>	m_auStopColor;
	float		m_fStopOpacity;				// zero to one
	deque<UINT8>	m_auStroke;
	tstring		m_strStrokeUrl;
	deque<float>	m_afStrokeDashArray;
	float		m_fStrokeDashOffset;
	UINT8		m_uStrokeDashOffsetUnit;
	UINT8		m_uStrokeLineCap;
	UINT8		m_uStrokeLineJoin;
	float		m_fStrokeMiterLimit;
	float		m_fStrokeOpacity;			// zero to one
	float		m_fStrokeWidth;
	UINT8		m_uStrokeWidthUnit;
	UINT8		m_uTextAnchor;
	UINT8		m_uTextRendering;
	deque<UINT8>	m_auViewportFill;
	float		m_fViewportFillOpacity;		// zero to one
	UINT8		m_uVectorEffect;
	UINT8		m_uVisibility;

	CSVGProperties();
	~CSVGProperties();

	lsrErr		SetAttrs(LPXNode pScene, CLASeR *pI);

	float		GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8		GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring		GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT		GetAttrType(tstring strAttrName);
	UINT8		GetAttrValType(UINT uAttrType);

	lsrErr		SetAttr(UINT uAttrType, tstring strValue);
	lsrErr		SetAttr(UINT uAttrType, float fValue);
};

class CSVGMediaProperties
{
public:
	float		m_fAudioLevel;					// zero to one
	UINT8		m_uDisplay;
	UINT8		m_uImageRendering;
	UINT8		m_uPointerEvents;
	UINT8		m_uShapeRendering;
	UINT8		m_uTextRendering;
	deque<UINT8>	m_auViewportFill;
	float			m_fViewportFillOpacity;			// zero to one
	UINT8			m_uVisibility;

	CSVGMediaProperties();
	~CSVGMediaProperties();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CSVGBasic
{
public:
	deque<UINT8>	m_auRequiredFeatures;
	tstring			m_strRequiredExtensions;
	tstring			m_strSystemLanguage;
	tstring			m_strRequiredFormats;
	tstring			m_strRequiredFonts;

	CSVGBasic();
	~CSVGBasic();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);

	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);
};

class CSVGLocatable
{
public:
	deque<float>	m_afRotation;
	deque<float>	m_afScale;
	deque<float>	m_afTranslation;
	deque<float>	m_afMatrix;
	tstring			m_strTransform;

	CSVGLocatable();
	~CSVGLocatable();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);

	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CSVGHref
{
public:
	tstring	m_strHref;
	tstring	m_strTitle;

	UINT8	m_uType;
	tstring	m_strRole;
	tstring	m_strArcrole;
	UINT8	m_uActuate;
	UINT8	m_uShow;

	CSVGHref();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);

	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);

	UINT	GetAttrType(tstring strAttrName);
	UINT8	GetAttrValType(UINT uAttrType);

	lsrErr	SetAttr(UINT uAttrType, tstring strValue);
};

class CSVGFocus
{
public:
	tstring	m_strNavRight;
	tstring	m_strNavNext;
	tstring	m_strNavUp;
	tstring	m_strNavUpRight;
	tstring	m_strNavUpLeft;
	tstring	m_strNavPrev;
	tstring	m_strNavDown;
	tstring	m_strNavDownRight;
	tstring	m_strNavDownLeft;
	tstring	m_strNavLeft;
	UINT8	m_uFocusable;

	CSVGFocus();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);

	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);

	UINT	GetAttrType(tstring strAttrName);
	UINT8	GetAttrValType(UINT uAttrType);

	lsrErr	SetAttr(UINT uAttrType, tstring strValue);
};

class CSVGAnimateAttr
{
public:
	tstring			m_strFrom;
	deque<float>	m_afFrom;
	deque<float>	m_afBy;
	deque<float>	m_afValues;
	UINT8			m_uCalcMode;
	deque<float>	m_afKeyTimes;
	tstring			m_strKeySplines;
	UINT8			m_uAdditive;
	UINT8			m_uAccumulate;

	CSVGAnimateAttr();
	~CSVGAnimateAttr();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);

	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);
};

class CSVGSetAttr
{
public:
	UINT8			m_uEnabled;
	deque<tstring>	m_astrBegin;
	deque<tstring>	m_astrEnd;
	float			m_fDur;
	float			m_fMin;
	float			m_fMax;
	tstring			m_strAttributeType;
	float			m_fRepeatCount;
	float			m_fRepeatDur;
	UINT8			m_uRestart;
	UINT8			m_uFill;
	deque<float>	m_afTo;

	CSVGSetAttr();
	~CSVGSetAttr();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);
	deque<tstring>	GetStrListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);
};

/************************************************************************/
// LASeR amd2 ComplexType
class CLSRAdaptationAttr
{
protected:
	UINT			m_uMinDisplaySize;
	deque<float>	m_afMinMemory;
	UINT8			m_uPriority;
	UINT			m_uMinResolution;
	UINT			m_uMinSize;
	UINT8			m_uMinSizeBehavior;
	tstring			m_strConstraintFilter;

public:
	CLSRAdaptationAttr();
	~CLSRAdaptationAttr();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);

	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	UINT			GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);
};

#endif