///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "laser_error.h"
#include "Datatypes.h"

#include <algorithm>


UINT8 ReadBooleanType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("false")))		return 0;
	else if (!_tcscmp(szValue, _T("true")))	return 1;
	else	*pErr = LASeRValidationErr;		return 0;
}

deque<UINT8> ReadInt8ListType(const TCHAR *szValue, const TCHAR chUnit, lsrErr *pErr)
{
	deque<UINT8> auList;

	if (szValue == NULL)
	{
		*pErr = LASeRValidationErr;
		return auList;
	}

	// position of number's beginning
	TCHAR *pBegin = _tcschr((TCHAR *)szValue, '(');

	if (pBegin)
		++pBegin;

	else
		pBegin = (TCHAR *)szValue;

	TCHAR *pUnit = NULL;						// position of blank

	while (1)
	{
		pUnit = _tcschr(pBegin, chUnit);		// missing processing null character

		auList.push_back((UINT8)_tcstoi64(pBegin, &pUnit, 10));

		pBegin = pUnit + 1;

		if (*pUnit != chUnit)
			break;
	}

	return auList;
}

deque<UINT> ReadIntListType(const TCHAR *szValue, lsrErr *pErr)
{
	deque<UINT> auList;

	if (szValue == NULL)
	{
		*pErr = LASeRValidationErr;
		return auList;
	}

	// position of number's beginning
	TCHAR *pBegin = _tcschr((TCHAR *)szValue, '(');

	if (pBegin)
		++pBegin;

	else
		pBegin = (TCHAR *)szValue;

	TCHAR	*pBlank	= NULL;						// position of blank
	TCHAR	*pComma	= NULL;						// position of comma
	TCHAR	*pToken	= NULL;

	while (1)
	{
		pBlank = _tcschr(pBegin, ' ');
		pComma = _tcschr(pBegin, ',');

		/************************************************************************/
		if ((pBlank != NULL) && (pComma == NULL))
			pToken = pBlank;

		else if ((pBlank == NULL) && (pComma != NULL))
			pToken = pComma;

		else if ((pBlank != NULL) && (pComma != NULL))
			pToken = (pBlank < pComma) ? pBlank : pComma;
		/************************************************************************/

		auList.push_back((UINT)_tcstoi64(pBegin, &pToken, 10));

		pBegin = pToken + 1;

		if ((*pToken != ' ') && (*pToken != ','))
			break;
	}

	return auList;
}

deque<float> ReadFloatListType(const TCHAR *szValue, lsrErr *pErr)
{
	deque<float> afList;

	if (szValue == NULL)
	{
		*pErr = LASeRValidationErr;
		return afList;
	}

	// position of number's beginning
	TCHAR *pBegin = _tcschr((TCHAR *)szValue, '(');

	if (pBegin)
		++pBegin;

	else
		pBegin = (TCHAR *)szValue;

	TCHAR	*pBlank	= NULL;						// position of blank
	TCHAR	*pComma	= NULL;						// position of comma
	TCHAR	*pToken	= NULL;

	while (1)
	{
		pBlank = _tcschr(pBegin, ' ');
		pComma = _tcschr(pBegin, ',');

		/************************************************************************/
		if ((pBlank != NULL) && (pComma == NULL))
			pToken = pBlank;

		else if ((pBlank == NULL) && (pComma != NULL))
			pToken = pComma;

		else if ((pBlank != NULL) && (pComma != NULL))
			pToken = (pBlank < pComma) ? pBlank : pComma;
		/************************************************************************/

		afList.push_back((float)_tcstod(pBegin, &pToken));

		pBegin = pToken + 1;

		if ((*pToken != ' ') && (*pToken != ','))
			break;
	}

	return afList;
}

deque<float> ReadFloatListType(const TCHAR *szValue, const TCHAR chUnit, lsrErr *pErr)
{
	deque<float> afList;

	if (szValue == NULL)
	{
		*pErr = LASeRValidationErr;
		return afList;
	}

	TCHAR	*pBegin	= (TCHAR *)szValue;			// position of number's beginning
	TCHAR	*pUnit	= NULL;						// position of blank

	while (1)
	{
		pUnit = _tcschr(pBegin, chUnit);		// missing processing null character

		afList.push_back((float)_tcstod(pBegin, &pUnit));

		pBegin = pUnit + 1;

		if (*pUnit != chUnit)
			break;
	}

	return afList;
}

deque<tstring> ReadStringListType(const TCHAR *szValue, const TCHAR chUnit, lsrErr *pErr)
{
	deque<tstring>	astrList;
	tstring			strTemp = _T("");

	if (szValue == NULL)
	{
		*pErr = LASeRValidationErr;
		return astrList;
	}

	const UINT	uLength	= (UINT)_tcslen(szValue);
	TCHAR		*pBegin	= (TCHAR *)&szValue[0];	// position of string's beginning
	TCHAR		*pBlank	= pBegin;				// position of blank

	while (1)
	{
		pBlank	= (TCHAR *)find((const TCHAR *)pBegin, &szValue[uLength], chUnit);

		copy(pBegin, pBlank, back_inserter(strTemp));
		astrList.push_back(strTemp);

		strTemp	= _T("");
		pBegin	= pBlank + 1;

		if (*pBlank != chUnit)
			break;
	}

	return astrList;
}

/************************************************************************/
// ev
UINT8 ReadPropagateType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("continue")))	return 0;
	else if (!_tcscmp(szValue, _T("stop")))	return 1;
	else	*pErr = LASeRValidationErr;		return 0;
}

UINT8 ReadDefaultActionType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("perform")))		return 0;
	else if (!_tcscmp(szValue, _T("cancel")))	return 1;
	else	*pErr = LASeRValidationErr;			return 0;
}

/************************************************************************/
// xml
UINT8 ReadSpaceType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("default")))		return 0;
	else if (!_tcscmp(szValue, _T("preserve")))	return 1;
	else	*pErr = LASeRValidationErr;			return 0;
}

/************************************************************************/
// xlink
UINT8 ReadTypeType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("simple")))		return 0;
	else if (!_tcscmp(szValue, _T("extended")))	return 1;
	else if (!_tcscmp(szValue, _T("locator")))	return 2;
	else if (!_tcscmp(szValue, _T("arc")))		return 3;
	else if (!_tcscmp(szValue, _T("resource")))	return 4;
	else if (!_tcscmp(szValue, _T("title")))	return 5;
	else if (!_tcscmp(szValue, _T("none")))		return 6;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadShowType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("new")))			return 0;
	else if (!_tcscmp(szValue, _T("replace")))	return 1;
	else if (!_tcscmp(szValue, _T("embed")))	return 2;
	else if (!_tcscmp(szValue, _T("other")))	return 3;
	else if (!_tcscmp(szValue, _T("none")))		return 4;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadActuateType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("onLoad")))			return 0;
	else if (!_tcscmp(szValue, _T("onRequest")))	return 1;
	else if (!_tcscmp(szValue, _T("other")))		return 2;
	else if (!_tcscmp(szValue, _T("none")))			return 3;
	else	*pErr = LASeRValidationErr;				return 0;
}

/************************************************************************/
// datatype2
deque<UINT8> ReadColorType(const TCHAR *szValue, lsrErr *pErr)
{
	deque<UINT8> auList;
	auList.assign(3, 0);

	if (szValue == NULL)
	{
		*pErr = LASeRValidationErr;
		return auList;
	}

	if ((_tcslen(szValue) > 4) && (!_tcsnicmp(szValue, _T("rgb("), 4)))
		auList = ReadInt8ListType(&szValue[4], ',', pErr);

	/************************************************************************/
	// if the szValue hasn't 'rgb' string
	else if (!_tcscmp(szValue, _T("blue")))
	{auList[0] = 0;		auList[1] = 0;		auList[2] = 255;}

	else if (!_tcscmp(szValue, _T("cyan")))
	{auList[0] = 0;		auList[1] = 255;	auList[2] = 255;}

	else if (!_tcscmp(szValue, _T("green")))
	{auList[0] = 0;		auList[1] = 255;	auList[2] = 0;}

	else if (!_tcscmp(szValue, _T("magenta")))
	{auList[0] = 255;	auList[1] = 0;		auList[2] = 255;}

	else if (!_tcscmp(szValue, _T("pink")))
	{auList[0] = 255;	auList[1] = 192;	auList[2] = 203;}

	else if (!_tcscmp(szValue, _T("red")))
	{auList[0] = 255;	auList[1] = 0;		auList[2] = 0;}

	else if (!_tcscmp(szValue, _T("yellow")))
	{auList[0] = 255;	auList[1] = 255;	auList[2] = 0;}
	/************************************************************************/

	return auList;
}

UINT8 ReadFontStyleType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("normal")))		return 0;
	else if (!_tcscmp(szValue, _T("italic")))	return 1;
	else if (!_tcscmp(szValue, _T("oblique")))	return 2;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadTextDecorationType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("none")))							return 0;
	else if (!_tcscmp(szValue, _T("underline")))				return 1;
	else if (!_tcscmp(szValue, _T("LASeR_OUTLINE")))			return 3;
	else if (!_tcscmp(szValue, _T("LASeR_EMBOSS")))				return 4;
	else if (!_tcscmp(szValue, _T("LASeR_ENGRAVE")))			return 5;
	else if (!_tcscmp(szValue, _T("LASeR_LEFTDROPSHADOW")))		return 6;
	else if (!_tcscmp(szValue, _T("LASeR_RIGHTDROPSHADOW")))	return 7;
	else	*pErr = LASeRValidationErr;							return 0;
}

UINT8 ReadCalcModeType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("discrete")))		return 0;
	else if (!_tcscmp(szValue, _T("linear")))	return 1;
	else if (!_tcscmp(szValue, _T("paced")))	return 2;
	else if (!_tcscmp(szValue, _T("spline")))	return 3;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadVisibilityType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("visible")))		return 0;
	else if (!_tcscmp(szValue, _T("hidden")))	return 1;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadOverflowType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("visible")))		return 0;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadDisplayType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("inline")))					return 0;
	else if (!_tcscmp(szValue, _T("none")))					return 1;
	else if (!_tcscmp(szValue, _T("block")))				return 3;
	else if (!_tcscmp(szValue, _T("list-item")))			return 4;
	else if (!_tcscmp(szValue, _T("run-in")))				return 5;
	else if (!_tcscmp(szValue, _T("compact")))				return 6;
	else if (!_tcscmp(szValue, _T("marker")))				return 7;
	else if (!_tcscmp(szValue, _T("table")))				return 8;
	else if (!_tcscmp(szValue, _T("inline-table")))			return 9;
	else if (!_tcscmp(szValue, _T("table-row-group")))		return 10;
	else if (!_tcscmp(szValue, _T("table-header-group")))	return 11;
	else if (!_tcscmp(szValue, _T("table-footer-group")))	return 12;
	else if (!_tcscmp(szValue, _T("table-row")))			return 13;
	else if (!_tcscmp(szValue, _T("table-column-group")))	return 14;
	else if (!_tcscmp(szValue, _T("table-column")))			return 15;
	else if (!_tcscmp(szValue, _T("table-cell")))			return 16;
	else if (!_tcscmp(szValue, _T("table-caption")))		return 17;
	else	*pErr = LASeRValidationErr;						return 0;
}

UINT8 ReadTextAnchorType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("start")))			return 0;
	else if (!_tcscmp(szValue, _T("middle")))	return 1;
	else if (!_tcscmp(szValue, _T("end")))		return 2;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadEventType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("focusin")))					return 0;
	else if (!_tcscmp(szValue, _T("focusout")))				return 1;
	else if (!_tcscmp(szValue, _T("activate")))				return 2;
	else if (!_tcscmp(szValue, _T("mousedown")))			return 3;
	else if (!_tcscmp(szValue, _T("mouseup")))				return 4;
	else if (!_tcscmp(szValue, _T("click")))				return 5;
	else if (!_tcscmp(szValue, _T("beginEvent")))			return 6;
	else if (!_tcscmp(szValue, _T("endEvent")))				return 7;
	else if (!_tcscmp(szValue, _T("repeatEvent")))			return 8;
	else if (!_tcscmp(szValue, _T("zoom")))					return 9;
	else if (!_tcscmp(szValue, _T("scroll")))				return 10;
	else if (!_tcscmp(szValue, _T("resize")))				return 11;
	else if (!_tcscmp(szValue, _T("error")))				return 12;
	else if (!_tcscmp(szValue, _T("abort")))				return 13;
	else if (!_tcscmp(szValue, _T("load")))					return 14;
	else if (!_tcscmp(szValue, _T("unload")))				return 15;
	else if (!_tcscmp(szValue, _T("keydown")))				return 16;
	else if (!_tcscmp(szValue, _T("keyup")))				return 17;
	else if (!_tcscmp(szValue, _T("accessKey")))			return 18;
	else if (!_tcscmp(szValue, _T("longAccessKey")))		return 19;
	else if (!_tcscmp(szValue, _T("textinput")))			return 20;
	else if (!_tcscmp(szValue, _T("mouseover")))			return 21;
	else if (!_tcscmp(szValue, _T("mouseout")))				return 22;
	else if (!_tcscmp(szValue, _T("mousemove")))			return 23;
	else if (!_tcscmp(szValue, _T("pause")))				return 24;
	else if (!_tcscmp(szValue, _T("resume")))				return 25;
	else if (!_tcscmp(szValue, _T("pausedEvent")))			return 26;
	else if (!_tcscmp(szValue, _T("resumedEvent")))			return 27;
	else if (!_tcscmp(szValue, _T("activatedEvent")))		return 28;
	else if (!_tcscmp(szValue, _T("deactivatedEvent")))		return 29;
	else if (!_tcscmp(szValue, _T("LASeRConnecting")))		return 30;
	else if (!_tcscmp(szValue, _T("LASeRBuffering")))		return 31;
	else if (!_tcscmp(szValue, _T("LASeREndStream")))		return 32;
	else if (!_tcscmp(szValue, _T("LASeRPlayable")))		return 33;
	else if (!_tcscmp(szValue, _T("executionTime")))		return 34;

	// non IS
	else if (_tcsstr(szValue, _T("displaySizeChanged")))	return 35;
	else if (_tcsstr(szValue, _T("MemoryStatus")))      	return 36;
	else	*pErr = LASeRValidationErr;						return 0;
}

UINT8 ReadFillRuleType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("evenodd")))		return 0;
	else if (!_tcscmp(szValue, _T("nonzero")))	return 1;
	else	*pErr = LASeRValidationErr;			return 0;
}

deque<float> ReadTransformType(const TCHAR *szValue, const TCHAR *szAttr, lsrErr *pErr)
{
	deque<float> afList;

	if (szValue == NULL)
	{
		*pErr = LASeRValidationErr;
		return afList;
	}

	const TCHAR *pRotate = _tcsstr(szValue, szAttr);

	if (pRotate)
	{
		const TCHAR *pRoundBracket	= _tcschr(pRotate + 1, ')');

		TCHAR *pComma	= (TCHAR *)find(pRotate + 1, pRoundBracket, ',');
		TCHAR *pBlank	= (TCHAR *)find(pRotate + 1, pRoundBracket, ' ');

		// comma used
		if (pComma != pRoundBracket)
			// _tcslen(szAttr) : rotate, translate, scale
			afList = ReadFloatListType(pRotate + _tcslen(szAttr) + 1, ',', pErr);

		// blank used
		else if (pBlank != pRoundBracket)
			afList = ReadFloatListType(pRotate + _tcslen(szAttr) + 1, ' ', pErr);

		// only one
		else
			afList.push_back((float)_tcstod(pRotate + _tcslen(szAttr) + 1, (TCHAR **)&pRoundBracket));
	}

	return afList;
}

UINT8 ReadAnimatableType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("opacity")))						return 0;
	else if (!_tcscmp(szValue, _T("choice")))					return 1;
	else if (!_tcscmp(szValue, _T("color-rendering")))			return 2;
	else if (!_tcscmp(szValue, _T("display")))					return 3;
	else if (!_tcscmp(szValue, _T("font-style")))				return 4;
	else if (!_tcscmp(szValue, _T("preserveAspectRatio")))		return 5;
	else if (!_tcscmp(szValue, _T("visibility")))				return 6;
	else if (!_tcscmp(szValue, _T("editable")))					return 7;
	else if (!_tcscmp(szValue, _T("d")))						return 8;
	else if (!_tcscmp(szValue, _T("fill")))						return 9;
	else if (!_tcscmp(szValue, _T("fill-opacity")))				return 10;
	else if (!_tcscmp(szValue, _T("font-family")))				return 11;
	else if (!_tcscmp(szValue, _T("font-size")))				return 12;
	else if (!_tcscmp(szValue, _T("height")))					return 13;
	else if (!_tcscmp(szValue, _T("xlink:href")))				return 14;
	else if (!_tcscmp(szValue, _T("points")))					return 15;
	else if (!_tcscmp(szValue, _T("stroke")))					return 16;
	else if (!_tcscmp(szValue, _T("stroke-opacity")))			return 17;
	else if (!_tcscmp(szValue, _T("stroke-width")))				return 18;
	else if (!_tcscmp(szValue, _T("width")))					return 19;
	else if (!_tcscmp(szValue, _T("viewBox")))					return 20;
	else if (!_tcscmp(szValue, _T("x1")))						return 21;
	else if (!_tcscmp(szValue, _T("y1")))						return 22;
	else if (!_tcscmp(szValue, _T("x2")))						return 23;
	else if (!_tcscmp(szValue, _T("y2")))						return 24;
	else if (!_tcscmp(szValue, _T("cx")))						return 25;
	else if (!_tcscmp(szValue, _T("cy")))						return 26;
	else if (!_tcscmp(szValue, _T("x")))						return 27;
	else if (!_tcscmp(szValue, _T("y")))						return 28;
	else if (!_tcscmp(szValue, _T("rx")))						return 29;
	else if (!_tcscmp(szValue, _T("ry")))						return 30;
	else if (!_tcscmp(szValue, _T("r")))						return 31;
	else if (!_tcscmp(szValue, _T("transform")))				return 32;
	else if (!_tcscmp(szValue, _T("size")))						return 33;
	else if (!_tcscmp(szValue, _T("display-align")))			return 34;
	else if (!_tcscmp(szValue, _T("color")))					return 35;
	else if (!_tcscmp(szValue, _T("audio-level")))				return 36;
	else if (!_tcscmp(szValue, _T("fill-rule")))				return 37;
	else if (!_tcscmp(szValue, _T("font-style")))				return 38;
	else if (!_tcscmp(szValue, _T("font-weight")))				return 39;
	else if (!_tcscmp(szValue, _T("image-rendering")))			return 40;
	else if (!_tcscmp(szValue, _T("line-increment")))			return 41;
	else if (!_tcscmp(szValue, _T("pointer-events")))			return 42;
	else if (!_tcscmp(szValue, _T("shape-rendering")))			return 43;
	else if (!_tcscmp(szValue, _T("stop-color")))				return 44;
	else if (!_tcscmp(szValue, _T("stop-opacity")))				return 45;
	else if (!_tcscmp(szValue, _T("solid-color")))				return 46;
	else if (!_tcscmp(szValue, _T("solid-opacity")))			return 47;
	else if (!_tcscmp(szValue, _T("stroke-dasharray")))			return 48;
	else if (!_tcscmp(szValue, _T("stroke-dashoffset")))		return 49;
	else if (!_tcscmp(szValue, _T("stroke-linecap")))			return 50;
	else if (!_tcscmp(szValue, _T("stroke-linejoin")))			return 51;
	else if (!_tcscmp(szValue, _T("stroke-miterlimit")))		return 52;
	else if (!_tcscmp(szValue, _T("text-anchor")))				return 53;
	else if (!_tcscmp(szValue, _T("vector-effect")))			return 54;
	else if (!_tcscmp(szValue, _T("viewport-fill")))			return 55;
	else if (!_tcscmp(szValue, _T("viewport-fill-opacity")))	return 56;
	else if (!_tcscmp(szValue, _T("nav-right")))				return 57;
	else if (!_tcscmp(szValue, _T("nav-next")))					return 58;
	else if (!_tcscmp(szValue, _T("nav-up")))					return 59;
	else if (!_tcscmp(szValue, _T("nav-up-right")))				return 60;
	else if (!_tcscmp(szValue, _T("nav-up-left")))				return 61;
	else if (!_tcscmp(szValue, _T("nav-prev")))					return 62;
	else if (!_tcscmp(szValue, _T("nav-down")))					return 63;
	else if (!_tcscmp(szValue, _T("nav-down-right")))			return 64;
	else if (!_tcscmp(szValue, _T("nav-down-left")))			return 65;
	else if (!_tcscmp(szValue, _T("nav-left")))					return 66;
	else if (!_tcscmp(szValue, _T("focusable")))				return 67;
	else if (!_tcscmp(szValue, _T("gradientUnits")))			return 68;
	else if (!_tcscmp(szValue, _T("pathLength")))				return 69;
	else if (!_tcscmp(szValue, _T("rotate")))					return 70;
	else if (!_tcscmp(szValue, _T("target")))					return 71;
	else if (!_tcscmp(szValue, _T("type")))						return 72;
	else	*pErr = LASeRValidationErr;							return 0;
}

UINT8 ReadRotateType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("auto")))				return 0;
	else if (!_tcscmp(szValue, _T("auto-reverse")))	return 1;
	else	*pErr = LASeRValidationErr;				return 2;
}

UINT8 ReadRestartType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("always")))				return 0;
	else if (!_tcscmp(szValue, _T("whenNotActive")))	return 1;
	else if (!_tcscmp(szValue, _T("never")))			return 2;
	else	*pErr = LASeRValidationErr;					return 0;
}

UINT8 ReadRotscatraType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("translate")))		return 0;
	else if (!_tcscmp(szValue, _T("scale")))	return 1;
	else if (!_tcscmp(szValue, _T("rotate")))	return 2;
	else if (!_tcscmp(szValue, _T("skewX")))	return 3;
	else if (!_tcscmp(szValue, _T("skewY")))	return 4;
	else	*pErr = LASeRValidationErr;			return 0;
}

INT8 ReadResolutionType(int iValue, lsrErr *pErr)
{
	if ((iValue >= -7) || (iValue <= 8))	return (INT8)iValue;
	else	*pErr = LASeRValidationErr;		return 0;
}

float ReadTimesType(const TCHAR *szValue, lsrErr *pErr)
{
	if (szValue == NULL)
	{
		*pErr = LASeRValidationErr;
		return 0.0;
	}

	// find the position of unit
	TCHAR *pFind = (TCHAR *)_tcsstr(szValue, _T("ms"));

	if (pFind)
		return (float)(_tcstod(szValue, &pFind) / 1000);

	else
		return (float)_tcstod(szValue, &pFind);
}

deque<float> ReadPointListType(const TCHAR *szValue, lsrErr *pErr)
{
	deque<float>	afTemp, afOut;
	tstring			strTemp = _T("");

	if (szValue == NULL)
	{
		*pErr = LASeRValidationErr;
		return afOut;
	}

	const UINT	uLength	= (UINT)_tcslen(szValue);
	TCHAR		*pBegin	= (TCHAR *)&szValue[0];	// position of string's beginning
	TCHAR		*pBlank	= pBegin;				// position of blank

	while (1)
	{
		pBlank = (TCHAR *)find((const TCHAR *)pBegin, &szValue[uLength], ' ');

		copy(pBegin, pBlank, back_inserter(strTemp));
		afTemp = ReadFloatListType(strTemp.c_str(), ',', pErr);

		afOut.push_back(afTemp[0]);
		afOut.push_back(afTemp[1]);

		afTemp.clear();
		strTemp	= _T("");
		pBegin	= pBlank + 1;

		if (*pBlank != ' ')
			break;
	}

	return afOut;
}

UINT8 ReadAnimFillType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("freeze")))		return 0;
	else if (!_tcscmp(szValue, _T("remove")))	return 1;
	else	*pErr = LASeRValidationErr;			return 0;
}

float ReadRepeatType(const TCHAR *szValue, lsrErr *pErr)
{
	if (szValue == NULL)
	{
		*pErr = LASeRValidationErr;
		return 0.0;
	}

	if (!_tcscmp(szValue, _T("indefinite")))	return -1.0;

	return (float)_tstof(szValue);
}

UINT8 ReadStrokeLineCapType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("butt")))			return 0;
	else if (!_tcscmp(szValue, _T("round")))	return 1;
	else if (!_tcscmp(szValue, _T("square")))	return 2;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadStrokeLineJoinType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("miter")))			return 0;
	else if (!_tcscmp(szValue, _T("round")))	return 1;
	else if (!_tcscmp(szValue, _T("bevel")))	return 2;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadPointerEventsType(const TCHAR *szValue, lsrErr *pErr)
{
//	if (!_tcscmp(szValue, _T("boundingBox")))		    return 0;
	if (!_tcscmp(szValue, _T("visiblePainted")))		return 1;
	else if (!_tcscmp(szValue, _T("visibleFill")))		return 2;
	else if (!_tcscmp(szValue, _T("visibleStroke")))	return 3;
	else if (!_tcscmp(szValue, _T("visible")))			return 4;
	else if (!_tcscmp(szValue, _T("painted")))			return 5;
	else if (!_tcscmp(szValue, _T("fill")))				return 6;
	else if (!_tcscmp(szValue, _T("stroke")))			return 7;
	else if (!_tcscmp(szValue, _T("all")))				return 8;
	else if (!_tcscmp(szValue, _T("none")))				return 9;
	else	*pErr = LASeRValidationErr;					return 0;
}

UINT8 ReadFontWeightType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("normal")))		return 0;
	else if (!_tcscmp(szValue, _T("bold")))		return 1;
	else if (!_tcscmp(szValue, _T("bolder")))	return 2;
	else if (!_tcscmp(szValue, _T("lighter")))	return 3;
	else if (!_tcscmp(szValue, _T("100")))		return 4;
	else if (!_tcscmp(szValue, _T("200")))		return 5;
	else if (!_tcscmp(szValue, _T("300")))		return 6;
	else if (!_tcscmp(szValue, _T("400")))		return 7;
	else if (!_tcscmp(szValue, _T("500")))		return 8;
	else if (!_tcscmp(szValue, _T("600")))		return 9;
	else if (!_tcscmp(szValue, _T("700")))		return 10;
	else if (!_tcscmp(szValue, _T("800")))		return 11;
	else if (!_tcscmp(szValue, _T("900")))		return 12;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadFontVariantType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("normal")))			return 0;
	else if (!_tcscmp(szValue, _T("small-caps")))	return 1;
	else	*pErr = LASeRValidationErr;				return 0;
}

UINT8 ReadColorRenderingType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("auto")))					return 0;
	else if (!_tcscmp(szValue, _T("optimizeSpeed")))	return 1;
	else if (!_tcscmp(szValue, _T("optimizeQuality")))	return 2;
	else	*pErr = LASeRValidationErr;					return 0;
}

UINT8 ReadImageRenderingType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("auto")))					return 0;
	else if (!_tcscmp(szValue, _T("optimizeSpeed")))	return 1;
	else if (!_tcscmp(szValue, _T("optimizeQuality")))	return 2;
	else	*pErr = LASeRValidationErr;					return 0;
}

UINT8 ReadShapeRenderingType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("auto")))						return 0;
	else if (!_tcscmp(szValue, _T("optimizeSpeed")))		return 1;
	else if (!_tcscmp(szValue, _T("crispEdges")))			return 2;
	else if (!_tcscmp(szValue, _T("geometricPrecision")))	return 3;
	else	*pErr = LASeRValidationErr;						return 0;
}

UINT8 ReadTextRenderingType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("auto")))						return 0;
	else if (!_tcscmp(szValue, _T("optimizeSpeed")))		return 1;
	else if (!_tcscmp(szValue, _T("optimizeLegibility")))	return 2;
	else if (!_tcscmp(szValue, _T("geometricPrecision")))	return 3;
	else	*pErr = LASeRValidationErr;						return 0;
}

deque<UINT8> ReadFeatureListType(const TCHAR *szValue, lsrErr *pErr)
{
	deque<tstring>	auStringList;
	deque<UINT8>		auEnumList;
	UINT8				uListSize;

	auStringList = ReadStringListType(szValue, ' ', pErr);

	uListSize = (UINT8)auStringList.size();

	for (UINT8 i = 0; i < uListSize; ++i)
	{
		if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#SVG")))											auEnumList.push_back(0);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#SVGDOM")))									auEnumList.push_back(1);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#SVG-static")))								auEnumList.push_back(2);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#SVGDOM-static")))								auEnumList.push_back(3);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#SVG-animation")))								auEnumList.push_back(4);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#SVGDOM-animation")))							auEnumList.push_back(5);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#SVG-dynamic")))								auEnumList.push_back(6);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#SVGDOM-dynamic")))							auEnumList.push_back(7);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#CoreAttribute")))								auEnumList.push_back(8);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#Structure")))									auEnumList.push_back(9);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#BasicStructure")))							auEnumList.push_back(10);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#ContainerAttribute")))						auEnumList.push_back(11);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#ConditionalProcessing")))						auEnumList.push_back(12);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#Image")))										auEnumList.push_back(13);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#Style")))										auEnumList.push_back(14);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#ViewportAttribute")))							auEnumList.push_back(15);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#Shape")))										auEnumList.push_back(16);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#Text")))										auEnumList.push_back(17);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#BasicText")))									auEnumList.push_back(18);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#PaintAttribute")))							auEnumList.push_back(19);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#BasicPaintAttribute")))						auEnumList.push_back(20);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#OpacityAttribute")))							auEnumList.push_back(21);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#GraphicsAttribute")))							auEnumList.push_back(22);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#BasicGraphicsAttribute")))					auEnumList.push_back(23);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#Marker")))									auEnumList.push_back(24);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#ColorProfile")))								auEnumList.push_back(25);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#Gradient")))									auEnumList.push_back(26);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#Pattern")))									auEnumList.push_back(27);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#Clip")))										auEnumList.push_back(28);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#BasicClip")))									auEnumList.push_back(29);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#Mask")))										auEnumList.push_back(30);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#Filter")))									auEnumList.push_back(31);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#BasicFilter")))								auEnumList.push_back(32);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#DocumentEventsAttribute")))					auEnumList.push_back(33);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#GraphicalEventsAttribute")))					auEnumList.push_back(34);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#AnimationEventsAttribute")))					auEnumList.push_back(35);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#Cursor")))									auEnumList.push_back(36);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#Hyperlinking")))								auEnumList.push_back(37);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#XlinkAttribute")))							auEnumList.push_back(38);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#ExternalResourcesRequired")))					auEnumList.push_back(39);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#View")))										auEnumList.push_back(40);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#Script")))									auEnumList.push_back(41);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#Animation")))									auEnumList.push_back(42);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#Font")))										auEnumList.push_back(43);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#BasicFont")))									auEnumList.push_back(44);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVG11/feature#Extensibility")))								auEnumList.push_back(45);
		else if (!_tcscmp(szValue, _T("org.w3c.svg")))																	auEnumList.push_back(46);
		else if (!_tcscmp(szValue, _T("org.w3c.dom.svg")))																auEnumList.push_back(47);
		else if (!_tcscmp(szValue, _T("org.w3c.svg.static")))															auEnumList.push_back(48);
		else if (!_tcscmp(szValue, _T("org.w3c.dom.svg.static")))														auEnumList.push_back(49);
		else if (!_tcscmp(szValue, _T("org.w3c.svg.animation")))														auEnumList.push_back(50);
		else if (!_tcscmp(szValue, _T("org.w3c.dom.svg.animation")))													auEnumList.push_back(51);
		else if (!_tcscmp(szValue, _T("org.w3c.svg.dynamic")))															auEnumList.push_back(52);
		else if (!_tcscmp(szValue, _T("org.w3c.dom.svg.dynamic")))														auEnumList.push_back(53);
		else if (!_tcscmp(szValue, _T("org.w3c.svg.all")))																auEnumList.push_back(54);
		else if (!_tcscmp(szValue, _T("org.w3c.dom.svg.all")))															auEnumList.push_back(55);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVGMobile/Tiny/feature#base")))								auEnumList.push_back(56);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVGMobile/Tiny/feature#interactivity")))					auEnumList.push_back(57);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVGMobile/Tiny/feature#all")))								auEnumList.push_back(58);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVGMobile/Basic/feature#base")))							auEnumList.push_back(59);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVGMobile/Basic/feature#interactivity")))					auEnumList.push_back(60);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVGMobile/Basic/feature#SVGBasicDomCore")))					auEnumList.push_back(61);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVGMobile/Basic/feature#SVGBasicDomExtended")))				auEnumList.push_back(62);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVGMobile/Basic/feature#css")))								auEnumList.push_back(63);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/TR/SVGMobile/Basic/feature#all")))								auEnumList.push_back(64);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#SVG-static")))						auEnumList.push_back(65);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#SVG-static-DOM")))					auEnumList.push_back(66);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#SVG-animated")))						auEnumList.push_back(67);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#SVG-all")))							auEnumList.push_back(68);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#CoreAttribute")))					auEnumList.push_back(69);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#NavigationAttribute")))				auEnumList.push_back(70);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#Structure")))						auEnumList.push_back(71);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#ConditionalProcessing")))			auEnumList.push_back(72);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#ConditionalProcessingAttribute")))	auEnumList.push_back(73);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#Image")))							auEnumList.push_back(74);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#Prefetch")))							auEnumList.push_back(75);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#Discard")))							auEnumList.push_back(76);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#Shape")))							auEnumList.push_back(77);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#Text")))								auEnumList.push_back(78);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#PaintAttribute")))					auEnumList.push_back(79);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#OpacityAttribute")))					auEnumList.push_back(80);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#GraphicsAttribute")))				auEnumList.push_back(81);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#Gradient")))							auEnumList.push_back(82);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#SolidColor")))						auEnumList.push_back(83);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#Hyperlinking")))						auEnumList.push_back(84);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#XlinkAttribute")))					auEnumList.push_back(85);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#ExternalResourcesRequired")))		auEnumList.push_back(86);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#Scripting")))						auEnumList.push_back(87);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#Handler")))							auEnumList.push_back(88);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#Listener")))							auEnumList.push_back(89);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#TimedAnimation")))					auEnumList.push_back(90);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#Animation")))						auEnumList.push_back(91);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#Audio")))							auEnumList.push_back(92);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#Video")))							auEnumList.push_back(93);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#Font")))								auEnumList.push_back(94);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#Extensibility")))					auEnumList.push_back(95);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#MediaAttribute")))					auEnumList.push_back(96);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#TextFlow")))							auEnumList.push_back(97);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#TransformedVideo")))					auEnumList.push_back(98);
		else if (!_tcscmp(szValue, _T("http://www.w3.org/Graphics/SVG/feature/1.2/#ComposedVideo")))					auEnumList.push_back(99);
	}

	if (auStringList.size() != auEnumList.size())
		*pErr = LASeRValidationErr;

	return auEnumList;
}

UINT8 ReadPointsCodecType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("ExpGolombPointsCodec")))	return 0;
	else	*pErr = LASeRValidationErr;					return 0;
}

UINT8 ReadProfileType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("full")))		return 0;
	else if (!_tcscmp(szValue, _T("mini")))	return 1;
	else	*pErr = LASeRValidationErr;		return 0;
}

UINT8 ReadSyncBehaviorType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("canSlip")))			return 0;
	else if (!_tcscmp(szValue, _T("locked")))		return 1;
	else if (!_tcscmp(szValue, _T("independent")))	return 2;
	else if (!_tcscmp(szValue, _T("default")))		return 3;
	else	*pErr = LASeRValidationErr;				return 0;
}

UINT8 ReadSyncBehaviorDefaultType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("canSlip")))			return 0;
	else if (!_tcscmp(szValue, _T("locked")))		return 1;
	else if (!_tcscmp(szValue, _T("independent")))	return 2;
	else	*pErr = LASeRValidationErr;				return 0;
}

UINT8 ReadGradientUnitsType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("userSpaceOnUse")))			return 0;
	else if (!_tcscmp(szValue, _T("objectBoundingBox")))	return 1;
	else	*pErr = LASeRValidationErr;						return 0;
}

UINT8 ReadVectorType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("none")))						return 0;
	else if (!_tcscmp(szValue, _T("non-scaling-stroke")))	return 1;
	else	*pErr = LASeRValidationErr;						return 0;
}

deque<float> ReadAnimatedValuesType(const TCHAR *szValue, lsrErr *pErr)
{
	deque<float>	afTemp, afOut;
	tstring			strTemp = _T("");

	if (szValue == NULL)
	{
		*pErr = LASeRValidationErr;
		return afOut;
	}

	const UINT	uLength	= (UINT)_tcslen(szValue);
	TCHAR		*pBegin	= (TCHAR *)&szValue[0];	// position of string's beginning
	TCHAR		*pSemicolon	= pBegin;			// position of semicolon

	while (1)
	{
		pSemicolon = (TCHAR *)find((const TCHAR *)pBegin, &szValue[uLength], ';');

		copy(pBegin, pSemicolon, back_inserter(strTemp));
		afTemp = ReadFloatListType(strTemp.c_str(), ' ', pErr);

		for (UINT8 i = 0; i < afTemp.size(); ++i)
			afOut.push_back(afTemp[i]);

		afTemp.clear();
		strTemp	= _T("");
		pBegin	= pSemicolon + 1;

		if (*pSemicolon != ';')
			break;
	}

	return afOut;
}

UINT8 ReadPreserveAspectRatioType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("none")))					return 0;
	else if (!_tcscmp(szValue, _T("xMinYMin")))			return 1;
	else if (!_tcscmp(szValue, _T("xMidYMin")))			return 2;
	else if (!_tcscmp(szValue, _T("xMaxYMin")))			return 3;
	else if (!_tcscmp(szValue, _T("xMinYMid")))			return 4;
	else if (!_tcscmp(szValue, _T("xMidYMid")))			return 5;
	else if (!_tcscmp(szValue, _T("xMaxYMid")))			return 6;
	else if (!_tcscmp(szValue, _T("xMinYMax")))			return 7;
	else if (!_tcscmp(szValue, _T("xMidYMax")))			return 8;
	else if (!_tcscmp(szValue, _T("xMaxYMax")))			return 9;
	else if (!_tcscmp(szValue, _T("xMinYMin meet")))	return 10;
	else if (!_tcscmp(szValue, _T("xMidYMin meet")))	return 11;
	else if (!_tcscmp(szValue, _T("xMaxYMin meet")))	return 12;
	else if (!_tcscmp(szValue, _T("xMinYMid meet")))	return 13;
	else if (!_tcscmp(szValue, _T("xMidYMid meet")))	return 14;
	else if (!_tcscmp(szValue, _T("xMaxYMid meet")))	return 15;
	else if (!_tcscmp(szValue, _T("xMinYMax meet")))	return 16;
	else if (!_tcscmp(szValue, _T("xMidYMax meet")))	return 17;
	else if (!_tcscmp(szValue, _T("xMaxYMax meet")))	return 18;
	else	*pErr = LASeRValidationErr;					return 0;
}

float ReadValueWithUnitsType(const TCHAR *szValue, UINT8 *uUnit, lsrErr *pErr)
{
	if (szValue == NULL)
	{
		*pErr = LASeRValidationErr;
		return 0;
	}

	TCHAR *pFind = NULL;

	if (_tcsstr(szValue, _T("in")))			{pFind = (TCHAR *)_tcsstr(szValue, _T("in"));		*uUnit = 0;}
	else if (_tcsstr(szValue, _T("cm")))	{pFind = (TCHAR *)_tcsstr(szValue, _T("cm"));		*uUnit = 1;}
	else if (_tcsstr(szValue, _T("mm")))	{pFind = (TCHAR *)_tcsstr(szValue, _T("mm"));		*uUnit = 2;}
	else if (_tcsstr(szValue, _T("pt")))	{pFind = (TCHAR *)_tcsstr(szValue, _T("pt"));		*uUnit = 3;}
	else if (_tcsstr(szValue, _T("pc")))	{pFind = (TCHAR *)_tcsstr(szValue, _T("pc"));		*uUnit = 4;}
	else if (_tcsstr(szValue, _T("px")))	{pFind = (TCHAR *)_tcsstr(szValue, _T("px"));		*uUnit = 5;}
	else if (_tcsstr(szValue, _T("%")))		{pFind = (TCHAR *)_tcsstr(szValue, _T("%"));		*uUnit = 6;}
	else									{pFind = (TCHAR *)&szValue[(UINT)_tcslen(szValue)]; *uUnit = 5;}

	return (float)_tcstod(szValue, &pFind);
}

UINT8 ReadOverlayType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("top")))		return 0;
	else if (!_tcscmp(szValue, _T("none")))	return 1;
	else	*pErr = LASeRValidationErr;		return 0;
}

UINT8 ReadScriptType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("application/ecmascript")))		return 0;
	else if (!_tcscmp(szValue, _T("application/jar-archive")))	return 1;
	else	*pErr = LASeRValidationErr;							return 0;
}

UINT8 ReadTransformBehaviorType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("geometric")))			return 0;
	else if (!_tcscmp(szValue, _T("pinned")))		return 1;
	else if (!_tcscmp(szValue, _T("pinned_90")))	return 2;
	else if (!_tcscmp(szValue, _T("pinned_180")))	return 3;
	else if (!_tcscmp(szValue, _T("pinned_270")))	return 4;
	else	*pErr = LASeRValidationErr;				return 0;
}

UINT8 ReadDisplayAlignType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("before")))		return 0;
	else if (!_tcscmp(szValue, _T("center")))	return 1;
	else if (!_tcscmp(szValue, _T("after")))	return 2;
	else if (!_tcscmp(szValue, _T("auto")))		return 3;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadFocusableType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("true")))			return 0;
	else if (!_tcscmp(szValue, _T("false")))	return 1;
	else if (!_tcscmp(szValue, _T("auto")))		return 2;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadFocusHighlightType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("none")))		return 0;
	else if (!_tcscmp(szValue, _T("auto")))	return 1;
	else	*pErr = LASeRValidationErr;		return 0;
}

UINT8 ReadEditableType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("none")))			return 0;
	else if (!_tcscmp(szValue, _T("simple")))	return 1;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadInitialVisibilityType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("whenStarted")))	return 0;
	else if (!_tcscmp(szValue, _T("always")))	return 1;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadPlaybackOrderType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("all")))				return 0;
	else if (!_tcscmp(szValue, _T("forwardOnly")))	return 1;
	else	*pErr = LASeRValidationErr;				return 0;
}

UINT8 ReadTimeLineBeginType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("onLoad")))		return 0;
	else if (!_tcscmp(szValue, _T("onStart")))	return 1;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadAccumulateType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("sum")))		return 0;
	else if (!_tcscmp(szValue, _T("none")))	return 1;
	else	*pErr = LASeRValidationErr;		return 0;
}

UINT8 ReadAdditiveType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("sum")))			return 0;
	else if (!_tcscmp(szValue, _T("replace")))	return 1;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadZoomAndPanType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("disable")))		return 0;
	else if (!_tcscmp(szValue, _T("magnify")))	return 1;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadPhaseType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("default")))		return 0;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadStreamSourceModeType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("replace")))		return 0;
	else if (!_tcscmp(szValue, _T("useOld")))	return 1;
	else if (!_tcscmp(szValue, _T("keepOld")))	return 2;
	else if (!_tcscmp(szValue, _T("playList")))	return 3;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadDirectionType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("up")))			return 0;
	else if (!_tcscmp(szValue, _T("down")))		return 1;
	else if (!_tcscmp(szValue, _T("left")))		return 2;
	else if (!_tcscmp(szValue, _T("right")))	return 3;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadSecurityType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("new")))			return 0;
	else if (!_tcscmp(szValue, _T("parent")))	return 1;
	else if (!_tcscmp(szValue, _T("zz1")))		return 2;
	else if (!_tcscmp(szValue, _T("zz2")))		return 3;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadFlowType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("any")))			return 0;
	else if (!_tcscmp(szValue, _T("complete")))	return 1;
	else if (!_tcscmp(szValue, _T("segment")))	return 2;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadMinSizeBehaviorType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("freeze")))		return 0;
	else if (!_tcscmp(szValue, _T("remove")))	return 1;
	else	*pErr = LASeRValidationErr;			return 0;
}

UINT8 ReadParsingModeType(const TCHAR *szValue, lsrErr *pErr)
{
	if (!_tcscmp(szValue, _T("descending")))		return 0;
	else if (!_tcscmp(szValue, _T("ascending")))	return 1;
	else if (!_tcscmp(szValue, _T("incremental")))	return 2;
	else	*pErr = LASeRValidationErr;				return 0;
}