///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#ifndef _DATATYPES_H_
#define _DATATYPES_H_

#include "tools.h"

#include <deque>


UINT8			ReadBooleanType(const TCHAR *szValue, lsrErr *pErr);
deque<UINT8>	ReadInt8ListType(const TCHAR *szValue, const TCHAR chUnit, lsrErr *pErr);
deque<UINT>		ReadIntListType(const TCHAR *szValue, lsrErr *pErr);
deque<float>	ReadFloatListType(const TCHAR *szValue, lsrErr *pErr);
deque<float>	ReadFloatListType(const TCHAR *szValue, const TCHAR chUnit, lsrErr *pErr);
deque<tstring>	ReadStringListType(const TCHAR *szValue, const TCHAR chUnit, lsrErr *pErr);
UINT			ReadLengthUnitType(const TCHAR *szValue, UINT8 *pUnit, lsrErr *pErr);

// ev
UINT8			ReadPropagateType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadDefaultActionType(const TCHAR *szValue, lsrErr *pErr);

// xml
UINT8			ReadSpaceType(const TCHAR *szValue, lsrErr *pErr);

// xlink
UINT8			ReadTypeType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadShowType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadActuateType(const TCHAR *szValue, lsrErr *pErr);

// datatypes2
deque<UINT8>	ReadColorType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadFontStyleType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadTextDecorationType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadCalcModeType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadVisibilityType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadOverflowType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadDisplayType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadTextAnchorType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadEventType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadFillRuleType(const TCHAR *szValue, lsrErr *pErr);
deque<float>	ReadTransformType(const TCHAR *szValue, const TCHAR *szAttr, lsrErr *pErr);
UINT8			ReadAnimatableType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadRotateType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadRestartType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadRotscatraType(const TCHAR *szValue, lsrErr *pErr);
INT8			ReadResolutionType(int iValue, lsrErr *pErr);
float			ReadTimesType(const TCHAR *szValue, lsrErr *pErr);
deque<float>	ReadPointListType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadAnimFillType(const TCHAR *szValue, lsrErr *pErr);
float			ReadRepeatType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadStrokeLineCapType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadStrokeLineJoinType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadPointerEventsType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadFontWeightType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadFontVariantType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadColorRenderingType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadImageRenderingType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadShapeRenderingType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadTextRenderingType(const TCHAR *szValue, lsrErr *pErr);
deque<UINT8>	ReadFeatureListType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadPointsCodecType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadProfileType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadSyncBehaviorType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadSyncBehaviorDefaultType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadGradientUnitsType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadVectorType(const TCHAR *szValue, lsrErr *pErr);
deque<float>	ReadAnimatedValuesType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadPreserveAspectRatioType(const TCHAR *szValue, lsrErr *pErr);
float			ReadValueWithUnitsType(const TCHAR *szValue, UINT8 *uUnit, lsrErr *pErr);
UINT8			ReadOverlayType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadScriptType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadTransformBehaviorType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadDisplayAlignType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadFocusableType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadFocusHighlightType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadEditableType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadInitialVisibilityType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadPlaybackOrderType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadTimeLineBeginType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadAccumulateType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadAdditiveType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadZoomAndPanType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadPhaseType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadStreamSourceModeType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadDirectionType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadSecurityType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadFlowType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadMinSizeBehaviorType(const TCHAR *szValue, lsrErr *pErr);
UINT8			ReadParsingModeType(const TCHAR *szValue, lsrErr *pErr);

#endif