///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "LASeR.h"

#include <algorithm>


CLASeR::CLASeR()
{
	m_uElementType		= 0;
	m_pParent			= NULL;
	m_uNumOfChildren	= 0;
	m_bHasID			= 0;
	m_bDeactivation		= 0;
	m_uMyIndex			= 0;
}

CLASeR::~CLASeR()
{
	// memory deletion
	for (deque<CLASeR *>::iterator i = m_apChildren.begin(); i < m_apChildren.end(); ++i)
	{DELETEMEMORY(*i);}

	m_apChildren.clear();
}

/************************************************************************/
// CreateElement
// Desc		: Create New LASeR elements
// Params	: DOM tree, New LASeR Element pointer
// History
//	PSH (2008. 7. 14)	- first release
//	PSH (2008. 8. 2)	- move to LASeR.cpp
/************************************************************************/
lsrErr CreateElement(LPXNode pReader, CLASeR **outLASeRElement)
{
	lsrErr err = LASeRNoErr;

	CLASeR	*pNewLASeRElement = NULL;
	*outLASeRElement = NULL;

	/************************************************************************/
	// setting new LASeR element
	if (pReader->name == _T("a"))
		pNewLASeRElement = new CSVGA();

	else if (pReader->name == _T("animate"))
		pNewLASeRElement = new CSVGAnimate(SVG_animate);

	else if (pReader->name == _T("animateColor"))
		pNewLASeRElement = new CSVGAnimate(SVG_animateColor);

	else if (pReader->name == _T("animateMotion"))
		pNewLASeRElement = new CSVGAnimateMotion();

	else if (pReader->name == _T("animateTransform"))
		pNewLASeRElement = new CSVGAnimateTransform();

	else if (pReader->name == _T("circle"))
		pNewLASeRElement = new CSVGCircle();

	else if (pReader->name == _T("defs"))
		pNewLASeRElement = new CSVGDefs();

	else if (pReader->name == _T("desc"))
		pNewLASeRElement = new CSVGDesc(SVG_desc);

	else if (pReader->name == _T("ellipse"))
		pNewLASeRElement = new CSVGEllipse();

	else if (pReader->name == _T("foreignObject"))
		pNewLASeRElement = new CSVGForeignObject();

	else if (pReader->name == _T("g"))
		pNewLASeRElement = new CSVGG();

	else if (pReader->name == _T("image"))
		pNewLASeRElement = new CSVGImage();

	else if (pReader->name == _T("line"))
		pNewLASeRElement = new CSVGLine();

	else if (pReader->name == _T("linearGradient"))
		pNewLASeRElement = new CSVGLinearGradient();

	else if (pReader->name == _T("desc"))
		pNewLASeRElement = new CSVGDesc(SVG_metadata);

	else if (pReader->name == _T("mpath"))
		pNewLASeRElement = new CSVGMPath();

	else if (pReader->name == _T("path"))
		pNewLASeRElement = new CSVGPath();

	else if (pReader->name == _T("polygon"))
		pNewLASeRElement = new CSVGPolygon(SVG_polygon);

	else if (pReader->name == _T("polyline"))
		pNewLASeRElement = new CSVGPolygon(SVG_polyline);

	else if (pReader->name == _T("radialGradient"))
		pNewLASeRElement = new CSVGRadialGradient();

	else if (pReader->name == _T("rect"))
		pNewLASeRElement = new CSVGRect();

	else if (pReader->name == _T("script"))
		pNewLASeRElement = new CSVGScript();

	else if (pReader->name == _T("set"))
		pNewLASeRElement = new CSVGSet();

	else if (pReader->name == _T("stop"))
		pNewLASeRElement = new CSVGStop();

	else if (pReader->name == _T("svg"))
		pNewLASeRElement = new CSVG();

	else if (pReader->name == _T("switch"))
		pNewLASeRElement = new CSVGSwitch();

	else if (pReader->name == _T("text"))
		pNewLASeRElement = new CSVGText();

	else if (pReader->name == _T("title"))
		pNewLASeRElement = new CSVGDesc(SVG_title);

	else if (pReader->name == _T("use"))
		pNewLASeRElement = new CSVGUse();

	else if (pReader->name == _T("video"))
		pNewLASeRElement = new CSVGVideo();

	else if (pReader->name == _T("ev:listener"))
		pNewLASeRElement = new CEVListener();

	// amd1
	else if (pReader->name == _T("animation"))
		pNewLASeRElement = new CSVGAnimation();

	else if (pReader->name == _T("discard"))
		pNewLASeRElement = new CSVGDiscard();

	else if (pReader->name == _T("font"))
		pNewLASeRElement = new CSVGTFont();

	else if (pReader->name == _T("font-face"))
		pNewLASeRElement = new CSVGTFontFace();

	else if (pReader->name == _T("font-face-src"))
		pNewLASeRElement = new CSVGTFontFaceSrc();

	else if (pReader->name == _T("font-face-uri"))
		pNewLASeRElement = new CSVGTFontFaceUri();

	else if (pReader->name == _T("glyph"))
		pNewLASeRElement = new CSVGTGlyph();

	else if (pReader->name == _T("handler"))
		pNewLASeRElement = new CSVGHandler();

	else if (pReader->name == _T("hkern"))
		pNewLASeRElement = new CSVGTHkern();

	else if (pReader->name == _T("missing-glyph"))
		pNewLASeRElement = new CSVGTMissingGlyph();

	else if (pReader->name == _T("prefetch"))
		pNewLASeRElement = new CSVGPrefetch();

	else if (pReader->name == _T("solidColor"))
		pNewLASeRElement = new CSVGSolidColor();

	else if (pReader->name == _T("tBreak"))
		pNewLASeRElement = new CSVGTBreak();

	else if (pReader->name == _T("textArea"))
		pNewLASeRElement = new CSVGTextArea();

	else if (pReader->name == _T("lsr:conditional"))
		pNewLASeRElement = new CLSRConditional();

	else if (pReader->name == _T("lsr:cursorManager"))
		pNewLASeRElement = new CLSRCursorManager();

	else if (pReader->name == _T("lsr:rectClip"))
		pNewLASeRElement = new CLSRRectClip();

	else if (pReader->name == _T("lsr:selector"))
		pNewLASeRElement = new CLSRSelector();

	else if (pReader->name == _T("lsr:animateScroll"))
		pNewLASeRElement = new CLSRAnimateScroll();

	else if (pReader->name == _T("lsr:setScroll"))
		pNewLASeRElement = new CLSRSetScroll();

	else if (pReader->name == _T("lsr:updates"))
		pNewLASeRElement = new CLSRUpdates();

	else if (pReader->name == _T("lsr:parsingSwitch"))
		pNewLASeRElement = new CLSRParsingSwitch();

	else if (pReader->name == _T("lsr:NewScene"))
		pNewLASeRElement = new CNewScene();

	else if (pReader->name == _T("lsr:RefreshScene"))
		pNewLASeRElement = new CRefreshScene();

	else if (pReader->name == _T("lsr:Insert"))
		pNewLASeRElement = new CInsert();

	else if (pReader->name == _T("lsr:Delete"))
		pNewLASeRElement = new CDelete();

	else if (pReader->name == _T("lsr:Replace"))
		pNewLASeRElement = new CReplace();

	else if (pReader->name == _T("lsr:Add"))
		pNewLASeRElement = new CAdd();

	else if (pReader->name == _T("lsr:Save"))
		pNewLASeRElement = new CSave();

	else if (pReader->name == _T("lsr:Restore"))
		pNewLASeRElement = new CRestore();

	else if (pReader->name == _T("lsr:Clean"))
		pNewLASeRElement = new CClean();

	else if (pReader->name == _T("lsr:SendEvent"))
		pNewLASeRElement = new CSendEvent();

	else if (pReader->name == _T("lsr:Activate"))
		pNewLASeRElement = new CActivate();

	else if (pReader->name == _T("lsr:Deactivate"))
		pNewLASeRElement = new CDeactivate();

	else if (pReader->name == _T("lsr:ReleaseResource"))
		pNewLASeRElement = new CReleaseResource();

	else if (pReader->name == _T("lsr:AdaptiveUpdateGroup"))
		pNewLASeRElement = new CAdaptiveUpdateGroup();
	
	else if (pReader->name == _T("lsr:LASeRHeader"))
		pNewLASeRElement = new CLASeRHeader();

	else
		err	= LASeRDOMTreeErr;

	*outLASeRElement = pNewLASeRElement;

	return err;
}

/************************************************************************/
// GetChildren
// Desc		: function to get the specific child
// Param	: child index
// History
//	PSH (2009. 2. 12)	- first release
/************************************************************************/
CLASeR *CLASeR::GetChildren(UINT8 uIndex)
{
	if (uIndex < m_uNumOfChildren)
		return m_apChildren[uIndex];

	else
		return NULL;
}

/************************************************************************/
// SetChildrenSize
// Desc		: function to set the children size
// Param	: new children size
// History
//	PSH (2009. 2. 12)	- first release
/************************************************************************/
lsrErr CLASeR::SetChildrenSize(UINT8 uSize)
{
	if (uSize > 255)
		return LASeRDOMTreeErr;

	else
		m_uNumOfChildren = uSize;

	return LASeRNoErr;
}

/************************************************************************/
// SetActivate & SetDeactivate
// Desc		: function to add the children
// Param	: DOM tree
// History
//	PSH (2008. 12. 1)	- first release
/************************************************************************/
void CLASeR::SetActivate()
{
	m_bDeactivation = 0;

	for (UINT8 i = 0; i < m_uNumOfChildren; ++i)
		GetChildren(i)->SetActivate();
}

void CLASeR::SetDeactivate()
{
	m_bDeactivation = 1;

	for (UINT8 i = 0; i < m_uNumOfChildren; ++i)
		GetChildren(i)->SetDeactivate();
}

/************************************************************************/
// AddChildren
// Desc		: function to add the children
// Param	: DOM tree
// History
//	PSH (2008. 7. 14)	- first release
/************************************************************************/
lsrErr CLASeR::AddChildren(LPXNode pScene)
{
	lsrErr err = LASeRNoErr;

	m_uNumOfChildren = (UINT8)pScene->GetChildCount();

	for (UINT8 i = 0; i < m_uNumOfChildren; ++i)
	{
		CLASeR *pNewLASeRElement = NULL;

		// make LASeR Element
		err = CreateElement(pScene->GetChild(i), &pNewLASeRElement);
		CONFIRM(err);

		// check the right child
		err = CheckRightChild(pNewLASeRElement->GetElementType());
		CONFIRM(err);

		pNewLASeRElement->SetParent(this);
		pNewLASeRElement->SetIndex(i);
		// inherit parent's activation property
		pNewLASeRElement->InheritParentActivation();

		// insert child
		m_apChildren.push_back(pNewLASeRElement);

		err = pNewLASeRElement->SetAttrs(pScene->GetChild(i), pNewLASeRElement);
		CONFIRM(err);
	}

	// compare DOM tree to Scene tree
	if (m_apChildren.size() != m_uNumOfChildren)
		err = LASeRValidationErr;

	return err;
}

/************************************************************************/
// DeleteChild
// Desc		: function to delete a child
// Param	: child's index to be deleted
// History
//	PSH (2008. 11. 11)	- first release
/************************************************************************/
lsrErr CLASeR::DeleteChild(UINT8 uIndex)
{
	lsrErr err = LASeRNoErr;

	UINT uChildrenSize = GetChildrenSize();

	if (uIndex > uChildrenSize)
	{
		err = LASeRBadParamErr;
		return err;
	}

	CLASeR *pChild = GetChildren(uIndex);
	
	// shift left
	for (UINT8 i = uIndex + 1; i < uChildrenSize; ++i)
		m_apChildren[i]->SetIndex(i - 1);

	pChild->SetParent(NULL);

	m_apChildren.erase(remove(m_apChildren.begin(), m_apChildren.end(), pChild), m_apChildren.end());
	SetChildrenSize((UINT8)(uChildrenSize - 1));

	DELETEMEMORY(pChild);

	return err;
}

/************************************************************************/
// InsertChild
// Desc		: function to insert a child
// Param	: child's index to be added
// History
//	PSH (2008. 11. 12)	- first release
/************************************************************************/
lsrErr CLASeR::InsertChild(UINT uIndex, CLASeR *pToBeAdded)
{
	lsrErr err = LASeRNoErr;

	UINT8 uChildrenSize = GetChildrenSize();

	if (uIndex >= 256)
	{
		pToBeAdded->SetActivate();
		pToBeAdded->SetIndex(uChildrenSize);

		m_apChildren.push_back(pToBeAdded);

		pToBeAdded->SetParent(this);
		SetChildrenSize((UINT8)(uChildrenSize + 1));
	}

	else if (uIndex > (UINT)(uChildrenSize + 1))
	{
		err = LASeRBadParamErr;
		return err;
	}

	else
	{
		CLASeR *pToBePut = GetChildren((UINT8)uIndex);

		// shift right
		for (UINT8 i = (UINT8)uIndex + 1; i < uChildrenSize; ++i)
			m_apChildren[i]->SetIndex(i + 1);

		pToBeAdded->SetActivate();
		pToBeAdded->SetIndex((UINT8)uIndex);

		// to be put iterator
		deque<CLASeR *>::iterator iter = find(m_apChildren.begin(), m_apChildren.end(), pToBePut);
		m_apChildren.insert(iter, pToBeAdded);

		pToBeAdded->SetParent(this);
		SetChildrenSize(uChildrenSize + 1);
	}

	return err;
}

/************************************************************************/
// ReplaceChild
// Desc		: function to replace a child
// Param	: child's index to be replaced
// History
//	PSH (2008. 11. 13)	- first release
/************************************************************************/
lsrErr CLASeR::ReplaceChild(UINT uIndex, CLASeR *pToBeReplaced)
{
	lsrErr err = LASeRNoErr;

	UINT8 uChildrenSize = GetChildrenSize();

	if (uIndex > (UINT)uChildrenSize)
	{
		err = LASeRBadParamErr;
		return err;
	}

	CLASeR *pChild = GetChildren((UINT8)uIndex);

	// replace parent pointer
	pToBeReplaced->SetParent(pChild->GetParent());
	// replace child index
	pToBeReplaced->SetIndex(pChild->GetIndex());

	DELETEMEMORY(pChild);

	return LASeRNoErr;
}

/************************************************************************/
// SetParent
// Desc		: function to set a SAF parent
// Param	: a parent's void pointer, parent's element type
// History
//	PSH (2008. 7. 23)	- first release
/************************************************************************/
void CLASeRUnit::SetParent(void *pParent, UINT8 uParentElementType)
{
	m_pSAFParent			= pParent;
	m_uParentElementType	= uParentElementType;
}