///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#ifndef _LSR_H_
#define _LSR_H_

#include "svg.h"


class CLASeRHeader : public CLASeR
{
protected:
	void		*m_pSAFParent;
	UINT8		m_uParentElementType;

	UINT8		m_uProfile;
	UINT8		m_uLevel;
	INT8		m_nResolution;
	UINT16		m_uTimeResoluton;
	UINT8		m_uCoordBits;
	UINT8		m_uScaleBitsMinusCoordBits;
	UINT8		m_uColorComponentBits;			// 4 bits
	UINT8		m_bUseFullRequestHost;
	UINT8		m_uPathComponents;				// 4 bits
	UINT8		m_bNewSceneIndicator;
	UINT8		m_uTimeEndcoding;
	UINT8		m_uPointsCodecType;				// 2 bits

	// ?
	UINT8		m_uAdaptationType;
	UINT		m_uAdaptationTypeValue;
	deque<UINT>	m_auAdaptationConstraints;

public:
	CLASeRHeader();
	~CLASeRHeader();

	void		SetParent(void *pParent, UINT8 uParentElementType);
	void		*GetParent()					{return m_pSAFParent;}
	UINT8		GetParentElementType() const	{return m_uParentElementType;}

	lsrErr		SetAttrs(LPXNode pScene, CLASeR *pI);

	UINT8		GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	UINT		GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT>	GetUINTListAttr(UINT uAttrType, lsrErr *pErr);
};

/************************************************************************/
// Elements
class CLSRConditional : public CSVGObject
{
protected:
	CLSRAdaptationAttr	*m_pAdaptationAttr;

	UINT8				m_bExternalResourcesRequired;
	deque<tstring>		m_astrBegin;
	UINT8				m_bEnabled;

public:
	CLSRConditional();
	~CLSRConditional();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	UINT			GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);
	deque<tstring>	GetStrListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);
};

class CLSRCursorManager : public CSVGObject
{
protected:
	CSVGHref	*m_pHref;
	float		m_fX;
	float		m_fY;

public:
	CLSRCursorManager();
	~CLSRCursorManager();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);

	float	GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);

	UINT	GetAttrType(tstring strAttrName);
	UINT8	GetAttrValType(UINT uAttrType);

	lsrErr	SetAttr(UINT uAttrType, tstring strValue);
	lsrErr	SetAttr(UINT uAttrType, float fValue);
};

class CLSRRectClip : public CSVGG
{
protected:
	deque<float>	m_afSize;
	float			m_fX;
	float			m_fY;

public:
	CLSRRectClip();
	~CLSRRectClip();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CLSRSelector : public CSVGG
{
protected:
	int	m_nChoice;

public:
	CLSRSelector();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	UINT			GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, UINT uValue);
};

class CLSRSimpleLayout : public CSVGG
{
protected:
	deque<float>	m_afDelta;

public:
	CLSRSimpleLayout();
	~CLSRSimpleLayout();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
};

// amd1
class CLSRAnimateScroll : public CSVGObject
{
protected:
	CSVGHref		*m_pHref;

	deque<float>	m_afBy;
	deque<float>	m_afFrom;
	deque<float>	m_afTo;
	float			m_fDelayAtStart;
	float			m_fDelayAtEnd;
	float			m_fSpeed;
	UINT8			m_uDirection;
	deque<tstring>	m_astrBegin;
	float			m_fDur;
	deque<tstring>	m_astrEnd;
	UINT8			m_uFill;
	UINT8			m_uRestart;
	float			m_fRepeatCount;
	float			m_fRepeatDur;

public:
	CLSRAnimateScroll();
	~CLSRAnimateScroll();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);
	deque<tstring>	GetStrListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
};

class CLSRSetScroll : public CSVGObject
{
protected:
	deque<tstring>	m_astrBegin;
	deque<float>	m_afIncrement;
	deque<float>	m_afTo;
	UINT8			m_uDirection;
	UINT8			m_uFill;

public:
	CLSRSetScroll();
	~CLSRSetScroll();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);

	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);
	deque<tstring>	GetStrListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
};

class CLSRUpdates : public CSVGObject
{
protected:
	deque<tstring>	m_astrBegin;
	deque<tstring>	m_astrEnd;
	UINT8			m_bExternalResourcesRequired;
	float			m_fDur;
	float			m_fRepeatCount;
	float			m_fRepeatDur;
	UINT8			m_uRestart;
	UINT8			m_uSyncBehavior;
	float			m_fSyncTolerance;
	UINT8			m_bSyncMaster;
	tstring			m_strType;
	UINT8			m_uSecurity;
	UINT8			m_uFlow;
	tstring			m_strSyncReference;
	UINT			m_uClipBegin;
	UINT			m_uClipEnd;

public:
	CLSRUpdates();
	~CLSRUpdates();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	UINT			GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<tstring>	GetStrListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
};

// amd2
class CLSRParsingSwitch : public CSVGG
{
protected:
	UINT8			m_uMode;

public:
	CLSRParsingSwitch();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
};

/************************************************************************/
// LASeRUnit
class CLASeRUnit : public CLASeR
{
public:
	void	*m_pSAFParent;
	UINT8	m_uParentElementType;

	CLASeRUnit()
	{
		CLASeR::CLASeR();

		m_pSAFParent			= NULL;
		m_uParentElementType	= 0;
	}

	void	SetParent(void *pParent, UINT8 uParentElementType);

	void	*GetParent()
	{
		if (m_pParent)
			return (void *)m_pParent;

		else
			return m_pSAFParent;
	}

	UINT8	GetParentElementType() const	{return m_uParentElementType;}
};

// commands
class CNewScene : public CLASeRUnit
{
protected:
	CSVG	*m_pSVG;								// child

public:
	CNewScene();

	lsrErr	AddChildren(LPXNode pScene);
	lsrErr	CheckRightChild(UINT8 uElementType);
};

class CAdd : public CLASeRUnit
{
protected:
	tstring	m_strRef;
	tstring	m_strAttributeName;
	UINT8	m_uAttrType;
	tstring m_strValue;
	tstring	m_strOperandElementID;
	tstring	m_strOperandAttribute;

public:
	CAdd();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);

	UINT	GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);
};

class CRefreshScene : public CLASeRUnit
{
protected:
	CSVG	*m_pSVG;								// child

	UINT	m_uTime;

public:
	CRefreshScene();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);

	lsrErr	AddChildren(LPXNode pScene);
	lsrErr	CheckRightChild(UINT8 uElementType);

	UINT	GetUINTAttr(UINT uAttrType, lsrErr *pErr);
};

class CInsert : public CLASeRUnit
{
protected:
	tstring	m_strRef;
	tstring	m_strAttributeName;
	tstring	m_strValue;
	UINT	m_uIndex;

public:
	CInsert();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);

	UINT	GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);
};

class CReplace : public CLASeRUnit
{
protected:
	tstring	m_strRef;
	tstring	m_strAttributeName;
	tstring	m_strValue;
	UINT	m_uIndex;
	tstring	m_strOperandElementID;
	tstring	m_strOperandAttribute;

public:
	CReplace();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);

	UINT	GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);
};

class CDelete : public CLASeRUnit
{
protected:
	tstring	m_strRef;
	UINT	m_uIndex;
	tstring	m_strAttributeName;

public:
	CDelete();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);

	UINT	GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);
};

class CSave : public CLASeRUnit
{
protected:
	tstring	m_strGroupID;
	tstring	m_strElements;
	tstring m_strAttributes;

public:
	CSave();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);

	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);
};

class CRestore : public CLASeRUnit
{
protected:
	tstring	m_strGroupID;

public:
	CRestore();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);

	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);
};

class CClean : public CLASeRUnit
{
protected:
	tstring	m_strGroupID;

public:
	CClean();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);

	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);
};

class CSendEvent : public CLASeRUnit
{
protected:
	tstring			m_strRef;
	UINT8			m_uEvent;
	deque<float>	m_afPointValue;
	UINT			m_uIntValue;
	tstring			m_strStringValue;

public:
	CSendEvent();
	~CSendEvent();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);
};

class CActivate : public CLASeRUnit
{
protected:
	tstring	m_strRef;

public:
	CActivate();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);

	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);
};

class CDeactivate : public CLASeRUnit
{
protected:
	tstring	m_strRef;

public:
	CDeactivate();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);

	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);
};

class CReleaseResource : public CLASeRUnit
{
protected:
	tstring	m_strRef;

public:
	CReleaseResource();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);

	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);
};

class CAdaptiveUpdateGroup : public CLASeRUnit
{
protected:
	deque<UINT>	m_auConstraints;

public:
	CAdaptiveUpdateGroup();
	~CAdaptiveUpdateGroup();

	lsrErr		SetAttrs(LPXNode pScene, CLASeR *pI);

	deque<UINT>	GetUINTListAttr(UINT uAttrType, lsrErr *pErr);
};
/************************************************************************/

#endif