///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "SAF.h"


CSAF::CSAF()
{
	m_uElementType		= 0;
	m_pParent			= NULL;
	m_uNumOfChildren	= 0;
	m_bHasStreamID		= 0;
	m_uMyIndex			= 0;
}

CSAF::~CSAF()
{
	// memory deletion
	for (deque<CSAF *>::iterator i = m_apChildren.begin(); i < m_apChildren.end(); ++i)
	{DELETEMEMORY(*i);}

	m_apChildren.clear();
}

/************************************************************************/
// CreateElement
// Desc		: Create New SAF elements
// Params	: DOM tree, New SAF Element pointer
// History
//	PSH (2008. 7. 14)	- first release
//	PSH (2008. 8. 2)	- move to SAF.cpp
/************************************************************************/
lsrErr CreateElement(LPXNode pReader, CSAF **outSAFElement)
{
	lsrErr err = LASeRNoErr;

	CSAF *pNewSAFElement = NULL;
	*outSAFElement = NULL;

	/************************************************************************/
	// memory allocating for new SAF element
	if (pReader->name == _T("saf:SAFSession"))
		pNewSAFElement = new CSAFSession();

	else if (pReader->name == _T("saf:sceneHeader"))
		pNewSAFElement = new CSceneHeader();

	else if (pReader->name == _T("saf:mediaHeader"))
		pNewSAFElement = new CMediaHeader();

	else if (pReader->name == _T("saf:imageHeader"))
		pNewSAFElement = new CImageHeader();

	else if (pReader->name == _T("saf:endOfStream"))
		pNewSAFElement = new CEndOfStream();

	else if (pReader->name == _T("saf:sceneUnit"))
		pNewSAFElement = new CSceneUnit();

	else if (pReader->name == _T("saf:mediaUnit"))
		pNewSAFElement = new CMediaUnit();

	else if (pReader->name == _T("saf:imageUnit"))
		pNewSAFElement = new CImageUnit();

	else if (pReader->name == _T("saf:endOfSAFSession"))
		pNewSAFElement = new CEndOfSAFSession();

	else
		err	= LASeRDOMTreeErr;

	/************************************************************************/
	// set attributes
	if (pNewSAFElement != NULL)
	{
		err = pNewSAFElement->SetAttrs(pReader);
		CONFIRM(err);
	}

	*outSAFElement = pNewSAFElement;

	return err;
}