///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#ifndef _SAF_H_
#define _SAF_H_

#include <deque>

#include "laser_error.h"
#include "base.h"
#include "LASeR.h"


class CSAFUnit : public CSAF
{
protected:
	UINT	m_uTime;							// 30 bits
	tstring	m_strRef;
	tstring	m_strSource;
	UINT	m_uStartOffset;
	UINT	m_uLength;
	UINT8	m_bRap;

public:
	CSAFUnit();

	lsrErr	SetAttrs(LPXNode pScene);

	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	UINT	GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);

protected:
	UINT	GetAttrType(UINT uAttrType);
};

/************************************************************************/
// 7.6. SimpleDecoderConfigDescriptor
class CESHeader : public CSAFUnit
{
protected:
	tstring	m_strStreamID;
	tstring	m_strDecSpecificInfo;
	UINT	m_uStartOffset;
	UINT	m_uLength;
	UINT8	m_bTransient;
	UINT	m_uBufferSize;
	UINT8	m_uObjectTypeIndication;
	UINT8	m_uStreamType;
	UINT	m_uTimeStampResolution;
	tstring	m_strContentType;

public:
	CESHeader();

	lsrErr	SetAttrs(LPXNode pScene);

	tstring	GetStreamID() const	{return m_strStreamID;}

	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	UINT	GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);
};

class CSceneHeader : public CESHeader
{
protected:
	CLASeRHeader	*m_pLASeRHeader;			// only one

public:
	CSceneHeader();
	~CSceneHeader();

	CLASeRHeader	*GetChildren()			{return m_pLASeRHeader;}
	UINT8			GetChildSize() const	{return 1;}

	lsrErr	SetAttrs(LPXNode pScene);

protected:
	lsrErr	AddChildren(LPXNode pScene);
};

class CMediaHeader : public CESHeader
{
public:
	CMediaHeader();

	lsrErr	SetAttrs(LPXNode pScene);
};

class CImageHeader : public CESHeader
{
public:
	CImageHeader();

	lsrErr	SetAttrs(LPXNode pScene);
};
/************************************************************************/

class CEndOfStream : public CSAF
{
protected:
	tstring	m_strRef;

public:
	CEndOfStream();

	lsrErr	SetAttrs(LPXNode pScene);
};

/************************************************************************/
// Access Unit
class CSceneUnit : public CSAFUnit
{
protected:
	deque<CLASeRUnit *>	m_apLASeRChildren;

	UINT8				m_bResetEncodingContext;

public:
	CSceneUnit();
	~CSceneUnit();

	// nonsense
	CLASeRUnit	*GetChildren(UINT uIndex)	{return m_apLASeRChildren[uIndex];}
	UINT		GetChildSize() const		{return (UINT)m_apLASeRChildren.size();}

	lsrErr		SetAttrs(LPXNode pScene);
	lsrErr		AddChildren(LPXNode pScene);
};

class CMediaUnit : public CSAFUnit
{
public:
	CMediaUnit();

	lsrErr	SetAttrs(LPXNode pScene);
};

class CImageUnit : public CSAFUnit
{
public:
	CImageUnit();

	lsrErr	SetAttrs(LPXNode pScene);
};
/************************************************************************/

class CEndOfSAFSession : public CSAF
{
public:
	CEndOfSAFSession()	{m_uElementType = SAF_EndOfSAFSession;}
};

class CCacheUnit : public CSAF
{
protected:
	UINT8	m_bReplace;
	UINT8	m_bPermanent;
	UINT16	m_uUrlLength;
	char	*m_szUrl;
};

class CRemoteStreamHeader : public CESHeader
{
protected:
	UINT16	m_uUrlLength;
	char	*m_szUrl;
};

class CGroupingDescriptor : public CSAF
{
protected:
	deque<tstring>	m_astrStreamIDs;			// required

public:
	CGroupingDescriptor(); 
	~CGroupingDescriptor();

	lsrErr	SetAttrs(LPXNode pScene);
};

class CSAFSession : public CSAF
{
protected:
	// children
	// exactly one
	CSceneHeader					*m_pSceneHeader;

	// unbounded
	deque<CMediaHeader *>			m_apMediaHeader;
	deque<CImageHeader *>			m_apImageHeader;
	deque<CEndOfStream *>			m_apEndOfStream;
	deque<CSceneUnit *>				m_apSceneUnit;
	deque<CMediaUnit *>				m_apMediaUnit;
	deque<CImageUnit *>				m_apImageUnit;
	deque<CCacheUnit *>				m_apCacheUnit;
	deque<CRemoteStreamHeader *>	m_apRemoteStreamHeader;

	// zero or one
	CEndOfSAFSession				*m_pEndOfSAFSession;
	
public:
	CSAFSession();
	~CSAFSession();

	lsrErr		SetAttrs(LPXNode pScene);
	lsrErr		AddChildren(LPXNode pScene);

	CSceneUnit	*GetSceneUnit(UINT8 uIndex)	{return m_apSceneUnit[uIndex];}
};

#endif