///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#ifndef _LSR_BASE_H_
#define _LSR_BASE_H_

#include "laser_error.h"
#include "XMLite.h"


class CLASeR
{
protected:
	UINT8			m_uElementType;

	CLASeR			*m_pParent;

	deque<CLASeR *>	m_apChildren;				// children list
	UINT8			m_uNumOfChildren;

	UINT8			m_bHasID;					// id flag
	UINT8			m_bDeactivation;			// deactivate flag

	UINT8			m_uMyIndex;					// my children index of my parent node

public:
	CLASeR();
	virtual ~CLASeR();

	UINT8			GetElementType() const			{return m_uElementType;} 

	virtual void	SetParent(CLASeR *pParent)		{m_pParent = pParent;}
	CLASeR			*GetParent()					{return m_pParent;}
	virtual UINT8	GetParentElementType() const	{return m_pParent->GetElementType();}

	deque<CLASeR *>	GetChildren() const				{return m_apChildren;}
	CLASeR			*GetChildren(UINT8 uIndex);

	lsrErr			SetChildrenSize(UINT8 uSize);
	UINT8			GetChildrenSize() const			{return m_uNumOfChildren;}

	UINT8			CheckHavingID() const			{return m_bHasID;}

	void			SetActivate();
	void			SetDeactivate();
	UINT8			GetActivation() const			{return m_bDeactivation;}
	void			InheritParentActivation()		{m_bDeactivation = m_pParent->GetActivation();}

	void			SetIndex(UINT8 uIndex)			{m_uMyIndex = uIndex;}
	UINT8			GetIndex() const				{return m_uMyIndex;}

	lsrErr			DeleteChild(UINT8 uIndex);
	lsrErr			InsertChild(UINT uIndex, CLASeR *pToBeAdded);
	lsrErr			ReplaceChild(UINT uIndex, CLASeR *pToBeReplaced);

	virtual lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI)	{AddChildren(pScene); pI; return LASeRNoErr;}
	virtual lsrErr	AddChildren(LPXNode pScene);
	virtual lsrErr	CheckRightChild(UINT8 uElementType)	{uElementType; return LASeRNoErr;}

	virtual float			GetFloatAttr(UINT uAttrType, lsrErr *pErr)
	{uAttrType; *pErr = LASeRAttrNotFoundErr; return 0.0;}
	virtual UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
	{uAttrType; *pErr = LASeRAttrNotFoundErr; return 0;}
	virtual UINT			GetUINTAttr(UINT uAttrType, lsrErr *pErr)
	{uAttrType; *pErr = LASeRAttrNotFoundErr; return 0;}
	virtual tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr)
	{uAttrType; *pErr = LASeRAttrNotFoundErr; return _T("");}
	virtual deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
	{uAttrType; *pErr = LASeRAttrNotFoundErr; deque<UINT8> auTemp;	return auTemp;}
	virtual deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
	{uAttrType; *pErr = LASeRAttrNotFoundErr; deque<float> afTemp;	return afTemp;}
	virtual deque<tstring>	GetStrListAttr(UINT uAttrType, lsrErr *pErr)
	{uAttrType; *pErr = LASeRAttrNotFoundErr; deque<tstring> astrTemp;	return astrTemp;}

	virtual UINT	GetAttrType(tstring strAttrName)	{strAttrName; return 0;}
	virtual UINT8	GetAttrValType(UINT uAttrType)		{uAttrType; return 0;}

	virtual lsrErr	SetAttr(UINT uAttrType, tstring strValue)
	{uAttrType; strValue; return LASeRFuncNotFoundErr;}
	virtual lsrErr	SetAttr(UINT uAttrType, float fValue)
	{uAttrType; fValue; return LASeRFuncNotFoundErr;}

	virtual	tstring	GetID() const			{return _T("");}
	virtual void	SetID(tstring strNewID)	{strNewID;}
};

class CSAF
{
protected:
	UINT8			m_uElementType;

	CSAF			*m_pParent;

	deque<CSAF *>	m_apChildren;				// children list
	UINT8			m_uNumOfChildren;

	UINT8			m_bHasStreamID;				// ID flag

	UINT8			m_uMyIndex;					// my children index of my parent node

public:
	CSAF();
	virtual ~CSAF();

	UINT8			GetElementType() const			{return m_uElementType;}

	void			SetParent(CSAF *pParent)		{m_pParent = pParent;}
	CSAF			*GetParent()					{return m_pParent;}
	UINT8			GetParentElementType() const	{return m_pParent->GetElementType();}

	deque<CSAF *>	GetChildren() const				{return m_apChildren;}
	CSAF			*GetChildren(UINT8 uIndex)		{return m_apChildren[uIndex];}
	virtual UINT8	GetChildrenSize() const			{return m_uNumOfChildren;}

	UINT8			CheckHavingID() const			{return m_bHasStreamID;}

	void			SetIndex(UINT8 uIndex)			{m_uMyIndex = uIndex;}
	UINT8			GetIndex() const				{return m_uMyIndex;}

	virtual lsrErr	AddChildren(LPXNode pScene)	{pScene; return LASeRNoErr;}
	virtual lsrErr	SetAttrs(LPXNode pScene)	{pScene; return LASeRNoErr;}
};

lsrErr CreateElement(LPXNode pReader, CLASeR **outLASeRElement);
lsrErr CreateElement(LPXNode pReader, CSAF **outSAFElement);

static deque<CLASeR *> apReusedElements;		// to remember reused elements

enum
{
	// elements
	SVG_a			= 1,
	SVG_animate,
	SVG_animateColor,
	SVG_animateMotion,
	SVG_animateTransform,
	SVG_audio,
	SVG_circle,
	SVG_defs,
	SVG_desc,
	SVG_ellipse,
	SVG_foreignObject,
	SVG_g,
	SVG_image,
	SVG_line,
	SVG_linearGradient,
	EV_listener,
	SVG_metadata,
	SVG_mpath,
	SVG_path,
	SVG_polygon,
	SVG_polyline,
	SVG_radialGradient,  
	SVG_rect,
	SVG_script,
	SVG_set,
	SVG_stop,
	SVG_svg,
	SVG_switch,
	SVG_text,
	SVG_title,
	SVG_tspan,
	SVG_use,
	SVG_video,

	// amd1
	SVG_animation,
	SVG_discard,
	SVGT_font,
	SVGT_fontface,
	SVGT_fontfacesrc,
	SVGT_fontfaceuri,
	SVGT_glyph,
	SVG_handler,
	SVGT_hkern,
	SVGT_missingglyph,
	SVG_prefetch,
	SVG_solidColor,
	SVG_tBreak,
	SVG_textArea,

	LSR_conditional	= 61,
	LSR_cursorManager,
	LSR_rectClip,
	LSR_selector,
	LSR_simpleLayout,
	// amd1
	LSR_streamSource,
	LSR_animateScroll,
	LSR_setScroll,
	LSR_updates,
	// amd2
	LSR_parsingSwitch,

	LSR_StringIDTable,

	// commands
	LSR_Add			= 81,
	LSR_Clean,
	LSR_Delete,
	LSR_Insert,
	LSR_NewScene,
	LSR_RefreshScene,
	LSR_Replace,
	LSR_Restore,
	LSR_Save,
	LSR_SendEvent,
	// amd1
	LSR_Activate,
	LSR_Deactivate,
	LSR_ReleaseResource,
	// amd2
	LSR_AdaptiveUpdateGroup,

	LSR_LASeRHeader	= 99,
};

enum
{
	SAF_SAFSession	= 100,
	SAF_SceneHeader,
	SAF_MediaHeader,
	SAF_ImageHeader,
	SAF_EndOfStream,
	SAF_SceneUnit,
	SAF_MediaUnit,
	SAF_ImageUnit,
	SAF_EndOfSAFSession,
	SAF_CacheUnit,
	SAF_RemoteStreamHeader,
	SAF_GroupingDescriptor,
	SAF_FirstFragmentUnit,
	SAF_FragmentUnit,
	SAF_SAFConfiguration,
	SAF_StopCache,
};

// attribute
enum
{
	// animatable
	ATTR_OPACITY				= 0,
	ATTR_CHOICE,
	ATTR_COLOR_RENDERING,
	ATTR_DISPLAY,
	ATTR_FONT_STYLE,		
	ATTR_PRESERVEASPECTRATIO,
	ATTR_VISIBILITY,
	ATTR_EDITABLE,
	ATTR_D,
	ATTR_FILL,
	ATTR_FILL_OPACITY,
	ATTR_FONT_FAMILY,
	ATTR_FONT_SIZE,
	ATTR_HEIGHT,
	ATTR_XLINK_HREF,
	ATTR_POINTS,
	ATTR_STROKE,
	ATTR_STROKE_OPACITY,
	ATTR_STROKE_WIDTH,
	ATTR_WIDTH,
	ATTR_VIEWBOX,
	ATTR_X1,
	ATTR_Y1,
	ATTR_X2,
	ATTR_Y2,
	ATTR_CX,
	ATTR_CY,
	ATTR_X,
	ATTR_Y,
	ATTR_RX,
	ATTR_RY,
	ATTR_R,
	ATTR_TRANSFORM,
	ATTR_SIZE,
	ATTR_DISPLAY_ALIGN,
	ATTR_COLOR,
	ATTR_AUDIO_LEVEL,
	ATTR_FILL_RULE,
	ATTR_FONT_STYLEX,							// ?? see laser-datatypes2.xsd
	ATTR_FONT_WEIGHT,
	ATTR_IMAGE_RENDERING,
	ATTR_LINE_INCREMENT,
	ATTR_POINTER_EVENTS,
	ATTR_SHAPE_RENDERING,
	ATTR_STOP_COLOR,
	ATTR_STOP_OPACITY,
	ATTR_SOLID_COLOR,
	ATTR_SOLID_OPACITY,
	ATTR_STROKE_DASHARRAY,
	ATTR_STROKE_DASHOFFSET,
	ATTR_STROKE_LINECAP,
	ATTR_STROKE_LINEJOIN,
	ATTR_STROKE_MITERLIMIT,
	ATTR_TEXT_ANCHOR,
	ATTR_VECTOR_EFFECT,
	ATTR_VIEWPORT_FILL,
	ATTR_VIEWPORT_FILL_OPACITY,
	ATTR_NAV_RIGHT,
	ATTR_NAV_NEXT,
	ATTR_NAV_UP,
	ATTR_NAV_UP_RIGHT,
	ATTR_NAV_UP_LEFT,
	ATTR_NAV_PREV,
	ATTR_NAV_DOWN,
	ATTR_NAV_DOWN_RIGHT,
	ATTR_NAV_DOWN_LEFT,
	ATTR_NAV_LEFT,
	ATTR_FOCUSABLE,
	ATTR_GRADIENTUNITS,
	ATTR_PATHLENGTH,
	ATTR_ROTATE,
	ATTR_TARGET,
	ATTR_TYPE,

	// svg object
	ATTR_ID,
	ATTR_XML_BASE,
	ATTR_XML_SPACE,
	ATTR_XML_LANG,
	ATTR_CLASS,
	ATTR_STYLE,
	// svg properties
	ATTR_TEXT_DECORATION,
	ATTR_FONT_VARIANT,
	ATTR_TEXT_RENDERING,
	// svg basic
	ATTR_REQUIREDFEATURES,
	ATTR_REQUIREDEXTENSIONS,
	ATTR_SYSTEMLANGUAGE,
	ATTR_REQUIREDFORMATS,
	ATTR_REQUIREDFONTS,
	// svg locatable
	ATTR_LSR_ROTATION,
	ATTR_LSR_SCALE,
	ATTR_LSR_TRANSLATION,
	ATTR_LSR_MATRIX,							// developer defined
	// svg href
	ATTR_XLINK_TITLE,
	ATTR_XLINK_TYPE,
	ATTR_XLINK_ROLE,
	ATTR_XLINK_ARCROLE,
	ATTR_XLINK_ACTUATE,
	ATTR_XLINK_SHOW,
	// svg animateAttr
	ATTR_FROM,
	ATTR_BY,
	ATTR_VALUES,
	ATTR_CALCMODE,
	ATTR_KEYTIMES,
	ATTR_KEYSPLINES,
	ATTR_ADDITIVE,
	ATTR_ACCUMULATE,
	// svg setAttrs
	ATTR_LSR_ENABLED,
	ATTR_BEGIN,
	ATTR_END,
	ATTR_DUR,
	ATTR_MIN,
	ATTR_MAX,
	ATTR_ATTRIBUTETYPE,
	ATTR_REPEATCOUNT,
	ATTR_REPEATDUR,
	ATTR_RESTART,
	ATTR_TO,
	// laser adaptationAttr
	ATTR_LSR_MINDISPLAYSIZE,
	ATTR_LSR_MINMEMORY,
	ATTR_LSR_PRIORITY,
	ATTR_LSR_MINRESOLUTION,
	ATTR_LSR_MINSIZE,
	ATTR_LSR_MINSIZEBEHAVIOR,
	ATTR_LSR_CONSTRAINTFILTER,

	// svg animate, animateColor
	ATTR_ATTRIBUTENAME,
	// svg animateMotion
	ATTR_PATH,
	ATTR_KEYPOINTS,
	// LASeR animateScroll
	ATTR_LSR_BY,
	ATTR_LSR_FROM,
	ATTR_LSR_TO,
	ATTR_LSR_DELAYATSTART,
	ATTR_LSR_DELAYATEND,
	ATTR_LSR_SPEED,
	ATTR_LSR_DIRECTION,
	// svg audio
	ATTR_EXTERNALRESOURCESREQUIRED,
	ATTR_SYNCBEHAVIOR,
	ATTR_SYNCTOLERANCE,
	ATTR_SYNCMASTER,
	ATTR_LSR_SYNCREFERENCE,
	ATTR_SMIL_CLIPBEGIN,
	ATTR_SMIL_CLIPEND,
	// svg circle
	ATTR_FOCUSHIGHLIGHT,
	// svg desc
	ATTR_TEXTCONTENT,
	// svg image
	ATTR_LSR_TRANSFORMBEHAVIOR,
	// LASeR selector
	ATTR_LSR_CHOICE,
	// LASeR simpleLayout
	ATTR_LSR_DELTA,
	// svg script
	ATTR_SCRIPT_TYPE,
	// svg stop
	ATTR_OFFSET,
	// svg svg
	ATTR_ZOOMANDPAN,
	ATTR_SYNCBEHAVIORDEFAULT,
	ATTR_SYNCTOLERANCEDEFAULT,
	ATTR_BASEPROFILE,
	ATTR_CONTENTSCRIPTTYPE,
	ATTR_PLAYBACKORDER,
	ATTR_SNAPSHOTTIME,
	ATTR_TIMELINEBEGIN,
	ATTR_VERSION,
	ATTR_LSR_FULLSCREEN,
	// svg use
	ATTR_OVERFLOW,
	// svg video
	ATTR_OVERLAY,
	ATTR_TRANSFORMBEHAVIOR,
	ATTR_INITIALVISIBILITY,
	// ev listener
	ATTR_EVENT,
	ATTR_HANDLER,
	ATTR_OBSERVER,
	ATTR_PHASE,
	ATTR_PROPAGATE,
	ATTR_DEFAULTICON,
	// svg font
	ATTR_HORIZ_ADV_X,
	ATTR_HORIZ_ORIGIN_X,
	// svg fontFace
	ATTR_ACCENT_HEIGHT,
	ATTR_ALPHABETIC,
	ATTR_ASCENT,
	ATTR_BBOX,
	ATTR_CAP_HEIGHT,
	ATTR_DESCENT,
	ATTR_FONT_STRETCH,
	ATTR_HANGING,
	ATTR_IDEOGRAPHIC,
	ATTR_MATHEMATICAL,
	ATTR_OVERLINE_POSITION,
	ATTR_OVERLINE_THICKNESS,
	ATTR_PANOSE_1,
	ATTR_SLOPE,
	ATTR_STEMH,
	ATTR_STEMV,
	ATTR_STRIKE_THROUGH_POSITION,
	ATTR_STRIKE_THROUGH_THICKNESS,
	ATTR_UNDERLINE_POSITION,
	ATTR_UNDERLINE_THICKNESS,
	ATTR_UNICODE_RANGE,
	ATTR_UNITS_PER_EM,
	ATTR_WIDTHS,
	ATTR_X_HEIGHT,
	// svg glyph
	ATTR_ARABIC_FORM,
	ATTR_GLYPH_NAME,
	ATTR_LANG,
	ATTR_UNICODE,
	// svg hkern
	ATTR_G1,
	ATTR_G2,
	ATTR_K,
	ATTR_U1,
	ATTR_U2,
	// svg prefetch
	ATTR_BANDWIDTH,
	ATTR_MEDIACHARACTERENCODING,
	ATTR_MEDIACONTENTENCODINGS,
	ATTR_MEDIASIZE,
	ATTR_MEDIATIME,
	// LASeR setScroll
	ATTR_LSR_INCREMENT,
	// LASeR updates
	ATTR_LSR_SECURITY,
	ATTR_LSR_FLOW,
	// LASeR parsingSwitch
	ATTR_LSR_MODE,

	// LASeRHeader
	ATTR_PROFILE,
	ATTR_LEVEL,
	ATTR_RESOLUTION,
	ATTR_TIMERESOLUTION,
	ATTR_COORDBITS,
	ATTR_SCALEBITSMINUSCOORDBITS,
	ATTR_COLORCOMPONENTBITS,
	ATTR_USEFULLREQUESTHOST,
	ATTR_PATHCOMPONENTS,
	ATTR_NEWSCENEINDICATOR,
	ATTR_TIMEENCODING,
	ATTR_POINTSCODECTYPE,

	ATTR_ADAPTATIONTYPE,
	ATTR_ADAPTATIONTYPEVALUE,
	ATTR_ADAPTATIONCONSTRAINTS,

	// LASeR Add
	ATTR_REF,
	ATTR_VALUE,
	ATTR_OPERANDELEMENTID,
	ATTR_OPERANDATTRIBUTE,

	// LASeR Clean
	ATTR_GROUPID,

	// LASeR Save
	ATTR_ELEMENTS,
	ATTR_ATTRIBUTES,

	// LASeR Replace
	ATTR_INDEX,

	// LASeR AdaptiveUpdateGroup
	ATTR_CONSTRAINTS,

	// SAFUnit
	ATTR_TIME,
	ATTR_SOURCE,
	ATTR_STARTOFFSET,
	ATTR_LENGTH,
	ATTR_RAP,

	// ESHeader
	ATTR_STREAMID,
	ATTR_DECSPECIFICINFO,
	ATTR_TRANSIENT,
	ATTR_BUFFERSIZE,
	ATTR_OBJECTTYPEINDICATION,
	ATTR_STREAMTYPE,
	ATTR_TIMESTAMPRESOLUTION,
	ATTR_CONTENTTYPE,

	// custom
	ATTR_WIDTH_UNIT,
	ATTR_HEIGHT_UNIT,
	ATTR_EVENT_PARAM,
};

enum											// attribute variable type
{
	FLOAT_TYPE			= 1,
	UINT8_TYPE,
	UINT32_TYPE,
	STRING_TYPE,
	UINT8_LIST_TYPE,
	UINT32_LIST_TYPE,
	STRING_LIST_TYPE,
	FLOAT_LIST_TYPE,
};

#endif