///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "..\ComplexType.h"
#include "..\Datatypes.h"


CLSRAdaptationAttr::CLSRAdaptationAttr()
{
	m_uMinDisplaySize		= 0;
	m_uPriority				= 0;
	m_uMinResolution		= 0;
	m_uMinSize				= 0;
	m_uMinSizeBehavior		= 0;
	m_strConstraintFilter	= _T("");
}

CLSRAdaptationAttr::~CLSRAdaptationAttr()
{
	m_afMinMemory.clear();
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'adaptationAttr' attribute group to set the attributes
// Param	: DOM tree
// History
//	PSH (2008. 11. 20)	- first release
//	PSH (2009. 6. 14)	- minMemory modification
/************************************************************************/
lsrErr CLSRAdaptationAttr::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR	*szTemp	= NULL;

	/************************************************************************/
	// set attribute
	// minDisplaySize
	m_uMinDisplaySize = XStr2Int(pScene->GetAttrValue(_T("minDisplaySize")));

	// minMemory
	szTemp = pScene->GetAttrValue(_T("minMemory"));
	if (szTemp)
	{
		m_afMinMemory = ReadFloatListType(szTemp, &err);
		CONFIRM(err);
	}

	// priority
	m_uPriority = (UINT8)XStr2Int(pScene->GetAttrValue(_T("priority")));

	// minResolution
	m_uMinResolution = XStr2Int(pScene->GetAttrValue(_T("minResolution")));

	// minSize
	m_uMinSize = XStr2Int(pScene->GetAttrValue(_T("minSize")));

	// minSizeBehaviour
	szTemp = pScene->GetAttrValue(_T("minSizeBehaviour"));
	if (szTemp)
	{
		m_uMinSizeBehavior = ReadMinSizeBehaviorType(szTemp, &err);
		CONFIRM(err);
	}

	// constraintFilter
	szTemp = pScene->GetAttrValue(_T("constraintFilter"));
	if (szTemp)
		m_strConstraintFilter = szTemp;
	/************************************************************************/

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'adaptationAttr' attribute group to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 11. 20)	- first release
/************************************************************************/
UINT8 CLSRAdaptationAttr::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_LSR_PRIORITY:
		return m_uPriority;
		break;

	case ATTR_LSR_MINSIZEBEHAVIOR:
		return m_uMinSizeBehavior;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return 0;
}

UINT CLSRAdaptationAttr::GetUINTAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_LSR_MINDISPLAYSIZE:
		return m_uMinDisplaySize;
		break;

	case ATTR_LSR_MINRESOLUTION:
		return m_uMinResolution;
		break;

	case ATTR_LSR_MINSIZE:
		return m_uMinSize;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return 0;
}

tstring CLSRAdaptationAttr::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_LSR_CONSTRAINTFILTER:
		return m_strConstraintFilter;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return _T("");
}

deque<float> CLSRAdaptationAttr::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_LSR_MINMEMORY:
		return m_afMinMemory;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	// nonsense
	deque<float> afTemp;
	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'adaptationAttr' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 11. 20)       - first release
/************************************************************************/
UINT CLSRAdaptationAttr::GetAttrType(tstring strAttrName)
{
	if (strAttrName == _T("minDisplaySize"))		return ATTR_LSR_MINDISPLAYSIZE;
	else if (strAttrName == _T("minMemory"))		return ATTR_LSR_MINMEMORY;
	else if (strAttrName == _T("priority"))			return ATTR_LSR_PRIORITY;
	else if (strAttrName == _T("minResolution"))	return ATTR_LSR_MINRESOLUTION;
	else if (strAttrName == _T("minSize"))			return ATTR_LSR_MINSIZE;
	else if (strAttrName == _T("minSizeBehaviour"))	return ATTR_LSR_MINSIZEBEHAVIOR;
	else if (strAttrName == _T("constraintFilter"))	return ATTR_LSR_CONSTRAINTFILTER;
	else											return 0;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'adaptationAttr' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 20)       - first release
/************************************************************************/
UINT8 CLSRAdaptationAttr::GetAttrValType(UINT uAttrType)
{
	switch (uAttrType)
	{
	case ATTR_LSR_MINDISPLAYSIZE:
	case ATTR_LSR_MINRESOLUTION:
	case ATTR_LSR_MINSIZE:
		return UINT32_TYPE;
		break;

	case ATTR_LSR_MINMEMORY:
	case ATTR_LSR_CONSTRAINTFILTER:
		return STRING_TYPE;
		break;

	case ATTR_LSR_PRIORITY:
	case ATTR_LSR_MINSIZEBEHAVIOR:
		return UINT8_TYPE;
		break;

	default:
		return 0;
		break;
	}
}