///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "..\ComplexType.h"
#include "..\Datatypes.h"


CSVGAnimateAttr::CSVGAnimateAttr()
{
	m_strFrom		= _T("");
	m_uCalcMode		= 1;
	m_strKeySplines	= _T("");
	m_uAdditive		= 1;
	m_uAccumulate	= 1;
}

CSVGAnimateAttr::~CSVGAnimateAttr()
{
	m_afBy.clear();
	m_afValues.clear();
	m_afKeyTimes.clear();
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'animateAttr' attribute group to set the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 18)	- first release
//	PSH (2008. 10. 31)	- add the rest
/************************************************************************/
lsrErr CSVGAnimateAttr::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR	*szTemp	= NULL;

	/************************************************************************/
	// set attribute
	// from
	szTemp = pScene->GetAttrValue(_T("from"));
	if (szTemp)
	{
		m_afFrom = ReadFloatListType(szTemp, &err);
		CONFIRM(err);
	}

	// by
	szTemp = pScene->GetAttrValue(_T("by"));
	if (szTemp)
	{
		m_afBy = ReadAnimatedValuesType(szTemp, &err);
		CONFIRM(err);
	}

	// values
	szTemp = pScene->GetAttrValue(_T("values"));
	if (szTemp)
	{
		m_afValues = ReadAnimatedValuesType(szTemp, &err);
		CONFIRM(err);
	}

	// calMode
	szTemp = pScene->GetAttrValue(_T("calcMode"));
	if (szTemp)
	{
		m_uCalcMode = ReadCalcModeType(szTemp, &err);
		CONFIRM(err);
	}

	// keyTimes
	szTemp = pScene->GetAttrValue(_T("keyTimes"));
	if (szTemp)
	{
		m_afKeyTimes = ReadFloatListType(szTemp, ';', &err);
		CONFIRM(err);
	}

	// keySplines
	szTemp = pScene->GetAttrValue(_T("keySplines"));
	if (szTemp)
		m_strKeySplines = szTemp;

	// additive
	szTemp = pScene->GetAttrValue(_T("additive"));
	if (szTemp)
	{
		m_uAdditive = ReadAdditiveType(szTemp, &err);
		CONFIRM(err);
	}

	// accumulate
	szTemp = pScene->GetAttrValue(_T("accumulate"));
	if (szTemp)
	{
		m_uAccumulate = ReadAccumulateType(szTemp, &err);
		CONFIRM(err);
	}
	/************************************************************************/

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'animateAttr' attribute group to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 8. 28)	- first release
/************************************************************************/
UINT8 CSVGAnimateAttr::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_CALCMODE:
		return m_uCalcMode;
		break;

	case ATTR_ADDITIVE:
		return m_uAdditive;
		break;

	case ATTR_ACCUMULATE:
		return m_uAccumulate;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return 0;
}

tstring CSVGAnimateAttr::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_KEYSPLINES:
		return m_strKeySplines;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return _T("");
}

deque<float> CSVGAnimateAttr::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_FROM:
		return m_afFrom;
		break;

	case ATTR_BY:
		return m_afBy;
		break;

	case ATTR_VALUES:
		return m_afValues;
		break;

	case ATTR_KEYTIMES:
		return m_afKeyTimes;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	deque<float> afTemp;
	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'animateAttr' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CSVGAnimateAttr::GetAttrType(tstring strAttrName)
{
	if (strAttrName == _T("from"))				return ATTR_FROM;
	else if (strAttrName == _T("by"))			return ATTR_BY;
	else if (strAttrName == _T("values"))		return ATTR_VALUES;
	else if (strAttrName == _T("calcMode"))		return ATTR_CALCMODE;
	else if (strAttrName == _T("keyTimes"))		return ATTR_KEYTIMES;
	else if (strAttrName == _T("keySplines"))	return ATTR_KEYSPLINES;
	else if (strAttrName == _T("additive"))		return ATTR_ADDITIVE;
	else if (strAttrName == _T("accumulate"))	return ATTR_ACCUMULATE;
	else										return 0;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'animateAttr' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 7)       - first release
/************************************************************************/
UINT8 CSVGAnimateAttr::GetAttrValType(UINT uAttrType)
{
	switch (uAttrType)
	{
	case ATTR_BY:
	case ATTR_VALUES:
	case ATTR_KEYTIMES:
		return FLOAT_LIST_TYPE;
		break;

	case ATTR_CALCMODE:
	case ATTR_ADDITIVE:
	case ATTR_ACCUMULATE:
		return UINT8_TYPE;
		break;

	case ATTR_FROM:
	case ATTR_KEYSPLINES:
		return STRING_TYPE;
		break;

	default:
		return 0;
		break;
	}
}