///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "..\ComplexType.h"
#include "..\Datatypes.h"


CSVGBasic::CSVGBasic()
{
	m_strRequiredExtensions	= _T("");
	m_strSystemLanguage		= _T("");
	m_strRequiredFormats	= _T("");
	m_strRequiredFonts		= _T("");
}

CSVGBasic::~CSVGBasic()
{
	m_auRequiredFeatures.clear();
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'basic' attribute group to set the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 16)	- first release
/************************************************************************/
lsrErr CSVGBasic::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR	*szTemp	= NULL;

	/************************************************************************/
	// set attribute
	// requiredFeatures
	szTemp = pScene->GetAttrValue(_T("requiredFeatures"));
	if (szTemp)
	{
		m_auRequiredFeatures = ReadFeatureListType(szTemp, &err);
		CONFIRM(err);
	}

	// requiredExtensions
	szTemp = pScene->GetAttrValue(_T("requiredExtensions"));
	if (szTemp)
		m_strRequiredExtensions = szTemp;

	// systemLanguage
	szTemp = pScene->GetAttrValue(_T("systemLanguage"));
	if (szTemp)
		m_strSystemLanguage = szTemp;

	// requiredFormats
	szTemp = pScene->GetAttrValue(_T("requiredFormats"));
	if (szTemp)
		m_strRequiredFormats = szTemp;

	// requiredFonts
	szTemp = pScene->GetAttrValue(_T("requiredFonts"));
	if (szTemp)
		m_strRequiredFonts = szTemp;
	/************************************************************************/

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'basic' attribute group to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 8. 28)	- first release
/************************************************************************/
tstring CSVGBasic::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_REQUIREDEXTENSIONS:
		return m_strRequiredExtensions;
		break;

	case ATTR_SYSTEMLANGUAGE:
		return m_strSystemLanguage;
		break;

	case ATTR_REQUIREDFORMATS:
		return m_strRequiredFormats;
		break;

	case ATTR_REQUIREDFONTS:
		return m_strRequiredFonts;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return _T("");
}

deque<UINT8> CSVGBasic::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_REQUIREDFEATURES:
		return m_auRequiredFeatures;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	deque<UINT8> auTemp;
	return auTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'basic' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CSVGBasic::GetAttrType(tstring strAttrName)
{
	if (strAttrName == _T("requiredFeatures"))			return ATTR_REQUIREDFEATURES;
	else if (strAttrName == _T("requiredExtensions"))	return ATTR_REQUIREDEXTENSIONS;
	else if (strAttrName == _T("systemLanguage"))		return ATTR_SYSTEMLANGUAGE;
	else if (strAttrName == _T("requiredFormats"))		return ATTR_REQUIREDFORMATS;
	else if (strAttrName == _T("requiredFonts"))		return ATTR_REQUIREDFONTS;
	else												return 0;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'basic' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 7)       - first release
/************************************************************************/
UINT8 CSVGBasic::GetAttrValType(UINT uAttrType)
{
	switch (uAttrType)
	{
	case ATTR_REQUIREDFEATURES:
		return UINT8_LIST_TYPE;
		break;

	case ATTR_REQUIREDEXTENSIONS:
	case ATTR_SYSTEMLANGUAGE:
	case ATTR_REQUIREDFORMATS:
	case ATTR_REQUIREDFONTS:
		return STRING_TYPE;
		break;

	default:
		return 0;
		break;
	}
}