///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "..\ComplexType.h"
#include "..\Datatypes.h"

#include <algorithm>


CSVGFocus::CSVGFocus()
{
	m_strNavRight		= _T("");
	m_strNavNext		= _T("");
	m_strNavUp			= _T("");
	m_strNavUpRight		= _T("");
	m_strNavUpLeft		= _T("");
	m_strNavPrev		= _T("");
	m_strNavDown		= _T("");
	m_strNavDownRight	= _T("");
	m_strNavDownLeft	= _T("");
	m_strNavLeft		= _T("");
	m_uFocusable		= 0;
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'focus' attribute group to set the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 16)	- first release
/************************************************************************/
lsrErr CSVGFocus::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR	*szTemp	= NULL;

	/************************************************************************/
	// set attribute
	// nav-right
	szTemp = pScene->GetAttrValue(_T("nav-right"));
	if (szTemp)
	{
		// if url used
		if (!_tcsnicmp(szTemp, _T("url(#"), 5))
		{
			m_strNavRight = &szTemp[5];

			// remove ')'
			m_strNavRight.erase(remove(m_strNavRight.begin(), m_strNavRight.end(), ')'), m_strNavRight.end());
		}

		else
			m_strNavRight = szTemp;
	}

	// nav-next
	szTemp = pScene->GetAttrValue(_T("nav-next"));
	if (szTemp)
	{
		// if url used
		if (!_tcsnicmp(szTemp, _T("url(#"), 5))
		{
			m_strNavNext = &szTemp[5];

			// remove ')'
			m_strNavNext.erase(remove(m_strNavNext.begin(), m_strNavNext.end(), ')'), m_strNavNext.end());
		}

		else
			m_strNavNext = szTemp;
	}

	// nav-up
	szTemp = pScene->GetAttrValue(_T("nav-up"));
	if (szTemp)
	{
		// if url used
		if (!_tcsnicmp(szTemp, _T("url(#"), 5))
		{
			m_strNavUp = &szTemp[5];

			// remove ')'
			m_strNavUp.erase(remove(m_strNavUp.begin(), m_strNavUp.end(), ')'), m_strNavUp.end());
		}

		else
			m_strNavUp = szTemp;
	}

	// nav-up-right
	szTemp = pScene->GetAttrValue(_T("nav-up-right"));
	if (szTemp)
	{
		// if url used
		if (!_tcsnicmp(szTemp, _T("url(#"), 5))
		{
			m_strNavUpRight = &szTemp[5];

			// remove ')'
			m_strNavUpRight.erase(remove(m_strNavUpRight.begin(), m_strNavUpRight.end(), ')'), m_strNavUpRight.end());
		}

		else
			m_strNavUpRight = szTemp;
	}

	// nav-up-left
	szTemp = pScene->GetAttrValue(_T("nav-up-left"));
	if (szTemp)
	{
		// if url used
		if (!_tcsnicmp(szTemp, _T("url(#"), 5))
		{
			m_strNavUpLeft = &szTemp[5];

			// remove ')'
			m_strNavUpLeft.erase(remove(m_strNavUpLeft.begin(), m_strNavUpLeft.end(), ')'), m_strNavUpLeft.end());
		}

		else
			m_strNavUpLeft = szTemp;
	}

	// nav-prev
	szTemp = pScene->GetAttrValue(_T("nav-prev"));
	if (szTemp)
	{
		// if url used
		if (!_tcsnicmp(szTemp, _T("url(#"), 5))
		{
			m_strNavPrev = &szTemp[5];

			// remove ')'
			m_strNavPrev.erase(remove(m_strNavPrev.begin(), m_strNavPrev.end(), ')'), m_strNavPrev.end());
		}

		else
			m_strNavPrev = szTemp;
	}

	// nav-down
	szTemp = pScene->GetAttrValue(_T("nav-down"));
	if (szTemp)
	{
		// if url used
		if (!_tcsnicmp(szTemp, _T("url(#"), 5))
		{
			m_strNavDown = &szTemp[5];

			// remove ')'
			m_strNavDown.erase(remove(m_strNavDown.begin(), m_strNavDown.end(), ')'), m_strNavDown.end());
		}

		else
			m_strNavDown = szTemp;
	}

	// nav-down-right
	szTemp = pScene->GetAttrValue(_T("nav-down-right"));
	if (szTemp)
	{
		// if url used
		if (!_tcsnicmp(szTemp, _T("url(#"), 5))
		{
			m_strNavDownRight = &szTemp[5];

			// remove ')'
			m_strNavDownRight.erase(remove(m_strNavDownRight.begin(), m_strNavDownRight.end(), ')'), m_strNavDownRight.end());
		}

		else
			m_strNavDownRight = szTemp;
	}

	// nav-down-left
	szTemp = pScene->GetAttrValue(_T("nav-down-left"));
	if (szTemp)
	{
		// if url used
		if (!_tcsnicmp(szTemp, _T("url(#"), 5))
		{
			m_strNavDownRight = &szTemp[5];

			// remove ')'
			m_strNavDownLeft.erase(remove(m_strNavDownLeft.begin(), m_strNavDownLeft.end(), ')'), m_strNavDownLeft.end());
		}

		else
			m_strNavDownLeft = szTemp;
	}

	// nav-left
	szTemp = pScene->GetAttrValue(_T("nav-left"));
	if (szTemp)
	{
		// if url used
		if (!_tcsnicmp(szTemp, _T("url(#"), 5))
		{
			m_strNavLeft = &szTemp[5];

			// remove ')'
			m_strNavLeft.erase(remove(m_strNavLeft.begin(), m_strNavLeft.end(), ')'), m_strNavLeft.end());
		}

		else
			m_strNavLeft = szTemp;
	}

	// focusable
	szTemp = pScene->GetAttrValue(_T("focusable"));
	if (szTemp)
	{
		m_uFocusable = ReadFocusableType(szTemp, &err);
		CONFIRM(err);
	}
	/************************************************************************/

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'focus' attribute group to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 8. 28)	- first release
/************************************************************************/
UINT8 CSVGFocus::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_FOCUSABLE:
		return m_uFocusable;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return 0;
}

tstring CSVGFocus::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_NAV_RIGHT:
		return m_strNavRight;
		break;

	case ATTR_NAV_NEXT:
		return m_strNavNext;
		break;

	case ATTR_NAV_UP:
		return m_strNavUp;
		break;

	case ATTR_NAV_UP_RIGHT:
		return m_strNavUpRight;
		break;

	case ATTR_NAV_UP_LEFT:
		return m_strNavUpLeft;
		break;

	case ATTR_NAV_PREV:
		return m_strNavPrev;
		break;

	case ATTR_NAV_DOWN:
		return m_strNavDown;
		break;

	case ATTR_NAV_DOWN_RIGHT:
		return m_strNavDownRight;
		break;

	case ATTR_NAV_DOWN_LEFT:
		return m_strNavDownLeft;
		break;

	case ATTR_NAV_LEFT:
		return m_strNavLeft;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return _T("");
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'href' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CSVGFocus::GetAttrType(tstring strAttrName)
{
	if (strAttrName == _T("nav-right"))				return ATTR_LSR_ROTATION;
	else if (strAttrName == _T("nav-next"))			return ATTR_LSR_SCALE;
	else if (strAttrName == _T("nav-up"))			return ATTR_LSR_TRANSLATION;
	else if (strAttrName == _T("nav-up-right"))		return ATTR_TRANSFORM;
	else if (strAttrName == _T("nav-up-left"))		return ATTR_LSR_SCALE;
	else if (strAttrName == _T("nav-prev"))			return ATTR_LSR_TRANSLATION;
	else if (strAttrName == _T("nav-down"))			return ATTR_TRANSFORM;
	else if (strAttrName == _T("nav-down-right"))	return ATTR_LSR_SCALE;
	else if (strAttrName == _T("nav-down-left"))	return ATTR_LSR_TRANSLATION;
	else if (strAttrName == _T("nav-left"))			return ATTR_TRANSFORM;
	else if (strAttrName == _T("focusable"))		return ATTR_LSR_SCALE;
	else											return 0;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'href' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 7)       - first release
/************************************************************************/
UINT8 CSVGFocus::GetAttrValType(UINT uAttrType)
{
	switch (uAttrType)
	{
	case ATTR_NAV_RIGHT:
	case ATTR_NAV_NEXT:
	case ATTR_NAV_UP:
	case ATTR_NAV_UP_RIGHT:
	case ATTR_NAV_UP_LEFT:
	case ATTR_NAV_PREV:
	case ATTR_NAV_DOWN:
	case ATTR_NAV_DOWN_RIGHT:
	case ATTR_NAV_DOWN_LEFT:
	case ATTR_NAV_LEFT:
		return STRING_TYPE;
		break;

	case ATTR_FOCUSABLE:
		return UINT8_TYPE;
		break;

	default:
		return 0;
		break;
	}
}

/************************************************************************/
// SetAttr
// Desc		: function of 'focus' attribute group to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 8. 28)	- first release
/************************************************************************/
lsrErr CSVGFocus::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_NAV_RIGHT:
		m_strNavRight = strValue;
		break;

	case ATTR_NAV_NEXT:
		m_strNavNext = strValue;
		break;

	case ATTR_NAV_UP:
		m_strNavUp = strValue;
		break;

	case ATTR_NAV_UP_RIGHT:
		m_strNavUpRight = strValue;
		break;

	case ATTR_NAV_UP_LEFT:
		m_strNavUpLeft = strValue;
		break;

	case ATTR_NAV_PREV:
		m_strNavPrev = strValue;
		break;

	case ATTR_NAV_DOWN:
		m_strNavDown = strValue;
		break;

	case ATTR_NAV_DOWN_RIGHT:
		m_strNavDownRight = strValue;
		break;

	case ATTR_NAV_DOWN_LEFT:
		m_strNavDownLeft = strValue;
		break;

	case ATTR_NAV_LEFT:
		m_strNavLeft = strValue;
		break;

	case ATTR_FOCUSABLE:
		m_uFocusable = ReadFocusableType(strValue.c_str(), &err);
		CONFIRM(err);
		break;

	default:
		err = LASeRAttrNotFoundErr;
		break;
	}

	return err;
}