///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "..\ComplexType.h"
#include "..\Datatypes.h"

#include <algorithm>


CSVGHref::CSVGHref()
{
	m_strHref		= _T("");
	m_strTitle		= _T("");
	m_uType			= 0;
	m_strRole		= _T("");
	m_strArcrole	= _T("");
	m_uActuate		= 0;
	m_uShow			= 0;
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'href' attribute group to set the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 16)	- first release
/************************************************************************/
lsrErr CSVGHref::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR	*szTemp	= NULL;

	/************************************************************************/
	// set attribute
	// xlink:href
	szTemp = pScene->GetAttrValue(_T("xlink:href"));
	if (szTemp)
	{
		m_strHref = szTemp;
		m_strHref.erase(remove(m_strHref.begin(), m_strHref.end(), '#'), m_strHref.end());
	}

	// xlink:title
	szTemp = pScene->GetAttrValue(_T("xlink:title"));
	if (szTemp)
		m_strTitle = szTemp;

	// xlink:type
	szTemp = pScene->GetAttrValue(_T("xlink:type"));
	if (szTemp)
	{
		m_uType = ReadTypeType(szTemp, &err);
		CONFIRM(err);
	}

	// xlink:role
	szTemp = pScene->GetAttrValue(_T("xlink:role"));
	if (szTemp)
		m_strRole = szTemp;

	// xlink:arcrole
	szTemp = pScene->GetAttrValue(_T("xlink:arcrole"));
	if (szTemp)
		m_strArcrole = szTemp;

	// xlink:actuate
	szTemp = pScene->GetAttrValue(_T("xlink:actuate"));
	if (szTemp)
	{
		m_uActuate = ReadActuateType(szTemp, &err);
		CONFIRM(err);
	}

	// xlink:show
	szTemp = pScene->GetAttrValue(_T("xlink:show"));
	if (szTemp)
	{
		m_uShow = ReadShowType(szTemp, &err);
		CONFIRM(err);
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'href' attribute group to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 8. 28)	- first release
/************************************************************************/
UINT8 CSVGHref::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_XLINK_TYPE:
		return m_uType;
		break;

	case ATTR_XLINK_ACTUATE:
		return m_uActuate;
		break;

	case ATTR_XLINK_SHOW:
		return m_uShow;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return 0;
}

tstring CSVGHref::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_XLINK_HREF:
		return m_strHref;
		break;

	case ATTR_XLINK_TITLE:
		return m_strTitle;
		break;

	case ATTR_XLINK_ROLE:
		return m_strRole;
		break;

	case ATTR_XLINK_ARCROLE:
		return m_strArcrole;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return _T("");
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'href' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CSVGHref::GetAttrType(tstring strAttrName)
{
	if (strAttrName == _T("xlink:href"))			return ATTR_XLINK_HREF;
	else if (strAttrName == _T("xlink:title"))		return ATTR_XLINK_TITLE;
	else if (strAttrName == _T("xlink:type"))		return ATTR_XLINK_TYPE;
	else if (strAttrName == _T("xlink:role"))		return ATTR_XLINK_ROLE;
	else if (strAttrName == _T("xlink:arcrole"))	return ATTR_XLINK_ARCROLE;
	else if (strAttrName == _T("xlink:actuate"))	return ATTR_XLINK_ACTUATE;
	else if (strAttrName == _T("xlink:show"))		return ATTR_XLINK_SHOW;
	else											return 0;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'href' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 7)       - first release
/************************************************************************/
UINT8 CSVGHref::GetAttrValType(UINT uAttrType)
{
	switch (uAttrType)
	{
	case ATTR_XLINK_HREF:
	case ATTR_XLINK_TITLE:
	case ATTR_XLINK_ROLE:
	case ATTR_XLINK_ARCROLE:
		return STRING_TYPE;
		break;

	case ATTR_XLINK_TYPE:
	case ATTR_XLINK_ACTUATE:
	case ATTR_XLINK_SHOW:
		return UINT8_TYPE;
		break;

	default:
		return 0;
		break;
	}
}

/************************************************************************/
// SetAttr
// Desc		: function of 'href' attribute group to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 8. 28)	- first release
/************************************************************************/
lsrErr CSVGHref::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_XLINK_HREF:
		m_strHref = strValue;
		break;

	default:
		err = LASeRAttrNotFoundErr;
		break;
	}

	return err;
}