///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "..\ComplexType.h"
#include "..\Datatypes.h"


CSVGLocatable::CSVGLocatable()
{
	m_afRotation.assign(2, 0);
	m_afScale.assign(2, 1);
	m_afTranslation.assign(2, 0);

	m_strTransform = _T("");
}

CSVGLocatable::~CSVGLocatable()
{
	m_afRotation.clear();
	m_afScale.clear();
	m_afTranslation.clear();
	m_afMatrix.clear();
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'locatable' attribute group to set the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 16)	- first release
/************************************************************************/
lsrErr CSVGLocatable::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR	*szTemp		= NULL;
	CLASeR		*pParent	= pI->GetParent();

	// temporary variable
	deque<float> afTemp;

	/************************************************************************/
	// set attribute
	// rotation
	szTemp = pScene->GetAttrValue(_T("lsr:rotation"));
	if (szTemp)
		m_afRotation = ReadTransformType(szTemp, _T("rotate"), &err);

	else if (pParent->GetElementType() == SVG_g)
	{
		afTemp = pParent->GetFloatListAttr(ATTR_LSR_ROTATION, &err);

		if (err == LASeRNoErr)					m_afRotation = afTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;

		afTemp.clear();
	}

	// scale
	szTemp = pScene->GetAttrValue(_T("lsr:scale"));
	if (szTemp)
		m_afScale = ReadTransformType(szTemp, _T("scale"), &err);

	else if (pParent->GetElementType() == SVG_g)
	{
		afTemp = pParent->GetFloatListAttr(ATTR_LSR_SCALE, &err);

		if (err == LASeRNoErr)					m_afScale = afTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;

		afTemp.clear();
	}

	// translation
	szTemp = pScene->GetAttrValue(_T("lsr:translation"));
	if (szTemp)
		m_afTranslation = ReadTransformType(szTemp, _T("translate"), &err);

	else if (pParent->GetElementType() == SVG_g)
	{
		afTemp = pParent->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);

		if (err == LASeRNoErr)					m_afTranslation = afTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;

		afTemp.clear();
	}

	// transform
	szTemp = pScene->GetAttrValue(_T("transform"));
	if (szTemp)
	{
		m_strTransform = szTemp;

		if (_tcsstr(szTemp, _T("rotate")))
			m_afRotation = ReadTransformType(szTemp, _T("rotate"), &err);

		if (_tcsstr(szTemp, _T("scale")))
			m_afScale = ReadTransformType(szTemp, _T("scale"), &err);

		if (_tcsstr(szTemp, _T("translate")))
			m_afTranslation = ReadTransformType(szTemp, _T("translate"), &err);

		if (_tcsstr(szTemp, _T("matrix")))
			m_afMatrix = ReadTransformType(szTemp, _T("matrix"), &err);
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'locatable' attribute group to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 8. 28)	- first release
/************************************************************************/
tstring CSVGLocatable::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_TRANSFORM:
		return m_strTransform;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return _T("");
}

deque<float> CSVGLocatable::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<float> afTemp;

	switch (uAttrType)
	{
	case ATTR_LSR_ROTATION:
		return m_afRotation;
		break;

	case ATTR_LSR_SCALE:
		return m_afScale;
		break;

	case ATTR_LSR_TRANSLATION:
		return m_afTranslation;
		break;

	case ATTR_LSR_MATRIX:
		return m_afMatrix;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'locatable' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CSVGLocatable::GetAttrType(tstring strAttrName)
{
	if (strAttrName == _T("lsr:rotation"))			return ATTR_LSR_ROTATION;
	else if (strAttrName == _T("lsr:scale"))		return ATTR_LSR_SCALE;
	else if (strAttrName == _T("lsr:translation"))	return ATTR_LSR_TRANSLATION;
	else if (strAttrName == _T("transform"))		return ATTR_TRANSFORM;
	else											return 0;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'locatable' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 7)       - first release
/************************************************************************/
UINT8 CSVGLocatable::GetAttrValType(UINT uAttrType)
{
	switch (uAttrType)
	{
	case ATTR_LSR_ROTATION:
	case ATTR_LSR_SCALE:
	case ATTR_LSR_TRANSLATION:
		return FLOAT_TYPE;
		break;

	case ATTR_TRANSFORM:
		return STRING_TYPE;
		break;

	default:
		return 0;
		break;
	}
}

/************************************************************************/
// SetAttr
// Desc		: function of 'locatable' attribute group to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 8. 28)	- first release
/************************************************************************/
lsrErr CSVGLocatable::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_TRANSFORM:
		m_strTransform = strValue;
		CONFIRM(err);
		break;

	default:
		err = LASeRAttrNotFoundErr;
		break;
	}

	return err;
}

lsrErr CSVGLocatable::SetAttr(UINT uAttrType, float fValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_LSR_ROTATION:
		m_afRotation.clear();
		m_afRotation.push_back(fValue);
		break;

	case ATTR_LSR_SCALE:
		m_afScale.clear();
		m_afScale.push_back(fValue);
		m_afScale.push_back(fValue);
		break;

	default:
		err = LASeRAttrNotFoundErr;
		break;
	}

	return err;
}