///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "..\ComplexType.h"
#include "..\Datatypes.h"


CSVGMediaProperties::CSVGMediaProperties()
{
	m_fAudioLevel			= 1.0;
	m_uDisplay				= 0;
	m_uImageRendering		= 0;
	m_uPointerEvents		= 1;
	m_uShapeRendering		= 0;
	m_uTextRendering		= 0;
	m_fViewportFillOpacity	= 1.0;
	m_uVisibility			= 0;
}

CSVGMediaProperties::~CSVGMediaProperties()
{
	m_auViewportFill.clear();
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'mediaProperties' attribute group to set the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 16)	- first release
/************************************************************************/
lsrErr CSVGMediaProperties::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR	*szTemp		= NULL;
	CLASeR		*pParent	= pI->GetParent();

	// temporary variable
	UINT8			uTemp	= 0;
	tstring			strTemp	= _T("");
	deque<UINT8>	auTemp;
	deque<float>	afTemp;

	/************************************************************************/
	// set attribute
	// audio-level
	szTemp = pScene->GetAttrValue(_T("audio-level"));
	if (szTemp)
	{
		if (!_tcscmp(szTemp, _T("inherit")))
		{
			m_fAudioLevel = pParent->GetFloatAttr(ATTR_AUDIO_LEVEL, &err);
			CONFIRM(err);
		}

		else
		{
			m_fAudioLevel = (float)_tstof(szTemp);

			// zero to one
			if ((m_fAudioLevel < 0) || (m_fAudioLevel > 1))
			{
				err = LASeRAttrNotFoundErr;
				return err;
			}
		}
	}

	// display
	szTemp = pScene->GetAttrValue(_T("display"));
	if (szTemp)
	{
		if (!_tcscmp(szTemp, _T("inherit")))
		{
			m_uDisplay = pParent->GetUINT8Attr(ATTR_DISPLAY, &err);
			CONFIRM(err);
		}

		m_uDisplay = ReadDisplayType(szTemp, &err);
		CONFIRM(err);
	}

	/************************************************************************/
	// image-rendering
	szTemp = pScene->GetAttrValue(_T("image-rendering"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uImageRendering = ReadImageRenderingType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_IMAGE_RENDERING, &err);

		if (err == LASeRNoErr)
			m_uImageRendering = uTemp;
	}

	/************************************************************************/
	// pointer-events
	szTemp = pScene->GetAttrValue(_T("pointer-events"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uPointerEvents = ReadPointerEventsType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_POINTER_EVENTS, &err);

		if (err == LASeRNoErr)
			m_uPointerEvents = uTemp;
	}

	/************************************************************************/
	// shape-rendering
	szTemp = pScene->GetAttrValue(_T("shape-rendering"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uShapeRendering = ReadShapeRenderingType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_SHAPE_RENDERING, &err);

		if (err == LASeRNoErr)
			m_uShapeRendering = uTemp;
	}

	/************************************************************************/
	// text-rendering
	szTemp = pScene->GetAttrValue(_T("text-rendering"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uTextRendering = ReadTextRenderingType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_TEXT_RENDERING, &err);

		if (err == LASeRNoErr)
			m_uTextRendering = uTemp;
	}
	/************************************************************************/

	// viewport-fill
	szTemp = pScene->GetAttrValue(_T("viewport-fill"));
	if (szTemp)
	{
		if (!_tcscmp(szTemp, _T("inherit")))
		{
			m_auViewportFill = pParent->GetUINT8ListAttr(ATTR_VIEWPORT_FILL, &err);
			CONFIRM(err);
		}

		else
		{
			if (_tcscmp(szTemp, _T("none")))
			{
				m_auViewportFill = ReadColorType(szTemp, &err);
				CONFIRM(err);
			}
		}
	}

	// viewport-fill-opacity
	szTemp = pScene->GetAttrValue(_T("viewport-fill-opacity"));
	if (szTemp)
	{
		if (!_tcscmp(szTemp, _T("inherit")))
		{
			m_fViewportFillOpacity = pParent->GetFloatAttr(ATTR_VIEWPORT_FILL_OPACITY, &err);
			CONFIRM(err);
		}

		else
		{
			m_fViewportFillOpacity = (float)_tstof(szTemp);

			// zero to one
			if ((m_fViewportFillOpacity < 0) || (m_fViewportFillOpacity > 1))
			{
				err = LASeRAttrNotFoundErr;
				return err;
			}
		}
	}

	/************************************************************************/
	// visibility
	szTemp = pScene->GetAttrValue(_T("visibility"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uVisibility = ReadVisibilityType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_VISIBILITY, &err);

		if (err == LASeRNoErr)
			m_uVisibility = uTemp;
	}
	/************************************************************************/

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'mediaProperties' attribute group to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 8. 28)	- first release
/************************************************************************/
float CSVGMediaProperties::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_AUDIO_LEVEL:
		return m_fAudioLevel;
		break;

	case ATTR_VIEWPORT_FILL_OPACITY:
		return m_fViewportFillOpacity;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return 0.0;
}

UINT8 CSVGMediaProperties::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_DISPLAY:
		return m_uDisplay;
		break;

	case ATTR_IMAGE_RENDERING:
		return m_uImageRendering;
		break;

	case ATTR_POINTER_EVENTS:
		return m_uPointerEvents;
		break;

	case ATTR_SHAPE_RENDERING:
		return m_uShapeRendering;
		break;

	case ATTR_TEXT_RENDERING:
		return m_uTextRendering;
		break;

	case ATTR_VISIBILITY:
		return m_uVisibility;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return 0;
}

deque<UINT8> CSVGMediaProperties::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_VIEWPORT_FILL:
		return m_auViewportFill;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	deque<UINT8> auTemp;
	return auTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'mediaProperties' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CSVGMediaProperties::GetAttrType(tstring strAttrName)
{
	if (strAttrName == _T("audio-level"))					return ATTR_AUDIO_LEVEL;
	else if (strAttrName == _T("display"))					return ATTR_DISPLAY;
	else if (strAttrName == _T("image-rendering"))			return ATTR_IMAGE_RENDERING;
	else if (strAttrName == _T("pointer-events"))			return ATTR_POINTER_EVENTS;
	else if (strAttrName == _T("shape-rendering"))			return ATTR_SHAPE_RENDERING;
	else if (strAttrName == _T("text-rendering"))			return ATTR_TEXT_RENDERING;
	else if (strAttrName == _T("viewport-fill"))			return ATTR_VIEWPORT_FILL;
	else if (strAttrName == _T("viewport-fill-opacity"))	return ATTR_VIEWPORT_FILL_OPACITY;
	else if (strAttrName == _T("visibility"))				return ATTR_VISIBILITY;
	else													return 0;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'mediaProperties' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 7)       - first release
/************************************************************************/
UINT8 CSVGMediaProperties::GetAttrValType(UINT uAttrType)
{
	switch (uAttrType)
	{
	case ATTR_AUDIO_LEVEL:
	case ATTR_VIEWPORT_FILL_OPACITY:
		return FLOAT_TYPE;
		break;

	case ATTR_DISPLAY:
	case ATTR_IMAGE_RENDERING:
	case ATTR_POINTER_EVENTS:
	case ATTR_SHAPE_RENDERING:
	case ATTR_TEXT_RENDERING:
	case ATTR_VISIBILITY:
		return UINT8_TYPE;
		break;

	case ATTR_VIEWPORT_FILL:
		return UINT8_LIST_TYPE;
		break;

	default:
		return 0;
		break;
	}
}

/************************************************************************/
// SetAttr
// Desc		: function of 'mediaProperties' attribute group to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 8. 28)	- first release
/************************************************************************/
lsrErr CSVGMediaProperties::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szValue = strValue.c_str();

	switch (uAttrType)
	{
	case ATTR_AUDIO_LEVEL:
		m_fAudioLevel = (float)_tstof(szValue);

		// zero to one
		if ((m_fAudioLevel < 0) || (m_fAudioLevel > 1))
		{
			err = LASeRAttrNotFoundErr;
			return err;
		}

		break;

	case ATTR_DISPLAY:
		m_uDisplay = ReadDisplayType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_IMAGE_RENDERING:
		m_uImageRendering = ReadImageRenderingType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_POINTER_EVENTS:
		m_uPointerEvents = ReadPointerEventsType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_SHAPE_RENDERING:
		m_uShapeRendering = ReadShapeRenderingType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_VIEWPORT_FILL:
		m_auViewportFill = ReadColorType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_VIEWPORT_FILL_OPACITY:
		m_fViewportFillOpacity = (float)_tstof(szValue);

		// zero to one
		if ((m_fViewportFillOpacity < 0) || (m_fViewportFillOpacity > 1))
		{
			err = LASeRAttrNotFoundErr;
			return err;
		}

		break;

	case ATTR_VISIBILITY:
		m_uVisibility = ReadVisibilityType(szValue, &err);
		CONFIRM(err);
		break;

	default:
		err = LASeRAttrNotFoundErr;
		break;
	}

	return err;
}

lsrErr CSVGMediaProperties::SetAttr(UINT uAttrType, float fValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_AUDIO_LEVEL:
		m_fAudioLevel = fValue;

		// zero to one
		if ((m_fAudioLevel < 0) || (m_fAudioLevel > 1))
		{
			err = LASeRAttrNotFoundErr;
			return err;
		}

		break;

	case ATTR_VIEWPORT_FILL_OPACITY:
		m_fViewportFillOpacity = fValue;

		// zero to one
		if ((m_fViewportFillOpacity < 0) || (m_fViewportFillOpacity > 1))
		{
			err = LASeRAttrNotFoundErr;
			return err;
		}

		break;

	default:
		err = LASeRAttrNotFoundErr;
		break;
	}

	return err;
}