///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "..\ComplexType.h"
#include "..\Datatypes.h"


CSVGObject::CSVGObject()
{
	CLASeR::CLASeR();

	m_strID					= _T("");
	m_pXMLBase				= new CXMLBase();
	m_strClass				= _T("");
	m_strStyle				= _T("");

	m_pXMLBase->m_strBase	= _T("");
	m_pXMLBase->m_uSpace	= 0;
	m_pXMLBase->m_strLang	= _T("");
}

CSVGObject::~CSVGObject()
{
	DELETEMEMORY(m_pXMLBase);
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'object' attribute group to set the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 16)	- first release
/************************************************************************/
lsrErr CSVGObject::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pXMLBase)
	{
		err = LASeRNoMoreMemoryErr;
		return err;
	}

	const TCHAR	*szTemp	= NULL;

	/************************************************************************/
	// set attribute
	// id
	szTemp = pScene->GetAttrValue(_T("id"));
	if (szTemp)
	{
		m_strID		= szTemp;
		m_bHasID	= 1;
	}

	// xml:base
	szTemp = pScene->GetAttrValue(_T("xml:base"));
	if (szTemp)
		m_pXMLBase->m_strBase = szTemp;

	// xml:space
	szTemp = pScene->GetAttrValue(_T("xml:space"));
	if (szTemp)
	{
		m_pXMLBase->m_uSpace = ReadSpaceType(szTemp, &err);
		CONFIRM(err);
	}

	// xml:lang
	szTemp = pScene->GetAttrValue(_T("xml:lang"));
	if (szTemp)
		m_pXMLBase->m_strLang = szTemp;

	// class
	szTemp = pScene->GetAttrValue(_T("class"));
	if (szTemp)
		m_strClass = szTemp;

	// style
	szTemp = pScene->GetAttrValue(_T("style"));
	if (szTemp)
		m_strClass = szTemp;

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'object' attribute group to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 8. 28)	- first release
/************************************************************************/
UINT8 CSVGObject::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_XML_SPACE:
		return m_pXMLBase->m_uSpace;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return 0;
}

tstring CSVGObject::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_ID:
		return m_strID;
		break;

	case ATTR_XML_BASE:
		return m_pXMLBase->m_strBase;
		break;

	case ATTR_XML_LANG:
		return m_pXMLBase->m_strLang;
		break;

	case ATTR_CLASS:
		return m_strClass;
		break;

	case ATTR_STYLE:
		return m_strStyle;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return _T("");
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'object' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CSVGObject::GetAttrType(tstring strAttrName)
{
	if (strAttrName == _T("id"))				return ATTR_ID;
	else if (strAttrName == _T("xml:base"))		return ATTR_XML_BASE;
	else if (strAttrName == _T("xml:space"))	return ATTR_XML_SPACE;
	else if (strAttrName == _T("xml:lang"))		return ATTR_XML_LANG;
	else if (strAttrName == _T("class"))		return ATTR_CLASS;
	else if (strAttrName == _T("style"))		return ATTR_STYLE;
	else										return 0;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'object' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 7)       - first release
/************************************************************************/
UINT8 CSVGObject::GetAttrValType(UINT uAttrType)
{
	switch (uAttrType)
	{
	case ATTR_ID:
	case ATTR_XML_BASE:
	case ATTR_XML_LANG:
	case ATTR_CLASS:
	case ATTR_STYLE:
		return STRING_TYPE;
		break;

	case ATTR_XML_SPACE:
		return UINT8_TYPE;
		break;

	default:
		return 0;
		break;
	}
}

/************************************************************************/
// SetID
// Desc         : function of 'object' attribute group to set new id
// Param        : new id string
// History
//	PSH (2009. 12. 17)       - first release
/************************************************************************/
void CSVGObject::SetID(tstring strNewID)
{
	m_strID = strNewID;

	if (strNewID == _T(""))
		m_bHasID = 0;

	else
		m_bHasID = 1;
}