///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "..\ComplexType.h"
#include "..\Datatypes.h"


CSVGProperties::CSVGProperties()
{
	m_fAudioLevel			= 1.0;
	m_auColor.assign(3, 0);
	m_uColorRendering		= 0;
	m_uDisplay				= 0;
	m_uDisplayAlign			= 0;
	m_auFill.assign(3, 0);
	m_strFillUrl			= _T("");
	m_fFillOpacity			= 1.0;
	m_uFillRule				= 0;
	m_strFontFamily			= _T("arial");
	m_fFontSize				= 10.0;				// developer defined, medium?
	m_uFontStyle			= 0;
	m_uTextDecoration		= 0;	
	m_uFontWeight			= 0;
	m_uFontVariant			= 0;
	m_uImageRendering		= 0;
	m_fLineIncrement		= 0.0;
	m_uPointerEvents		= 1;
	m_uShapeRendering		= 0;
	m_auSolidColor.assign(3, 0);
	m_fSolidOpacity			= 1.0;
	m_auStopColor.assign(3, 0);
	m_fStopOpacity			= 1.0;
	m_strStrokeUrl			= _T("");
	m_fStrokeDashOffset		= 0.0;
	m_uStrokeLineCap		= 0;
	m_uStrokeLineJoin		= 0;
	m_fStrokeMiterLimit		= 4.0;
	m_fStrokeOpacity		= 1.0;
	m_fStrokeWidth			= 1.0;
	m_uTextAnchor			= 0;
	m_uTextRendering		= 0;
	m_fViewportFillOpacity	= 1.0;
	m_uVectorEffect			= 0;
	m_uVisibility			= 0;
}

CSVGProperties::~CSVGProperties()
{
	m_auColor.clear();
	m_auFill.clear();
	m_auSolidColor.clear();
	m_auStroke.clear();
	m_afStrokeDashArray.clear();
	m_auViewportFill.clear();
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'properties' attribute group to set the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 16)	- first release
/************************************************************************/
lsrErr CSVGProperties::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR	*szTemp		= NULL;
	CLASeR		*pParent	= pI->GetParent();

	// temporary variable
	float			fTemp	= 0.0;
	UINT8			uTemp	= 0;
	tstring			strTemp	= _T("");
	deque<UINT8>	auTemp;
	deque<float>	afTemp;

	/************************************************************************/
	// set attributes
	// audio-level
	szTemp = pScene->GetAttrValue(_T("audio-level"));
	if (szTemp)
	{
		if (!_tcscmp(szTemp, _T("inherit")))
		{
			m_fAudioLevel = pParent->GetFloatAttr(ATTR_AUDIO_LEVEL, &err);
			CONFIRM(err);
		}

		else
		{
			m_fAudioLevel = (float)_tstof(szTemp);

			// zero to one
			if ((m_fAudioLevel < 0) || (m_fAudioLevel > 1))
			{
				err = LASeRAttrNotFoundErr;
				return err;
			}
		}
	}

	/************************************************************************/
	// color
	szTemp = pScene->GetAttrValue(_T("color"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_auColor = ReadColorType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		auTemp = pParent->GetUINT8ListAttr(ATTR_COLOR, &err);

		if (err == LASeRNoErr)					m_auColor = auTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;

		auTemp.clear();
	}

	/************************************************************************/
	// color-rendering
	szTemp = pScene->GetAttrValue(_T("color-rendering"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uColorRendering = ReadColorRenderingType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_COLOR_RENDERING, &err);

		if (err == LASeRNoErr)					m_uColorRendering = uTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}
	/************************************************************************/

	// display
	szTemp = pScene->GetAttrValue(_T("display"));
	if (szTemp)
	{
		if (!_tcscmp(szTemp, _T("inherit")))
		{
			m_uDisplay = pParent->GetUINT8Attr(ATTR_DISPLAY, &err);
			CONFIRM(err);
		}

		m_uDisplay = ReadDisplayType(szTemp, &err);
		CONFIRM(err);
	}

	/************************************************************************/
	// display-align
	szTemp = pScene->GetAttrValue(_T("display-align"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uDisplayAlign = ReadDisplayAlignType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_DISPLAY_ALIGN, &err);

		if (err == LASeRNoErr)					m_uDisplayAlign = uTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}

	/************************************************************************/
	// fill
	szTemp = pScene->GetAttrValue(_T("fill"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		// if "currentColor", get the color variable
		if (!_tcscmp(szTemp, _T("currentColor")))
			m_auFill = m_auColor;

		// if "none"
		else if (!_tcscmp(szTemp, _T("none")))
			m_auFill.clear();

		// if url used
		else if (!_tcsnicmp(szTemp, _T("url(#"), 5))
			m_strFillUrl = &szTemp[5];

		else
		{
			m_auFill = ReadColorType(szTemp, &err);
			CONFIRM(err);
		}
	}

	else
	{
		auTemp = pParent->GetUINT8ListAttr(ATTR_FILL, &err);

		if (err == LASeRNoErr)					m_auFill = auTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;

		auTemp.clear();
	}

	/************************************************************************/
	// fill-opacity
	szTemp = pScene->GetAttrValue(_T("fill-opacity"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_fFillOpacity = (float)_tstof(szTemp);

		// zero to one
		if ((m_fFillOpacity < 0) || (m_fFillOpacity > 1))
		{
			err = LASeRAttrNotFoundErr;
			return err;
		}
	}

	else
	{
		fTemp = pParent->GetFloatAttr(ATTR_FILL_OPACITY, &err);
		
		if (err == LASeRNoErr)					m_fFillOpacity = fTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}
	
	/************************************************************************/
	// fill-rule
	szTemp = pScene->GetAttrValue(_T("fill-rule"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uFillRule = ReadFillRuleType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_FILL_RULE, &err);
		
		if (err == LASeRNoErr)					m_uFillRule = uTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}

	/************************************************************************/
	// font-family
	szTemp = pScene->GetAttrValue(_T("font-family"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
		m_strFontFamily = szTemp;

	else
	{
		strTemp = pParent->GetStrAttr(ATTR_FONT_FAMILY, &err);

		if (err == LASeRNoErr)					m_strFontFamily = strTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}

	/************************************************************************/
	// font-size
	szTemp = pScene->GetAttrValue(_T("font-size"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
		m_fFontSize = (float)_tstof(szTemp);

	else
	{
		fTemp = pParent->GetFloatAttr(ATTR_FONT_SIZE, &err);

		if (err == LASeRNoErr)					m_fFontSize = fTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}

	/************************************************************************/
	// font-style
	szTemp = pScene->GetAttrValue(_T("font-style"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uFontStyle = ReadFontStyleType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_FONT_STYLE, &err);

		if (err == LASeRNoErr)					m_uFontStyle = uTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}
	/************************************************************************/

	// text-decoration
	szTemp = pScene->GetAttrValue(_T("text-decoration"));
	if (szTemp)
	{
		if (!_tcscmp(szTemp, _T("inherit")))
		{
			m_uTextDecoration = pParent->GetUINT8Attr(ATTR_TEXT_DECORATION, &err);
			CONFIRM(err);
		}

		else
		{
			m_uTextDecoration = ReadTextDecorationType(szTemp, &err);
			CONFIRM(err);
		}
	}

	/************************************************************************/
	// font-weight
	szTemp = pScene->GetAttrValue(_T("font-weight"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uFontWeight = ReadFontWeightType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_FONT_WEIGHT, &err);

		if (err == LASeRNoErr)					m_uFontWeight = uTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}

	/************************************************************************/
	// font-variant
	szTemp = pScene->GetAttrValue(_T("font-variant"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uFontVariant = ReadFontVariantType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_FONT_VARIANT, &err);

		if (err == LASeRNoErr)					m_uFontVariant = uTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}

	/************************************************************************/
	// image-rendering
	szTemp = pScene->GetAttrValue(_T("image-rendering"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uImageRendering = ReadImageRenderingType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_IMAGE_RENDERING, &err);

		if (err == LASeRNoErr)					m_uImageRendering = uTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}

	/************************************************************************/
	// line-increment
	szTemp = pScene->GetAttrValue(_T("line-increment"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		if (!_tcscmp(szTemp, _T("auto")))	m_fLineIncrement = 0;
		else								m_fLineIncrement = (float)_tstof(szTemp);
	}

	else
	{
		fTemp = pParent->GetFloatAttr(ATTR_LINE_INCREMENT, &err);

		if (err == LASeRNoErr)					m_fLineIncrement = fTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}

	/************************************************************************/
	// pointer-events
	szTemp = pScene->GetAttrValue(_T("pointer-events"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uPointerEvents = ReadPointerEventsType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_POINTER_EVENTS, &err);

		if (err == LASeRNoErr)					m_uPointerEvents = uTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}

	/************************************************************************/
	// shape-rendering
	szTemp = pScene->GetAttrValue(_T("shape-rendering"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uShapeRendering = ReadShapeRenderingType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_SHAPE_RENDERING, &err);

		if (err == LASeRNoErr)					m_uShapeRendering = uTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}
	/************************************************************************/

	// solid-color
	szTemp = pScene->GetAttrValue(_T("solid-color"));
	if (szTemp)
	{
		if (!_tcscmp(szTemp, _T("inherit")))
		{
			m_auSolidColor = pParent->GetUINT8ListAttr(ATTR_SOLID_COLOR, &err);
			CONFIRM(err);
		}

		else
		{
			// if "currentColor", get the color variable
			if (!_tcscmp(szTemp, _T("currentColor")))
				m_auSolidColor = m_auColor;

			else
			{
				m_auSolidColor = ReadColorType(szTemp, &err);
				CONFIRM(err);
			}
		}
	}

	// solid-opacity
	szTemp = pScene->GetAttrValue(_T("solid-opacity"));
	if (szTemp)
	{
		if (!_tcscmp(szTemp, _T("inherit")))
		{
			m_fSolidOpacity = pParent->GetFloatAttr(ATTR_SOLID_OPACITY, &err);
			CONFIRM(err);
		}

		else
		{
			m_fSolidOpacity = (float)_tstof(szTemp);

			// zero to one
			if ((m_fSolidOpacity < 0) || (m_fSolidOpacity > 1))
			{
				err = LASeRAttrNotFoundErr;
				return err;
			}
		}
	}

	// stop-color
	szTemp = pScene->GetAttrValue(_T("stop-color"));
	if (szTemp)
	{
		if (!_tcscmp(szTemp, _T("inherit")))
		{
			m_auStopColor = pParent->GetUINT8ListAttr(ATTR_STOP_COLOR, &err);
			CONFIRM(err);
		}

		else
		{
			// if "currentColor", get the color variable
			if (!_tcscmp(szTemp, _T("currentColor")))
				m_auStopColor = m_auColor;

			else
			{
				m_auStopColor = ReadColorType(szTemp, &err);
				CONFIRM(err);
			}
		}
	}

	// stop-opacity
	szTemp = pScene->GetAttrValue(_T("stop-opacity"));
	if (szTemp)
	{
		if (!_tcscmp(szTemp, _T("inherit")))
		{
			m_fStopOpacity = pParent->GetFloatAttr(ATTR_STOP_OPACITY, &err);
			CONFIRM(err);
		}

		else
		{
			m_fStopOpacity = (float)_tstof(szTemp);

			// zero to one
			if ((m_fStopOpacity < 0) || (m_fStopOpacity > 1))
			{
				err = LASeRAttrNotFoundErr;
				return err;
			}
		}
	}

	/************************************************************************/
	// stroke
	szTemp = pScene->GetAttrValue(_T("stroke"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		// if "currentColor", get the color variable
		if (!_tcscmp(szTemp, _T("currentColor")))
			m_auStroke = m_auColor;

		// if "none"
		else if (!_tcscmp(szTemp, _T("none")))
			m_auStroke.clear();

		// if url used
		else if (!_tcsnicmp(szTemp, _T("url(#"), 5))
			m_strStrokeUrl = &szTemp[5];

		else
		{
			m_auStroke = ReadColorType(szTemp, &err);
			CONFIRM(err);
		}
	}

	else
	{
		auTemp = pParent->GetUINT8ListAttr(ATTR_STROKE, &err);

		if (err == LASeRNoErr)					m_auStroke = auTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;

		auTemp.clear();
	}

	/************************************************************************/
	// stroke-dasharray
	szTemp = pScene->GetAttrValue(_T("stroke-dasharray"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		if (_tcscmp(szTemp, _T("none")))
		{
			deque<tstring> m_astrTemp;

			m_astrTemp = ReadStringListType(szTemp, ',', &err);
			CONFIRM(err);


		}
	}

	else
	{
		afTemp = pParent->GetFloatListAttr(ATTR_STROKE_DASHARRAY, &err);

		if (err == LASeRNoErr)					m_afStrokeDashArray = afTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;

		afTemp.clear();
	}

	/************************************************************************/
	// stroke-dashoffset
	szTemp = pScene->GetAttrValue(_T("stroke-dashoffset"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
		m_fStrokeDashOffset = (float)_tstof(szTemp);

	else
	{
		fTemp = pParent->GetFloatAttr(ATTR_STROKE_DASHOFFSET, &err);

		if (err == LASeRNoErr)					m_fStrokeDashOffset = fTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}

	/************************************************************************/
	// stroke-linecap
	szTemp = pScene->GetAttrValue(_T("stroke-linecap"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uStrokeLineCap = ReadStrokeLineCapType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_STROKE_LINECAP, &err);

		if (err == LASeRNoErr)					m_uStrokeLineCap = uTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}

	/************************************************************************/
	// stroke-linejoin
	szTemp = pScene->GetAttrValue(_T("stroke-linejoin"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uStrokeLineJoin = ReadStrokeLineJoinType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_STROKE_LINEJOIN, &err);

		if (err == LASeRNoErr)					m_uStrokeLineJoin = uTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}

	/************************************************************************/
	// stroke-miterlimit
	szTemp = pScene->GetAttrValue(_T("stroke-miterlimit"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
		m_fStrokeMiterLimit = (float)_tstof(szTemp);

	else
	{
		fTemp = pParent->GetFloatAttr(ATTR_STROKE_MITERLIMIT, &err);

		if (err == LASeRNoErr)					m_fStrokeMiterLimit = fTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}

	/************************************************************************/
	// stroke-opacity
	szTemp = pScene->GetAttrValue(_T("stroke-opacity"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_fStrokeOpacity = (float)_tstof(szTemp);

		// zero to one
		if ((m_fStrokeOpacity < 0) || (m_fStrokeOpacity > 1))
		{
			err = LASeRAttrNotFoundErr;
			return err;
		}
	}

	else
	{
		fTemp = pParent->GetFloatAttr(ATTR_STROKE_OPACITY, &err);

		if (err == LASeRNoErr)					m_fStrokeOpacity = fTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}

	/************************************************************************/
	// stroke-width
	szTemp = pScene->GetAttrValue(_T("stroke-width"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
		m_fStrokeWidth = (float)_tstof(szTemp);

	else
	{
		fTemp = pParent->GetFloatAttr(ATTR_STROKE_WIDTH, &err);

		if (err == LASeRNoErr)					m_fStrokeWidth = fTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}

	/************************************************************************/
	// text-anchor
	szTemp = pScene->GetAttrValue(_T("text-anchor"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uTextAnchor = ReadTextAnchorType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_TEXT_ANCHOR, &err);

		if (err == LASeRNoErr)					m_uTextAnchor = uTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}

	/************************************************************************/
	// text-rendering
	szTemp = pScene->GetAttrValue(_T("text-rendering"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uTextRendering = ReadTextRenderingType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_TEXT_RENDERING, &err);

		if (err == LASeRNoErr)					m_uTextRendering = uTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}
	/************************************************************************/

	// viewport-fill
	szTemp = pScene->GetAttrValue(_T("viewport-fill"));
	if (szTemp)
	{
		if (!_tcscmp(szTemp, _T("inherit")))
		{
			m_auViewportFill = pParent->GetUINT8ListAttr(ATTR_VIEWPORT_FILL, &err);
			CONFIRM(err);
		}

		else
		{
			if (_tcscmp(szTemp, _T("none")))
			{
				m_auViewportFill = ReadColorType(szTemp, &err);
				CONFIRM(err);
			}
		}
	}

	// viewport-fill-opacity
	szTemp = pScene->GetAttrValue(_T("viewport-fill-opacity"));
	if (szTemp)
	{
		if (!_tcscmp(szTemp, _T("inherit")))
		{
			m_fViewportFillOpacity = pParent->GetFloatAttr(ATTR_VIEWPORT_FILL_OPACITY, &err);
			CONFIRM(err);
		}

		else
		{
			m_fViewportFillOpacity = (float)_tstof(szTemp);

			// zero to one
			if ((m_fViewportFillOpacity < 0) || (m_fViewportFillOpacity > 1))
			{
				err = LASeRAttrNotFoundErr;
				return err;
			}
		}
	}

	// vector-effect
	szTemp = pScene->GetAttrValue(_T("vector-effect"));
	if (szTemp)
	{
		if (!_tcscmp(szTemp, _T("inherit")))
		{
			m_uVectorEffect = pParent->GetUINT8Attr(ATTR_VECTOR_EFFECT, &err);
			CONFIRM(err);
		}

		else
		{
			m_uVectorEffect = ReadVectorType(szTemp, &err);
			CONFIRM(err);
		}
	}

	/************************************************************************/
	// visibility
	szTemp = pScene->GetAttrValue(_T("visibility"));
	if (szTemp && _tcscmp(szTemp, _T("inherit")))
	{
		m_uVisibility = ReadVisibilityType(szTemp, &err);
		CONFIRM(err);
	}

	else
	{
		uTemp = pParent->GetUINT8Attr(ATTR_VISIBILITY, &err);

		if (err == LASeRNoErr)					m_uVisibility = uTemp;
		else if (err == LASeRAttrNotFoundErr)	err = LASeRNoErr;
	}
	/************************************************************************/

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'properties' attribute group to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 8. 26)	- first release
/************************************************************************/
float CSVGProperties::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_AUDIO_LEVEL:
		return m_fAudioLevel;
		break;

	case ATTR_FILL_OPACITY:
		return m_fFillOpacity;
		break;

	case ATTR_FONT_SIZE:
		return m_fFontSize;
		break;

	case ATTR_LINE_INCREMENT:
		return m_fLineIncrement;
		break;

	case ATTR_SOLID_OPACITY:
		return m_fSolidOpacity;
		break;

	case ATTR_STOP_OPACITY:
		return m_fStopOpacity;
		break;

	case ATTR_STROKE_DASHOFFSET:
		return m_fStrokeDashOffset;
		break;

	case ATTR_STROKE_MITERLIMIT:
		return m_fStrokeMiterLimit;
		break;

	case ATTR_STROKE_OPACITY:
		return m_fStrokeOpacity;
		break;

	case ATTR_STROKE_WIDTH:
		return m_fStrokeWidth;
		break;

	case ATTR_VIEWPORT_FILL_OPACITY:
		return m_fViewportFillOpacity;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return 0.0;
}

UINT8 CSVGProperties::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_COLOR_RENDERING:
		return m_uColorRendering;
		break;

	case ATTR_DISPLAY:
		return m_uDisplay;
		break;

	case ATTR_DISPLAY_ALIGN:
		return m_uDisplayAlign;
		break;

	case ATTR_FILL_RULE:
		return m_uFillRule;
		break;

	case ATTR_FONT_STYLE:
		return m_uFontStyle;
		break;

	case ATTR_TEXT_DECORATION:
		return m_uTextDecoration;
		break;

	case ATTR_FONT_WEIGHT:
		return m_uFontWeight;
		break;

	case ATTR_FONT_VARIANT:
		return m_uFontVariant;
		break;

	case ATTR_IMAGE_RENDERING:
		return m_uImageRendering;
		break;

	case ATTR_POINTER_EVENTS:
		return m_uPointerEvents;
		break;

	case ATTR_SHAPE_RENDERING:
		return m_uShapeRendering;
		break;

	case ATTR_STROKE_LINECAP:
		return m_uStrokeLineCap;
		break;

	case ATTR_STROKE_LINEJOIN:
		return m_uStrokeLineJoin;
		break;

	case ATTR_TEXT_ANCHOR:
		return m_uTextAnchor;
		break;

	case ATTR_TEXT_RENDERING:
		return m_uTextRendering;
		break;

	case ATTR_VECTOR_EFFECT:
		return m_uVectorEffect;
		break;

	case ATTR_VISIBILITY:
		return m_uVisibility;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return 0;
}

tstring CSVGProperties::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_FILL:
		return m_strFillUrl;
		break;

	case ATTR_FONT_FAMILY:
		return m_strFontFamily;
		break;

	case ATTR_STROKE:
		return m_strStrokeUrl;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return _T("");
}

deque<UINT8> CSVGProperties::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_COLOR:
		return m_auColor;
		break;

	case ATTR_FILL:
		return m_auFill;
		break;

	case ATTR_SOLID_COLOR:
		return m_auSolidColor;
		break;

	case ATTR_STOP_COLOR:
		return m_auStopColor;
		break;

	case ATTR_STROKE:
		return m_auStroke;
		break;

	case ATTR_VIEWPORT_FILL:
		return m_auViewportFill;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	deque<UINT8> auTemp;
	return auTemp;
}

deque<float> CSVGProperties::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_STROKE_DASHARRAY:
		return m_afStrokeDashArray;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	deque<float> afTemp;
	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'properties' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CSVGProperties::GetAttrType(tstring strAttrName)
{
	if (strAttrName == _T("audio-level"))					return ATTR_AUDIO_LEVEL;
	else if (strAttrName == _T("color"))					return ATTR_COLOR;
	else if (strAttrName == _T("color-rendering"))			return ATTR_COLOR_RENDERING;
	else if (strAttrName == _T("display"))					return ATTR_DISPLAY;
	else if (strAttrName == _T("display-align"))			return ATTR_DISPLAY_ALIGN;
	else if (strAttrName == _T("fill"))						return ATTR_FILL;
	else if (strAttrName == _T("fill-opacity"))				return ATTR_FILL_OPACITY;
	else if (strAttrName == _T("fill-rule"))				return ATTR_FILL_RULE;
	else if (strAttrName == _T("font-family"))				return ATTR_FONT_FAMILY;
	else if (strAttrName == _T("font-size"))				return ATTR_FONT_SIZE;
	else if (strAttrName == _T("font-style"))				return ATTR_FONT_STYLE;
	else if (strAttrName == _T("text-decoration"))			return ATTR_TEXT_DECORATION;
	else if (strAttrName == _T("font-weight"))				return ATTR_FONT_WEIGHT;
	else if (strAttrName == _T("font-variant"))				return ATTR_FONT_VARIANT;
	else if (strAttrName == _T("image-rendering"))			return ATTR_IMAGE_RENDERING;
	else if (strAttrName == _T("line-increment"))			return ATTR_LINE_INCREMENT;
	else if (strAttrName == _T("pointer-events"))			return ATTR_POINTER_EVENTS;
	else if (strAttrName == _T("shape-rendering"))			return ATTR_SHAPE_RENDERING;
	else if (strAttrName == _T("solid-color"))				return ATTR_SOLID_COLOR;
	else if (strAttrName == _T("solid-opacity"))			return ATTR_SOLID_OPACITY;
	else if (strAttrName == _T("stop-color"))				return ATTR_STOP_COLOR;
	else if (strAttrName == _T("stop-opacity"))				return ATTR_STOP_OPACITY;
	else if (strAttrName == _T("stroke"))					return ATTR_STROKE;
	else if (strAttrName == _T("stroke-dasharray"))			return ATTR_STROKE_DASHARRAY;
	else if (strAttrName == _T("stroke-dashoffset"))		return ATTR_STROKE_DASHOFFSET;
	else if (strAttrName == _T("stroke-linecap"))			return ATTR_STROKE_LINECAP;
	else if (strAttrName == _T("stroke-linejoin"))			return ATTR_STROKE_LINEJOIN;
	else if (strAttrName == _T("stroke-miterlimit"))		return ATTR_STROKE_MITERLIMIT;
	else if (strAttrName == _T("stroke-opacity"))			return ATTR_STROKE_OPACITY;
	else if (strAttrName == _T("stroke-width"))				return ATTR_STROKE_WIDTH;
	else if (strAttrName == _T("text-anchor"))				return ATTR_TEXT_ANCHOR;
	else if (strAttrName == _T("text-rendering"))			return ATTR_TEXT_RENDERING;
	else if (strAttrName == _T("viewport-fill"))			return ATTR_VIEWPORT_FILL;
	else if (strAttrName == _T("viewport-fill-opacity"))	return ATTR_VIEWPORT_FILL_OPACITY;
	else if (strAttrName == _T("vector-effect"))			return ATTR_VECTOR_EFFECT;
	else if (strAttrName == _T("visibility"))				return ATTR_VISIBILITY;
	else													return 0;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'properties' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 7)       - first release
/************************************************************************/
UINT8 CSVGProperties::GetAttrValType(UINT uAttrType)
{
	switch (uAttrType)
	{
	case ATTR_AUDIO_LEVEL:
	case ATTR_FILL_OPACITY:
	case ATTR_FONT_SIZE:
	case ATTR_LINE_INCREMENT:
	case ATTR_SOLID_OPACITY:
	case ATTR_STOP_OPACITY:
	case ATTR_STROKE_DASHOFFSET:
	case ATTR_STROKE_MITERLIMIT:
	case ATTR_STROKE_OPACITY:
	case ATTR_STROKE_WIDTH:
	case ATTR_VIEWPORT_FILL_OPACITY:
		return FLOAT_TYPE;
		break;

	case ATTR_COLOR:
	case ATTR_FILL:
	case ATTR_SOLID_COLOR:
	case ATTR_STOP_COLOR:
	case ATTR_STROKE:
	case ATTR_VIEWPORT_FILL:
		return UINT8_LIST_TYPE;
		break;

	case ATTR_COLOR_RENDERING:
	case ATTR_DISPLAY:
	case ATTR_DISPLAY_ALIGN:
	case ATTR_FILL_RULE:
	case ATTR_FONT_STYLE:
	case ATTR_TEXT_DECORATION:
	case ATTR_FONT_WEIGHT:
	case ATTR_FONT_VARIANT:
	case ATTR_IMAGE_RENDERING:
	case ATTR_POINTER_EVENTS:
	case ATTR_SHAPE_RENDERING:
	case ATTR_STROKE_LINECAP:
	case ATTR_STROKE_LINEJOIN:
	case ATTR_TEXT_ANCHOR:
	case ATTR_TEXT_RENDERING:
	case ATTR_VECTOR_EFFECT:
	case ATTR_VISIBILITY:
		return UINT8_TYPE;
		break;

	case ATTR_FONT_FAMILY:
		return STRING_TYPE;
		break;

	case ATTR_STROKE_DASHARRAY:
		return FLOAT_LIST_TYPE;
		break;

	default:
		return 0;
		break;
	}
}

/************************************************************************/
// SetAttr
// Desc		: function of 'properties' attribute group to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 8. 26)	- first release
/************************************************************************/
lsrErr CSVGProperties::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szValue = strValue.c_str();

	switch (uAttrType)
	{
	case ATTR_AUDIO_LEVEL:
		m_fAudioLevel = (float)_tstof(szValue);

		// zero to one
		if ((m_fAudioLevel < 0) || (m_fAudioLevel > 1))
		{
			err = LASeRAttrNotFoundErr;
			return err;
		}

		break;

	case ATTR_COLOR:
		m_auColor = ReadColorType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_COLOR_RENDERING:
		m_uColorRendering = ReadColorRenderingType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_DISPLAY:
		m_uDisplay = ReadDisplayType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_DISPLAY_ALIGN:
		m_uDisplayAlign = ReadDisplayAlignType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_FILL:
		// if "currentColor", get the color variable
		if (!_tcscmp(szValue, _T("currentColor")))
			m_auFill = m_auColor;

		// if "none"
		else if (!_tcscmp(szValue, _T("none")))
			m_auFill.clear();

		else
		{
			m_auFill = ReadColorType(szValue, &err);
			CONFIRM(err);
		}

		break;

	case ATTR_FILL_OPACITY:
		m_fFillOpacity = (float)_tstof(szValue);

		// zero to one
		if ((m_fFillOpacity < 0) || (m_fFillOpacity > 1))
		{
			err = LASeRAttrNotFoundErr;
			return err;
		}

		break;

	case ATTR_FILL_RULE:
		m_uFillRule = ReadFillRuleType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_FONT_FAMILY:
		m_strFontFamily = strValue;
		break;

	case ATTR_FONT_SIZE:
		m_fFontSize = (float)_tstof(szValue);
		break;

	case ATTR_FONT_STYLE:
		m_uFontStyle = ReadFontStyleType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_FONT_WEIGHT:
		m_uFontWeight = ReadFontWeightType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_IMAGE_RENDERING:
		m_uImageRendering = ReadImageRenderingType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_LINE_INCREMENT:
		if (!_tcscmp(szValue, _T("auto")))	m_fLineIncrement = 0;
		else								m_fLineIncrement = (float)_tstof(szValue);
		break;

	case ATTR_POINTER_EVENTS:
		m_uPointerEvents = ReadPointerEventsType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_SHAPE_RENDERING:
		m_uShapeRendering = ReadShapeRenderingType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_SOLID_COLOR:
		// if "currentColor", get the 'color' variable
		if (!_tcscmp(szValue, "currentColor"))
			m_auSolidColor = m_auColor;

		else
		{
			m_auSolidColor = ReadColorType(szValue, &err);
			CONFIRM(err);
		}

		break;

	case ATTR_SOLID_OPACITY:
		m_fSolidOpacity = (float)_tstof(szValue);

		// zero to one
		if ((m_fSolidOpacity < 0) || (m_fSolidOpacity > 1))
		{
			err = LASeRAttrNotFoundErr;
			return err;
		}

		break;

	case ATTR_STOP_COLOR:
		// if "currentColor", get the 'color' variable
		if (!_tcscmp(szValue, "currentColor"))
			m_auStopColor = m_auColor;

		else
		{
			m_auStopColor = ReadColorType(szValue, &err);
			CONFIRM(err);
		}

		break;

	case ATTR_STOP_OPACITY:
		m_fStopOpacity = (float)_tstof(szValue);

		// zero to one
		if ((m_fStopOpacity < 0) || (m_fStopOpacity > 1))
		{
			err = LASeRAttrNotFoundErr;
			return err;
		}

		break;

	case ATTR_STROKE:
		// if "currentColor", get the color variable
		if (!_tcscmp(szValue, _T("currentColor")))
			m_auStroke = m_auColor;

		// if "none"
		else if (!_tcscmp(szValue, _T("none")))
			m_auStroke.clear();

		else
		{
			m_auStroke = ReadColorType(szValue, &err);
			CONFIRM(err);
		}

		break;

	case ATTR_STROKE_DASHARRAY:
		m_afStrokeDashArray = ReadFloatListType(szValue, ',', &err);
		CONFIRM(err);
		break;

	case ATTR_STROKE_DASHOFFSET:
		m_fStrokeDashOffset = (float)_tstof(szValue);
		break;

	case ATTR_STROKE_LINECAP:
		m_uStrokeLineCap = ReadStrokeLineCapType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_STROKE_LINEJOIN:
		m_uStrokeLineJoin = ReadStrokeLineJoinType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_STROKE_MITERLIMIT:
		m_fStrokeMiterLimit = (float)_tstof(szValue);
		break;

	case ATTR_STROKE_OPACITY:
		m_fStrokeOpacity = (float)_tstof(szValue);
		break;

	case ATTR_STROKE_WIDTH:
		m_fStrokeWidth = (float)_tstof(szValue);
		break;

	case ATTR_TEXT_ANCHOR:
		m_uTextAnchor = ReadTextAnchorType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_VIEWPORT_FILL:
		// if not "none" used
		if (_tcscmp(szValue, _T("none")))
		{
			m_auSolidColor = ReadColorType(szValue, &err);
			CONFIRM(err);
		}

		else
			m_auSolidColor.clear();

		break;

	case ATTR_VIEWPORT_FILL_OPACITY:
		m_fViewportFillOpacity = (float)_tstof(szValue);

		// zero to one
		if ((m_fViewportFillOpacity < 0) || (m_fViewportFillOpacity > 1))
		{
			err = LASeRAttrNotFoundErr;
			return err;
		}

		break;

	case ATTR_VECTOR_EFFECT:
		m_uVectorEffect = ReadVectorType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_VISIBILITY:
		m_uVisibility = ReadVisibilityType(szValue, &err);
		CONFIRM(err);
		break;

	default:
		err = LASeRAttrNotFoundErr;
		break;
	}

	return err;
}

lsrErr CSVGProperties::SetAttr(UINT uAttrType, float fValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_AUDIO_LEVEL:
		m_fAudioLevel = fValue;

		// zero to one
		if ((m_fAudioLevel < 0) || (m_fAudioLevel > 1))
		{
			err = LASeRAttrNotFoundErr;
			return err;
		}

		break;

	case ATTR_FILL_OPACITY:
		m_fFillOpacity = fValue;

		// zero to one
		if ((m_fFillOpacity < 0) || (m_fFillOpacity > 1))
		{
			err = LASeRAttrNotFoundErr;
			return err;
		}

		break;

	case ATTR_FONT_SIZE:
		m_fFontSize = fValue;
		break;

	case ATTR_LINE_INCREMENT:
		m_fLineIncrement = fValue;
		break;

	case ATTR_SOLID_OPACITY:
		m_fSolidOpacity = fValue;

		// zero to one
		if ((m_fSolidOpacity < 0) || (m_fSolidOpacity > 1))
		{
			err = LASeRAttrNotFoundErr;
			return err;
		}

		break;

	case ATTR_STOP_OPACITY:
		m_fStopOpacity = fValue;

		// zero to one
		if ((m_fStopOpacity < 0) || (m_fStopOpacity > 1))
		{
			err = LASeRAttrNotFoundErr;
			return err;
		}

		break;

	case ATTR_STROKE_DASHOFFSET:
		m_fStrokeDashOffset = fValue;
		break;

	case ATTR_STROKE_MITERLIMIT:
		m_fStrokeMiterLimit = fValue;
		break;

	case ATTR_STROKE_OPACITY:
		m_fStrokeOpacity = fValue;

		// zero to one
		if ((m_fViewportFillOpacity < 0) || (m_fViewportFillOpacity > 1))
		{
			err = LASeRAttrNotFoundErr;
			return err;
		}

		break;

	case ATTR_STROKE_WIDTH:
		m_fStrokeWidth = fValue;
		break;

	case ATTR_VIEWPORT_FILL_OPACITY:
		m_fViewportFillOpacity = fValue;
		break;

	default:
		err = LASeRAttrNotFoundErr;
		break;
	}

	return err;
}