///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "..\ComplexType.h"
#include "..\Datatypes.h"


CSVGSetAttr::CSVGSetAttr()
{
	m_uEnabled			= 0;
	m_astrBegin.assign(1, _T("0"));
	m_fDur				= 0.0;
	m_fMin				= 0.0;
	m_fMax				= 0.0;
	m_strAttributeType	= _T("");
	m_fRepeatCount		= 1.0;
	m_fRepeatDur		= 0.0;
	m_uRestart			= 0;
	m_uFill				= 0;
}

CSVGSetAttr::~CSVGSetAttr()
{
	m_astrBegin.clear();
	m_astrEnd.clear();
	m_afTo.clear();
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'SetAttrs' attribute group to set the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 18)	- first release
/************************************************************************/
lsrErr CSVGSetAttr::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR	*szTemp	= NULL;

	/************************************************************************/
	// set attribute
	// begin
	szTemp = pScene->GetAttrValue(_T("begin"));
	if (szTemp)
	{
		m_astrBegin = ReadStringListType(szTemp, ';', &err);
		CONFIRM(err);
	}

	// end
	szTemp = pScene->GetAttrValue(_T("end"));
	if (szTemp)
	{
		m_astrEnd = ReadStringListType(szTemp, ';', &err);
		CONFIRM(err);
	}

	// dur
	szTemp = pScene->GetAttrValue(_T("dur"));
	if (szTemp)
	{
		m_fDur = ReadTimesType(szTemp, &err);
		CONFIRM(err);
	}

	// min
	szTemp = pScene->GetAttrValue(_T("min"));
	if (szTemp)
	{
		m_fMin = ReadTimesType(szTemp, &err);
		CONFIRM(err);
	}

	// max
	szTemp = pScene->GetAttrValue(_T("max"));
	if (szTemp)
	{
		m_fMax = ReadTimesType(szTemp, &err);
		CONFIRM(err);
	}

	// attributeType
	szTemp = pScene->GetAttrValue(_T("attributeType"));
	if (szTemp)
		m_strAttributeType = szTemp;

	// repeatCount
	szTemp = pScene->GetAttrValue(_T("repeatCount"));
	if (szTemp)
	{
		m_fRepeatCount = ReadRepeatType(szTemp, &err);
		CONFIRM(err);
	}

	// repeatDur
	szTemp = pScene->GetAttrValue(_T("repeatDur"));
	if (szTemp)
	{
		m_fRepeatDur = ReadRepeatType(szTemp, &err);
		CONFIRM(err);
	}

	// restart
	szTemp = pScene->GetAttrValue(_T("restart"));
	if (szTemp)
	{
		m_uRestart = ReadRestartType(szTemp, &err);
		CONFIRM(err);
	}

	// fill
	szTemp = pScene->GetAttrValue(_T("fill"));
	if (szTemp)
	{
		m_uFill = ReadAnimFillType(szTemp, &err);
		CONFIRM(err);
	}

	// to
	szTemp = pScene->GetAttrValue(_T("to"));
	if (szTemp)
	{
		m_afTo = ReadFloatListType(szTemp, &err);
		CONFIRM(err);
	}
	/************************************************************************/

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'SetAttrs' attribute group to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 8. 28)	- first release
/************************************************************************/
float CSVGSetAttr::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_DUR:
		return m_fDur;
		break;

	case ATTR_MIN:
		return m_fMin;
		break;

	case ATTR_MAX:
		return m_fMax;
		break;

	case ATTR_REPEATCOUNT:
		return m_fRepeatCount;
		break;

	case ATTR_REPEATDUR:
		return m_fRepeatDur;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return 0.0;
}

UINT8 CSVGSetAttr::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_LSR_ENABLED:
		return m_uEnabled;
		break;

	case ATTR_RESTART:
		return m_uRestart;
		break;

	case ATTR_FILL:
		return m_uFill;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return 0;
}

tstring CSVGSetAttr::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_ATTRIBUTETYPE:
		return m_strAttributeType;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return _T("");
}

deque<float> CSVGSetAttr::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_TO:
		return m_afTo;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	// nonsense
	deque<float> afTemp;
	return afTemp;
}

deque<tstring> CSVGSetAttr::GetStrListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_BEGIN:
		return m_astrBegin;
		break;

	case ATTR_END:
		return m_astrEnd;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	// nonsense
	deque<tstring>	astrTemp;
	return astrTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'SetAttrs' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CSVGSetAttr::GetAttrType(tstring strAttrName)
{
	if (strAttrName == _T("lsr:enabled"))			return ATTR_LSR_ENABLED;
	else if (strAttrName == _T("begin"))			return ATTR_BEGIN;
	else if (strAttrName == _T("end"))				return ATTR_END;
	else if (strAttrName == _T("dur"))				return ATTR_DUR;
	else if (strAttrName == _T("min"))				return ATTR_MIN;
	else if (strAttrName == _T("max"))				return ATTR_MAX;
	else if (strAttrName == _T("attributeType"))	return ATTR_ATTRIBUTETYPE;
	else if (strAttrName == _T("repeatCount"))		return ATTR_REPEATCOUNT;
	else if (strAttrName == _T("repeatDur"))		return ATTR_REPEATDUR;
	else if (strAttrName == _T("restart"))			return ATTR_RESTART;
	else if (strAttrName == _T("fill"))				return ATTR_FILL;
	else if (strAttrName == _T("to"))				return ATTR_TO;
	else											return 0;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'SetAttrs' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 7)       - first release
/************************************************************************/
UINT8 CSVGSetAttr::GetAttrValType(UINT uAttrType)
{
	switch (uAttrType)
	{
	case ATTR_LSR_ENABLED:
	case ATTR_RESTART:
	case ATTR_FILL:
		return UINT8_TYPE;
		break;

	case ATTR_BEGIN:
	case ATTR_END:
		return STRING_LIST_TYPE;
		break;

	case ATTR_DUR:
	case ATTR_MIN:
	case ATTR_MAX:
	case ATTR_REPEATCOUNT:
	case ATTR_REPEATDUR:
		return FLOAT_TYPE;
		break;

	case ATTR_ATTRIBUTETYPE:
	case ATTR_TO:
		return STRING_TYPE;
		break;

	default:
		return 0;
		break;
	}
}