///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\LASeR.h"
#include "..\Datatypes.h"


CAdaptiveUpdateGroup::CAdaptiveUpdateGroup()
{
	CLASeRUnit::CLASeRUnit();

	m_uElementType	= LSR_AdaptiveUpdateGroup;
}

CAdaptiveUpdateGroup::~CAdaptiveUpdateGroup()
{
	m_auConstraints.clear();
}

/************************************************************************/
// SetAttr
// Desc		: function of 'AdaptiveUpdateGroup' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2009. 6. 30)	- first release
/************************************************************************/
lsrErr CAdaptiveUpdateGroup::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// adaptationConstraints
	szTemp = pScene->GetAttrValue(_T("adaptationConstraints"));
	if (szTemp)
	{
		m_auConstraints = ReadIntListType(szTemp, &err);
		CONFIRM(err);
	}

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	// if having children, deactive children
	if (GetChildrenSize() > 0)
	{
		for (deque<CLASeR *>::iterator i = m_apChildren.begin(); i < m_apChildren.end(); ++i)
		{
			if ((((*i)->GetElementType()) < LSR_Add) || (((*i)->GetElementType()) > LSR_AdaptiveUpdateGroup))
			{
				err = LASeRAdaptiveUpdateGroupErr;
				return err;
			}

			(*i)->SetDeactivate();
		}
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'AdaptiveUpdateGroup' attribute group to get attribute value using it's name
// Param	: target attribute name, return value
// History
//	PSH (2009. 6. 30)	- first release
/************************************************************************/
deque<UINT>	CAdaptiveUpdateGroup::GetUINTListAttr(UINT uAttrType, lsrErr *pErr)
{
	switch (uAttrType)
	{
	case ATTR_CONSTRAINTS:
		return m_auConstraints;
		break;

	default:
		*pErr = LASeRAdaptiveUpdateGroupErr;
		break;
	}

	deque<UINT> auTemp;
	return auTemp;
}