///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "..\LASeR.h"


CDelete::CDelete()
{
	CLASeRUnit::CLASeRUnit();

	m_uElementType		= LSR_Delete;

	m_strRef			= _T("");
	m_uIndex			= (UINT)-1;
	m_strAttributeName	= _T("children");
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'Delete' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 18)	- first release
/************************************************************************/
lsrErr CDelete::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// ref
	szTemp = pScene->GetAttrValue(_T("ref"));
	if (szTemp)
		m_strRef = szTemp;

	// index
	szTemp = pScene->GetAttrValue(_T("index"));
	if (szTemp)
		m_uIndex = (UINT)XStr2Int(pScene->GetAttrValue(_T("index")));

	// attributeName
	szTemp = pScene->GetAttrValue(_T("attributeName"));
	if (szTemp)
		m_strRef = szTemp;

	/************************************************************************/
	// 'required' confirmation
	if (m_strRef == _T(""))
		err = LASeRDeleteErr;

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'Delete' to get attribute value using it's name
// Param	: target attribute, return value
// History
//	PSH (2008. 11. 10)	- first release
/************************************************************************/
UINT CDelete::GetUINTAttr(UINT uAttrType, lsrErr *pErr)
{
	UINT uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_INDEX:
		return m_uIndex;
		break;

	default:
		*pErr = LASeRDeleteErr;
		break;
	}

	return uTemp;
}

tstring CDelete::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	tstring strTemp = _T("");

	switch (uAttrType)
	{
	case ATTR_REF:
		return m_strRef;
		break;

	case ATTR_ATTRIBUTENAME:
		return m_strAttributeName;
		break;

	default:
		*pErr = LASeRDeleteErr;
		break;
	}

	return strTemp;
}