///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\LASeR.h"


CNewScene::CNewScene()
{
	CLASeRUnit::CLASeRUnit();

	m_uElementType	= LSR_NewScene;

	m_pSVG			= NULL;
}

/************************************************************************/
// AddChildren
// Desc		: function to add the children
// Param	: new LASeR element pointer
// History
//	PSH (2008. 7. 14)	- first release
/************************************************************************/
lsrErr CNewScene::AddChildren(LPXNode pScene)
{
	lsrErr err = LASeRNoErr;

	const XNodes aChilds = pScene->GetChilds();

	CLASeR *pNewLASeRElement = NULL;

	// make LASeR Element
	err = CreateElement(aChilds[0], &pNewLASeRElement);
	CONFIRM(err);

	err = CheckRightChild(pNewLASeRElement->GetElementType());
	CONFIRM(err);

	m_pSVG = (CSVG *)pNewLASeRElement;

	m_pSVG->SetParent(this);
	m_pSVG->SetIndex(0);

	m_apChildren.push_back(m_pSVG);

	m_uNumOfChildren = 1;

	err = m_pSVG->SetAttrs(aChilds[0], pNewLASeRElement);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 12. 4)	- first release
/************************************************************************/
lsrErr CNewScene::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_svg:
		/* nothing */
		break;

	default:
		err = LASeRNewSceneErr;
		break;
	}

	return err;
}