///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "laser_error.h"
#include "..\LASeR.h"
#include "..\Datatypes.h"


CSendEvent::CSendEvent()
{
	CLASeRUnit::CLASeRUnit();

	m_uElementType		= LSR_SendEvent;

	m_strRef			= _T("");
	m_uEvent			= 0;
	m_afPointValue.assign(2, 0);
	m_uIntValue			= 0;
	m_strStringValue	= _T("");
}

CSendEvent::~CSendEvent()
{
	m_afPointValue.clear();
}

/************************************************************************/
// SetAttr
// Desc		: function of 'SendEvent' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 18)	- first release
/************************************************************************/
lsrErr CSendEvent::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// ref
	szTemp = pScene->GetAttrValue(_T("ref"));
	if (szTemp)
		m_strRef = szTemp;

	// event
	szTemp = pScene->GetAttrValue(_T("event"));
	if (szTemp)
		m_uEvent = ReadEventType(szTemp, &err);

	// pointvalue
	szTemp = pScene->GetAttrValue(_T("pointValue"));
	if (szTemp)
	{
		m_afPointValue = ReadFloatListType(szTemp, &err);
		CONFIRM(err);
	}

	// intvalue
	szTemp = pScene->GetAttrValue(_T("intValue"));
	if (szTemp)
	{
		m_uIntValue = (UINT)XStr2Int(pScene->GetAttrValue(_T("ref")));
	}

	// stringvalue
	szTemp = pScene->GetAttrValue(_T("stringValue"));
	if (szTemp)
		m_strStringValue = szTemp;

	/************************************************************************/
	// 'required' confirmation
	if (m_strRef == _T(""))
		err = LASeRSendEventErr;

	return err;
}