///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "..\svg.h"
#include "..\Datatypes.h"


CEVListener::CEVListener()
{
	CSVGObject::CSVGObject();

	m_uElementType		= EV_listener;

	m_bEnabled			= 0;
	m_uEvent			= 0;
	m_strHandler		= _T("");
	m_strObserver		= _T("");
	m_uPhase			= 0;
	m_strTarget			= _T("");
	m_uPropagate		= 0;
	m_uDefaultAction	= 0;
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'listener' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 19)	- first release
/************************************************************************/
lsrErr CEVListener::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR	*szTemp	= NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// lsr:enabled
	szTemp = pScene->GetAttrValue(_T("lsr:enabled"));
	if (szTemp)
	{
		m_bEnabled = ReadBooleanType(szTemp, &err);
		CONFIRM(err);
	}

	// event
	szTemp = pScene->GetAttrValue(_T("event"));
	if (szTemp)
	{
		m_uEvent = ReadEventType(szTemp, &err);
		CONFIRM(err);

		switch (m_uEvent)
		{
		case 35:	// displaySizeChanged
			{
				TCHAR *pRoundBraket = (TCHAR *)_tcschr(szTemp, '(');

				m_strEventParam = pRoundBraket + 1;

				m_strEventParam.erase(m_strEventParam.end() - 1);
			}
		}
	}

	// handler
	szTemp = pScene->GetAttrValue(_T("handler"));
	if (szTemp)
		m_strHandler = szTemp;

	// observer
	szTemp = pScene->GetAttrValue(_T("observer"));
	if (szTemp)
		m_strObserver = szTemp;

	// phase
	szTemp = pScene->GetAttrValue(_T("phase"));
	if (szTemp)
	{
		m_uPhase = ReadPhaseType(szTemp, &err);
		CONFIRM(err);
	}

	// target
	szTemp = pScene->GetAttrValue(_T("target"));
	if (szTemp)
		m_strTarget = szTemp;

	// propagate
	szTemp = pScene->GetAttrValue(_T("propagate"));
	if (szTemp)
	{
		m_uPropagate = ReadPropagateType(szTemp, &err);
		CONFIRM(err);
	}

	// defaultAction
	szTemp = pScene->GetAttrValue(_T("defaultAction"));
	if (szTemp)
	{
		m_uDefaultAction = ReadDefaultActionType(szTemp, &err);
		CONFIRM(err);
	}

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 11. 21)	- first release
/************************************************************************/
// must not to be called
lsrErr CEVListener::CheckRightChild(UINT8 uElementType)	{uElementType; return LASeRListenerErr;}

/************************************************************************/
// GetAttribute
// Desc		: function of 'listener' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 8. 28)	- first release
/************************************************************************/
UINT8 CEVListener::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_LSR_ENABLED:
		return m_bEnabled;
		break;

	case ATTR_EVENT:
		return m_uEvent;
		break;

	case ATTR_PHASE:
		return m_uPhase;
		break;

	case ATTR_PROPAGATE:
		return m_uPropagate;
		break;

	case ATTR_DEFAULTICON:
		return m_uDefaultAction;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRListenerErr;

		break;
	}

	return uTemp;
}

tstring CEVListener::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	switch (uAttrType)
	{
	case ATTR_EVENT_PARAM:
		return m_strEventParam;
		break;

	case ATTR_HANDLER:
		return m_strHandler;
		break;

	case ATTR_OBSERVER:
		return m_strObserver;
		break;

	case ATTR_TARGET:
		return m_strTarget;
		break;

	default:
		strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;

		*pErr = LASeRListenerErr;

		break;
	}

	return strTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'listener' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CEVListener::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("lsr:enabled"))			return ATTR_LSR_ENABLED;
	else if (strAttrName == _T("event"))			return ATTR_EVENT;
	else if (strAttrName == _T("handler"))			return ATTR_HANDLER;
	else if (strAttrName == _T("observer"))			return ATTR_OBSERVER;
	else if (strAttrName == _T("phase"))			return ATTR_PHASE;
	else if (strAttrName == _T("target"))			return ATTR_TARGET;
	else if (strAttrName == _T("propagate"))		return ATTR_PROPAGATE;
	else if (strAttrName == _T("defaultAction"))	return ATTR_DEFAULTICON;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'image' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 8)       - first release
/************************************************************************/
UINT8 CEVListener::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_LSR_ENABLED:
	case ATTR_EVENT:
	case ATTR_PHASE:
	case ATTR_PROPAGATE:
	case ATTR_DEFAULTICON:
		return UINT8_TYPE;
		break;

	case ATTR_HANDLER:
	case ATTR_OBSERVER:
	case ATTR_TARGET:
		return STRING_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'listener' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 8. 26)	- first release
/************************************************************************/
lsrErr CEVListener::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_TARGET:
		m_strTarget = strValue;
		break;

	default:
		err = LASeRListenerErr;
		break;
	}

	return err;
}