///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "laser_error.h"
#include "..\LASeR.h"
#include "..\Datatypes.h"


CLSRAnimateScroll::CLSRAnimateScroll()
{
	CSVGObject::CSVGObject();

	m_uElementType	= LSR_animateScroll;

	m_pHref			= new CSVGHref();

	m_fDelayAtStart	= 0.0;
	m_fDelayAtEnd	= 0.0;
	m_fSpeed		= 0.0;
	m_uDirection	= 0;
	m_fDur			= 0.0;
	m_uFill			= 0;
	m_uRestart		= 0;
	m_fRepeatCount	= 0.0;
	m_fRepeatDur	= 0.0;
}

CLSRAnimateScroll::~CLSRAnimateScroll()
{
	DELETEMEMORY(m_pHref);

	m_afBy.clear();
	m_afFrom.clear();
	m_afTo.clear();
	m_astrBegin.clear();
	m_astrEnd.clear();
}

/************************************************************************/
// SetAttribute
// Desc		: function of 'animateScroll' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 15)	- first release
/************************************************************************/
lsrErr CLSRAnimateScroll::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pHref)
	{
		err = LASeRAnimateScrollErr;
		return err;
	}

	const TCHAR	*szTemp	= NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg href
	err = m_pHref->SetAttrs(pScene, pI);
	CONFIRM(err);

	// by
	szTemp = pScene->GetAttrValue(_T("by"));
	if (szTemp)
	{
		m_afBy = ReadFloatListType(szTemp, &err);
		CONFIRM(err);
	}

	// from
	szTemp = pScene->GetAttrValue(_T("from"));
	if (szTemp)
	{
		m_afFrom = ReadFloatListType(szTemp, &err);
		CONFIRM(err);
	}

	// to
	szTemp = pScene->GetAttrValue(_T("to"));
	if (szTemp)
	{
		m_afTo = ReadFloatListType(szTemp, &err);
		CONFIRM(err);
	}

	// delayAtStart
	szTemp = pScene->GetAttrValue(_T("delayAtStart"));
	if (szTemp)
	{
		m_fDelayAtStart = ReadTimesType(szTemp, &err);
		CONFIRM(err);
	}

	// delayAtEnd
	szTemp = pScene->GetAttrValue(_T("delayAtEnd"));
	if (szTemp)
	{
		m_fDelayAtEnd = ReadTimesType(szTemp, &err);
		CONFIRM(err);
	}

	// speed
	szTemp = pScene->GetAttrValue(_T("speed"));
	if (szTemp)
	{
		m_fSpeed = ReadTimesType(szTemp, &err);
		CONFIRM(err);
	}

	// direction
	szTemp = pScene->GetAttrValue(_T("direction"));
	if (szTemp)
	{
		m_uDirection = ReadDirectionType(szTemp, &err);
		CONFIRM(err);
	}

	// begin
	szTemp = pScene->GetAttrValue(_T("begin"));
	if (szTemp)
	{
		m_astrBegin = ReadStringListType(szTemp, ';', &err);
		CONFIRM(err);
	}

	// dur
	szTemp = pScene->GetAttrValue(_T("dur"));
	if (szTemp)
	{
		m_fDur = ReadTimesType(szTemp, &err);
		CONFIRM(err);
	}

	// end
	szTemp = pScene->GetAttrValue(_T("end"));
	if (szTemp)
	{
		m_astrEnd = ReadStringListType(szTemp, ';', &err);
		CONFIRM(err);
	}

	// fill
	szTemp = pScene->GetAttrValue(_T("fill"));
	if (szTemp)
	{
		m_uFill = ReadAnimFillType(szTemp, &err);
		CONFIRM(err);
	}

	// restart
	szTemp = pScene->GetAttrValue(_T("restart"));
	if (szTemp)
	{
		m_uRestart = ReadRestartType(szTemp, &err);
		CONFIRM(err);
	}

	// repeatCount
	szTemp = pScene->GetAttrValue(_T("repeatCount"));
	if (szTemp)
	{
		m_fRepeatCount = ReadRepeatType(szTemp, &err);
		CONFIRM(err);
	}

	// repeatDur
	szTemp = pScene->GetAttrValue(_T("repeatDur"));
	if (szTemp)
	{
		m_fRepeatDur = ReadRepeatType(szTemp, &err);
		CONFIRM(err);
	}

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'animateScroll' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 12. 10)	- first release
/************************************************************************/
float CLSRAnimateScroll::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	switch (uAttrType)
	{
	case ATTR_LSR_DELAYATSTART:
		return m_fDelayAtStart;
		break;

	case ATTR_LSR_DELAYATEND:
		return m_fDelayAtEnd;
		break;

	case ATTR_LSR_SPEED:
		return m_fSpeed;
		break;

	case ATTR_DUR:
		return m_fDur;
		break;

	case ATTR_REPEATCOUNT:
		return m_fRepeatCount;
		break;

	case ATTR_REPEATDUR:
		return m_fRepeatDur;
		break;

	default:
		*pErr = LASeRAnimateScrollErr;
	}

	return fTemp;
}

UINT8 CLSRAnimateScroll::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_LSR_DIRECTION:
		return m_uDirection;
		break;

	case ATTR_FILL:
		return m_uFill;
		break;

	case ATTR_RESTART:
		return m_uRestart;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pHref->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRAnimateScrollErr;

		break;
	}

	return uTemp;
}

tstring CLSRAnimateScroll::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	tstring strTemp = _T("");

	strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pHref->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;

	*pErr = LASeRAnimateScrollErr;

	return strTemp;
}

deque<float> CLSRAnimateScroll::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	deque<float> afTemp;

	switch (uAttrType)
	{
	case ATTR_BY:
		return m_afBy;
		break;

	case ATTR_FROM:
		return m_afFrom;
		break;

	case ATTR_TO:
		return m_afTo;
		break;

	default:
		*pErr = LASeRAnimateScrollErr;

		break;
	}

	return afTemp;
}

deque<tstring> CLSRAnimateScroll::GetStrListAttr(UINT uAttrType, lsrErr *pErr)
{
	deque<tstring> astrTemp;

	switch (uAttrType)
	{
	case ATTR_BEGIN:
		return m_astrBegin;
		break;

	case ATTR_END:
		return m_astrEnd;
		break;

	default:
		*pErr = LASeRAnimateScrollErr;

		break;
	}

	return astrTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'animateScroll' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 12. 11)       - first release
/************************************************************************/
UINT CLSRAnimateScroll::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("by"))				return ATTR_BY;
	else if (strAttrName == _T("from"))			return ATTR_FROM;
	else if (strAttrName == _T("to"))			return ATTR_TO;
	else if (strAttrName == _T("delayAtStart"))	return ATTR_LSR_DELAYATSTART;
	else if (strAttrName == _T("delayAtEnd"))	return ATTR_LSR_DELAYATEND;
	else if (strAttrName == _T("speed"))		return ATTR_LSR_SPEED;
	else if (strAttrName == _T("direction"))	return ATTR_LSR_DIRECTION;
	else if (strAttrName == _T("begin"))		return ATTR_BEGIN;
	else if (strAttrName == _T("dur"))			return ATTR_DUR;
	else if (strAttrName == _T("end"))			return ATTR_END;
	else if (strAttrName == _T("fill"))			return ATTR_FILL;
	else if (strAttrName == _T("restart"))		return ATTR_RESTART;
	else if (strAttrName == _T("repeatCount"))	return ATTR_REPEATCOUNT;
	else if (strAttrName == _T("repeatDur"))	return ATTR_REPEATDUR;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'animateScroll' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 12. 11)       - first release
/************************************************************************/
UINT8 CLSRAnimateScroll::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_BY:
	case ATTR_FROM:
	case ATTR_TO:
		return FLOAT_LIST_TYPE;
		break;

	case ATTR_LSR_DELAYATSTART:
	case ATTR_LSR_DELAYATEND:
	case ATTR_LSR_SPEED:
	case ATTR_DUR:
	case ATTR_REPEATCOUNT:
	case ATTR_REPEATDUR:
		return FLOAT_TYPE;
		break;

	case ATTR_LSR_DIRECTION:
	case ATTR_FILL:
	case ATTR_RESTART:
		return UINT8_TYPE;
		break;

	case ATTR_BEGIN:
	case ATTR_END:
		return STRING_LIST_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'animateScroll' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 12. 11)	- first release
/************************************************************************/
lsrErr CLSRAnimateScroll::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_FILL:
		m_uFill = ReadAnimFillType(strValue.c_str(), &err);
		CONFIRM(err);
		break;

	default:
		err = m_pHref->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRAnimateScrollErr;

		break;
	}

	return err;
}