///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\LASeR.h"
#include "..\Datatypes.h"


CLSRConditional::CLSRConditional()
{
	CSVGObject::CSVGObject();

	m_uElementType					= LSR_conditional;

	m_pAdaptationAttr				= new CLSRAdaptationAttr();

	m_bExternalResourcesRequired	= 0;
	m_bEnabled						= 0;
}

CLSRConditional::~CLSRConditional()
{
	DELETEMEMORY(m_pAdaptationAttr);

	m_astrBegin.clear();
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'conditional' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 15)	- first release
/************************************************************************/
lsrErr CLSRConditional::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pAdaptationAttr)
	{
		err = LASeRGErr;
		return err;
	}

	const TCHAR *szTemp = NULL;
	
	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// LASeR adaptationAttr
	err = m_pAdaptationAttr->SetAttrs(pScene, pI);
	CONFIRM(err);

	// externalResourcesRequired
	szTemp = pScene->GetAttrValue(_T("externalResourcesRequired"));
	if (szTemp)
	{
		m_bExternalResourcesRequired = ReadBooleanType(szTemp, &err);
		CONFIRM(err);
	}

	// begin
	szTemp = pScene->GetAttrValue(_T("begin"));
	if (szTemp)
	{
		m_astrBegin = ReadStringListType(szTemp, ';', &err);
		CONFIRM(err);
	}

	// lsr:enabled
	szTemp = pScene->GetAttrValue(_T("lsr:enabled"));
	if (szTemp)
	{
		m_bEnabled = ReadBooleanType(szTemp, &err);
		CONFIRM(err);
	}

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 11. 21)	- first release
/************************************************************************/
lsrErr CLSRConditional::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	if ((uElementType < LSR_Add) || (uElementType > LSR_ReleaseResource))
		err = LASeRConditionalErr;

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'conditional' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 9. 29)	- first release
/************************************************************************/
UINT8 CLSRConditional::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_EXTERNALRESOURCESREQUIRED:
		return m_bExternalResourcesRequired;
		break;

	case ATTR_LSR_ENABLED:
		return m_bEnabled;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pAdaptationAttr->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRConditionalErr;

		break;
	}

	return uTemp;
}

UINT CLSRConditional::GetUINTAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT uTemp = 0;

	uTemp = m_pAdaptationAttr->GetUINTAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;

	*pErr = LASeRGErr;

	return uTemp;
}

tstring CLSRConditional::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pAdaptationAttr->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;

	*pErr = LASeRConditionalErr;

	return strTemp;
}

deque<float> CLSRConditional::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<float> afTemp;

	afTemp = m_pAdaptationAttr->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;

	*pErr = LASeRGErr;

	return afTemp;
}

deque<tstring> CLSRConditional::GetStrListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<tstring> astrTemp;

	switch (uAttrType)
	{
	case ATTR_BEGIN:
		return m_astrBegin;
		break;

	default:
		*pErr = LASeRConditionalErr;
		break;
	}

	return astrTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'conditional' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CLSRConditional::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("externalResourcesRequired"))		return ATTR_EXTERNALRESOURCESREQUIRED;
	else if (strAttrName == _T("begin"))					return ATTR_BEGIN;
	else if (strAttrName == _T("lsr:enabled"))				return ATTR_LSR_ENABLED;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pAdaptationAttr->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'image' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 8)       - first release
/************************************************************************/
UINT8 CLSRConditional::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_EXTERNALRESOURCESREQUIRED:
	case ATTR_LSR_ENABLED:
		return UINT8_TYPE;
		break;

	case ATTR_BEGIN:
		return STRING_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pAdaptationAttr->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}