///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\LASeR.h"


CLSRCursorManager::CLSRCursorManager()
{
	CSVGObject::CSVGObject();

	m_uElementType	= LSR_cursorManager;

	m_pHref			= new CSVGHref();
	m_fX			= 0.0;
	m_fY			= 0.0;
}

CLSRCursorManager::~CLSRCursorManager()
{
	DELETEMEMORY(m_pHref);
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'cursorManager' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 11. 19)	- first release
/************************************************************************/
lsrErr CLSRCursorManager::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pHref)
	{
		err = LASeRAErr;
		return err;
	}

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg href
	err = m_pHref->SetAttrs(pScene, pI);
	CONFIRM(err);

	// x
	szTemp = pScene->GetAttrValue(_T("x"));
	if (szTemp)
		m_fX = (float)_tstof(szTemp);

	// y
	szTemp = pScene->GetAttrValue(_T("y"));
	if (szTemp)
		m_fY = (float)_tstof(szTemp);

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'cursorManager' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 11. 8)	- first release
/************************************************************************/
float CLSRCursorManager::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	switch (uAttrType)
	{
	case ATTR_X:
		return m_fX;
		break;

	case ATTR_Y:
		return m_fY;
		break;

	default:
		*pErr = LASeRCursorManagerErr;
		break;
	}

	return fTemp;
}

UINT8 CLSRCursorManager::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;

	uTemp = m_pHref->GetUINT8Attr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;
	else *pErr = LASeRNoErr;

	*pErr = LASeRCursorManagerErr;

	return uTemp;
}

tstring CLSRCursorManager::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pHref->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;

	*pErr = LASeRCursorManagerErr;

	return strTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'cursorManager' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 11. 19)       - first release
/************************************************************************/
UINT CLSRCursorManager::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("x"))			return ATTR_X;
	else if (strAttrName == _T("y"))	return ATTR_Y;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'cursorManager' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 19)       - first release
/************************************************************************/
UINT8 CLSRCursorManager::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_X:
	case ATTR_Y:
		return FLOAT_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'cursorManager' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 11. 19)	- first release
/************************************************************************/
lsrErr CLSRCursorManager::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szValue = strValue.c_str();

	switch (uAttrType)
	{
	case ATTR_X:
		m_fX = (float)_tstof(szValue);
		break;

	case ATTR_Y:
		m_fY = (float)_tstof(szValue);
		break;

	default:
		err = m_pHref->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRCursorManagerErr;

		break;
	}

	return err;
}

lsrErr CLSRCursorManager::SetAttr(UINT uAttrType, float fValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_X:
		m_fX = fValue;
		break;

	case ATTR_Y:
		m_fY = fValue;
		break;

	default:
		err = LASeRCursorManagerErr;
		break;
	}

	return err;
}