///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\LASeR.h"
#include "..\Datatypes.h"


CLSRParsingSwitch::CLSRParsingSwitch()
{
	CSVGG::CSVGG();

	m_uElementType	= LSR_parsingSwitch;

	m_uMode			= 0;
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'parsingSwitch' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2009. 6. 25)	- first release
/************************************************************************/
lsrErr CLSRParsingSwitch::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// svg g
	err = CSVGG::SetAttrs(pScene, pI);
	CONFIRM(err);

	// mode
	szTemp = pScene->GetAttrValue(_T("mode"));
	if (szTemp)
	{
		m_uMode = ReadParsingModeType(szTemp, &err);
		CONFIRM(err);
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'parsingSwitch' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2007. 6. 25)	- first release
/************************************************************************/
float CLSRParsingSwitch::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	fTemp = CSVGG::GetFloatAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return fTemp;

	*pErr = LASeRParsingSwitchErr;

	return fTemp;
}

UINT8 CLSRParsingSwitch::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_LSR_MODE:
		return m_uMode;
		break;

	default:
		uTemp = CSVGG::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRParsingSwitchErr;

		break;
	}

	return uTemp;
}

tstring CLSRParsingSwitch::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	strTemp = CSVGG::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;

	*pErr = LASeRParsingSwitchErr;

	return strTemp;
}

deque<UINT8> CLSRParsingSwitch::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<UINT8> auTemp;

	auTemp = CSVGG::GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;

	*pErr = LASeRParsingSwitchErr;

	return auTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'simpleLayout' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 12. 7)	- first release
/************************************************************************/
UINT CLSRParsingSwitch::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("mode"))		return ATTR_LSR_MODE;
	else
	{
		uTemp = CSVGG::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'ParsingSwitch' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2009. 6. 25)       - first release
/************************************************************************/
UINT8 CLSRParsingSwitch::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_LSR_MODE:
		return UINT8_TYPE;
		break;

	default:
		uTemp = CSVGG::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'ParsingSwitch' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2009. 6. 25)	- first release
/************************************************************************/
lsrErr CLSRParsingSwitch::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	err = CSVGG::SetAttr(uAttrType, strValue);
	if (err == LASeRNoErr)	return err;

	err = LASeRParsingSwitchErr;

	return err;
}