///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\LASeR.h"
#include "..\Datatypes.h"


CLSRRectClip::CLSRRectClip()
{
	CSVGG::CSVGG();

	m_uElementType	= LSR_rectClip;

	m_fX			= 0.0;
	m_fY			= 0.0;
}

CLSRRectClip::~CLSRRectClip()
{
	m_afSize.clear();
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'rectClip' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 4)	- first release
/************************************************************************/
lsrErr CLSRRectClip::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// svg g
	err = CSVGG::SetAttrs(pScene, pI);
	CONFIRM(err);

	// size
	szTemp = pScene->GetAttrValue(_T("size"));
	if (szTemp)
	{
		m_afSize = ReadFloatListType(szTemp, ' ', &err);
		CONFIRM(err);

		if (m_afSize.size() != 2)			// restriction
		{
			err =  LASeRValidationErr;
			return err;
		}
	}

	// x
	szTemp = pScene->GetAttrValue(_T("x"));
	if (szTemp)
		m_fX = (float)_tstof(szTemp);

	// y
	szTemp = pScene->GetAttrValue(_T("y"));
	if (szTemp)
		m_fY = (float)_tstof(szTemp);

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'rectClip' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 11. 8)	- first release
/************************************************************************/
float CLSRRectClip::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	switch (uAttrType)
	{
	case ATTR_X:
		return m_fX;
		break;

	case ATTR_Y:
		return m_fY;
		break;

	default:
		fTemp = CSVGG::GetFloatAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return fTemp;

		*pErr = LASeRRectClipErr;

		break;
	}

	return fTemp;
}

UINT8 CLSRRectClip::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	uTemp = CSVGG::GetUINT8Attr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;

	*pErr = LASeRRectClipErr;

	return uTemp;
}

tstring CLSRRectClip::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	strTemp = CSVGG::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	
	*pErr = LASeRRectClipErr;

	return strTemp;
}

deque<UINT8> CLSRRectClip::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<UINT8> auTemp;

	auTemp = CSVGG::GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;

	*pErr = LASeRRectClipErr;

	return auTemp;
}

deque<float> CLSRRectClip::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<float> afTemp;

	switch (uAttrType)
	{
	case ATTR_SIZE:
		return m_afSize;
		break;

	default:
		afTemp = CSVGG::GetFloatListAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return afTemp;
		
		*pErr = LASeRRectClipErr;

		break;
	}

	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'rectClip' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CLSRRectClip::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("size"))		return ATTR_SIZE;
	else if (strAttrName == _T("x"))	return ATTR_X;
	else if (strAttrName == _T("y"))	return ATTR_Y;
	else
	{
		uTemp = CSVGG::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'rectClip' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 8)       - first release
/************************************************************************/
UINT8 CLSRRectClip::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_SIZE:
		return FLOAT_LIST_TYPE;
		break;

	case ATTR_X:
	case ATTR_Y:
		return FLOAT_TYPE;
		break;

	default:
		uTemp = CSVGG::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'rectClip' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 11. 10)	- first release
/************************************************************************/
lsrErr CLSRRectClip::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szValue = strValue.c_str();

	switch (uAttrType)
	{
	case ATTR_SIZE:
		m_afSize = ReadFloatListType(szValue, ' ', &err);
		CONFIRM(err);

		if (m_afSize.size() != 2)			// restriction
		{
			err =  LASeRValidationErr;
			return err;
		}
		break;

	case ATTR_X:
		m_fX = (float)_tstof(szValue);
		break;

	case ATTR_Y:
		m_fY = (float)_tstof(szValue);
		break;

	default:
		err = CSVGG::SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRRectClipErr;

		break;
	}

	return err;
}

lsrErr CLSRRectClip::SetAttr(UINT uAttrType, float fValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_X:
		m_fX = fValue;
		break;

	case ATTR_Y:
		m_fY = fValue;
		break;

	default:
		err = CSVGG::SetAttr(uAttrType, fValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRRectClipErr;
		break;
	}

	return err;
}