///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\LASeR.h"


CLSRSelector::CLSRSelector()
{
	CSVGG::CSVGG();

	m_uElementType	= LSR_selector;

	m_nChoice		= -1;
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'selector' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 12. 4)	- first release
/************************************************************************/
lsrErr CLSRSelector::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szTemp = NULL;
	
	// initial activation processing. inherit children
	SetDeactivate();

	/************************************************************************/
	// set attribute
	// svg g
	err = CSVGG::SetAttrs(pScene, pI);
	CONFIRM(err);

	// choice
	szTemp = pScene->GetAttrValue(_T("choice"));
	if (szTemp)
	{
		// if "choice", get the color variable
		if (!_tcscmp(szTemp, _T("none")))
			m_nChoice = -1;

		// if "none"
		else if (!_tcscmp(szTemp, _T("all")))
			m_nChoice = 256;

		else
			m_nChoice = XStr2Int(pScene->GetAttrValue(_T("choice")));
	}

	/************************************************************************/
	// rendering mode selector
	if ((m_nChoice >= 0) && (m_nChoice < m_uNumOfChildren))
		GetChildren((UINT8)m_nChoice)->SetActivate();

	else if (m_nChoice == 256)
	{
		for (UINT8 i = 0; i < m_uNumOfChildren; ++i)
			GetChildren(i)->SetActivate();
	}

	/* else - default setting */

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'selector' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 12. 4)	- first release
/************************************************************************/
float CLSRSelector::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	fTemp = CSVGG::GetFloatAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return fTemp;

	*pErr = LASeRSelectorErr;

	return fTemp;
}

UINT8 CLSRSelector::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	uTemp = CSVGG::GetUINT8Attr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;

	*pErr = LASeRSelectorErr;

	return uTemp;
}

UINT CLSRSelector::GetUINTAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_LSR_CHOICE:
		return (UINT)m_nChoice;
		break;

	default:
		*pErr = LASeRSelectorErr;
		break;
	}

	return uTemp;
}

tstring CLSRSelector::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	strTemp = CSVGG::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;

	*pErr = LASeRSelectorErr;

	return strTemp;
}

deque<UINT8> CLSRSelector::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<UINT8> auTemp;

	auTemp = CSVGG::GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;

	*pErr = LASeRRectClipErr;

	return auTemp;
}

deque<float> CLSRSelector::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<float> afTemp;

	afTemp = CSVGG::GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;

	*pErr = LASeRSelectorErr;

	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'selector' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 12. 6)	- first release
/************************************************************************/
UINT CLSRSelector::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("choice"))		return ATTR_LSR_CHOICE;
	else
	{
		uTemp = CSVGG::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'selector' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 12. 6)       - first release
/************************************************************************/
UINT8 CLSRSelector::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_LSR_CHOICE:
		return UINT32_TYPE;
		break;

	default:
		uTemp = CSVGG::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'selector' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 12. 6)	- first release
/************************************************************************/
lsrErr CLSRSelector::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szValue = strValue.c_str();

	switch (uAttrType)
	{
	case ATTR_LSR_CHOICE:
		// if "choice", get the color variable
		if (!_tcscmp(szValue, _T("none")))
			m_nChoice = -1;

		// if "none"
		else if (!_tcscmp(szValue, _T("all")))
			m_nChoice = 256;

		else
			m_nChoice = _tstoi(szValue);

		break;

	default:
		err = CSVGG::SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRSelectorErr;

		break;
	}

	return err;
}

lsrErr CLSRSelector::SetAttr(UINT uAttrType, UINT uValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_LSR_CHOICE:
		m_nChoice = (int)uValue;
		break;

	default:
		err = LASeRSelectorErr;
		break;
	}

	return err;
}