///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\LASeR.h"
#include "..\Datatypes.h"


CLSRSetScroll::CLSRSetScroll()
{
	CSVGObject::CSVGObject();

	m_uElementType	= LSR_setScroll;

	m_uDirection	= 0;
	m_uFill			= 0;
}

CLSRSetScroll::~CLSRSetScroll()
{
	m_astrBegin.clear();
	m_afIncrement.clear();
	m_afTo.clear();
}

/************************************************************************/
// SetAttribute
// Desc		: function of 'setScroll' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 15)	- first release
/************************************************************************/
lsrErr CLSRSetScroll::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR	*szTemp	= NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// begin
	szTemp = pScene->GetAttrValue(_T("begin"));
	if (szTemp)
	{
		m_astrBegin = ReadStringListType(szTemp, ';', &err);
		CONFIRM(err);
	}

	// increment
	szTemp = pScene->GetAttrValue(_T("increment"));
	if (szTemp)
	{
		m_afIncrement = ReadFloatListType(szTemp, ' ', &err);
		CONFIRM(err);
	}

	// to
	szTemp = pScene->GetAttrValue(_T("to"));
	if (szTemp)
	{
		m_afTo = ReadFloatListType(szTemp, ' ', &err);
		CONFIRM(err);
	}

	// direction
	szTemp = pScene->GetAttrValue(_T("direction"));
	if (szTemp)
	{
		m_uDirection = ReadDirectionType(szTemp, &err);
		CONFIRM(err);
	}

	// fill
	szTemp = pScene->GetAttrValue(_T("fill"));
	if (szTemp)
	{
		m_uFill = ReadAnimFillType(szTemp, &err);
		CONFIRM(err);
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'setScroll' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 12. 29)	- first release
/************************************************************************/
UINT8 CLSRSetScroll::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_LSR_DIRECTION:
		return m_uDirection;
		break;

	case ATTR_FILL:
		return m_uFill;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRSetScrollErr;

		break;
	}

	return uTemp;
}

tstring CLSRSetScroll::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	tstring strTemp = _T("");

	strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;

	*pErr = LASeRSetScrollErr;

	return strTemp;
}

deque<float> CLSRSetScroll::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	deque<float> afTemp;

	switch (uAttrType)
	{
	case ATTR_LSR_INCREMENT:
		return m_afIncrement;
		break;

	case ATTR_TO:
		return m_afTo;
		break;

	default:
		*pErr = LASeRSetScrollErr;
		break;
	}

	return afTemp;
}

deque<tstring> CLSRSetScroll::GetStrListAttr(UINT uAttrType, lsrErr *pErr)
{
	deque<tstring> astrTemp;

	switch (uAttrType)
	{
	case ATTR_BEGIN:
		return m_astrBegin;
		break;

	default:
		*pErr = LASeRSetScrollErr;

		break;
	}

	return astrTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'setScroll' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 12. 29)       - first release
/************************************************************************/
UINT CLSRSetScroll::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("begin"))				return ATTR_BEGIN;
	else if (strAttrName == _T("increment"))	return ATTR_LSR_INCREMENT;
	else if (strAttrName == _T("to"))			return ATTR_TO;
	else if (strAttrName == _T("direction"))	return ATTR_LSR_DIRECTION;
	else if (strAttrName == _T("fill"))			return ATTR_FILL;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'setScroll' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 12. 29)       - first release
/************************************************************************/
UINT8 CLSRSetScroll::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_BEGIN:
		return STRING_LIST_TYPE;
		break;

	case ATTR_LSR_INCREMENT:
	case ATTR_TO:
		return FLOAT_LIST_TYPE;
		break;

	case ATTR_LSR_DIRECTION:
	case ATTR_FILL:
		return UINT8_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'setScroll' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 12. 29)	- first release
/************************************************************************/
lsrErr CLSRSetScroll::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_FILL:
		m_uFill = ReadAnimFillType(strValue.c_str(), &err);
		CONFIRM(err);
		break;

	default:
		err = LASeRSetScrollErr;
		break;
	}

	return err;
}