#include "laser_error.h"
#include "..\LASeR.h"
#include "..\Datatypes.h"


CLSRStreamSource::CLSRStreamSource()
{
	CSVGObject::CSVGObject();

	m_uElementType	= LSR_streamSource;

	m_uSourceIndex	= 0;
	m_fWidth		= 0.0;
	m_fHeight		= 0.0;
	m_uMode			= 0;
}

CLSRStreamSource::~CLSRStreamSource()
{
	m_astrSources.clear();
}

/************************************************************************/
// SetAttribute
// Desc		: function of 'streamSource' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 8)	- first release
/************************************************************************/
lsrErr CLSRStreamSource::SetAttributes(LPXNode pScene)
{
	lsrErr err = LASeRNoErr;

	const TCHAR	*szTemp	= NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttributes(pScene);
	CONFIRM(err);

	// sources
	szTemp = pScene->GetAttrValue(_T("sources"));
	if (szTemp)
	{
		m_astrSources = ReadStringListType(szTemp, _T(' '), &err);
		CONFIRM(err);
	}

	// sourceIndex
	m_uSourceIndex = XStr2Int(pScene->GetAttrValue(_T("sourceIndex")));

	// width
	szTemp = pScene->GetAttrValue(_T("width"));
	if (szTemp)
	{
		const u32	uLength = (u32)_tcslen(szTemp);
		TCHAR		*pEnd	= (TCHAR *)(szTemp + uLength);
		m_fWidth = (float)_tcstod(szTemp, &pEnd);
	}

	// height
	szTemp = pScene->GetAttrValue(_T("height"));
	if (szTemp)
	{
		const u32	uLength = (u32)_tcslen(szTemp);
		TCHAR		*pEnd	= (TCHAR *)(szTemp + uLength);
		m_fHeight = (float)_tcstod(szTemp, &pEnd);
	}

	// mode
	szTemp = pScene->GetAttrValue(_T("mode"));
	if (szTemp)
	{
		m_uMode = ReadStreamSourceModeType(szTemp, &err);
		CONFIRM(err);
	}

	/************************************************************************/
	// 'required' confirmation
	if (m_astrSources.size() == 0)
		err = LASeRValidationErr;

	return err;
}