///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"


CSVGA::CSVGA()
{
	CSVGG::CSVGG();

	m_uElementType	= SVG_a;

	m_pHref			= new CSVGHref();

	m_strTarget		= _T("");
}

CSVGA::~CSVGA()
{
	DELETEMEMORY(m_pHref);
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'a' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 18)	- first release
//	PSH (2008. 9. 5)	- add target
/************************************************************************/
lsrErr CSVGA::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pHref)
	{
		err = LASeRAErr;
		return err;
	}

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// svg g
	err = CSVGG::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg href
	err = m_pHref->SetAttrs(pScene, pI);
	CONFIRM(err);

	// target
	szTemp = pScene->GetAttrValue(_T("target"));
	if (szTemp)
		m_strTarget = szTemp;

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 11. 21)	- first release
/************************************************************************/
lsrErr CSVGA::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	if (uElementType == SVG_a)
		err = LASeRAErr;

	err = GetParent()->CheckRightChild(uElementType);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'a' to get attribute value using it's type
// Param	: attribute name
// History
//	PSH (2008. 8. 29)	- first release
/************************************************************************/
float CSVGA::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	fTemp = CSVGG::GetFloatAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return fTemp;

	*pErr = LASeRAErr;

	return fTemp;
}

UINT8 CSVGA::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	uTemp = CSVGG::GetUINT8Attr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;
	else *pErr = LASeRNoErr;

	uTemp = m_pHref->GetUINT8Attr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;

	*pErr = LASeRAErr;

	return uTemp;
}

tstring CSVGA::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	switch (uAttrType)
	{
	case ATTR_TARGET:
		return m_strTarget;
		break;

	default:
		strTemp = CSVGG::GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		else *pErr = LASeRNoErr;

		strTemp = m_pHref->GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;

		*pErr = LASeRAErr;

		break;
	}

	return strTemp;
}

deque<UINT8> CSVGA::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<UINT8> auTemp;

	auTemp = CSVGG::GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;

	*pErr = LASeRAErr;

	return auTemp;
}

deque<float> CSVGA::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<float> afTemp;

	afTemp = CSVGG::GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;

	*pErr = LASeRAErr;

	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'a' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CSVGA::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("target"))	return ATTR_TARGET;
	else
	{
		uTemp = CSVGG::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'a' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 8)       - first release
/************************************************************************/
UINT8 CSVGA::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_TARGET:
		return STRING_TYPE;
		break;

	default:
		uTemp = CSVGG::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'a' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 8. 29)	- first release
/************************************************************************/
lsrErr CSVGA::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_TARGET:
		m_strTarget = strValue;
		break;

	default:
		err = CSVGG::SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;

		err = m_pHref->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRAErr;

		break;
	}

	return err;
}