///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"
#include "..\Datatypes.h"


CSVGAnimateMotion::CSVGAnimateMotion()
{
	CSVGObject::CSVGObject();

	m_uElementType	= SVG_animateMotion;

	m_pSetAttr		= new CSVGSetAttr();
	m_pHref			= new CSVGHref();
	m_pAnimateAttr	= new CSVGAnimateAttr();

	m_strPath		= _T("");
	m_uRotate		= 0;
}

CSVGAnimateMotion::~CSVGAnimateMotion()
{
	DELETEMEMORY(m_pSetAttr);
	DELETEMEMORY(m_pHref);
	DELETEMEMORY(m_pAnimateAttr);

	m_afKeyPoints.clear();
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'animateMotion' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 11. 18)	- first release
/************************************************************************/
lsrErr CSVGAnimateMotion::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pSetAttr || !m_pHref || !m_pAnimateAttr)
	{
		err = LASeRAnimateMotionErr;
		return err;
	}

	const TCHAR *szTemp = NULL;
	
	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg setAttrs
	err = m_pSetAttr->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg href
	err = m_pHref->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg animateAttr
	err = m_pAnimateAttr->SetAttrs(pScene, pI);
	CONFIRM(err);

	// path
	szTemp = pScene->GetAttrValue(_T("path"));
	if (szTemp)
		m_strPath = szTemp;

	// keyPoints
	szTemp = pScene->GetAttrValue(_T("keyPoints"));
	if (szTemp)
	{
		m_afKeyPoints = ReadFloatListType(szTemp, ' ', &err);
		CONFIRM(err);
	}

	// rotate
	szTemp = pScene->GetAttrValue(_T("rotate"));
	if (szTemp)
	{
		m_uRotate = ReadRotateType(szTemp,&err);
		CONFIRM(err);
	}

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 11. 21)	- first release
/************************************************************************/
lsrErr CSVGAnimateMotion::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_desc:
	case SVG_handler:
	case SVG_metadata:
	case SVG_mpath:
	case SVG_switch:
	case SVG_title:
		/* nothing */
		break;

	default:
		err = LASeRAnimateMotionErr;
		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'animateMotion' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 11. 18)	- first release
/************************************************************************/
float CSVGAnimateMotion::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	fTemp = m_pSetAttr->GetFloatAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return fTemp;
	
	*pErr = LASeRAnimateMotionErr;

	return fTemp;
}

UINT8 CSVGAnimateMotion::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_ROTATE:
		return m_uRotate;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pHref->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRAnimateMotionErr;

		break;
	}

	return uTemp;
}

tstring CSVGAnimateMotion::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	tstring strTemp = _T("");

	switch (uAttrType)
	{
	case ATTR_PATH:
		return m_strPath;
		break;

	default:
		strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		else *pErr = LASeRNoErr;

		strTemp = m_pHref->GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		else *pErr = LASeRNoErr;

		strTemp = m_pAnimateAttr->GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		else *pErr = LASeRNoErr;

		strTemp = m_pSetAttr->GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;

		*pErr = LASeRAnimateMotionErr;

		break;
	}

	return strTemp;
}

deque<float> CSVGAnimateMotion::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	deque<float> afTemp;

	switch (uAttrType)
	{
	case ATTR_KEYPOINTS:
		return m_afKeyPoints;
		break;

	default:
		afTemp = m_pAnimateAttr->GetFloatListAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return afTemp;
		else *pErr = LASeRNoErr;

		afTemp = m_pSetAttr->GetFloatListAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return afTemp;

		*pErr = LASeRAnimateMotionErr;

		break;
	}

	return afTemp;
}

deque<tstring> CSVGAnimateMotion::GetStrListAttr(UINT uAttrType, lsrErr *pErr)
{
	deque<tstring> astrTemp;

	astrTemp = m_pSetAttr->GetStrListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return astrTemp;

	*pErr = LASeRAnimateMotionErr;

	return astrTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'animateMotion' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 11. 18)       - first release
/************************************************************************/
UINT CSVGAnimateMotion::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("path"))				return ATTR_PATH;
	else if (strAttrName == _T("keyPoints"))	return ATTR_KEYPOINTS;
	else if (strAttrName == _T("rotate"))		return ATTR_ROTATE;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pAnimateAttr->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pSetAttr->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'animateMotion' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 18)       - first release
/************************************************************************/
UINT8 CSVGAnimateMotion::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_PATH:
		return STRING_TYPE;
		break;

	case ATTR_KEYPOINTS:
		return FLOAT_LIST_TYPE;
		break;

	case ATTR_ROTATE:
		return UINT8_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pAnimateAttr->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pSetAttr->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'animateMotion' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 11. 18)	- first release
/************************************************************************/
lsrErr CSVGAnimateMotion::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_ROTATE:
		m_uRotate = ReadRotateType(strValue.c_str(), &err);
		CONFIRM(err);
		break;

	default:
		err = m_pHref->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRAnimateMotionErr;

		break;
	}

	return err;
}