///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"
#include "..\Datatypes.h"


CSVGAnimateTransform::CSVGAnimateTransform()
{
	CSVGObject::CSVGObject();

	m_uElementType		= SVG_animateTransform;

	m_pSetAttr			= new CSVGSetAttr();
	m_pHref				= new CSVGHref();
	m_pAnimateAttr		= new CSVGAnimateAttr();

	m_uAttributeName	= 0;
	m_uType				= 0;
}

CSVGAnimateTransform::~CSVGAnimateTransform()
{
	DELETEMEMORY(m_pSetAttr);
	DELETEMEMORY(m_pHref);
	DELETEMEMORY(m_pAnimateAttr);
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'animateTransform' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 16)	- first release
//	PSH (2008. 9. 5)	- add type
/************************************************************************/
lsrErr CSVGAnimateTransform::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pSetAttr || !m_pHref || !m_pAnimateAttr)
	{
		err = LASeRAnimateTransformErr;
		return err;
	}

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg SetAttrs
	err = m_pSetAttr->SetAttrs(pScene, pI);
	CONFIRM(err);

	// attributeName
	szTemp = pScene->GetAttrValue(_T("attributeName"));
	if (szTemp)
	{
		m_uAttributeName = ReadAnimatableType(szTemp, &err);
		CONFIRM(err);
	}

	// svg href
	err = m_pHref->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg animateAttr
	err = m_pAnimateAttr->SetAttrs(pScene, pI);
	CONFIRM(err);

	// type
	szTemp = pScene->GetAttrValue(_T("type"));
	if (szTemp)
	{
		m_uType = ReadRotscatraType(szTemp, &err);
		CONFIRM(err);
	}

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 11. 21)	- first release
/************************************************************************/
lsrErr CSVGAnimateTransform::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_desc:
	case SVG_handler:
	case SVG_metadata:
	case SVG_switch:
	case SVG_title:
		/* nothing */
		break;

	default:
		err = LASeRAnimateTransformErr;
		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'animateTransform' to get attribute value using it's type
// Param	: attribute name
// History
//	PSH (2008. 9. 5)	- first release
/************************************************************************/
float CSVGAnimateTransform::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	fTemp = m_pSetAttr->GetFloatAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return fTemp;
	
	*pErr = LASeRAnimateTransformErr;

	return fTemp;
}

UINT8 CSVGAnimateTransform::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_ATTRIBUTENAME:
		return m_uAttributeName;
		break;

	case ATTR_TYPE:
		return m_uType;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pHref->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		
		*pErr = LASeRAnimateTransformErr;

		break;
	}

	return uTemp;
}

tstring CSVGAnimateTransform::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pHref->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pAnimateAttr->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pSetAttr->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	
	*pErr = LASeRAnimateTransformErr;

	return strTemp;
}

deque<float> CSVGAnimateTransform::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<float> afTemp;

	afTemp = m_pAnimateAttr->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;
	else *pErr = LASeRNoErr;

	afTemp = m_pSetAttr->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;
	
	*pErr = LASeRAnimateTransformErr;

	return afTemp;
}

deque<tstring> CSVGAnimateTransform::GetStrListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<tstring> astrTemp;

	astrTemp = m_pSetAttr->GetStrListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return astrTemp;
	
	*pErr = LASeRAnimateTransformErr;

	return astrTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'animateTransform' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CSVGAnimateTransform::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("attributeName"))	return ATTR_ATTRIBUTENAME;
	else if (strAttrName == _T("type"))		return ATTR_TYPE;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pAnimateAttr->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pSetAttr->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'animateTransform' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 8)       - first release
/************************************************************************/
UINT8 CSVGAnimateTransform::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_ATTRIBUTENAME:
	case ATTR_TYPE:
		return UINT8_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pAnimateAttr->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pSetAttr->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'animateTransform' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 9. 5)	- first release
/************************************************************************/
lsrErr CSVGAnimateTransform::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	err = m_pHref->SetAttr(uAttrType, strValue);
	if (err == LASeRNoErr)	return err;

	err = LASeRAnimateTransformErr;

	return err;
}