///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"
#include "..\Datatypes.h"


CSVGAudio::CSVGAudio()
{
	CSVGObject::CSVGObject();

	m_uElementType					= SVG_audio;

	m_pMediaProperties				= new CSVGMediaProperties();
	m_pBasic						= new CSVGBasic();
	m_pHref							= new CSVGHref();

	m_bExternalResourcesRequired	= 0;
	m_fDur							= 0.0;
	m_fMin							= 0.0;
	m_fMax							= 0.0;
	m_fRepeatCount					= 0.0;
	m_fRepeatDur					= 0.0;
	m_uSyncBehavior					= 0;
	m_fSyncTolerance				= 0.0;
	m_bSyncMaster					= 0;
	m_strSyncReference				= _T("");
	m_strType						= _T("");
	m_uClipBegin					= 0;
	m_uClipEnd						= 0;
	m_uRestart						= 0;
}

CSVGAudio::~CSVGAudio()
{
	DELETEMEMORY(m_pMediaProperties);
	DELETEMEMORY(m_pBasic);
	DELETEMEMORY(m_pHref);

	m_astrBegin.clear();
	m_astrEnd.clear();
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'audio' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 16)	- first release
/************************************************************************/
lsrErr CSVGAudio::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pMediaProperties || !m_pBasic || !m_pHref)
	{
		err = LASeRAudioErr;
		return err;
	}

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg mediaProperties
	err = m_pMediaProperties->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg basic
	err = m_pBasic->SetAttrs(pScene, pI);
	CONFIRM(err);

	// externalResourcesRequired
	szTemp = pScene->GetAttrValue(_T("externalResourcesRequired"));
	if (szTemp)
	{
		m_bExternalResourcesRequired = ReadBooleanType(szTemp, &err);
		CONFIRM(err);
	}

	// begin
	szTemp = pScene->GetAttrValue(_T("begin"));
	if (szTemp)
	{
		m_astrBegin = ReadStringListType(szTemp, ';', &err);
		CONFIRM(err);
	}

	// end
	szTemp = pScene->GetAttrValue(_T("end"));
	if (szTemp)
	{
		m_astrEnd = ReadStringListType(szTemp, ';', &err);
		CONFIRM(err);
	}

	// dur
	szTemp = pScene->GetAttrValue(_T("dur"));
	if (szTemp)
	{
		m_fDur = ReadTimesType(szTemp, &err);
		CONFIRM(err);
	}

	// min
	szTemp = pScene->GetAttrValue(_T("min"));
	if (szTemp)
	{
		m_fMin = ReadTimesType(szTemp, &err);
		CONFIRM(err);
	}

	// max
	szTemp = pScene->GetAttrValue(_T("max"));
	if (szTemp)
	{
		m_fMax = ReadTimesType(szTemp, &err);
		CONFIRM(err);
	}

	// svg href
	err = m_pHref->SetAttrs(pScene, pI);
	CONFIRM(err);

	// repeatCount
	szTemp = pScene->GetAttrValue(_T("repeatCount"));
	if (szTemp)
	{
		m_fRepeatCount = ReadRepeatType(szTemp, &err);
		CONFIRM(err);
	}

	// repeatDur
	szTemp = pScene->GetAttrValue(_T("repeatDur"));
	if (szTemp)
	{
		m_fRepeatDur = ReadRepeatType(szTemp, &err);
		CONFIRM(err);
	}

	// syncBehavior
	szTemp = pScene->GetAttrValue(_T("syncBehavior"));
	if (szTemp)
	{
		m_uSyncBehavior = ReadSyncBehaviorType(szTemp, &err);
		CONFIRM(err);
	}

	// syncTolerance
	szTemp = pScene->GetAttrValue(_T("syncTolerance"));
	if (szTemp)
	{
		if (!_tcscmp(szTemp, _T("default")))
		{
			for (deque<CLASeR *>::iterator i = apReusedElements.begin();
				i < apReusedElements.end(); ++i)
			{
				if ((*i)->GetElementType() == SVG_svg)
				{

					CSVG *pTemp = (CSVG *)*i;

					m_fSyncTolerance = pTemp->GetFloatAttr(ATTR_SYNCTOLERANCEDEFAULT, &err);
					CONFIRM(err);
				}
			}
		}

		else
			m_fSyncTolerance = (float)_tstof(szTemp);
	}

	// syncMaster
	szTemp = pScene->GetAttrValue(_T("syncMaster"));
	if (szTemp)
	{
		m_bSyncMaster = ReadBooleanType(szTemp, &err);
		CONFIRM(err);
	}

	// lsr:syncReference
	szTemp = pScene->GetAttrValue(_T("lsr:syncReference"));
	if (szTemp)
		m_strSyncReference = szTemp;

	// type
	szTemp = pScene->GetAttrValue(_T("type"));
	if (szTemp)
		m_strType = szTemp;

	// smil:clipBegin
	m_uClipBegin = XStr2Int(pScene->GetAttrValue(_T("smil:clipBegin")));

	// smil:clipEnd
	m_uClipEnd = XStr2Int(pScene->GetAttrValue(_T("smil:clipEnd")));

	// restart
	szTemp = pScene->GetAttrValue(_T("restart"));
	if (szTemp)
	{
		m_uRestart = ReadRestartType(szTemp, &err);
		CONFIRM(err);
	}

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 11. 21)	- first release
/************************************************************************/
lsrErr CSVGAudio::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_animate:
	case SVG_animateColor:
	case SVG_animateMotion:
	case SVG_animateTransform:
	case SVG_desc:
	case SVG_discard:
	case SVG_handler:
	case SVG_metadata:
	case SVG_set:
	case SVG_switch:
	case SVG_title:
		/* nothing */
		break;

	default:
		err = LASeRAudioErr;
		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'audio' to get attribute value using it's type
// Param	: attribute name
// History
//	PSH (2008. 9. 5)	- first release
/************************************************************************/
float CSVGAudio::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	switch (uAttrType)
	{
	case ATTR_DUR:
		return m_fDur;
		break;

	case ATTR_MIN:
		return m_fMin;
		break;

	case ATTR_MAX:
		return m_fMax;
		break;

	case ATTR_REPEATCOUNT:
		return m_fRepeatCount;
		break;

	case ATTR_REPEATDUR:
		return m_fRepeatDur;
		break;

	case ATTR_SYNCTOLERANCE:
		return m_fSyncTolerance;
		break;

	default:
		fTemp = m_pMediaProperties->GetFloatAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return fTemp;

		*pErr = LASeRAudioErr;

		break;
	}

	return fTemp;
}

UINT8 CSVGAudio::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_EXTERNALRESOURCESREQUIRED:
		return m_bExternalResourcesRequired;
		break;

	case ATTR_SYNCBEHAVIOR:
		return m_uSyncBehavior;
		break;

	case ATTR_SYNCMASTER:
		return m_bSyncMaster;
		break;

	case ATTR_RESTART:
		return m_uRestart;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pMediaProperties->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pHref->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRAudioErr;

		break;
	}

	return uTemp;
}

UINT CSVGAudio::GetUINTAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_SMIL_CLIPBEGIN:
		return m_uClipBegin;
		break;

	case ATTR_SMIL_CLIPEND:
		return m_uClipEnd;
		break;

	default:
		*pErr = LASeRAudioErr;
		break;
	}

	return 0;
}

tstring CSVGAudio::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	switch (uAttrType)
	{
	case ATTR_LSR_SYNCREFERENCE:
		return m_strSyncReference;
		break;

	case ATTR_TYPE:
		return m_strType;
		break;

	default:
		strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		else *pErr = LASeRNoErr;

		strTemp = m_pBasic->GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		else *pErr = LASeRNoErr;

		strTemp = m_pHref->GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		
		*pErr = LASeRAudioErr;

		break;
	}

	return strTemp;
}

deque<UINT8> CSVGAudio::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<UINT8> auTemp;

	auTemp = m_pMediaProperties->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;
	else *pErr = LASeRNoErr;

	auTemp = m_pBasic->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;
	
	*pErr = LASeRAudioErr;

	return auTemp;
}

deque<tstring> CSVGAudio::GetStrListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<tstring> astrTemp;

	switch (uAttrType)
	{
	case ATTR_BEGIN:
		return m_astrBegin;
		break;

	case ATTR_END:
		return m_astrEnd;
		break;

	default:
		*pErr = LASeRAudioErr;
		break;
	}

	return astrTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'audio' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CSVGAudio::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("externalResourcesRequired"))	return ATTR_EXTERNALRESOURCESREQUIRED;
	else if (strAttrName == _T("begin"))				return ATTR_BEGIN;
	else if (strAttrName == _T("end"))					return ATTR_END;
	else if (strAttrName == _T("dur"))					return ATTR_DUR;
	else if (strAttrName == _T("min"))					return ATTR_MIN;
	else if (strAttrName == _T("max"))					return ATTR_MAX;
	else if (strAttrName == _T("repeatCount"))			return ATTR_REPEATCOUNT;
	else if (strAttrName == _T("repeatDur"))			return ATTR_REPEATDUR;
	else if (strAttrName == _T("syncBehavior"))			return ATTR_SYNCBEHAVIOR;
	else if (strAttrName == _T("syncTolerance"))		return ATTR_SYNCTOLERANCE;
	else if (strAttrName == _T("syncMaster"))			return ATTR_SYNCMASTER;
	else if (strAttrName == _T("lsr:syncReference"))	return ATTR_LSR_SYNCREFERENCE;
	else if (strAttrName == _T("type"))					return ATTR_TYPE;
	else if (strAttrName == _T("smil:clipBegin"))		return ATTR_SMIL_CLIPBEGIN;
	else if (strAttrName == _T("smil:clipEnd"))			return ATTR_SMIL_CLIPEND;
	else if (strAttrName == _T("restart"))				return ATTR_RESTART;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pMediaProperties->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pBasic->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'audio' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 8)       - first release
/************************************************************************/
UINT8 CSVGAudio::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_EXTERNALRESOURCESREQUIRED:
	case ATTR_SYNCBEHAVIOR:
	case ATTR_SYNCMASTER:
	case ATTR_RESTART:
		return UINT8_TYPE;
		break;

	case ATTR_BEGIN:
	case ATTR_END:
		return STRING_LIST_TYPE;
		break;

	case ATTR_DUR:
	case ATTR_MIN:
	case ATTR_MAX:
	case ATTR_REPEATCOUNT:
	case ATTR_REPEATDUR:
	case ATTR_SYNCTOLERANCE:
		return FLOAT_TYPE;
		break;

	case ATTR_LSR_SYNCREFERENCE:
	case ATTR_TYPE:
		return STRING_TYPE;
		break;

	case ATTR_SMIL_CLIPBEGIN:
	case ATTR_SMIL_CLIPEND:
		return UINT32_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pMediaProperties->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pBasic->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'audio' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 9. 5)	- first release
/************************************************************************/
lsrErr CSVGAudio::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_TYPE:
		m_strType = strValue;
		break;

	default:
		err = m_pMediaProperties->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;
		else err = LASeRNoErr;

		err = m_pHref->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRAudioErr;

		break;
	}

	return err;
}

lsrErr CSVGAudio::SetAttr(UINT uAttrType, float fValue)
{
	lsrErr err = LASeRNoErr;

	err = m_pMediaProperties->SetAttr(uAttrType, fValue);
	if (err == LASeRNoErr)	return err;
	
	err = LASeRAudioErr;

	return err;
}