///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"


CSVGDefs::CSVGDefs()
{
	CSVGObject::CSVGObject();

	m_uElementType	= SVG_defs;

	m_pProperties	= new CSVGProperties();
}

CSVGDefs::~CSVGDefs()
{
	DELETEMEMORY(m_pProperties);
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'defs' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 11. 19)	- first release
/************************************************************************/
lsrErr CSVGDefs::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pProperties)
	{
		err = LASeRDefsErr;
		return err;
	}

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg properties
	err = m_pProperties->SetAttrs(pScene, pI);
	CONFIRM(err);

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 11. 21)	- first release
/************************************************************************/
lsrErr CSVGDefs::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_mpath:
	case SVG_stop:
	case SVG_svg:
	case SVG_tspan:
	case SVGT_fontfacesrc:
	case SVGT_fontfaceuri:
	case SVGT_glyph:
	case SVGT_hkern:
	case SVGT_missingglyph:
	case SVG_tBreak:
		err = LASeRDefsErr;
		return err;
		break;

	default:
		if ((uElementType < SVG_a) || (uElementType > SVG_textArea))
			err = LASeRDefsErr;

		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'defs' to get attribute value using it's type
// Param	: attribute name
// History
//	PSH (2008. 11. 19)	- first release
/************************************************************************/
float CSVGDefs::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	fTemp = m_pProperties->GetFloatAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return fTemp;
	
	*pErr = LASeRDefsErr;

	return fTemp;
}

UINT8 CSVGDefs::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;
	else *pErr = LASeRNoErr;

	uTemp = m_pProperties->GetUINT8Attr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;
	
	*pErr = LASeRDefsErr;

	return uTemp;
}

tstring CSVGDefs::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pProperties->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	
	*pErr = LASeRDefsErr;

	return strTemp;
}

deque<UINT8> CSVGDefs::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<UINT8> auTemp;

	auTemp = m_pProperties->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;
	
	*pErr = LASeRDefsErr;

	return auTemp;
}

deque<float> CSVGDefs::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<float> afTemp;

	afTemp = m_pProperties->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;
	
	*pErr = LASeRDefsErr;

	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'desc' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 11. 19)       - first release
/************************************************************************/
UINT CSVGDefs::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("textContent"))	return ATTR_TEXTCONTENT;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'desc' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 19)       - first release
/************************************************************************/
UINT8 CSVGDefs::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_TEXTCONTENT:
		return STRING_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'defs' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 8. 28)	- first release
/************************************************************************/
lsrErr CSVGDefs::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	err = m_pProperties->SetAttr(uAttrType, strValue);
	if (err == LASeRNoErr)	return err;

	err = LASeRDefsErr;

	return err;
}