///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"


CSVGDesc::CSVGDesc(UINT8 uElementType)
{
	CSVGObject::CSVGObject();

	m_uElementType		= uElementType;

	m_strTextContent	= _T("");
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'desc' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 11. 19)	- first release
/************************************************************************/
lsrErr CSVGDesc::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	/************************************************************************/
	// set textContent
	m_strTextContent = pScene->value;

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 11. 21)	- first release
/************************************************************************/
// must not to be called
lsrErr CSVGDesc::CheckRightChild(UINT8 uElementType)	{uElementType; return LASeRDescErr;}

/************************************************************************/
// GetAttribute
// Desc		: function of 'desc' to get attribute value using it's type
// Param	: attribute name
// History
//	PSH (2008. 11. 19)	- first release
/************************************************************************/
UINT8 CSVGDesc::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;
	
	*pErr = LASeRDescErr;

	return uTemp;
}

tstring CSVGDesc::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	switch (uAttrType)
	{
	case ATTR_TEXTCONTENT:
		return m_strTextContent;
		break;

	default:
		strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		
		*pErr = LASeRDescErr;

		break;
	}

	return strTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'desc' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 11. 19)       - first release
/************************************************************************/
UINT CSVGDesc::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("textContent"))	return ATTR_EDITABLE;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'desc' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 19)       - first release
/************************************************************************/
UINT8 CSVGDesc::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	uTemp = CSVGObject::GetAttrValType(uAttrType);
	if (uTemp != 0)	return uTemp;

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'desc' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 11. 19)	- first release
/************************************************************************/
lsrErr CSVGDesc::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_TEXTCONTENT:
		m_strTextContent = strValue;
		break;

	default:
		err = LASeRDescErr;
		break;
	}

	return err;
}