///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"
#include "..\Datatypes.h"


CSVGHandler::CSVGHandler()
{
	CSVGObject::CSVGObject();

	m_uElementType				= SVG_handler;

	m_bExternalResourceRequired	= 0;
	m_strType					= _T("");
	m_uEvent					= 0;
}

/************************************************************************/
// SetAttribute
// Desc		: function of 'handler' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 4)	- first release
//	PSH (2008. 8. 4)	- delete adding children function
/************************************************************************/
lsrErr CSVGHandler::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// externalResourcesRequired
	szTemp = pScene->GetAttrValue(_T("externalResourcesRequired"));
	if (szTemp)
	{
		m_bExternalResourceRequired = ReadBooleanType(szTemp, &err);
		CONFIRM(err);
	}

	// type
	szTemp = pScene->GetAttrValue(_T("type"));
	if (szTemp)
		m_strType = szTemp;

	// ev:event
	szTemp = pScene->GetAttrValue(_T("ev:event"));
	if (szTemp)
	{
		m_uEvent = ReadEventType(szTemp, &err);
		CONFIRM(err);
	}

	/************************************************************************/
	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 12. 18)	- first release
/************************************************************************/
lsrErr CSVGHandler::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_desc:
	case SVG_metadata:
	case SVG_switch:
	case SVG_title:
		/* nothing */
		break;

	default:
		err = LASeRHandlerErr;
		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'handler' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 12. 18)	- first release
/************************************************************************/
UINT8 CSVGHandler::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_EXTERNALRESOURCESREQUIRED:
		return m_bExternalResourceRequired;
		break;

	case ATTR_EVENT:
		return m_uEvent;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRHandlerErr;

		break;
	}

	return uTemp;
}

tstring CSVGHandler::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	switch (uAttrType)
	{
	case ATTR_TYPE:
		return m_strType;
		break;

	case ATTR_TEXTCONTENT:
		return m_strTextContent;
		break;

	default:
		strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;

		*pErr = LASeRHandlerErr;
	}

	return strTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'handler' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 12. 18)       - first release
/************************************************************************/
UINT CSVGHandler::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("externalResourcesRequired"))	return ATTR_EXTERNALRESOURCESREQUIRED;
	else if (strAttrName == _T("type"))					return ATTR_ACCENT_HEIGHT;
	else if (strAttrName == _T("event"))				return ATTR_EVENT;
	else if (strAttrName == _T("textContent"))			return ATTR_TEXTCONTENT;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'handler' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 12. 18)       - first release
/************************************************************************/
UINT8 CSVGHandler::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_EXTERNALRESOURCESREQUIRED:
	case ATTR_EVENT:
		return UINT8_TYPE;
		break;

	case ATTR_TYPE:
	case ATTR_TEXTCONTENT:
		return STRING_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'handler' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 12. 18)	- first release
/************************************************************************/
lsrErr CSVGHandler::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_TEXTCONTENT:
		m_strTextContent = strValue;
		break;

	default:
		err = LASeRHandlerErr;
		break;
	}

	return err;
}