///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"
#include "..\Datatypes.h"


CSVGLine::CSVGLine()
{
	CSVGObject::CSVGObject();

	m_uElementType		= SVG_line;

	m_pProperties		= new CSVGProperties();
	m_pBasic			= new CSVGBasic();
	m_pLocatable		= new CSVGLocatable();
	m_pFocus			= new CSVGFocus();

	m_uFocusHighlight	= 0;
	m_fX1				= 0.0;
	m_fY1				= 0.0;
	m_fX2				= 0.0;
	m_fY2				= 0.0;
}

CSVGLine::~CSVGLine()
{
	DELETEMEMORY(m_pProperties);
	DELETEMEMORY(m_pBasic);
	DELETEMEMORY(m_pLocatable);
	DELETEMEMORY(m_pFocus);
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'line' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 11. 19)	- first release
/************************************************************************/
lsrErr CSVGLine::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pProperties || !m_pBasic || !m_pLocatable || !m_pFocus)
	{
		err = LASeRLineErr;
		return err;
	}

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg properties
	err = m_pProperties->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg basic
	err = m_pBasic->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg locatable
	err = m_pLocatable->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg focus
	err = m_pFocus->SetAttrs(pScene, pI);
	CONFIRM(err);

	// focusHighlight
	szTemp = pScene->GetAttrValue(_T("focusHighlight"));
	if (szTemp)
	{
		m_uFocusHighlight = ReadFocusHighlightType(szTemp, &err);
		CONFIRM(err);
	}

	// x1
	szTemp = pScene->GetAttrValue(_T("x1"));
	if (szTemp)
		m_fX1 = (float)_tstof(szTemp);

	// y1
	szTemp = pScene->GetAttrValue(_T("y1"));
	if (szTemp)
		m_fY1 = (float)_tstof(szTemp);

	// x2
	szTemp = pScene->GetAttrValue(_T("x2"));
	if (szTemp)
		m_fX2 = (float)_tstof(szTemp);

	// y2
	szTemp = pScene->GetAttrValue(_T("y2"));
	if (szTemp)
		m_fY2 = (float)_tstof(szTemp);

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 11. 21)	- first release
/************************************************************************/
lsrErr CSVGLine::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_animate:
	case SVG_animateColor:
	case SVG_animateMotion:
	case SVG_animateTransform:
	case SVG_desc:
	case SVG_discard:
	case SVG_handler:
	case SVG_metadata:
	case SVG_set:
	case SVG_switch:
	case SVG_title:
		/* nothing */
		break;

	default:
		err = LASeRLineErr;
		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'line' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 8. 29)	- first release
/************************************************************************/
float CSVGLine::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	switch (uAttrType)
	{
	case ATTR_X1:
		return m_fX1;
		break;

	case ATTR_Y1:
		return m_fY1;
		break;

	case ATTR_X2:
		return m_fX2;
		break;

	case ATTR_Y2:
		return m_fY2;
		break;

	default:
		fTemp = m_pProperties->GetFloatAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return fTemp;

		*pErr = LASeRLineErr;

		break;
	}

	return fTemp;
}

UINT8 CSVGLine::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_FOCUSHIGHLIGHT:
		return m_uFocusHighlight;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pProperties->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pFocus->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRLineErr;

		break;
	}

	return uTemp;
}

tstring CSVGLine::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pProperties->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pBasic->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pLocatable->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pFocus->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;

	*pErr = LASeRLineErr;

	return strTemp;
}

deque<UINT8> CSVGLine::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<UINT8> auTemp;

	auTemp = m_pProperties->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;
	else *pErr = LASeRNoErr;

	auTemp = m_pBasic->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;

	*pErr = LASeRLineErr;

	return auTemp;
}

deque<float> CSVGLine::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<float> afTemp;

	afTemp = m_pProperties->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;
	else *pErr = LASeRNoErr;

	afTemp = m_pLocatable->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;
	
	*pErr = LASeRLineErr;

	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'line' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 11. 19)       - first release
/************************************************************************/
UINT CSVGLine::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("focusHighlight"))	return ATTR_FOCUSHIGHLIGHT;
	else if (strAttrName == _T("x1"))			return ATTR_X1;
	else if (strAttrName == _T("y1"))			return ATTR_Y1;
	else if (strAttrName == _T("x2"))			return ATTR_X2;
	else if (strAttrName == _T("y2"))			return ATTR_Y2;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pBasic->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pLocatable->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pFocus->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'line' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 19)       - first release
/************************************************************************/
UINT8 CSVGLine::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_FOCUSHIGHLIGHT:
		return UINT8_TYPE;
		break;

	case ATTR_X1:
	case ATTR_Y1:
	case ATTR_X2:
	case ATTR_Y2:
		return FLOAT_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pBasic->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pLocatable->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pFocus->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'path' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 8. 29)	- first release
/************************************************************************/
lsrErr CSVGLine::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szValue = strValue.c_str();

	switch (uAttrType)
	{
	case ATTR_X1:
		m_fX1 = (float)_tstof(szValue);
		break;

	case ATTR_Y1:
		m_fY1 = (float)_tstof(szValue);
		break;

	case ATTR_X2:
		m_fX2 = (float)_tstof(szValue);
		break;

	case ATTR_Y2:
		m_fY2 = (float)_tstof(szValue);
		break;

	default:
		err = m_pProperties->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;
		else err = LASeRNoErr;

		err = m_pLocatable->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;
		else err = LASeRNoErr;

		err = m_pFocus->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRLineErr;

		break;
	}

	return err;
}

lsrErr CSVGLine::SetAttr(UINT uAttrType, float fValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_X1:
		m_fX1 = fValue;
		break;

	case ATTR_Y1:
		m_fY1 = fValue;
		break;

	case ATTR_X2:
		m_fX2 = fValue;
		break;

	case ATTR_Y2:
		m_fY2 = fValue;
		break;

	default:
		err = m_pProperties->SetAttr(uAttrType, fValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRLineErr;

		break;
	}

	return err;
}