///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"


CSVGPrefetch::CSVGPrefetch()
{
	CSVGObject::CSVGObject();

	m_uElementType				= SVG_prefetch;

	m_pHref						= new CSVGHref();

	m_uBandwidth				= 0;
	m_strMediaCharacterEncoding	= _T("");
	m_strMediaContentEncodings	= _T("");
	m_uMediaSize				= 0;
	m_uMediaTime				= 0;
}

CSVGPrefetch::~CSVGPrefetch()
{
	DELETEMEMORY(m_pHref);
}

/************************************************************************/
// SetAttribute
// Desc		: function of 'prefetch' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 15)	- first release
/************************************************************************/
lsrErr CSVGPrefetch::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pHref)
	{
		err = LASeRPrefetchErr;
		return err;
	}

	const TCHAR	*szTemp	= NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg href
	err = m_pHref->SetAttrs(pScene, pI);
	CONFIRM(err);

	// bandwidth
	szTemp = pScene->GetAttrValue(_T("bandwidth"));
	if (szTemp)
		m_uBandwidth = XStr2Int(pScene->GetAttrValue(_T("bandwidth")));

	// mediaCharacterEncoding
	szTemp = pScene->GetAttrValue(_T("mediaCharacterEncoding"));
	if (szTemp)
		m_strMediaCharacterEncoding = szTemp;

	// mediaContentEncodings
	szTemp = pScene->GetAttrValue(_T("mediaContentEncodings"));
	if (szTemp)
		m_strMediaContentEncodings = szTemp;

	// mediaSize
	szTemp = pScene->GetAttrValue(_T("mediaSize"));
	if (szTemp)
		m_uMediaSize = XStr2Int(pScene->GetAttrValue(_T("mediaSize")));

	// mediaTime
	szTemp = pScene->GetAttrValue(_T("mediaTime"));
	if (szTemp)
		m_uMediaTime = XStr2Int(pScene->GetAttrValue(_T("mediaTime")));

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 12. 29)	- first release
/************************************************************************/
lsrErr CSVGPrefetch::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_desc:
	case SVG_metadata:
	case SVG_switch:
	case SVG_title:
		/* nothing */
		break;

	default:
		err = LASeRPrefetchErr;
		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'prefetch' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 12. 29)	- first release
/************************************************************************/
UINT8 CSVGPrefetch::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;
	else *pErr = LASeRNoErr;

	uTemp = m_pHref->GetUINT8Attr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;

	*pErr = LASeRPrefetchErr;

	return uTemp;
}

UINT CSVGPrefetch::GetUINTAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_BANDWIDTH:
		return m_uBandwidth;
		break;

	case ATTR_MEDIASIZE:
		return m_uMediaSize;
		break;

	case ATTR_MEDIATIME:
		return m_uMediaTime;
		break;

	default:
		*pErr = LASeRPrefetchErr;
		break;
	}

	return uTemp;
}

tstring CSVGPrefetch::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	switch (uAttrType)
	{
	case ATTR_MEDIACHARACTERENCODING:
		return m_strMediaCharacterEncoding;
		break;

	case ATTR_MEDIACONTENTENCODINGS:
		return m_strMediaContentEncodings;
		break;

	default:
		strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		else *pErr = LASeRNoErr;

		strTemp = m_pHref->GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;

		*pErr = LASeRPrefetchErr;

		break;
	}

	return strTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'prefetch' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 12. 29)	- first release
/************************************************************************/
UINT CSVGPrefetch::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("bandwidth"))						return ATTR_BANDWIDTH;
	else if (strAttrName == _T("mediaCharacterEncoding"))	return ATTR_MEDIACHARACTERENCODING;
	else if (strAttrName == _T("mediaContentEncodings"))	return ATTR_MEDIACONTENTENCODINGS;
	else if (strAttrName == _T("mediaSize"))				return ATTR_MEDIASIZE;
	else if (strAttrName == _T("mediaTime"))				return ATTR_MEDIATIME;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'prefetch' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 12. 29)	- first release
/************************************************************************/
UINT8 CSVGPrefetch::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_BANDWIDTH:
	case ATTR_MEDIASIZE:
	case ATTR_MEDIATIME:
		return UINT32_TYPE;
		break;

	case ATTR_MEDIACHARACTERENCODING:
	case ATTR_MEDIACONTENTENCODINGS:
		return STRING_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'prefetch' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 12. 18)	- first release
/************************************************************************/
lsrErr CSVGPrefetch::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	err = m_pHref->SetAttr(uAttrType, strValue);
	if (err == LASeRNoErr)	return err;

	err = LASeRPrefetchErr;

	return err;
}