///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"
#include "..\Datatypes.h"


CSVGRadialGradient::CSVGRadialGradient()
{
	CSVGObject::CSVGObject();

	m_uElementType		= SVG_radialGradient;

	m_pProperties		= new CSVGProperties();

	m_uGradientUnits	= 0;
	m_fCX				= 0.0;
	m_fCY				= 0.0;
	m_fR				= 0.0;
}

CSVGRadialGradient::~CSVGRadialGradient()
{
	DELETEMEMORY(m_pProperties);
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'radialGradient' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 11. 20)	- first release
/************************************************************************/
lsrErr CSVGRadialGradient::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pProperties)
	{
		err = LASeRRadialGradientErr;
		return err;
	}

	const TCHAR *szTemp = NULL;
	
	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg properties
	err = m_pProperties->SetAttrs(pScene, pI);
	CONFIRM(err);

	// gradientUnits
	szTemp = pScene->GetAttrValue(_T("gradientUnits"));
	if (szTemp)
	{
		m_uGradientUnits = ReadGradientUnitsType(szTemp, &err);
		CONFIRM(err);
	}

	// cx
	szTemp = pScene->GetAttrValue(_T("cx"));
	if (szTemp)
		m_fCX = (float)_tstof(szTemp);

	// cy
	szTemp = pScene->GetAttrValue(_T("cy"));
	if (szTemp)
		m_fCY = (float)_tstof(szTemp);

	// r
	szTemp = pScene->GetAttrValue(_T("r"));
	if (szTemp)
		m_fR = (float)_tstof(szTemp);

	/************************************************************************/
	// 'required' confirmation
	if (m_fR == 0)
		err = LASeRRadialGradientErr;

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 11. 21)	- first release
/************************************************************************/
lsrErr CSVGRadialGradient::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_animate:
	case SVG_animateColor:
	case SVG_animateMotion:
	case SVG_animateTransform:
	case SVG_desc:
	case SVG_discard:
	case SVG_metadata:
	case SVG_set:
	case SVG_stop:
	case SVG_switch:
	case SVG_title:
		/* nothing */
		break;

	default:
		err = LASeRRadialGradientErr;
		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'radialGradient' to get attribute value using it's type
// Param	: attribute name
// History
//	PSH (2008. 11. 20)	- first release
/************************************************************************/
float CSVGRadialGradient::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	switch (uAttrType)
	{
	case ATTR_CX:
		return m_fCX;
		break;

	case ATTR_CY:
		return m_fCY;
		break;

	case ATTR_R:
		return m_fR;
		break;

	default:
		fTemp = m_pProperties->GetFloatAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return fTemp;

		*pErr = LASeRRadialGradientErr;

		break;
	}

	return fTemp;
}

UINT8 CSVGRadialGradient::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_GRADIENTUNITS:
		return m_uGradientUnits;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pProperties->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRRadialGradientErr;

		break;
	}

	return uTemp;
}

tstring CSVGRadialGradient::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pProperties->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;

	*pErr = LASeRRadialGradientErr;

	return strTemp;
}

deque<UINT8> CSVGRadialGradient::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<UINT8> auTemp;

	auTemp = m_pProperties->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;

	*pErr = LASeRRadialGradientErr;

	return auTemp;
}

deque<float> CSVGRadialGradient::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<float> afTemp;

	afTemp = m_pProperties->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;

	*pErr = LASeRRadialGradientErr;

	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'radialGradient' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 11. 20)       - first release
/************************************************************************/
UINT CSVGRadialGradient::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("gradientUnits"))	return ATTR_FOCUSHIGHLIGHT;
	else if (strAttrName == _T("cx"))		return ATTR_CX;
	else if (strAttrName == _T("cy"))		return ATTR_CY;
	else if (strAttrName == _T("r"))		return ATTR_R;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'radialGradient' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 20)       - first release
/************************************************************************/
UINT8 CSVGRadialGradient::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_GRADIENTUNITS:
		return UINT8_TYPE;
		break;

	case ATTR_CX:
	case ATTR_CY:
	case ATTR_R:
		return FLOAT_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'radialGradient' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 11. 20)	- first release
/************************************************************************/
lsrErr CSVGRadialGradient::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szValue = strValue.c_str();

	switch (uAttrType)
	{
	case ATTR_GRADIENTUNITS:
		m_uGradientUnits = ReadGradientUnitsType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_CX:
		m_fCX = (float)_tstof(szValue);
		break;

	case ATTR_CY:
		m_fCY = (float)_tstof(szValue);
		break;

	case ATTR_R:
		m_fR = (float)_tstof(szValue);
		break;

	default:
		err = m_pProperties->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRRadialGradientErr;

		break;
	}

	return err;
}

lsrErr CSVGRadialGradient::SetAttr(UINT uAttrType, float fValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_CX:
		m_fCX = fValue;
		break;

	case ATTR_CY:
		m_fCY = fValue;
		break;

	case ATTR_R:
		m_fR = fValue;
		break;

	default:
		err = m_pProperties->SetAttr(uAttrType, fValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRRadialGradientErr;

		break;
	}

	return err;
}