///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"


CSVGSolidColor::CSVGSolidColor()
{
	CSVGObject::CSVGObject();

	m_uElementType	= SVG_solidColor;

	m_pProperties	= new CSVGProperties();
}

CSVGSolidColor::~CSVGSolidColor()
{
	DELETEMEMORY(m_pProperties);
}

/************************************************************************/
// SetAttribute
// Desc		: function of 'solidColor' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 7. 30)	- first release
//	PSH (2008. 8. 4)	- delete adding children function
/************************************************************************/
lsrErr CSVGSolidColor::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pProperties)
	{
		err = LASeRSolidColorErr;
		return err;
	}

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg properties
	err = m_pProperties->SetAttrs(pScene, pI);
	CONFIRM(err);

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 12. 29)	- first release
/************************************************************************/
lsrErr CSVGSolidColor::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_animate:
	case SVG_animateColor:
	case SVG_animateMotion:
	case SVG_animateTransform:
	case SVG_desc:
	case SVG_discard:
	case SVG_metadata:
	case SVG_set:
	case SVG_switch:
	case SVG_title:
		/* nothing */
		break;

	default:
		err = LASeRSolidColorErr;
		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'solidColor' to get attribute value using it's type
// Param	: attribute name
// History
//	PSH (2008. 12. 29)	- first release
/************************************************************************/
float CSVGSolidColor::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	fTemp = m_pProperties->GetFloatAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return fTemp;

	*pErr = LASeRSolidColorErr;

	return fTemp;
}

UINT8 CSVGSolidColor::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;
	else *pErr = LASeRNoErr;

	uTemp = m_pProperties->GetUINT8Attr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;

	*pErr = LASeRSolidColorErr;

	return uTemp;
}

tstring CSVGSolidColor::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pProperties->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;

	*pErr = LASeRSolidColorErr;

	return strTemp;
}

deque<UINT8> CSVGSolidColor::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<UINT8> auTemp;

	auTemp = m_pProperties->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;

	*pErr = LASeRSolidColorErr;

	return auTemp;
}

deque<float> CSVGSolidColor::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<float> afTemp;

	afTemp = m_pProperties->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;

	*pErr = LASeRSolidColorErr;

	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'solidColor' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 12. 29)	- first release
/************************************************************************/
UINT CSVGSolidColor::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	uTemp = CSVGObject::GetAttrType(strAttrName);
	if (uTemp != 0)	return uTemp;

	uTemp = m_pProperties->GetAttrType(strAttrName);
	if (uTemp != 0)	return uTemp;

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'solidColor' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 12. 29)	- first release
/************************************************************************/
UINT8 CSVGSolidColor::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	uTemp = CSVGObject::GetAttrValType(uAttrType);
	if (uTemp != 0)	return uTemp;

	uTemp = m_pProperties->GetAttrValType(uAttrType);
	if (uTemp != 0)	return uTemp;

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'solidColor' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 12. 29)	- first release
/************************************************************************/
lsrErr CSVGSolidColor::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	err = m_pProperties->SetAttr(uAttrType, strValue);
	if (err == LASeRNoErr)	return err;

	err = LASeRSolidColorErr;

	return err;
}

lsrErr CSVGSolidColor::SetAttr(UINT uAttrType, float fValue)
{
	lsrErr err = LASeRNoErr;

	err = m_pProperties->SetAttr(uAttrType, fValue);
	if (err == LASeRNoErr)	return err;

	err = LASeRSolidColorErr;

	return err;
}