///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"
#include "..\Datatypes.h"

#include <algorithm>


CSVGSwitch::CSVGSwitch()
{
	CSVGObject::CSVGObject();

	m_uElementType					= SVG_switch;

	m_pProperties					= new CSVGProperties();
	m_pBasic						= new CSVGBasic();
	m_pLocatable					= new CSVGLocatable();
	m_pFocus						= new CSVGFocus();

	m_uFocusHighlight				= 0;
	m_bExternalResourcesRequired	= false;
}

CSVGSwitch::~CSVGSwitch()
{
	DELETEMEMORY(m_pProperties);
	DELETEMEMORY(m_pBasic);
	DELETEMEMORY(m_pLocatable);
	DELETEMEMORY(m_pFocus);
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'switch' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 12. 7)	- first release
/************************************************************************/
lsrErr CSVGSwitch::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pProperties || !m_pBasic || !m_pLocatable || !m_pFocus)
	{
		err = LASeRSwitchErr;
		return err;
	}

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg properties
	err = m_pProperties->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg basic
	err = m_pBasic->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg locatable
	err = m_pLocatable->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg focus
	err = m_pFocus->SetAttrs(pScene, pI);
	CONFIRM(err);

	// focusHighlight
	szTemp = pScene->GetAttrValue(_T("focusHighlight"));
	if (szTemp)
	{
		m_uFocusHighlight = ReadFocusHighlightType(szTemp, &err);
		CONFIRM(err);
	}

	// externalResourcesRequired
	szTemp = pScene->GetAttrValue(_T("externalResourcesRequired"));
	if (szTemp)
	{
		m_bExternalResourcesRequired = ReadBooleanType(szTemp, &err);
		CONFIRM(err);
	}

	/************************************************************************/
	// initial activation processing. inherit children
	SetDeactivate();

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	// activate myself
	SetActivate();

	/************************************************************************/
	// get condition (?)
	const XNodes aChilds = pScene->GetChilds();

	deque<LPCTSTR> aszCondition = aChilds[0]->GetAttrNames();

	UINT8 i, j;

	for (i = 0; i < aszCondition.size(); ++i)
	{
		// existance flag
		UINT8 uFlag = 1;

		for (j = 1; j < m_uNumOfChildren; ++j)
		{
			deque<LPCTSTR> aszTemp = aChilds[i]->GetAttrNames();

			if (find(aszTemp.begin(), aszTemp.end(), aszCondition[i]) == aszTemp.end())
			{
				uFlag = 0;

				break;
			}
		}

		if (uFlag == 1)
			m_auCondition.push_back(GetAttrType(aszCondition[i]));
	}

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 12. 7)	- first release
/************************************************************************/
lsrErr CSVGSwitch::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	err = GetParent()->CheckRightChild(uElementType);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'switch' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 12. 7)	- first release
/************************************************************************/
float CSVGSwitch::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	fTemp = m_pProperties->GetFloatAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return fTemp;

	*pErr = LASeRRectErr;

	return fTemp;
}

UINT8 CSVGSwitch::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_FOCUSHIGHLIGHT:
		return m_uFocusHighlight;
		break;

	case ATTR_EXTERNALRESOURCESREQUIRED:
		return m_bExternalResourcesRequired;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pProperties->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pFocus->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRRectErr;

		break;
	}

	return uTemp;
}

UINT CSVGSwitch::GetUINTAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT uTemp = 0;

	uTemp = m_pAdaptationAttr->GetUINTAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;

	*pErr = LASeRGErr;

	return uTemp;
}

tstring CSVGSwitch::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pProperties->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pBasic->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pLocatable->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pFocus->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pAdaptationAttr->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;

	*pErr = LASeRRectErr;

	return strTemp;
}

deque<UINT8> CSVGSwitch::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<UINT8> auTemp;

	auTemp = m_pProperties->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;
	else *pErr = LASeRNoErr;

	auTemp = m_pBasic->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;

	*pErr = LASeRRectErr;

	return auTemp;
}

deque<float> CSVGSwitch::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<float> afTemp;

	afTemp = m_pProperties->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;
	else *pErr = LASeRNoErr;

	afTemp = m_pLocatable->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;
	else *pErr = LASeRNoErr;

	afTemp = m_pAdaptationAttr->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;

	*pErr = LASeRRectErr;

	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'switch' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 12. 7)       - first release
/************************************************************************/
UINT CSVGSwitch::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("focusHighlight"))					return ATTR_FOCUSHIGHLIGHT;
	else if (strAttrName == _T("externalResourcesRequired"))	return ATTR_EXTERNALRESOURCESREQUIRED;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pBasic->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pLocatable->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pFocus->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pAdaptationAttr->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'switch' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 12. 7)       - first release
/************************************************************************/
UINT8 CSVGSwitch::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_FOCUSHIGHLIGHT:
	case ATTR_EXTERNALRESOURCESREQUIRED:
		return UINT8_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pBasic->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pLocatable->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pFocus->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pAdaptationAttr->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'switch' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 12. 7)	- first release
/************************************************************************/
lsrErr CSVGSwitch::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	err = m_pProperties->SetAttr(uAttrType, strValue);
	if (err == LASeRNoErr)	return err;
	else err = LASeRNoErr;

	err = m_pLocatable->SetAttr(uAttrType, strValue);
	if (err == LASeRNoErr)	return err;
	else err = LASeRNoErr;

	err = m_pFocus->SetAttr(uAttrType, strValue);
	if (err == LASeRNoErr)	return err;

	err = LASeRRectErr;

	return err;
}