///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\LASeR.h"
#include "..\Datatypes.h"


CSVGText::CSVGText()
{
	CSVGTSpan::CSVGTSpan();

	m_uElementType		= SVG_text;

	m_pLocatable		= new CSVGLocatable();

	m_uEditable			= 0;
	m_fX				= 0.0;
	m_fY				= 0.0;

	m_strTextContent	= _T("");
}

CSVGText::~CSVGText()
{
	DELETEMEMORY(m_pLocatable);

	m_afRotate.clear();
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'text' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 7. 14)	- first release
//	PSH (2008. 8. 4)	- delete adding children function
/************************************************************************/
lsrErr CSVGText::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pLocatable)
	{
		err = LASeRTextErr;
		return err;
	}

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	//svg tspan
	err = CSVGTSpan::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg locatable
	err = m_pLocatable->SetAttrs(pScene, pI);
	CONFIRM(err);

	// editable
	szTemp = pScene->GetAttrValue(_T("editable"));
	if (szTemp)
	{
		m_uEditable = ReadEditableType(szTemp, &err);
		CONFIRM(err);
	}

	// x
	szTemp = pScene->GetAttrValue(_T("x"));
	if (szTemp)
		m_fX = (float)_tstof(szTemp);

	// y
	szTemp = pScene->GetAttrValue(_T("y"));
	if (szTemp)
		m_fY = (float)_tstof(szTemp);

	// rotate
	szTemp = pScene->GetAttrValue(_T("rotate"));

	/************************************************************************/
	// set textContent
	m_strTextContent = pScene->value;

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 11. 21)	- first release
/************************************************************************/
lsrErr CSVGText::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_a:
	case SVG_animate:
	case SVG_animateColor:
	case SVG_animateMotion:
	case SVG_animateTransform:
	case SVG_desc:
	case SVG_discard:
	case SVG_handler:
	case SVG_metadata:
	case SVG_set:
	case SVG_switch:
	case SVG_title:
	case SVG_tspan:
		/* nothing */
		break;

	default:
		err = LASeRTextErr;
		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'text' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 8. 28)	- first release
/************************************************************************/
float CSVGText::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	switch (uAttrType)
	{
	case ATTR_X:
		return m_fX;
		break;

	case ATTR_Y:
		return m_fY;
		break;

	default:
		fTemp = CSVGTSpan::GetFloatAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return fTemp;

		*pErr = LASeRTextErr;

		break;
	}

	return fTemp;
}

UINT8 CSVGText::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_EDITABLE:
		return m_uEditable;
		break;

	default:
		uTemp = CSVGTSpan::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRTextErr;

		break;
	}

	return uTemp;
}

tstring CSVGText::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	switch (uAttrType)
	{
	case ATTR_TEXTCONTENT:
		return m_strTextContent;
		break;

	default:
		strTemp = CSVGTSpan::GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		else *pErr = LASeRNoErr;

		strTemp = m_pLocatable->GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;

		*pErr = LASeRTextErr;

		break;
	}

	return strTemp;
}

deque<UINT8> CSVGText::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<UINT8> auTemp;

	auTemp = CSVGTSpan::GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;

	*pErr = LASeRTextErr;

	return auTemp;
}

deque<float> CSVGText::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<float> afTemp;

	switch (uAttrType)
	{
	case ATTR_ROTATE:
		return m_afRotate;
		break;

	default:
		afTemp = CSVGTSpan::GetFloatListAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return afTemp;
		else *pErr = LASeRNoErr;

		afTemp = m_pLocatable->GetFloatListAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return afTemp;

		*pErr = LASeRTextErr;

		break;
	}

	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'tspan' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CSVGText::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("editable"))			return ATTR_EDITABLE;
	else if (strAttrName == _T("x"))			return ATTR_X;
	else if (strAttrName == _T("y"))			return ATTR_Y;
	else if (strAttrName == _T("rotate"))		return ATTR_ROTATE;
	else if (strAttrName == _T("textContent"))	return ATTR_TEXTCONTENT;
	else
	{
		uTemp = CSVGTSpan::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pLocatable->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'tspan' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 9)       - first release
/************************************************************************/
UINT8 CSVGText::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_X:
	case ATTR_Y:
		return FLOAT_TYPE;
		break;

	case ATTR_EDITABLE:
		return UINT8_TYPE;
		break;

	case ATTR_TEXTCONTENT:
		return STRING_TYPE;
		break;

	case ATTR_ROTATE:
		return FLOAT_LIST_TYPE;
		break;

	default:
		uTemp = CSVGTSpan::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pLocatable->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'text' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 8. 28)	- first release
/************************************************************************/
lsrErr CSVGText::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szValue = strValue.c_str();

	switch (uAttrType)
	{
	case ATTR_EDITABLE:
		m_uEditable = ReadEditableType(szValue, &err);
		CONFIRM(err);
		break;

	case ATTR_X:
		m_fX = (float)_tstof(szValue);
		break;

	case ATTR_Y:
		m_fY = (float)_tstof(szValue);
		break;

	case ATTR_ROTATE:
		m_afRotate;
		break;

	case ATTR_TEXTCONTENT:
		m_strTextContent = strValue;
		break;

	default:
		err = CSVGTSpan::SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;
		else err = LASeRNoErr;

		err = m_pLocatable->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRTextErr;

		break;
	}

	return err;
}

lsrErr CSVGText::SetAttr(UINT uAttrType, float fValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_X:
		m_fX = fValue;
		break;

	case ATTR_Y:
		m_fY = fValue;
		break;

	default:
		err = CSVGTSpan::SetAttr(uAttrType, fValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRTextErr;

		break;
	}

	return err;
}