///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"
#include "..\Datatypes.h"


CSVGTextArea::CSVGTextArea()
{
	CSVGObject::CSVGObject();

	m_uElementType		= SVG_textArea;

	m_pProperties		= new CSVGProperties();
	m_pBasic			= new CSVGBasic();
	m_pLocatable		= new CSVGLocatable();
	m_pFocus			= new CSVGFocus();

	m_uFocusHighlight	= 0;
	m_uEditable			= 0;
	m_fX				= 0.0;
	m_fY				= 0.0;
	m_fWidth			= 0.0;
	m_fHeight			= 0.0;

	m_strTextContent	= _T("");
}

CSVGTextArea::~CSVGTextArea()
{
	DELETEMEMORY(m_pProperties);
	DELETEMEMORY(m_pBasic);
	DELETEMEMORY(m_pLocatable);
	DELETEMEMORY(m_pFocus);
}

/************************************************************************/
// SetAttribute
// Desc		: function of 'textArea' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 17)	- first release
/************************************************************************/
lsrErr CSVGTextArea::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pProperties || !m_pFocus || !m_pBasic || !m_pLocatable)
	{
		err = LASeRNoMoreMemoryErr;
		return err;
	}

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg properties
	err = m_pProperties->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg basic
	err = m_pBasic->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg focus
	err = m_pFocus->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg locatable
	err = m_pLocatable->SetAttrs(pScene, pI);
	CONFIRM(err);

	// focusHighlight
	szTemp = pScene->GetAttrValue(_T("focusHighlight"));
	if (szTemp)
	{
		m_uFocusHighlight = ReadFocusHighlightType(szTemp, &err);
		CONFIRM(err);
	}

	// editable
	szTemp = pScene->GetAttrValue(_T("editable"));
	if (szTemp)
	{
		m_uEditable = ReadEditableType(szTemp, &err);
		CONFIRM(err);
	}

	// x
	szTemp = pScene->GetAttrValue(_T("x"));
	if (szTemp)
		m_fX = (float)_tstof(szTemp);

	// y
	szTemp = pScene->GetAttrValue(_T("y"));
	if (szTemp)
		m_fY = (float)_tstof(szTemp);

	// width
	szTemp = pScene->GetAttrValue(_T("width"));
	if (szTemp)
		m_fWidth = (float)_tstof(szTemp);

	// height
	szTemp = pScene->GetAttrValue(_T("height"));
	if (szTemp)
		m_fHeight = (float)_tstof(szTemp);

	/************************************************************************/
	// set textContent
	m_strTextContent = pScene->value;

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);		

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 12. 29)	- first release
/************************************************************************/
lsrErr CSVGTextArea::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_a:
	case SVG_animate:
	case SVG_animateColor:
	case SVG_animateMotion:
	case SVG_animateTransform:
	case SVG_desc:
	case SVG_discard:
	case SVG_handler:
	case SVG_metadata:
	case SVG_set:
	case SVG_switch:
	case SVG_tBreak:
	case SVG_title:
	case SVG_tspan:
		/* nothing */
		break;

	default:
		err = LASeRTextAreaErr;
		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'textArea' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 12. 29)	- first release
/************************************************************************/
float CSVGTextArea::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	switch (uAttrType)
	{
	case ATTR_X:
		return m_fX;
		break;

	case ATTR_Y:
		return m_fY;
		break;

	case ATTR_WIDTH:
		return m_fX;
		break;

	case ATTR_HEIGHT:
		return m_fY;
		break;

	default:
		fTemp = m_pProperties->GetFloatAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return fTemp;

		*pErr = LASeRTextAreaErr;

		break;
	}

	return fTemp;
}

UINT8 CSVGTextArea::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_FOCUSHIGHLIGHT:
		return m_uFocusHighlight;
		break;

	case ATTR_EDITABLE:
		return m_uEditable;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pProperties->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pFocus->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRTextAreaErr;

		break;
	}

	return uTemp;
}

tstring CSVGTextArea::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	switch (uAttrType)
	{
	case ATTR_TEXTCONTENT:
		return m_strTextContent;
		break;

	default:
		strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		else *pErr = LASeRNoErr;

		strTemp = m_pProperties->GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		else *pErr = LASeRNoErr;

		strTemp = m_pFocus->GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		else *pErr = LASeRNoErr;

		strTemp = m_pBasic->GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		else *pErr = LASeRNoErr;

		strTemp = m_pLocatable->GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;

		*pErr = LASeRTextAreaErr;

		break;
	}

	return strTemp;
}

deque<UINT8> CSVGTextArea::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<UINT8> auTemp;

	auTemp = m_pProperties->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;
	else *pErr = LASeRNoErr;

	auTemp = m_pBasic->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;

	*pErr = LASeRTextAreaErr;

	return auTemp;
}

deque<float> CSVGTextArea::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<float> afTemp;

	afTemp = m_pProperties->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;
	else *pErr = LASeRNoErr;

	afTemp = m_pLocatable->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;

	*pErr = LASeRTextAreaErr;

	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'textArea' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 12. 29) - first release
/************************************************************************/
UINT CSVGTextArea::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("focusHighlight"))	return ATTR_FOCUSHIGHLIGHT;
	else if (strAttrName == _T("editable"))		return ATTR_EDITABLE;
	else if (strAttrName == _T("x"))			return ATTR_X;
	else if (strAttrName == _T("y"))			return ATTR_Y;
	else if (strAttrName == _T("width"))		return ATTR_WIDTH;
	else if (strAttrName == _T("height"))		return ATTR_HEIGHT;
	else if (strAttrName == _T("textContent"))	return ATTR_TEXTCONTENT;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pBasic->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pLocatable->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pFocus->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'textArea' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 12. 30)	- first release
/************************************************************************/
UINT8 CSVGTextArea::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_FOCUSHIGHLIGHT:
	case ATTR_EDITABLE:
		return UINT8_TYPE;
		break;

	case ATTR_X:
	case ATTR_Y:
	case ATTR_WIDTH:
	case ATTR_HEIGHT:
		return FLOAT_TYPE;
		break;

	case ATTR_TEXTCONTENT:
		return STRING_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pBasic->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pLocatable->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pFocus->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'textArea' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 12. 30)	- first release
/************************************************************************/
lsrErr CSVGTextArea::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szValue = strValue.c_str();

	switch (uAttrType)
	{
	case ATTR_EDITABLE:
		m_uEditable = ReadEditableType(szValue, &err);
		break;

	case ATTR_X:
		m_fX = (float)_tstof(szValue);
		break;

	case ATTR_Y:
		m_fY = (float)_tstof(szValue);
		break;

	case ATTR_WIDTH:
		m_fWidth = (float)_tstof(szValue);
		break;

	case ATTR_HEIGHT:
		m_fHeight = (float)_tstof(szValue);
		break;

	case ATTR_TEXTCONTENT:
		m_strTextContent = strValue;
		break;

	default:
		err = m_pProperties->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;
		else err = LASeRNoErr;

		err = m_pFocus->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;
		else err = LASeRNoErr;

		err = m_pLocatable->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRTextAreaErr;

		break;
	}

	return err;
}

lsrErr CSVGTextArea::SetAttr(UINT uAttrType, float fValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_X:
		m_fX = fValue;
		break;

	case ATTR_Y:
		m_fY = fValue;
		break;

	case ATTR_WIDTH:
		m_fWidth = fValue;
		break;

	case ATTR_HEIGHT:
		m_fHeight = fValue;
		break;

	default:
		err = m_pProperties->SetAttr(uAttrType, fValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRTextAreaErr;

		break;
	}

	return err;
}