///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"


CSVGTSpan::CSVGTSpan()
{
	CSVGObject::CSVGObject();

	m_pProperties		= new CSVGProperties();
	m_pBasic			= new CSVGBasic();
	m_pFocus			= new CSVGFocus();

	m_uFocusHighlight	= 0;
}

CSVGTSpan::~CSVGTSpan()
{
	DELETEMEMORY(m_pProperties);
	DELETEMEMORY(m_pBasic);
	DELETEMEMORY(m_pFocus);
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'tspan' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 15)	- first release
/************************************************************************/
lsrErr CSVGTSpan::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pProperties || !m_pBasic || !m_pFocus)
	{
		err = LASeRTSpanErr;
		return err;
	}

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg properties
	err = m_pProperties->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg basic
	err = m_pBasic->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg focus
	err = m_pFocus->SetAttrs(pScene, pI);
	CONFIRM(err);

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 11. 21)	- first release
/************************************************************************/
lsrErr CSVGTSpan::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	err = GetParent()->CheckRightChild(uElementType);

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'tspan' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 9. 10)	- first release
/************************************************************************/
float CSVGTSpan::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	fTemp = m_pProperties->GetFloatAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return fTemp;

	*pErr = LASeRTSpanErr;

	return fTemp;
}

UINT8 CSVGTSpan::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_FOCUSHIGHLIGHT:
		return m_uFocusHighlight;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pProperties->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pFocus->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRTSpanErr;

		break;
	}

	return uTemp;
}

tstring CSVGTSpan::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pProperties->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pBasic->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pFocus->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;

	*pErr = LASeRTSpanErr;

	return strTemp;
}

deque<UINT8> CSVGTSpan::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<UINT8> auTemp;

	auTemp = m_pProperties->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;
	else *pErr = LASeRNoErr;

	auTemp = m_pBasic->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;

	*pErr = LASeRTSpanErr;

	return auTemp;
}

deque<float> CSVGTSpan::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<float> afTemp;

	afTemp = m_pProperties->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;

	*pErr = LASeRTSpanErr;

	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'tspan' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CSVGTSpan::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("focusHighlight"))	return ATTR_FOCUSHIGHLIGHT;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pBasic->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pFocus->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'tspan' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 9)       - first release
/************************************************************************/
UINT8 CSVGTSpan::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_FOCUSHIGHLIGHT:
		return UINT8_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pBasic->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pFocus->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'tspan' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 9. 9)	- first release
/************************************************************************/
lsrErr CSVGTSpan::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	err = m_pProperties->SetAttr(uAttrType, strValue);
	if (err == LASeRNoErr)	return err;
	else err = LASeRNoErr;

	err = m_pFocus->SetAttr(uAttrType, strValue);
	if (err == LASeRNoErr)	return err;

	err = LASeRTSpanErr;

	return err;
}

lsrErr CSVGTSpan::SetAttr(UINT uAttrType, float fValue)
{
	lsrErr err = LASeRNoErr;

	err = m_pProperties->SetAttr(uAttrType, fValue);
	if (err == LASeRNoErr)	return err;

	err = LASeRTSpanErr;

	return err;
}