///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"
#include "..\Datatypes.h"


CSVGTFontFace::CSVGTFontFace()
{
	CSVGObject::CSVGObject();

	m_uElementType					= SVGT_fontface;

	m_fAccentHeight					= 0.0;
	m_fAlphabetic					= 0.0;
	m_fAscent						= 0.0;
	m_strBBox						= _T("");
	m_fCapHeight					= 0.0;
	m_fDescent						= 0.0;
	m_bExternalResourcesRequired	= 0;
	m_strFontFamily					= _T("");
	m_strFontStretch				= _T("");
	m_strFontStyle					= _T("");
	m_strFontVariant				= _T("");
	m_strFontWeight					= _T("");
	m_fHanging						= 0.0;
	m_fIdeographic					= 0.0;
	m_fMathematical					= 0.0;
	m_fOverlinePosition				= 0.0;
	m_fOverlineThickness			= 0.0;
	m_fSlope						= 0.0;
	m_fStemH						= 0.0;
	m_fStemV						= 0.0;
	m_fStrikeThroughPosition		= 0.0;
	m_fStrikeThroughThickness		= 0.0;
	m_fUnderlinePostion				= 0.0;
	m_fUnderlineThickness			= 0.0;
	m_strUnicodeRange				= _T("");
	m_strUnitsPerEM					= _T("");
	m_strWidths						= _T("");
	m_strXHeight					= _T("");
}

CSVGTFontFace::~CSVGTFontFace()
{
	m_auPanose1.clear();
}

/************************************************************************/
// SetAttribute
// Desc		: function of 'font-face' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 15)	- first release
/************************************************************************/
lsrErr CSVGTFontFace::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// accent-height
	szTemp = pScene->GetAttrValue(_T("accent-height"));
	if (szTemp)
		m_fAccentHeight = (float)_tstof(szTemp);

	// alphabetic
	szTemp = pScene->GetAttrValue(_T("alphabetic"));
	if (szTemp)
		m_fAlphabetic = (float)_tstof(szTemp);

	// ascent
	szTemp = pScene->GetAttrValue(_T("ascent"));
	if (szTemp)
		m_fAscent = (float)_tstof(szTemp);

	// bbox
	szTemp = pScene->GetAttrValue(_T("bbox"));
	if (szTemp)
		m_strBBox = szTemp;

	// cap-height
	szTemp = pScene->GetAttrValue(_T("cap-height"));
	if (szTemp)
		m_fCapHeight = (float)_tstof(szTemp);

	// descent
	szTemp = pScene->GetAttrValue(_T("descent"));
	if (szTemp)
		m_fDescent = (float)_tstof(szTemp);

	// externalResourcesRequired
	szTemp = pScene->GetAttrValue(_T("externalResourcesRequired"));
	if (szTemp)
		m_bExternalResourcesRequired = ReadBooleanType(szTemp, &err);

	// font-family
	szTemp = pScene->GetAttrValue(_T("font-family"));
	if (szTemp)
		m_strFontFamily = szTemp;

	// font-stretch
	szTemp = pScene->GetAttrValue(_T("font-stretch"));
	if (szTemp)
		m_strFontStretch = szTemp;

	// font-style
	szTemp = pScene->GetAttrValue(_T("font-style"));
	if (szTemp)
		m_strFontStyle = szTemp;

	// font-variant
	szTemp = pScene->GetAttrValue(_T("font-variant"));
	if (szTemp)
		m_strFontVariant = szTemp;

	// font-weight
	szTemp = pScene->GetAttrValue(_T("font-weight"));
	if (szTemp)
		m_strFontWeight = szTemp;

	// hanging
	szTemp = pScene->GetAttrValue(_T("hanging"));
	if (szTemp)
		m_fHanging = (float)_tstof(szTemp);

	// ideographic
	szTemp = pScene->GetAttrValue(_T("ideographic"));
	if (szTemp)
		m_fIdeographic = (float)_tstof(szTemp);

	// mathematical
	szTemp = pScene->GetAttrValue(_T("mathematical"));
	if (szTemp)
		m_fMathematical = (float)_tstof(szTemp);

	// overline-position
	szTemp = pScene->GetAttrValue(_T("overline-position"));
	if (szTemp)
		m_fOverlinePosition = (float)_tstof(szTemp);

	// overline-thickness
	szTemp = pScene->GetAttrValue(_T("overline-thickness"));
	if (szTemp)
		m_fOverlineThickness = (float)_tstof(szTemp);

	// panose-1
	szTemp = pScene->GetAttrValue(_T("panose-1"));
	if (szTemp)
		m_auPanose1 = ReadInt8ListType(szTemp, ' ', &err);

	// slope
	szTemp = pScene->GetAttrValue(_T("slope"));
	if (szTemp)
		m_fSlope = (float)_tstof(szTemp);

	// stemh
	szTemp = pScene->GetAttrValue(_T("stemh"));
	if (szTemp)
		m_fStemH = (float)_tstof(szTemp);

	// stemv
	szTemp = pScene->GetAttrValue(_T("stemv"));
	if (szTemp)
		m_fStemV = (float)_tstof(szTemp);

	// strikethrough-position
	szTemp = pScene->GetAttrValue(_T("strikethrough-position"));
	if (szTemp)
		m_fStrikeThroughPosition = (float)_tstof(szTemp);

	// strikethrough-thickness
	szTemp = pScene->GetAttrValue(_T("strikethrough-thickness"));
	if (szTemp)
		m_fStrikeThroughThickness = (float)_tstof(szTemp);

	// underline-position
	szTemp = pScene->GetAttrValue(_T("underline-position"));
	if (szTemp)
		m_fUnderlinePostion = (float)_tstof(szTemp);

	// underline-thickness
	szTemp = pScene->GetAttrValue(_T("underline-thickness"));
	if (szTemp)
		m_fUnderlineThickness = (float)_tstof(szTemp);

	// unicode-range
	szTemp = pScene->GetAttrValue(_T("unicode-range"));
	if (szTemp)
		m_strUnicodeRange = szTemp;

	// units-per-em
	szTemp = pScene->GetAttrValue(_T("units-per-em"));
	if (szTemp)
		m_strUnitsPerEM = szTemp;

	// widths
	szTemp = pScene->GetAttrValue(_T("widths"));
	if (szTemp)
		m_strWidths = szTemp;

	// x-height
	szTemp = pScene->GetAttrValue(_T("x-height"));
	if (szTemp)
		m_strXHeight = szTemp;

	/************************************************************************/
	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 12. 16)	- first release
/************************************************************************/
lsrErr CSVGTFontFace::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_desc:
	case SVGT_fontfacesrc:
	case SVG_metadata:
	case SVG_switch:
	case SVG_title:
		/* nothing */
		break;

	default:
		err = LASeRFontFaceErr;
		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'font-face' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 12. 16)	- first release
/************************************************************************/
float CSVGTFontFace::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	switch (uAttrType)
	{
	case ATTR_ACCENT_HEIGHT:
		return m_fAccentHeight;
		break;

	case ATTR_ALPHABETIC:
		return m_fAlphabetic;
		break;

	case ATTR_ASCENT:
		return m_fAscent;
		break;

	case ATTR_CAP_HEIGHT:
		return m_fCapHeight;
		break;

	case ATTR_DESCENT:
		return m_fDescent;
		break;

	case ATTR_HANGING:
		return m_fHanging;
		break;

	case ATTR_IDEOGRAPHIC:
		return m_fIdeographic;
		break;

	case ATTR_MATHEMATICAL:
		return m_fMathematical;
		break;

	case ATTR_OVERLINE_POSITION:
		return m_fOverlinePosition;
		break;

	case ATTR_OVERLINE_THICKNESS:
		return m_fOverlineThickness;
		break;

	case ATTR_SLOPE:
		return m_fSlope;
		break;

	case ATTR_STEMH:
		return m_fStemH;
		break;

	case ATTR_STEMV:
		return m_fStemV;
		break;

	case ATTR_STRIKE_THROUGH_POSITION:
		return m_fStrikeThroughPosition;
		break;

	case ATTR_STRIKE_THROUGH_THICKNESS:
		return m_fStrikeThroughThickness;
		break;

	case ATTR_UNDERLINE_POSITION:
		return m_fUnderlinePostion;
		break;

	case ATTR_UNDERLINE_THICKNESS:
		return m_fUnderlineThickness;
		break;

	default:
		*pErr = LASeRFontFaceErr;

		break;
	}

	return fTemp;
}

UINT8 CSVGTFontFace::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_EXTERNALRESOURCESREQUIRED:
		return m_bExternalResourcesRequired;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRFontFaceErr;

		break;
	}

	return uTemp;
}

tstring CSVGTFontFace::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	switch (uAttrType)
	{
	case ATTR_BBOX:
		return m_strBBox;
		break;

	case ATTR_FONT_FAMILY:
		return m_strFontFamily;
		break;

	case ATTR_FONT_STRETCH:
		return m_strFontStretch;
		break;

	case ATTR_FONT_STYLE:
		return m_strFontStyle;
		break;

	case ATTR_FONT_VARIANT:
		return m_strFontVariant;
		break;

	case ATTR_FONT_WEIGHT:
		return m_strFontWeight;
		break;

	case ATTR_UNICODE_RANGE:
		return m_strUnicodeRange;
		break;

	case ATTR_UNITS_PER_EM:
		return m_strUnitsPerEM;
		break;

	case ATTR_WIDTHS:
		return m_strWidths;
		break;

	case ATTR_X_HEIGHT:
		return m_strXHeight;
		break;

	default:
		strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;

		*pErr = LASeRFontFaceErr;
	}

	return strTemp;
}

deque<UINT8> CSVGTFontFace::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<UINT8> auTemp;

	switch (uAttrType)
	{
	case ATTR_PANOSE_1:
		return m_auPanose1;
		break;

	default:
		*pErr = LASeRFontFaceErr;
		break;
	}

	return auTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'font-face' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 12. 16)       - first release
/************************************************************************/
UINT CSVGTFontFace::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("accent-height"))						return ATTR_ACCENT_HEIGHT;
	else if (strAttrName == _T("alphabetic"))					return ATTR_ALPHABETIC;
	else if (strAttrName == _T("ascent"))						return ATTR_ASCENT;
	else if (strAttrName == _T("bbox"))							return ATTR_BBOX;
	else if (strAttrName == _T("cap-height"))					return ATTR_CAP_HEIGHT;
	else if (strAttrName == _T("descent"))						return ATTR_DESCENT;
	else if (strAttrName == _T("externalResourcesRequired"))	return ATTR_EXTERNALRESOURCESREQUIRED;
	else if (strAttrName == _T("font-family"))					return ATTR_FONT_FAMILY;
	else if (strAttrName == _T("font-stretch"))					return ATTR_FONT_STRETCH;
	else if (strAttrName == _T("font-style"))					return ATTR_FONT_STYLE;
	else if (strAttrName == _T("font-variant"))					return ATTR_FONT_VARIANT;
	else if (strAttrName == _T("font-weight"))					return ATTR_FONT_WEIGHT;
	else if (strAttrName == _T("hanging"))						return ATTR_HANGING;
	else if (strAttrName == _T("ideographic"))					return ATTR_IDEOGRAPHIC;
	else if (strAttrName == _T("mathematical"))					return ATTR_MATHEMATICAL;
	else if (strAttrName == _T("overline-position"))			return ATTR_OVERLINE_POSITION;
	else if (strAttrName == _T("overline-thickness"))			return ATTR_OVERLINE_THICKNESS;
	else if (strAttrName == _T("panose-1"))						return ATTR_PANOSE_1;
	else if (strAttrName == _T("slope"))						return ATTR_SLOPE;
	else if (strAttrName == _T("stemh"))						return ATTR_STEMH;
	else if (strAttrName == _T("stemv"))						return ATTR_STEMV;
	else if (strAttrName == _T("strikethrough-position"))		return ATTR_STRIKE_THROUGH_POSITION;
	else if (strAttrName == _T("strikethrough-thickness"))		return ATTR_STRIKE_THROUGH_THICKNESS;
	else if (strAttrName == _T("underline-position"))			return ATTR_UNDERLINE_POSITION;
	else if (strAttrName == _T("underline-thickness"))			return ATTR_UNDERLINE_THICKNESS;
	else if (strAttrName == _T("unicode-range"))				return ATTR_UNICODE_RANGE;
	else if (strAttrName == _T("units-per-em"))					return ATTR_UNITS_PER_EM;
	else if (strAttrName == _T("widths"))						return ATTR_WIDTHS;
	else if (strAttrName == _T("x-height"))						return ATTR_X_HEIGHT;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'font-face' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 12. 16)       - first release
/************************************************************************/
UINT8 CSVGTFontFace::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_ACCENT_HEIGHT:
	case ATTR_ALPHABETIC:
	case ATTR_ASCENT:
	case ATTR_CAP_HEIGHT:
	case ATTR_DESCENT:
	case ATTR_HANGING:
	case ATTR_IDEOGRAPHIC:
	case ATTR_MATHEMATICAL:
	case ATTR_OVERLINE_POSITION:
	case ATTR_OVERLINE_THICKNESS:
	case ATTR_SLOPE:
	case ATTR_STEMH:
	case ATTR_STEMV:
	case ATTR_STRIKE_THROUGH_POSITION:
	case ATTR_STRIKE_THROUGH_THICKNESS:
	case ATTR_UNDERLINE_POSITION:
	case ATTR_UNDERLINE_THICKNESS:
		return FLOAT_TYPE;
		break;

	case ATTR_EXTERNALRESOURCESREQUIRED:
		return UINT8_TYPE;
		break;

	case ATTR_BBOX:
	case ATTR_FONT_FAMILY:
	case ATTR_FONT_STRETCH:
	case ATTR_FONT_STYLE:
	case ATTR_FONT_VARIANT:
	case ATTR_FONT_WEIGHT:
	case ATTR_UNICODE_RANGE:
	case ATTR_UNITS_PER_EM:
	case ATTR_WIDTHS:
	case ATTR_X_HEIGHT:
		return STRING_TYPE;
		break;

	case ATTR_PANOSE_1:
		return UINT8_LIST_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}