///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"
#include "..\Datatypes.h"


CSVGTGlyph::CSVGTGlyph()
{
	CSVGObject::CSVGObject();

	m_uElementType				= SVGT_glyph;

	m_strArabicForm				= _T("");
	m_strD						= _T("");
	m_strGlyphName				= _T("");
	m_fHorizAdvX				= 0.0;
	m_strLang					= _T("");
	m_strUnicode				= _T("");
	m_bExternalResourceRequired	= 0;
}

/************************************************************************/
// SetAttribute
// Desc		: function of 'glyph' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 4)	- first release
//	PSH (2008. 8. 4)	- delete adding children function
/************************************************************************/
lsrErr CSVGTGlyph::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// arabic-form
	szTemp = pScene->GetAttrValue(_T("arabic-form"));
	if (szTemp)
		m_strArabicForm = szTemp;

	// d
	szTemp = pScene->GetAttrValue(_T("d"));
	if (szTemp)
		m_strD = szTemp;

	// glyph-name
	szTemp = pScene->GetAttrValue(_T("glyph-name"));
	if (szTemp)
		m_strGlyphName = szTemp;

	// horiz-adv-x
	szTemp = pScene->GetAttrValue(_T("horiz-adv-x"));
	if (szTemp)
		m_fHorizAdvX = (float)_tstof(szTemp);

	// lang
	szTemp = pScene->GetAttrValue(_T("lang"));
	if (szTemp)
		m_strLang = szTemp;

	// unicode
	szTemp = pScene->GetAttrValue(_T("unicode"));
	if (szTemp)
		m_strUnicode = szTemp;

	// externalResourcesRequired
	szTemp = pScene->GetAttrValue(_T("externalResourcesRequired"));
	if (szTemp)
	{
		m_bExternalResourceRequired = ReadBooleanType(szTemp, &err);
		CONFIRM(err);
	}

	/************************************************************************/
	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 12. 16)	- first release
/************************************************************************/
lsrErr CSVGTGlyph::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_desc:
	case SVG_metadata:
	case SVG_switch:
	case SVG_title:
		/* nothing */
		break;

	default:
		err = LASeRGlyphErr;
		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'glyph' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 12. 16)	- first release
/************************************************************************/
float CSVGTGlyph::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	switch (uAttrType)
	{
	case ATTR_HORIZ_ADV_X:
		return m_fHorizAdvX;
		break;

	default:
		*pErr = LASeRGlyphErr;

		break;
	}

	return fTemp;
}

UINT8 CSVGTGlyph::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_EXTERNALRESOURCESREQUIRED:
		return m_bExternalResourceRequired;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRGlyphErr;

		break;
	}

	return uTemp;
}

tstring CSVGTGlyph::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	switch (uAttrType)
	{
	case ATTR_ARABIC_FORM:
		return m_strArabicForm;
		break;

	case ATTR_D:
		return m_strD;
		break;

	case ATTR_GLYPH_NAME:
		return m_strGlyphName;
		break;

	case ATTR_LANG:
		return m_strLang;
		break;

	case ATTR_UNICODE:
		return m_strUnicode;
		break;

	default:
		strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;

		*pErr = LASeRGlyphErr;
	}

	return strTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'glyph' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 12. 18)       - first release
/************************************************************************/
UINT CSVGTGlyph::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("arabic-form"))						return ATTR_ACCENT_HEIGHT;
	else if (strAttrName == _T("d"))							return ATTR_ALPHABETIC;
	else if (strAttrName == _T("glyph-name"))					return ATTR_ASCENT;
	else if (strAttrName == _T("horiz-adv-x"))					return ATTR_BBOX;
	else if (strAttrName == _T("lang"))							return ATTR_CAP_HEIGHT;
	else if (strAttrName == _T("unicode"))						return ATTR_DESCENT;
	else if (strAttrName == _T("externalResourcesRequired"))	return ATTR_EXTERNALRESOURCESREQUIRED;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'glyph' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 12. 18)       - first release
/************************************************************************/
UINT8 CSVGTGlyph::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_HORIZ_ADV_X:
		return FLOAT_TYPE;
		break;

	case ATTR_EXTERNALRESOURCESREQUIRED:
		return UINT8_TYPE;
		break;

	case ATTR_ARABIC_FORM:
	case ATTR_D:
	case ATTR_GLYPH_NAME:
	case ATTR_LANG:
	case ATTR_UNICODE:
		return STRING_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}