///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"


CSVGTHkern::CSVGTHkern()
{
	CSVGObject::CSVGObject();

	m_uElementType	= SVGT_hkern;

	m_strG1			= _T("");
	m_strG2			= _T("");
	m_fK			= 0.0;
	m_strU1			= _T("");
	m_strU2			= _T("");
}

/************************************************************************/
// SetAttribute
// Desc		: function of 'hkern' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 12. 28)	- first release
/************************************************************************/
lsrErr CSVGTHkern::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// g1
	szTemp = pScene->GetAttrValue(_T("g1"));
	if (szTemp)
		m_strG1 = szTemp;

	// g2
	szTemp = pScene->GetAttrValue(_T("g2"));
	if (szTemp)
		m_strG2 = szTemp;

	// k
	szTemp = pScene->GetAttrValue(_T("k"));
	if (szTemp)
		m_fK = (float)_tstof(szTemp);

	// u1
	szTemp = pScene->GetAttrValue(_T("u1"));
	if (szTemp)
		m_strU1 = szTemp;

	// u2
	szTemp = pScene->GetAttrValue(_T("u2"));
	if (szTemp)
		m_strU2 = szTemp;

	/************************************************************************/
	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 12. 28)	- first release
/************************************************************************/
lsrErr CSVGTHkern::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_desc:
	case SVG_metadata:
	case SVG_switch:
	case SVG_title:
		/* nothing */
		break;

	default:
		err = LASeRHkernErr;
		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'hkern' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 12. 28)	- first release
/************************************************************************/
float CSVGTHkern::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	switch (uAttrType)
	{
	case ATTR_K:
		return m_fK;
		break;

	default:
		*pErr = LASeRHkernErr;
		break;
	}

	return fTemp;
}

UINT8 CSVGTHkern::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;

	*pErr = LASeRHkernErr;

	return uTemp;
}

tstring CSVGTHkern::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	switch (uAttrType)
	{
	case ATTR_G1:
		return m_strG1;
		break;

	case ATTR_G2:
		return m_strG2;
		break;

	case ATTR_U1:
		return m_strU1;
		break;

	case ATTR_U2:
		return m_strU2;
		break;

	default:
		strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;

		*pErr = LASeRHkernErr;
		
		break;
	}

	return strTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'hkern' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 12. 28)       - first release
/************************************************************************/
UINT CSVGTHkern::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("g1"))		return ATTR_G1;
	else if (strAttrName == _T("g2"))	return ATTR_G2;
	else if (strAttrName == _T("k"))	return ATTR_K;
	else if (strAttrName == _T("u1"))	return ATTR_U1;
	else if (strAttrName == _T("u2"))	return ATTR_U2;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'hkern' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 12. 28)       - first release
/************************************************************************/
UINT8 CSVGTHkern::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_G1:
	case ATTR_G2:
	case ATTR_U1:
	case ATTR_U2:
		return STRING_TYPE;
		break;

	case ATTR_K:
		return FLOAT_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}