///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "laser_error.h"
#include "..\LASeR.h"
#include "..\Datatypes.h"


CLASeRHeader::CLASeRHeader()
{
	CLASeR::CLASeR();

	m_uElementType				= LSR_LASeRHeader;
	m_uMyIndex					= 1;

	m_uProfile					= 0;
	m_uLevel					= 0;
	m_nResolution				= 0;
	m_uTimeResoluton			= 0;
	m_uCoordBits				= 12;
	m_uScaleBitsMinusCoordBits	= 0;
	m_uColorComponentBits		= 8;
	m_bUseFullRequestHost		= 0;
	m_uPathComponents			= 15;
	m_bNewSceneIndicator		= 0;
	m_uTimeEndcoding			= 0;
	m_uPointsCodecType			= 0;

	m_uAdaptationType			= 0;
	m_uAdaptationTypeValue		= 0;
}

CLASeRHeader::~CLASeRHeader()
{
	m_auAdaptationConstraints.clear();
}

/************************************************************************/
// SetParent
// Desc		: function to set a SAF parent
// Param	: a parent's void pointer, parent's element type
// History
//	PSH (2008. 7. 23)	- first release
/************************************************************************/
void CLASeRHeader::SetParent(void *pParent, UINT8 uParentElementType)
{
	m_pSAFParent			= pParent;
	m_uParentElementType	= uParentElementType;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'LASeRHeader' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 7. 14)	- first release
//	PSH (2008. 9. 20)	- complete the setting attribute part
/************************************************************************/
lsrErr CLASeRHeader::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szTemp = NULL;

	pI;											// nonsense

	/************************************************************************/
	// set attribute
	// profile
	szTemp = pScene->GetAttrValue(_T("profile"));
	if (szTemp)
	{
		m_uProfile = ReadProfileType(szTemp, &err);
		CONFIRM(err);
	}

	// level
	szTemp = pScene->GetAttrValue(_T("level"));
	if (szTemp)
		m_uLevel = (UINT8)XStr2Int(pScene->GetAttrValue(_T("level")));

	if (m_uLevel > 15)
		err = LASeRValidationErr;

	// resolution
	szTemp = pScene->GetAttrValue(_T("resolution"));
	if (szTemp)
		m_nResolution = ReadResolutionType((UINT8)XStr2Int(pScene->GetAttrValue(_T("resolution"))), &err);

	// timeResolution
	szTemp = pScene->GetAttrValue(_T("timeResolution"));
	if (szTemp)
		m_uTimeResoluton = (UINT16)XStr2Int(pScene->GetAttrValue(_T("timeResolution")));

	// coordBits
	szTemp = pScene->GetAttrValue(_T("coordBits"));
	if (szTemp)
		m_uCoordBits = (UINT8)XStr2Int(pScene->GetAttrValue(_T("coordBits")));

	if (m_uCoordBits > 31)
		err = LASeRValidationErr;

	// scaleBits_minus_coordBits
	m_uScaleBitsMinusCoordBits = (UINT8)XStr2Int(pScene->GetAttrValue(_T("scaleBits_minus_coordBits")));

	if (m_uScaleBitsMinusCoordBits > 31)
		err = LASeRValidationErr;

	// colorComponentBits
	szTemp = pScene->GetAttrValue(_T("colorComponentBits"));
	if (szTemp)
		m_uColorComponentBits = (UINT8)XStr2Int(pScene->GetAttrValue(_T("colorComponentBits")));

	if (m_uColorComponentBits > 32)
		err = LASeRValidationErr;

	// useFullRequestHost
	szTemp = pScene->GetAttrValue(_T("useFullRequestHost"));
	if (szTemp)
	{
		m_bUseFullRequestHost = ReadBooleanType(szTemp, &err);
		CONFIRM(err);
	}

	// pathComponents
	szTemp = pScene->GetAttrValue(_T("pathComponents"));
	if (szTemp)
		m_uPathComponents = (UINT8)XStr2Int(pScene->GetAttrValue(_T("pathComponents")));

	if (m_uPathComponents > 15)
		err = LASeRValidationErr;

	// newSceneIndicator
	szTemp = pScene->GetAttrValue(_T("newSceneIndicator"));
	if (szTemp)
	{
		m_bNewSceneIndicator = ReadBooleanType(szTemp, &err);
		CONFIRM(err);
	}

	// timeEncoding

	// pointsCodec
	szTemp = pScene->GetAttrValue(_T("pointsCodec"));
	if (szTemp)
	{
		m_uPointsCodecType = ReadPointsCodecType(szTemp, &err);
		CONFIRM(err);
	}

	// adaptationType
	szTemp = pScene->GetAttrValue(_T("adaptationType"));
	if (szTemp)
		m_uAdaptationType = (UINT8)XStr2Int(pScene->GetAttrValue(_T("adaptationType")));

	// adaptationTypeValue
	szTemp = pScene->GetAttrValue(_T("adaptationTypeValue"));
	if (szTemp)
		m_uAdaptationTypeValue = XStr2Int(pScene->GetAttrValue(_T("adaptationTypeValue")));

	// adaptationConstraints
	szTemp = pScene->GetAttrValue(_T("adaptationConstraints"));
	if (szTemp)
	{
		m_auAdaptationConstraints = ReadIntListType(szTemp, &err);
		CONFIRM(err);
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'LASeRHeader' and it's children to get attribute value using it's name
// Param	: target attribute name, wanted value
// History
//	PSH (2008. 9. 29)	- first release
/************************************************************************/
UINT8 CLASeRHeader::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	switch (uAttrType)
	{
	case ATTR_PROFILE:
		return m_uProfile;
		break;

	case ATTR_LEVEL:
		return m_uLevel;
		break;

	case ATTR_RESOLUTION:
		return (UINT8)m_nResolution;
		break;

	case ATTR_COORDBITS:
		return m_uCoordBits;
		break;

	case ATTR_SCALEBITSMINUSCOORDBITS:
		return m_uScaleBitsMinusCoordBits;
		break;

	case ATTR_COLORCOMPONENTBITS:
		return m_uColorComponentBits;
		break;

	case ATTR_USEFULLREQUESTHOST:
		return m_bUseFullRequestHost;
		break;

	case ATTR_PATHCOMPONENTS:
		return m_uPathComponents;
		break;

	case ATTR_NEWSCENEINDICATOR:
		return m_bNewSceneIndicator;
		break;

	case ATTR_TIMEENCODING:
		return m_uTimeEndcoding;
		break;

	case ATTR_POINTSCODECTYPE:
		return m_uPointsCodecType;
		break;

	case ATTR_ADAPTATIONTYPE:
		return m_uAdaptationType;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return 0;
}

UINT CLASeRHeader::GetUINTAttr(UINT uAttrType, lsrErr *pErr)
{
	switch (uAttrType)
	{
	case ATTR_TIMERESOLUTION:
		return m_uTimeResoluton;
		break;

	case ATTR_ADAPTATIONTYPEVALUE:
		return m_uAdaptationTypeValue;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return 0;
}

deque<UINT> CLASeRHeader::GetUINTListAttr(UINT uAttrType, lsrErr *pErr)
{
	switch (uAttrType)
	{
	case ATTR_ADAPTATIONCONSTRAINTS:
		return m_auAdaptationConstraints;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	deque<UINT> auTemp;
	return auTemp;
}