///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#include "laser_error.h"
#include "..\SAF.h"


CESHeader::CESHeader()
{
	CSAFUnit::CSAFUnit();

	m_strStreamID			= _T("");
	m_strDecSpecificInfo	= _T("");
	m_uStartOffset			= 0;
	m_uLength				= 0;
	m_bTransient			= 0;
	m_uBufferSize			= 0;
	m_uObjectTypeIndication	= 0;
	m_uStreamType			= 0;
	m_uTimeStampResolution	= 0;
	m_strContentType		= _T("");
}

/************************************************************************/
// SetAttr
// Desc		: function of 'ESHeader' attribute group to set the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 17)	- first release
/************************************************************************/
lsrErr CESHeader::SetAttrs(LPXNode pScene)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// ESHeader
	err = CSAFUnit::SetAttrs(pScene);
	CONFIRM(err);

	// streamID
	szTemp = pScene->GetAttrValue(_T("streamID"));
	if (szTemp)
	{
		m_strStreamID	= szTemp;
		m_bHasStreamID	= 1;
	}

	// objectTypeIndication
	szTemp = pScene->GetAttrValue(_T("objectTypeIndication"));
	if (szTemp)
		m_uObjectTypeIndication = (UINT8)XStr2Int(pScene->GetAttrValue(_T("objectTypeIndication")));

	// streamType
	szTemp = pScene->GetAttrValue(_T("streamType"));
	if (szTemp)
		m_uStreamType = (UINT8)XStr2Int(pScene->GetAttrValue(_T("streamType")));

	// timeStampResolution
	szTemp = pScene->GetAttrValue(_T("timeStampResolution"));
	if (szTemp)
		m_uStreamType = (UINT8)XStr2Int(pScene->GetAttrValue(_T("timeStampResolution")));

	// bufferSize
	m_uBufferSize = (UINT8)XStr2Int(pScene->GetAttrValue(_T("bufferSize")));

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'ESHeader' and it's children to get attribute value using it's name
// Param	: target attribute name, wanted value
// History
//	PSH (2008. 8. 29)	- first release
/************************************************************************/
UINT8 CESHeader::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_TRANSIENT:
		return m_bTransient;
		break;

	case ATTR_OBJECTTYPEINDICATION:
		return m_uObjectTypeIndication;
		break;

	case ATTR_STREAMTYPE:
		return m_uStreamType;
		break;

	default:
		uTemp = CSAFUnit::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		break;
	}

	return 0;
}

UINT CESHeader::GetUINTAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_STARTOFFSET:
		return m_uStartOffset;
		break;

	case ATTR_LENGTH:
		return m_uLength;
		break;

	case ATTR_BUFFERSIZE:
		return m_uBufferSize;
		break;

	case ATTR_TIMESTAMPRESOLUTION:
		return m_uTimeStampResolution;
		break;

	default:
		uTemp = CSAFUnit::GetUINTAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		break;
	}

	return 0;
}

tstring CESHeader::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	switch (uAttrType)
	{
	case ATTR_STREAMID:
		return m_strStreamID;
		break;

	case ATTR_DECSPECIFICINFO:
		return m_strDecSpecificInfo;
		break;

	case ATTR_CONTENTTYPE:
		return m_strContentType;
		break;

	default:
		strTemp = CSAFUnit::GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;

		break;
	}

	return _T("");
}