///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "laser_error.h"
#include "..\SAF.h"


CSAFSession::CSAFSession()
{
	CSAF::CSAF();

	m_uElementType		= SAF_SAFSession;

	m_pSceneHeader		= NULL;
	m_pEndOfSAFSession	= NULL;
}

CSAFSession::~CSAFSession()
{
	m_apMediaHeader.clear();
	m_apImageHeader.clear();
	m_apEndOfStream.clear();
	m_apSceneUnit.clear();
	m_apMediaUnit.clear();
	m_apImageUnit.clear();
	m_apCacheUnit.clear();
	m_apCacheUnit.clear();
}

/************************************************************************/
// SetAttr
// Desc		: function of 'SAFSession' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 7. 14)	- first release
/************************************************************************/
lsrErr CSAFSession::SetAttrs(LPXNode pScene)
{
	lsrErr err = LASeRNoErr;

	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// AddChildren
// Desc		: function to add the children
// Param	: new SAF element pointer
// History
//	PSH (2008. 7. 14)	- first release
/************************************************************************/
lsrErr CSAFSession::AddChildren(LPXNode pScene)
{
	lsrErr err = LASeRNoErr;

	m_uNumOfChildren = (UINT8)pScene->GetChildCount();

	for (UINT8 i = 0; i < m_uNumOfChildren; ++i)
	{
		CSAF *pNewSAFElement = NULL;

		// to make SAF Element
		err = CreateElement(pScene->GetChild(i), &pNewSAFElement);
		CONFIRM(err);

		pNewSAFElement->SetParent(this);
		pNewSAFElement->SetIndex(i);

		// to connect member variable to element
		if (pNewSAFElement != NULL)
		{
			switch (pNewSAFElement->GetElementType())
			{
			case SAF_SceneHeader:
				m_pSceneHeader = (CSceneHeader *)pNewSAFElement;
				break;

			case SAF_ImageHeader:
				m_apImageHeader.push_back((CImageHeader *)pNewSAFElement);
				break;

			case SAF_MediaHeader:
				m_apMediaHeader.push_back((CMediaHeader *)pNewSAFElement);
				break;

			case SAF_EndOfStream:
				m_apEndOfStream.push_back((CEndOfStream *)pNewSAFElement);
				break;

			case SAF_SceneUnit:
				m_apSceneUnit.push_back((CSceneUnit *)pNewSAFElement);
				break;

			case SAF_ImageUnit:
				m_apImageUnit.push_back((CImageUnit *)pNewSAFElement);
				break;

			case SAF_MediaUnit:
				m_apMediaUnit.push_back((CMediaUnit *)pNewSAFElement);
				break;

			case SAF_EndOfSAFSession:
				m_pEndOfSAFSession = (CEndOfSAFSession *)pNewSAFElement;
				err = LASeREOF;
				break;

			default :
				err = LASeRValidationErr;
				break;
			}

			m_apChildren.push_back(pNewSAFElement);
		}
	}

	// compare DOM tree to Scene tree
	if (m_apChildren.size() != m_uNumOfChildren)
		err = LASeRDOMTreeErr;

	return err;
}