///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "laser_error.h"
#include "..\SAF.h"


CSAFUnit::CSAFUnit()
{
	CSAF::CSAF();

	m_uTime			= 0x3FFFFFFF;				// 2^30 - 1
	m_strRef		= _T("");
	m_strSource		= _T("");
	m_uStartOffset	= 0;
	m_uLength		= 0;
	m_bRap			= 0;
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'sceneUnit' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 18)	- first release
/************************************************************************/
lsrErr CSAFUnit::SetAttrs(LPXNode pScene)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// time
	szTemp = pScene->GetAttrValue(_T("time"));
	if (szTemp)
		m_uTime = (UINT8)XStr2Int(pScene->GetAttrValue(_T("time")));

	// ref
	szTemp = pScene->GetAttrValue(_T("ref"));
	if (szTemp)
		m_strRef = szTemp;

	// source
	szTemp = pScene->GetAttrValue(_T("source"));
	if (szTemp)
		m_strSource = szTemp;
	/************************************************************************/

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'SAFUnit' and it's children to get attribute value using it's name
// Param	: target attribute name, wanted value
// History
//	PSH (2008. 8. 29)	- first release
/************************************************************************/
UINT8 CSAFUnit::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_RAP:
		return m_bRap;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return 0;
}

UINT CSAFUnit::GetUINTAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_TIME:
		return m_uTime;
		break;

	case ATTR_STARTOFFSET:
		return m_uStartOffset;
		break;

	case ATTR_LENGTH:
		return m_uLength;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return 0;
}

tstring CSAFUnit::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_REF:
		return m_strRef;
		break;

	case ATTR_SOURCE:
		return m_strSource;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return _T("");
}