///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "laser_error.h"
#include "..\SAF.h"


CSceneHeader::CSceneHeader()
{
	CESHeader::CESHeader();

	m_uElementType			= SAF_SceneHeader;

	m_uObjectTypeIndication	= 9;				// fixed
	m_uStreamType			= 3;				// fixed
	m_uTimeStampResolution	= 1000;				// default

	m_pLASeRHeader			= NULL;
}

CSceneHeader::~CSceneHeader()
{
	DELETEMEMORY(m_pLASeRHeader);
}

/************************************************************************/
// SetAttr
// Desc		: function of 'sceneHeader' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 7. 14)	- first release
//	PSH (2008. 7. 22)	- add setting parent & counting children line
//	PSH (2008. 7. 28)	- add setting attribute function
/************************************************************************/
lsrErr CSceneHeader::SetAttrs(LPXNode pScene)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// set attribute
	// ESHeader
	err = CESHeader::SetAttrs(pScene);
	CONFIRM(err);
	/************************************************************************/

	// to connect member variable to element
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// AddChildren
// Desc		: function to add the LASeR children
// Param	: new LASeR element pointer
// History
//	PSH (2008. 7. 14)	- first release
/************************************************************************/
lsrErr CSceneHeader::AddChildren(LPXNode pScene)
{
	lsrErr err = LASeRNoErr;

	m_uNumOfChildren = (UINT8)pScene->GetChildCount();

	if (m_uNumOfChildren != 1)					// must have only 1 child
	{
		err = LASeRValidationErr;
		return err;
	}

	CLASeRHeader *pNewLASeRElement = NULL;

	// to make LASeR Element
	err = CreateElement(pScene->GetChild((UINT)0), (CLASeR **)&pNewLASeRElement);
	CONFIRM(err);

	// to set the parent
	pNewLASeRElement->SetParent((void *)this, m_uElementType);
	pNewLASeRElement->SetIndex(0);

	err = pNewLASeRElement->SetAttrs(pScene, pNewLASeRElement);
	CONFIRM(err);

	if (pNewLASeRElement != NULL)
	{
		switch (pNewLASeRElement->GetElementType())
		{
		case LSR_LASeRHeader :
			m_pLASeRHeader = pNewLASeRElement;
			break;

		default :
			err = LASeRValidationErr;
			break;
		}			
	}

	else
		err = LASeRValidationErr;

	return err;
}