///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "laser_error.h"
#include "..\SAF.h"


CSceneUnit::CSceneUnit()
{
	CSAFUnit::CSAFUnit();

	m_uElementType			= SAF_SceneUnit;

	m_bResetEncodingContext	= 0;
}

CSceneUnit::~CSceneUnit()
{
	// memory deletion
	for (deque<CLASeRUnit *>::iterator i = m_apLASeRChildren.begin(); i < m_apLASeRChildren.end(); ++i)
	{DELETEMEMORY(*i);}

	m_apLASeRChildren.clear();
}

/************************************************************************/
// SetAttr
// Desc		: function of 'sceneUnit' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 7. 14)	- first release
//	PSH (2008. 7. 22)	- add setting parent & counting children line
/************************************************************************/
lsrErr CSceneUnit::SetAttrs(LPXNode pScene)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// set attribute
	// SAFUnit
	err = CSAFUnit::SetAttrs(pScene);
	CONFIRM(err);
	/************************************************************************/

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// AddChildren
// Desc		: function to add the LASeR children
// Param	: new LASeR element pointer
// History
//	PSH (2008. 7. 14)	- first release
//	PSH (2008. 7. 22)	- deletion of deque variable
/************************************************************************/
lsrErr CSceneUnit::AddChildren(LPXNode pScene)
{
	lsrErr err = LASeRNoErr;

	m_uNumOfChildren = (UINT8)pScene->GetChildCount();

	for (UINT8 i = 0; i < m_uNumOfChildren; ++i)
	{
		CLASeR *pNewLASeRElement = NULL;

		// to make LASeR Element
		err = CreateElement(pScene->GetChild(i), &pNewLASeRElement);
		CONFIRM(err);

		err = pNewLASeRElement->SetAttrs(pScene->GetChild(i), pNewLASeRElement);
		CONFIRM(err);

		// to connect member variable to element
		if (pNewLASeRElement != NULL)
		{
			if ((pNewLASeRElement->GetElementType()) >= LSR_StringIDTable
				&& (pNewLASeRElement->GetElementType()) <= LSR_AdaptiveUpdateGroup)
				m_apLASeRChildren.push_back((CLASeRUnit *)pNewLASeRElement);

			else
				err = LASeRValidationErr;
		}

		m_apLASeRChildren[i]->SetParent((void *)this, m_uElementType);
		m_apLASeRChildren[i]->SetIndex(i);
	}

	return err;
}