///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//

#ifndef _SVG_H_
#define _SVG_H_

#include "ComplexType.h"


class CSVGG : public CSVGObject
{
protected:
	CSVGProperties		*m_pProperties;
	CSVGBasic			*m_pBasic;
	CSVGLocatable		*m_pLocatable;
	CSVGFocus			*m_pFocus;

	CLSRAdaptationAttr	*m_pAdaptationAttr;

	UINT8				m_uFocusHighlight;
	UINT8				m_bExternalResourcesRequired;

public:
	CSVGG();
	virtual ~CSVGG();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	UINT			GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CSVGA : public CSVGG
{
protected:
	CSVGHref	*m_pHref;

	tstring		m_strTarget;

public:
	CSVGA();
	~CSVGA();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
};

// include animateColor
class CSVGAnimate : public CSVGObject
{
protected:
	CSVGSetAttr		*m_pSetAttr;
	CSVGHref		*m_pHref;
	CSVGAnimateAttr	*m_pAnimateAttr;

	UINT8			m_uAttributeName;

public:
	CSVGAnimate()	{/* must not be called*/}
	CSVGAnimate(UINT8 uElementType);
	~CSVGAnimate();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);
	deque<tstring>	GetStrListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
};

class CSVGAnimateMotion : public CSVGObject
{
protected:
	CSVGSetAttr		*m_pSetAttr;
	CSVGHref		*m_pHref;
	CSVGAnimateAttr	*m_pAnimateAttr;

	tstring			m_strPath;
	deque<float>	m_afKeyPoints;
	UINT8			m_uRotate;

public:
	CSVGAnimateMotion();
	~CSVGAnimateMotion();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);
	deque<tstring>	GetStrListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
};

class CSVGAnimateTransform : public CSVGObject
{
protected:
	CSVGSetAttr		*m_pSetAttr;
	CSVGHref		*m_pHref;
	CSVGAnimateAttr	*m_pAnimateAttr;

	UINT8			m_uAttributeName;
	UINT8			m_uType;

public:
	CSVGAnimateTransform();
	~CSVGAnimateTransform();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);
	deque<tstring>	GetStrListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
};

class CSVGAudio : public CSVGObject
{
protected:
	CSVGMediaProperties	*m_pMediaProperties;
	CSVGBasic			*m_pBasic;
	CSVGHref			*m_pHref;

	UINT8				m_bExternalResourcesRequired;
	deque<tstring>		m_astrBegin;
	deque<tstring>		m_astrEnd;
	float				m_fDur;
	float				m_fMin;
	float				m_fMax;
	float				m_fRepeatCount;
	float				m_fRepeatDur;
	UINT8				m_uSyncBehavior;
	float				m_fSyncTolerance;
	UINT8				m_bSyncMaster;
	tstring				m_strSyncReference;
	tstring				m_strType;
	UINT				m_uClipBegin;
	UINT				m_uClipEnd;
	UINT8				m_uRestart;

public:
	CSVGAudio();
	virtual ~CSVGAudio();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	UINT			GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<tstring>	GetStrListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CSVGCircle : public CSVGObject
{
protected:
	CSVGProperties	*m_pProperties;
	CSVGBasic		*m_pBasic;
	CSVGLocatable	*m_pLocatable;
	CSVGFocus		*m_pFocus;

	UINT8			m_uFocusHighlight;
	float			m_fCX;
	float			m_fCY;
	float			m_fR;

public:
	CSVGCircle();
	~CSVGCircle();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CSVGDefs : public CSVGObject
{
protected:
	CSVGProperties	*m_pProperties;

public:
	CSVGDefs();
	~CSVGDefs();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
};

// include metadata, mpath, title
class CSVGDesc : public CSVGObject
{
protected:
	tstring	m_strTextContent;

public:
	CSVGDesc()	{/* must not be called*/}
	CSVGDesc(UINT8 uElementType);

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr	CheckRightChild(UINT8 uElementType);

	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);

	UINT	GetAttrType(tstring strAttrName);
	UINT8	GetAttrValType(UINT uAttrType);

	lsrErr	SetAttr(UINT uAttrType, tstring strValue);
};

class CSVGEllipse : public CSVGObject
{
protected:
	CSVGProperties	*m_pProperties;
	CSVGBasic		*m_pBasic;
	CSVGLocatable	*m_pLocatable;
	CSVGFocus		*m_pFocus;

	UINT8			m_uFocusHighlight;
	float			m_fRX;
	float			m_fRY;
	float			m_fCX;
	float			m_fCY;

public:
	CSVGEllipse();
	~CSVGEllipse();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CSVGForeignObject : public CSVGObject
{
protected:
	CSVGProperties	*m_pProperties;
	CSVGBasic		*m_pBasic;
	CSVGFocus		*m_pFocus;
	CSVGHref		*m_pHref;

	UINT8			m_uFocusHighlight;
	UINT8			m_bExternalResourcesRequired;
	float			m_fWidth;					// required
	float			m_fHeight;					// required
	float			m_fX;
	float			m_fY;

public:
	CSVGForeignObject();
	~CSVGForeignObject();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CSVGImage : public CSVGObject
{
protected:
	CSVGBasic			*m_pBasic;
	CSVGLocatable		*m_pLocatable;
	CSVGFocus			*m_pFocus;
	CSVGMediaProperties	*m_pMediaProperties;
	CSVGHref			*m_pHref;

	UINT8				m_bExternalResourcesRequired;
	UINT8				m_uFocusHighlight;
	float				m_fWidth;
	float				m_fHeight;
	float				m_fX;
	float				m_fY;
	UINT8				m_uTransformBehavior;
	float				m_fOpacity;
	tstring				m_strType;
	UINT8				m_uPreserveAspectRatio;

public:
	CSVGImage();
	~CSVGImage();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CSVGLine : public CSVGObject
{
protected:
	CSVGProperties	*m_pProperties;
	CSVGBasic		*m_pBasic;
	CSVGLocatable	*m_pLocatable;
	CSVGFocus		*m_pFocus;

	UINT8			m_uFocusHighlight;
	float			m_fX1;
	float			m_fY1;
	float			m_fX2;						// required
	float			m_fY2;						// required

public:
	CSVGLine();
	~CSVGLine();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CSVGLinearGradient : public CSVGObject
{
protected:
	CSVGProperties	*m_pProperties;

	UINT8			m_uGradientUnits;
	float			m_fX1;
	float			m_fY1;
	float			m_fX2;
	float			m_fY2;

public:
	CSVGLinearGradient();
	~CSVGLinearGradient();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CEVListener : public CSVGObject
{
protected:
	UINT8	m_bEnabled;
	UINT8	m_uEvent;
	tstring	m_strEventParam;
	tstring	m_strHandler;
	tstring	m_strObserver;
	UINT8	m_uPhase;
	tstring	m_strTarget;
	UINT8	m_uPropagate;
	UINT8	m_uDefaultAction;

public:
	CEVListener();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr	CheckRightChild(UINT8 uElementType);

	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);

	UINT	GetAttrType(tstring strAttrName);
	UINT8	GetAttrValType(UINT uAttrType);

	lsrErr	SetAttr(UINT uAttrType, tstring strValue);
};

class CSVGMPath : public CSVGObject
{
protected:
	CSVGHref	*m_pHref;

public:
	CSVGMPath();
	~CSVGMPath();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr	CheckRightChild(UINT8 uElementType);

	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);

	UINT	GetAttrType(tstring strAttrName);
	UINT8	GetAttrValType(UINT uAttrType);

	lsrErr	SetAttr(UINT uAttrType, tstring strValue);
};

class CSVGPath : public CSVGObject
{
protected:
	CSVGProperties	*m_pProperties;
	CSVGBasic		*m_pBasic;
	CSVGLocatable	*m_pLocatable;
	CSVGFocus		*m_pFocus;

	UINT8			m_uFocusHighlight;
	tstring			m_strD;						// required
	float			m_fPathLength;

public:
	CSVGPath();
	~CSVGPath();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

// include polyline
class CSVGPolygon : public CSVGObject
{
protected:
	CSVGProperties	*m_pProperties;
	CSVGBasic		*m_pBasic;
	CSVGLocatable	*m_pLocatable;
	CSVGFocus		*m_pFocus;
	UINT8			m_uFocusHighlight;
	tstring			m_strPoints;
	deque<float>	m_afPoints;

public:
	CSVGPolygon()	{/* must not be called*/}
	CSVGPolygon(UINT8 uElementType);
	~CSVGPolygon();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
};

class CSVGRadialGradient : public CSVGObject
{
protected:
	CSVGProperties	*m_pProperties;

	UINT8			m_uGradientUnits;
	float			m_fCX;
	float			m_fCY;
	float			m_fR;

public:
	CSVGRadialGradient();
	~CSVGRadialGradient();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CSVGRect : public CSVGObject
{
protected:
	CSVGProperties	*m_pProperties;
	CSVGBasic		*m_pBasic;
	CSVGLocatable	*m_pLocatable;
	CSVGFocus		*m_pFocus;
	UINT8			m_uFocusHighlight;
	float			m_fWidth;
	float			m_fHeight;
	float			m_fX;
	float			m_fY;
	float			m_fRX;
	float			m_fRY;

public:
	CSVGRect();
	~CSVGRect();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CSVGScript : public CSVGObject
{
protected:
	CSVGHref	*m_pHref;

	UINT8		m_bExternalResourcesRequired;
	UINT8		m_uType;

	tstring		m_strTextContent;

public:
	CSVGScript();
	~CSVGScript();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr	CheckRightChild(UINT8 uElementType);

	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);

	UINT	GetAttrType(tstring strAttrName);
	UINT8	GetAttrValType(UINT uAttrType);

	lsrErr	SetAttr(UINT uAttrType, tstring strValue);
};

class CSVGSet : public CSVGObject
{
protected:
	CSVGSetAttr		*m_pSetAttr;
	CSVGHref		*m_pHref;

	UINT8			m_uAttributeName;

public:
	CSVGSet();
	~CSVGSet();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<tstring>	GetStrListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
};

class CSVGStop : public CSVGObject
{
protected:
	CSVGProperties	*m_pProperties;

	float			m_fOffset;

public:
	CSVGStop();
	~CSVGStop();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CSVG : public CSVGObject
{
protected:
	CSVGProperties		*m_pProperties;
	CSVGFocus			*m_pFocus;
	CLSRAdaptationAttr	*m_pAdaptationAttr;

	UINT8				m_uFocusHighlight;
	UINT8				m_bExternalResourcesRequired;
	float				m_fWidth;				// required
	float				m_fHeight;				// required
	deque<float>		m_afViewBox;
	UINT8				m_uPreserveAspectRatio;
	UINT8				m_uZoomAndPan;
	UINT8				m_uSyncBehaviorDefault;
	float				m_fSyncToleranceDefault;
	tstring				m_strBaseProfile;
	tstring				m_strContentScriptType;
	UINT8				m_uPlaybackOrder;
	float				m_fSnapshotTime;
	UINT8				m_uTimelineBegin;
	tstring				m_strVersion;
	UINT8				m_bFullscreen;

	UINT8				m_uWidthUnit;
	UINT8				m_uHeightUnit;

public:
	CSVG();
	~CSVG();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	UINT			GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
};

class CSVGSwitch : public CSVGObject
{
protected:
	CSVGProperties		*m_pProperties;
	CSVGBasic			*m_pBasic;
	CSVGLocatable		*m_pLocatable;
	CSVGFocus			*m_pFocus;
	CLSRAdaptationAttr	*m_pAdaptationAttr;

	UINT8				m_uFocusHighlight;
	UINT8				m_bExternalResourcesRequired;

	deque<UINT>			m_auCondition;

public:
	CSVGSwitch();
	~CSVGSwitch();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	UINT			GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);

	deque<UINT>		GetCondition()	{return m_auCondition;}
};

class CSVGTSpan : public CSVGObject
{
protected:
	CSVGProperties	*m_pProperties;
	CSVGBasic		*m_pBasic;
	CSVGFocus		*m_pFocus;

	UINT8			m_uFocusHighlight;

public:
	CSVGTSpan();
	virtual ~CSVGTSpan();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CSVGText : public CSVGTSpan
{
protected:
	CSVGLocatable	*m_pLocatable;

	UINT8			m_uEditable;
	float			m_fX;
	float			m_fY;
	deque<float>	m_afRotate;

	tstring			m_strTextContent;

public:
	CSVGText();
	~CSVGText();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CSVGUse : public CSVGObject
{
protected:
	CSVGProperties	*m_pProperties;
	CSVGBasic		*m_pBasic;
	CSVGLocatable	*m_pLocatable;
	CSVGHref		*m_pHref;
	CSVGFocus		*m_pFocus;
	UINT8			m_uFocusHighlight;
	UINT8			m_bExternalResourcesRequired;
	UINT8			m_uOverflow;
	float			m_fX;
	float			m_fY;

public:
	CSVGUse();
	~CSVGUse();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CSVGVideo : public CSVGAudio
{
protected:
	CSVGLocatable	*m_pLocatable;
	CSVGFocus		*m_pFocus;

	UINT8			m_uFocusHighlight;
	float			m_fWidth;
	float			m_fHeight;
	float			m_fX;
	float			m_fY;
	UINT8			m_uOverlay;
	UINT8			m_uTransformBehavior;
	UINT8			m_uInitialVisibility;
	UINT8			m_uPreserveAspectRatio;
	UINT8			m_bFullscreen;

public:
	CSVGVideo();
	~CSVGVideo();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	UINT			GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);
	deque<tstring>	GetStrListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

// amd1
class CSVGAnimation : public CSVGAudio
{
protected:
	CSVGFocus		*m_pFocus;
	CSVGLocatable	*m_pLocatable;

	UINT8			m_uFocusHighlight;
	float			m_fWidth;
	float			m_fHeight;
	float			m_fX;
	float			m_fY;
	UINT8			m_uPreserveAspectRatio;
	UINT8			m_uInitialVisibility;
	UINT8			m_uFill;

public:
	CSVGAnimation();
	~CSVGAnimation();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	UINT			GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);
	deque<tstring>	GetStrListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CSVGDiscard : public CSVGObject
{
protected:
	deque<tstring>	m_astrBegin;

	CSVGHref		*m_pHref;

public:
	CSVGDiscard();
	~CSVGDiscard();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<tstring>	GetStrListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
};

class CSVGTFont : public CSVGObject
{
protected:
	UINT8	m_bExternalResourcesRequired;
	float	m_fHorizAdvX;
	float	m_fHorizOriginX;

public:
	CSVGTFont();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr	CheckRightChild(UINT8 uElementType);

	float	GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);

	UINT	GetAttrType(tstring strAttrName);
	UINT8	GetAttrValType(UINT uAttrType);
};

class CSVGTFontFace : public CSVGObject
{
protected:
	float			m_fAccentHeight;
	float			m_fAlphabetic;
	float			m_fAscent;
	tstring			m_strBBox;
	float			m_fCapHeight;
	float			m_fDescent;
	UINT8			m_bExternalResourcesRequired;
	tstring			m_strFontFamily;
	tstring			m_strFontStretch;
	tstring			m_strFontStyle;
	tstring			m_strFontVariant;
	tstring			m_strFontWeight;
	float			m_fHanging;
	float			m_fIdeographic;
	float			m_fMathematical;
	float			m_fOverlinePosition;
	float			m_fOverlineThickness;
	deque<UINT8>	m_auPanose1;
	float			m_fSlope;
	float			m_fStemH;
	float			m_fStemV;
	float			m_fStrikeThroughPosition;
	float			m_fStrikeThroughThickness;
	float			m_fUnderlinePostion;
	float			m_fUnderlineThickness;
	tstring			m_strUnicodeRange;
	tstring			m_strUnitsPerEM;
	tstring			m_strWidths;
	tstring			m_strXHeight;

public:
	CSVGTFontFace();
	~CSVGTFontFace();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);
};

class CSVGTFontFaceSrc : public CSVGObject
{
public:
	CSVGTFontFaceSrc();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr	CheckRightChild(UINT8 uElementType);

	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);

	UINT	GetAttrType(tstring strAttrName);
	UINT8	GetAttrValType(UINT uAttrType);
};

class CSVGTFontFaceUri : public CSVGObject
{
protected:
	UINT8		m_bExternalResourcesRequired;

	CSVGHref	*m_pHref;

public:
	CSVGTFontFaceUri();
	~CSVGTFontFaceUri();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr	CheckRightChild(UINT8 uElementType);

	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);

	UINT	GetAttrType(tstring strAttrName);
	UINT8	GetAttrValType(UINT uAttrType);

	lsrErr	SetAttr(UINT uAttrType, tstring strValue);
};

class CSVGTGlyph : public CSVGObject
{
protected:
	tstring	m_strArabicForm;
	tstring	m_strD;
	tstring	m_strGlyphName;
	float	m_fHorizAdvX;
	tstring	m_strLang;
	tstring	m_strUnicode;
	UINT8	m_bExternalResourceRequired;

public:
	CSVGTGlyph();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr	CheckRightChild(UINT8 uElementType);

	float	GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);

	UINT	GetAttrType(tstring strAttrName);
	UINT8	GetAttrValType(UINT uAttrType);
};

class CSVGTHkern : public CSVGObject
{
protected:
	tstring m_strG1;
	tstring m_strG2;
	float	m_fK;
	tstring m_strU1;
	tstring m_strU2;

public:
	CSVGTHkern();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr	CheckRightChild(UINT8 uElementType);

	float	GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);

	UINT	GetAttrType(tstring strAttrName);
	UINT8	GetAttrValType(UINT uAttrType);
};

class CSVGHandler : public CSVGObject
{
protected:
	UINT8	m_bExternalResourceRequired;
	tstring	m_strType;
	UINT8	m_uEvent;

	tstring m_strTextContent;

public:
	CSVGHandler();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr	CheckRightChild(UINT8 uElementType);

	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);

	UINT	GetAttrType(tstring strAttrName);
	UINT8	GetAttrValType(UINT uAttrType);

	lsrErr	SetAttr(UINT uAttrType, tstring strValue);
};

class CSVGTMissingGlyph : public CSVGObject
{
protected:
	tstring	m_strD;
	float	m_fHorizAdvX;

public:
	CSVGTMissingGlyph();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr	CheckRightChild(UINT8 uElementType);

	float	GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);

	UINT	GetAttrType(tstring strAttrName);
	UINT8	GetAttrValType(UINT uAttrType);

	lsrErr	SetAttr(UINT uAttrType, tstring strValue);
};

class CSVGPrefetch : public CSVGObject
{
protected:
	CSVGHref	*m_pHref;

	UINT		m_uBandwidth;
	tstring		m_strMediaCharacterEncoding;
	tstring		m_strMediaContentEncodings;
	UINT		m_uMediaSize;
	UINT		m_uMediaTime;

public:
	CSVGPrefetch();
	~CSVGPrefetch();

	lsrErr	SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr	CheckRightChild(UINT8 uElementType);

	UINT8	GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	UINT	GetUINTAttr(UINT uAttrType, lsrErr *pErr);
	tstring	GetStrAttr(UINT uAttrType, lsrErr *pErr);

	UINT	GetAttrType(tstring strAttrName);
	UINT8	GetAttrValType(UINT uAttrType);

	lsrErr	SetAttr(UINT uAttrType, tstring strValue);
};

class CSVGSolidColor : public CSVGObject
{
protected:
	CSVGProperties	*m_pProperties;

public:
	CSVGSolidColor();
	~CSVGSolidColor();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

class CSVGTBreak : public CSVGObject
{
public:
	CSVGTBreak();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);

	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);
};

class CSVGTextArea : public CSVGObject
{
protected:
	CSVGProperties	*m_pProperties;
	CSVGFocus		*m_pFocus;
	CSVGBasic		*m_pBasic;
	CSVGLocatable	*m_pLocatable;

	UINT8			m_uFocusHighlight;
	UINT8			m_uEditable;
	float			m_fX;
	float			m_fY;
	float			m_fWidth;
	float			m_fHeight;

	tstring			m_strTextContent;

public:
	CSVGTextArea();
	~CSVGTextArea();

	lsrErr			SetAttrs(LPXNode pScene, CLASeR *pI);
	lsrErr			CheckRightChild(UINT8 uElementType);

	float			GetFloatAttr(UINT uAttrType, lsrErr *pErr);
	UINT8			GetUINT8Attr(UINT uAttrType, lsrErr *pErr);
	tstring			GetStrAttr(UINT uAttrType, lsrErr *pErr);
	deque<UINT8>	GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr);
	deque<float>	GetFloatListAttr(UINT uAttrType, lsrErr *pErr);

	UINT			GetAttrType(tstring strAttrName);
	UINT8			GetAttrValType(UINT uAttrType);

	lsrErr			SetAttr(UINT uAttrType, tstring strValue);
	lsrErr			SetAttr(UINT uAttrType, float fValue);
};

#endif